/*******************************************************************************

Author : Mauro Laurenti
Version : 1.4

Created on Date : 18/11/2015
Last update     : 31/01/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author)
is intended for use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable copyright laws.
All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws (Italian or International ones), as well as to
civil liability for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the following
software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This file contains all the configuration words needed for the PIC16F877.
It can be easily adapted to other PICs according to the available configurations.

******  WARNING ******

The configurations must be changed to reflect the application needs!

*******************************************************************************/

#ifndef PIC16F877_CONFIG_H
#define PIC16F877_CONFIG_H

#ifdef COMPILER_XC8
		#include <xc.h>
#endif


//******************************************************************************
//                          LTlib configurations
//******************************************************************************
#define IO_LIBRARY_SUPPORTED
#define UART_LIBRARY_SUPPORTED
#define SPI_LIBRARY_SUPPORTED
#define I2C_LIBRARY_SUPPORTED
#define ADC_LIBRARY_SUPPORTED
#define EEPROM_LIBRARY_SUPPORTED

//******************************************************************************
//                            Module Settings
//******************************************************************************

//******************************************
//                IO
//******************************************
#define NUMBER_OF_IO_PORTS 5

#define PULL_UP_RESISTORS_AVAILABLE
#define PULL_UP_ENABLE_BIT OPTION_REGbits.nRBPU

//******************************************
//                UART
//******************************************
#define UART1_AVAILABLE

#define UART1_TX_LINE_TRIS TRISCbits.TRISC6            
#define UART1_RX_LINE_TRIS TRISCbits.TRISC7 

//******************************************
//                SPI
//******************************************
#define SPI1_AVAILABLE

#define SPI1_SCK_LINE_TRIS TRISCbits.TRISC3             
#define SPI1_SDI_LINE_TRIS TRISCbits.TRISC4  
#define SPI1_SDO_LINE_TRIS TRISCbits.TRISC5 
#define SPI1_SS_LINE_TRIS TRISAbits.TRISA4

//******************************************
//                I2C
//******************************************
#define I2C1_AVAILABLE

#define I2C1_SDA_LINE_TRIS TRISCbits.TRISC4         
#define I2C1_SCL_LINE_TRIS TRISCbits.TRISC3  

//Register renaming required for PIC16 
#define SSPCON1 SSPCON
#define SSPCON1bits SSPCONbits 

//******************************************
//                ADC
//******************************************
#define ADC_MODULE_TYPE_4
#define ADC_AVAILABLE

#define ADC_WITH_NO_ANSEL_REG
#define ADC_NUMBER_OF_CHANNELS 8
#define ADC_ENABLE_BIT ADCON0bits.ADON
#define ADC_START_CONVERSION_BIT ADCON0bits.GO
#define ADC_CHANNEL_SELECTION_REGISTER ADCON0
#define ADC_BUFFER_HIGH ADRESH
#define ADC_BUFFER_LOW ADRESL
#define ADC_JUSTIFICATION ADCON1bits.ADFM
#define ADC_SETTINGS_0 ADCON0
#define ADC_SETTINGS_1 ADCON1 

#define ADC_CH0 0x00000001 
#define ADC_CH1 0x00000002 
#define ADC_CH2 0x00000004
#define ADC_CH3 0x00000008
#define ADC_CH4 0x00000010
#define ADC_CH5 0x00000020
#define ADC_CH6 0x00000040
#define ADC_CH7 0x00000080

//******************************************
//                EEPROM
//******************************************
#define EEPROM_AVAILABLE

#define EEPROM_MODULE_SIZE 0xFF             


//******************************************************************************
// MCU configurations
//******************************************************************************
#ifndef MCC_USED
	#pragma config FOSC = HS        // Oscillator Selection bits (HS oscillator)
	#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
	#pragma config PWRTE = ON       // Power-up Timer Enable bit (PWRT enabled)
	#pragma config CP = OFF         // FLASH Program Memory Code Protection bits (Code protection off)
	#pragma config BOREN = ON       // Brown-out Reset Enable bit (BOR enabled)
	#pragma config LVP = OFF        // Low Voltage In-Circuit Serial Programming Enable bit (RB3 is digital I/O, HV on MCLR must be used for programming)
	#pragma config CPD = OFF        // Data EE Memory Code Protection (Code Protection off)
	#pragma config WRT = OFF        // FLASH Program Memory Write Enable (Unprotected program memory may not be written to by EECON control)
#endif
#endif
