/*******************************************************************************

Author : Mauro Laurenti
Version : 1.4

Created on Date : 19/06/2013
Last update     : 31/01/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author)
is intended for use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable copyright laws.
All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws (Italian or International ones), as well as to
civil liability for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the following
software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This file contains all the configuration words needed for the PIC18F14K50.
It can be easily adapted to other PICs according to the available configurations.

******  WARNING ******

The configurations must be changed to reflect the application needs!

*****************************************************************************/

#ifndef PIC18F14K50_CONFIG_H
#define PIC18F14K50_CONFIG_H

#ifdef COMPILER_XC8
	#include <xc.h>
#endif

//******************************************************************************
// LTlib configurations
//******************************************************************************
#define IO_LIBRARY_SUPPORTED
#define UART_LIBRARY_SUPPORTED
#define SPI_LIBRARY_SUPPORTED
#define I2C_LIBRARY_SUPPORTED
#define EEPROM_LIBRARY_SUPPORTED
#define ADC_LIBRARY_SUPPORTED


//******************************************************************************
//                            Module Settings
//******************************************************************************

//******************************************
//                IO
//******************************************
#define NUMBER_OF_IO_PORTS 3

#define PULL_UP_RESISTORS_AVAILABLE
#define PULL_UP_ENABLE_BIT INTCON2bits.RABPU

//******************************************
//                UART
//******************************************
#define UART1_AVAILABLE

#define UART1_TX_LINE_TRIS TRISBbits.RB7              
#define UART1_RX_LINE_TRIS TRISBbits.RB5  

//******************************************
//                SPI
//******************************************
#define SPI1_AVAILABLE

#define SPI1_SCK_LINE_TRIS TRISBbits.RB6             
#define SPI1_SDI_LINE_TRIS TRISBbits.RB4  
#define SPI1_SDO_LINE_TRIS TRISCbits.RC7   
#define SPI1_SS_LINE_TRIS TRISCbits.RC6  

//******************************************
//                I2C
//******************************************
#define I2C1_AVAILABLE

#define I2C1_SDA_LINE_TRIS TRISBbits.RB4              
#define I2C1_SCL_LINE_TRIS TRISBbits.RB6 

//******************************************
//                ADC
//******************************************
#define ADC_MODULE_TYPE_3
#define ADC_AVAILABLE

#define ADC_WITH_ANSEL_2_REG

#define ADC_NUMBER_OF_CHANNELS 12
#define ADC_ENABLE_BIT ADCON0bits.ADON
#define ADC_START_CONVERSION_BIT ADCON0bits.GO
#define ADC_CHANNEL_SELECTION_REGISTER ADCON0
#define ADC_BUFFER_HIGH ADRESH
#define ADC_BUFFER_LOW ADRESL
#define ADC_JUSTIFICATION ADCON2bits.ADFM
#define ADC_SETTINGS_0 ADCON0
#define ADC_SETTINGS_1 ADCON1 
#define ADC_SETTINGS_2 ADCON2 

// CH0, CH1 and CH2 are not implemented
#define ADC_CH0 0x00000001 
#define ADC_CH1 0x00000002 
#define ADC_CH2 0x00000004

#define ADC_CH3 0x00000008
#define ADC_CH4 0x00000010
#define ADC_CH5 0x00000020
#define ADC_CH6 0x00000040
#define ADC_CH7 0x00000080

#define ADC_CH8  0x00000100
#define ADC_CH9  0x00000200
#define ADC_CH10 0x00000400
#define ADC_CH11 0x00000800

//******************************************
//                EEPROM
//******************************************
#define EEPROM_AVAILABLE

#define EEPROM_MODULE_SIZE 0xFF  


//******************************************************************************
// MCU configuration
//******************************************************************************

#ifndef MCC_USED
	// CONFIG1L
	#pragma config CPUDIV = NOCLKDIV// CPU System Clock Selection bits (No CPU System Clock divide)
	#pragma config USBDIV = OFF     // USB Clock Selection bit (USB clock comes directly from the OSC1/OSC2 oscillator block; no divide)

	// CONFIG1H
	#pragma config FOSC = HS        // Oscillator Selection bits (HS oscillator)
	#pragma config PLLEN = ON       // 4 X PLL Enable bit (Oscillator multiplied by 4)
	#pragma config PCLKEN = ON      // Primary Clock Enable bit (Primary clock enabled)
	#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enable (Fail-Safe Clock Monitor disabled)
	#pragma config IESO = OFF       // Internal/External Oscillator Switchover bit (Oscillator Switchover mode disabled)

	// CONFIG2L
	#pragma config PWRTEN = OFF     // Power-up Timer Enable bit (PWRT disabled)
	#pragma config BOREN = SBORDIS  // Brown-out Reset Enable bits (Brown-out Reset enabled in hardware only (SBOREN is disabled))
	#pragma config BORV = 19        // Brown-out Reset Voltage bits (VBOR set to 1.9 V nominal)

	// CONFIG2H
	#pragma config WDTEN = OFF      // Watchdog Timer Enable bit (WDT is controlled by SWDTEN bit of the WDTCON register)
	#pragma config WDTPS = 32768    // Watchdog Timer Postscale Select bits (1:32768)

	// CONFIG3H
	#pragma config HFOFST = OFF     // HFINTOSC Fast Start-up bit (The system clock is held off until the HFINTOSC is stable.)
	#pragma config MCLRE = OFF      // MCLR Pin Enable bit (RA3 input pin enabled; MCLR disabled)

	// CONFIG4L
	#pragma config STVREN = ON      // Stack Full/Underflow Reset Enable bit (Stack full/underflow will cause Reset)
	#pragma config LVP = OFF        // Single-Supply ICSP Enable bit (Single-Supply ICSP disabled)
	#pragma config BBSIZ = OFF      // Boot Block Size Select bit (1kW boot block size)
	#pragma config XINST = OFF      // Extended Instruction Set Enable bit (Instruction set extension and Indexed Addressing mode disabled (Legacy mode))

	// CONFIG5L
	#pragma config CP0 = OFF        // Code Protection bit (Block 0 not code-protected)
	#pragma config CP1 = OFF        // Code Protection bit (Block 1 not code-protected)

	// CONFIG5H
	#pragma config CPB = OFF        // Boot Block Code Protection bit (Boot block not code-protected)
	#pragma config CPD = OFF        // Data EEPROM Code Protection bit (Data EEPROM not code-protected)

	// CONFIG6L
	#pragma config WRT0 = OFF       // Table Write Protection bit (Block 0 not write-protected)
	#pragma config WRT1 = OFF       // Table Write Protection bit (Block 1 not write-protected)

	// CONFIG6H
	#pragma config WRTC = OFF       // Configuration Register Write Protection bit (Configuration registers not write-protected)
	#pragma config WRTB = OFF       // Boot Block Write Protection bit (Boot block not write-protected)
	#pragma config WRTD = OFF       // Data EEPROM Write Protection bit (Data EEPROM not write-protected)

	// CONFIG7L
	#pragma config EBTR0 = OFF      // Table Read Protection bit (Block 0 not protected from table reads executed in other blocks)
	#pragma config EBTR1 = OFF      // Table Read Protection bit (Block 1 not protected from table reads executed in other blocks)

	// CONFIG7H
	#pragma config EBTRB = OFF      // Boot Block Table Read Protection bit (Boot block not protected from table reads executed in other blocks)
   
#endif
#endif
