/*******************************************************************************

Author : Mauro Laurenti
Version : 1.5

Created on Date : 18/11/2015
Last update     : 31/01/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author)
is intended for use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable copyright laws.
All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws (Italian or International ones), as well as to
civil liability for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the following
software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This file contains all the configuration words needed for the PIC18F4580.
It can be easily adapted to other PICs according to the available configurations.

******  WARNING ******

The configurations must be changed to reflect the application needs!

*******************************************************************************/


#ifndef PIC18F4580_CONFIG_H
#define PIC18F4580_CONFIG_H

#ifdef COMPILER_XC8
	#include <xc.h>
#endif


//******************************************************************************
// LTlib configurations
//******************************************************************************
#define IO_LIBRARY_SUPPORTED
#define UART_LIBRARY_SUPPORTED
#define SPI_LIBRARY_SUPPORTED
#define I2C_LIBRARY_SUPPORTED
#define EEPROM_LIBRARY_SUPPORTED
#define CAN_LIBRARY_SUPPORTED
#define ADC_LIBRARY_SUPPORTED
#define FLASH_LIBRARY_SUPPORTED


//******************************************************************************
//                            Module Settings
//******************************************************************************

//******************************************
//                IO
//******************************************
#define NUMBER_OF_IO_PORTS 5

#define PULL_UP_RESISTORS_AVAILABLE
#define PULL_UP_ENABLE_BIT INTCON2bits.RBPU

//******************************************
//                UART
//******************************************
#define UART1_AVAILABLE

#define UART1_TX_LINE_TRIS TRISCbits.RC6            
#define UART1_RX_LINE_TRIS TRISCbits.RC7 

//******************************************
//                SPI
//******************************************
#define SPI1_AVAILABLE

#define SPI1_SCK_LINE_TRIS TRISCbits.RC3             
#define SPI1_SDI_LINE_TRIS TRISCbits.RC4  
#define SPI1_SDO_LINE_TRIS TRISCbits.RC5   
#define SPI1_SS_LINE_TRIS TRISAbits.RA5 

//******************************************
//                I2C
//******************************************
#define I2C1_AVAILABLE

#define I2C1_SDA_LINE_TRIS TRISCbits.RC4              
#define I2C1_SCL_LINE_TRIS TRISCbits.RC3  

//******************************************
//                CAN
//******************************************
#define CAN1_AVAILABLE

#define CAN1_TX_LINE_TRIS TRISBbits.RB2             
#define CAN1_RX_LINE_TRIS TRISBbits.RB3  

//******************************************
//                ADC
//******************************************
#define ADC_MODULE_TYPE_1
#define ADC_AVAILABLE

#define ADC_WITH_NO_ANSEL_REG

#define ADC_NUMBER_OF_CHANNELS 11
#define ADC_ENABLE_BIT ADCON0bits.ADON
#define ADC_START_CONVERSION_BIT ADCON0bits.GO
#define ADC_CHANNEL_SELECTION_REGISTER ADCON0
#define ADC_BUFFER_HIGH ADRESH
#define ADC_BUFFER_LOW ADRESL
#define ADC_JUSTIFICATION ADCON2bits.ADFM
#define ADC_SETTINGS_0 ADCON0
#define ADC_SETTINGS_1 ADCON1 
#define ADC_SETTINGS_2 ADCON2 

#define ADC_CH0 0x00000001 
#define ADC_CH1 0x00000002 
#define ADC_CH2 0x00000004
#define ADC_CH3 0x00000008
#define ADC_CH4 0x00000010
#define ADC_CH5 0x00000020
#define ADC_CH6 0x00000040
#define ADC_CH7 0x00000080

#define ADC_CH8  0x00000100
#define ADC_CH9  0x00000200
#define ADC_CH10 0x00000400

//These ADC Channels are not implemented
#define ADC_CH11 0x00000000
#define ADC_CH12 0x00000000

//******************************************
//                EEPROM
//******************************************
#define EEPROM_AVAILABLE

#define EEPROM_MODULE_SIZE 0xFF

//******************************************
//                FLASH
//******************************************
#define FLASH_AVAILABLE

#define FLASH_ERASE_BLOCK 64           
#define FLASH_WRITE_BLOCK 8   

#define DEVICE_ID_ADDRESS 0x3FFFFE
#define DEVICE_ID_BYTES 0x02

#define DEVICE_REVISION_ADDRESS 0x3FFFFE
#define DEVICE_REVISION_BYTES 0x01


//******************************************************************************
// MCU configurations
//******************************************************************************
#ifndef MCC_USED
	// CONFIG1H
	#pragma config OSC = HS         // Oscillator Selection bits (HS oscillator)
	#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enable bit (Fail-Safe Clock Monitor disabled)
	#pragma config IESO = OFF       // Internal/External Oscillator Switchover bit (Oscillator Switchover mode disabled)

	// CONFIG2L
	#pragma config PWRT = OFF       // Power-up Timer Enable bit (PWRT disabled)
	#pragma config BOREN = OFF      // Brown-out Reset Enable bits (Brown-out Reset disabled in hardware and software)
	#pragma config BORV = 1         // Brown-out Reset Voltage bits (VBOR set to 4.3V)

	// CONFIG2H
	#pragma config WDT = OFF        // Watchdog Timer Enable bit (WDT disabled (control is placed on the SWDTEN bit))
	#pragma config WDTPS = 32768    // Watchdog Timer Postscale Select bits (1:32768)

	// CONFIG3H
	#pragma config PBADEN = OFF     // PORTB A/D Enable bit (PORTB<4:0> pins are configured as digital I/O on Reset)
	#pragma config LPT1OSC = OFF    // Low-Power Timer 1 Oscillator Enable bit (Timer1 configured for higher power operation)
	#pragma config MCLRE = ON       // MCLR Pin Enable bit (MCLR pin enabled; RE3 input pin disabled)

	// CONFIG4L
	#pragma config STVREN = ON      // Stack Full/Underflow Reset Enable bit (Stack full/underflow will cause Reset)
	#pragma config LVP = OFF        // Single-Supply ICSP Enable bit (Single-Supply ICSP disabled)
	#pragma config BBSIZ = 1024     // Boot Block Size Select bit (1K words (2K bytes) boot block)
	#pragma config XINST = OFF      // Extended Instruction Set Enable bit (Instruction set extension and Indexed Addressing mode disabled (Legacy mode))

	// CONFIG5L
	#pragma config CP0 = OFF        // Code Protection bit (Block 0 (000800-001FFFh) not code-protected)
	#pragma config CP1 = OFF        // Code Protection bit (Block 1 (002000-003FFFh) not code-protected)
	#pragma config CP2 = OFF        // Code Protection bit (Block 2 (004000-005FFFh) not code-protected)
	#pragma config CP3 = OFF        // Code Protection bit (Block 3 (006000-007FFFh) not code-protected)

	// CONFIG5H
	#pragma config CPB = OFF        // Boot Block Code Protection bit (Boot block (000000-0007FFh) not code-protected)
	#pragma config CPD = OFF        // Data EEPROM Code Protection bit (Data EEPROM not code-protected)

	// CONFIG6L
	#pragma config WRT0 = OFF       // Write Protection bit (Block 0 (000800-001FFFh) not write-protected)
	#pragma config WRT1 = OFF       // Write Protection bit (Block 1 (002000-003FFFh) not write-protected)
	#pragma config WRT2 = OFF       // Write Protection bit (Block 2 (004000-005FFFh) not write-protected)
	#pragma config WRT3 = OFF       // Write Protection bit (Block 3 (006000-007FFFh) not write-protected)

	// CONFIG6H
	#pragma config WRTC = OFF       // Configuration Register Write Protection bit (Configuration registers (300000-3000FFh) not write-protected)
	#pragma config WRTB = OFF       // Boot Block Write Protection bit (Boot block (000000-0007FFh) not write-protected)
	#pragma config WRTD = OFF       // Data EEPROM Write Protection bit (Data EEPROM not write-protected)

	// CONFIG7L
	#pragma config EBTR0 = OFF      // Table Read Protection bit (Block 0 (000800-001FFFh) not protected from table reads executed in other blocks)
	#pragma config EBTR1 = OFF      // Table Read Protection bit (Block 1 (002000-003FFFh) not protected from table reads executed in other blocks)
	#pragma config EBTR2 = OFF      // Table Read Protection bit (Block 2 (004000-005FFFh) not protected from table reads executed in other blocks)
	#pragma config EBTR3 = OFF      // Table Read Protection bit (Block 3 (006000-007FFFh) not protected from table reads executed in other blocks)

	// CONFIG7H
	#pragma config EBTRB = OFF      // Boot Block Table Read Protection bit (Boot block (000000-0007FFh) not protected from table reads executed in other blocks)
#endif
#endif
