/*******************************************************************************

Author : Mauro Laurenti
Version : 1.2
Created on Date : 04/02/2018
Last update     : 31/01/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author)
is intended for use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable copyright laws.
All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws (Italian or International ones), as well as to
civil liability for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the following
software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This file contains all the configuration words needed for the PIC18F45K22.
It can be easily adapted to other PICs according to the available configurations.

******  WARNING ******

The configurations must be changed to reflect the application needs!

*******************************************************************************/

#ifndef PIC18F45K22_CONFIG_H
#define PIC18F45K22_CONFIG_H

#ifdef COMPILER_XC8
	#include <xc.h>
#endif

//******************************************************************************
// LTlib configurations
//******************************************************************************

#define IO_LIBRARY_SUPPORTED
#define UART_LIBRARY_SUPPORTED
#define SPI_LIBRARY_SUPPORTED
#define I2C_LIBRARY_SUPPORTED
#define EEPROM_LIBRARY_SUPPORTED
#define ADC_LIBRARY_SUPPORTED
#define FLASH_LIBRARY_SUPPORTED


//******************************************************************************
//                            Module Settings
//******************************************************************************

//******************************************
//                IO
//******************************************
#define NUMBER_OF_IO_PORTS 5    

#define PULL_UP_RESISTORS_AVAILABLE
#define PULL_UP_ENABLE_BIT INTCON2bits.RBPU
#define PULL_UP_SINGLE_BIT_ENABLE
#define PULL_UP_ENABLE_REGISTER_B WPUB

//******************************************
//                UART
//******************************************
#define UART1_AVAILABLE
#define UART2_AVAILABLE

#define UART1_TX_LINE_TRIS TRISCbits.RC6              
#define UART1_RX_LINE_TRIS TRISCbits.RC7              
            
#define UART2_TX_LINE_TRIS TRISDbits.RD6              
#define UART2_RX_LINE_TRIS TRISDbits.RD7  

//******************************************
//                SPI
//******************************************
#define SPI1_AVAILABLE
#define SPI2_AVAILABLE

#define SPI1_SCK_LINE_TRIS TRISCbits.RC3             
#define SPI1_SDI_LINE_TRIS TRISCbits.RC4  
#define SPI1_SDO_LINE_TRIS TRISCbits.RC5   
#define SPI1_SS_LINE_TRIS TRISAbits.RA5  

#define SPI2_SCK_LINE_TRIS TRISDbits.RD0             
#define SPI2_SDI_LINE_TRIS TRISDbits.RD1   
#define SPI2_SDO_LINE_TRIS TRISDbits.RD4  
#define SPI2_SS_LINE_TRIS TRISDbits.RD3 

//******************************************
//                I2C
//******************************************
#define I2C1_AVAILABLE
#define I2C2_AVAILABLE

#define I2C1_SDA_LINE_TRIS TRISCbits.RC4              
#define I2C1_SCL_LINE_TRIS TRISCbits.RC3               
            
#define I2C2_SDA_LINE_TRIS TRISDbits.RD1              
#define I2C2_SCL_LINE_TRIS TRISDbits.RD0  

//******************************************
//                ADC
//******************************************
#define ADC_MODULE_TYPE_2
#define ADC_AVAILABLE

#define ADC_WITH_ANSEL_REG

#define ADC_NUMBER_OF_CHANNELS 28
#define ADC_ENABLE_BIT ADCON0bits.ADON
#define ADC_START_CONVERSION_BIT ADCON0bits.GO
#define ADC_CHANNEL_SELECTION_REGISTER ADCON0
#define ADC_BUFFER_HIGH ADRESH
#define ADC_BUFFER_LOW ADRESL
#define ADC_JUSTIFICATION ADCON2bits.ADFM
#define ADC_SETTINGS_0 ADCON0
#define ADC_SETTINGS_1 ADCON1 
#define ADC_SETTINGS_2 ADCON2 

#define ADC_CH0 0x00000001 
#define ADC_CH1 0x00000002 
#define ADC_CH2 0x00000004
#define ADC_CH3 0x00000008
#define ADC_CH4 0x00000010
#define ADC_CH5 0x00000020
#define ADC_CH6 0x00000040
#define ADC_CH7 0x00000080

#define ADC_CH8  0x00000100
#define ADC_CH9  0x00000200
#define ADC_CH10 0x00000400
#define ADC_CH11 0x00000800
#define ADC_CH12 0x00001000
#define ADC_CH13 0x00002000
#define ADC_CH14 0x00004000
#define ADC_CH15 0x00008000

#define ADC_CH16 0x00010000
#define ADC_CH17 0x00020000
#define ADC_CH18 0x00040000
#define ADC_CH19 0x00080000
#define ADC_CH20 0x00100000
#define ADC_CH21 0x00200000
#define ADC_CH22 0x00400000
#define ADC_CH23 0x00800000

#define ADC_CH24 0x01000000
#define ADC_CH25 0x02000000
#define ADC_CH26 0x04000000
#define ADC_CH27 0x08000000

//******************************************
//                EEPROM
//******************************************
#define EEPROM_AVAILABLE

#define EEPROM_MODULE_SIZE 0x3FF 

//******************************************
//                FLASH
//******************************************
#define FLASH_AVAILABLE

#define FLASH_ERASE_BLOCK 64           
#define FLASH_WRITE_BLOCK 64  

#define DEVICE_ID_ADDRESS 0x3FFFFE
#define DEVICE_ID_BYTES 0x02

#define DEVICE_REVISION_ADDRESS 0x3FFFFE
#define DEVICE_REVISION_BYTES 0x01


//******************************************************************************
// MCU configurations
//******************************************************************************

#ifndef MCC_USED
	// CONFIG1H
	#pragma config FOSC = HSHP      // Oscillator Selection bits (HS oscillator (high power > 16 MHz))
	#pragma config PLLCFG = OFF     // 4X PLL Enable (Oscillator used directly)
	#pragma config PRICLKEN = ON    // Primary clock enable bit (Primary clock is always enabled)
	#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enable bit (Fail-Safe Clock Monitor disabled)
	#pragma config IESO = OFF       // Internal/External Oscillator Switchover bit (Oscillator Switchover mode disabled)

	// CONFIG2L
	#pragma config PWRTEN = OFF     // Power-up Timer Enable bit (Power up timer disabled)
	#pragma config BOREN = SBORDIS  // Brown-out Reset Enable bits (Brown-out Reset enabled in hardware only (SBOREN is disabled))
	#pragma config BORV = 190       // Brown Out Reset Voltage bits (VBOR set to 1.90 V nominal)

	// CONFIG2H
	#pragma config WDTEN = OFF      // Watchdog Timer Enable bits (Watch dog timer is always disabled. SWDTEN has no effect.)
	#pragma config WDTPS = 32768    // Watchdog Timer Postscale Select bits (1:32768)

	// CONFIG3H
	#pragma config CCP2MX = PORTC1  // CCP2 MUX bit (CCP2 input/output is multiplexed with RC1)
	#pragma config PBADEN = OFF     // PORTB A/D Enable bit (PORTB<5:0> pins are configured as digital I/O on Reset)
	#pragma config CCP3MX = PORTE0  // P3A/CCP3 Mux bit (P3A/CCP3 input/output is mulitplexed with RE0)
	#pragma config HFOFST = OFF     // HFINTOSC Fast Start-up (HFINTOSC output and ready status are delayed by the oscillator stable status)
	#pragma config T3CMX = PORTC0   // Timer3 Clock input mux bit (T3CKI is on RC0)
	#pragma config P2BMX = PORTD2   // ECCP2 B output mux bit (P2B is on RD2)
	#pragma config MCLRE = EXTMCLR  // MCLR Pin Enable bit (MCLR pin enabled, RE3 input pin disabled)

	// CONFIG4L
	#pragma config STVREN = ON      // Stack Full/Underflow Reset Enable bit (Stack full/underflow will cause Reset)
	#pragma config LVP = ON         // Single-Supply ICSP Enable bit (Single-Supply ICSP enabled if MCLRE is also 1)
	#pragma config XINST = OFF      // Extended Instruction Set Enable bit (Instruction set extension and Indexed Addressing mode disabled (Legacy mode))

	// CONFIG5L
	#pragma config CP0 = OFF        // Code Protection Block 0 (Block 0 (000800-003FFFh) not code-protected)
	#pragma config CP1 = OFF        // Code Protection Block 1 (Block 1 (004000-007FFFh) not code-protected)
	#pragma config CP2 = OFF        // Code Protection Block 2 (Block 2 (008000-00BFFFh) not code-protected)
	#pragma config CP3 = OFF        // Code Protection Block 3 (Block 3 (00C000-00FFFFh) not code-protected)

	// CONFIG5H
	#pragma config CPB = OFF        // Boot Block Code Protection bit (Boot block (000000-0007FFh) not code-protected)
	#pragma config CPD = OFF        // Data EEPROM Code Protection bit (Data EEPROM not code-protected)

	// CONFIG6L
	#pragma config WRT0 = OFF       // Write Protection Block 0 (Block 0 (000800-003FFFh) not write-protected)
	#pragma config WRT1 = OFF       // Write Protection Block 1 (Block 1 (004000-007FFFh) not write-protected)
	#pragma config WRT2 = OFF       // Write Protection Block 2 (Block 2 (008000-00BFFFh) not write-protected)
	#pragma config WRT3 = OFF       // Write Protection Block 3 (Block 3 (00C000-00FFFFh) not write-protected)

	// CONFIG6H
	#pragma config WRTC = OFF       // Configuration Register Write Protection bit (Configuration registers (300000-3000FFh) not write-protected)
	#pragma config WRTB = OFF       // Boot Block Write Protection bit (Boot Block (000000-0007FFh) not write-protected)
	#pragma config WRTD = OFF       // Data EEPROM Write Protection bit (Data EEPROM not write-protected)

	// CONFIG7L
	#pragma config EBTR0 = OFF      // Table Read Protection Block 0 (Block 0 (000800-003FFFh) not protected from table reads executed in other blocks)
	#pragma config EBTR1 = OFF      // Table Read Protection Block 1 (Block 1 (004000-007FFFh) not protected from table reads executed in other blocks)
	#pragma config EBTR2 = OFF      // Table Read Protection Block 2 (Block 2 (008000-00BFFFh) not protected from table reads executed in other blocks)
	#pragma config EBTR3 = OFF      // Table Read Protection Block 3 (Block 3 (00C000-00FFFFh) not protected from table reads executed in other blocks)

	// CONFIG7H
	#pragma config EBTRB = OFF      // Boot Block Table Read Protection bit (Boot Block (000000-0007FFh) not protected from table reads executed in other blocks)
#endif
#endif
