/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0

Created on Date : 23/02/2023
Last update     : 23/02/2023

CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author)
is intended for use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable copyright laws.
All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws (Italian or International ones), as well as to
civil liability for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the following
software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This file contains all the configuration words needed for the PIC24FJ128GA204.
It can be easily adapted to other PICs according to the available configurations.

******  WARNING ******

The configurations must be changed to reflect the application needs!

*******************************************************************************/

#ifndef PIC24FJ128GA204_CONFIG_H
#define PIC24FJ128GA204_CONFIG_H

#ifdef COMPILER_XC16
	#include <xc.h>
#endif


//******************************************************************************
// LTlib configurations
//******************************************************************************
#define IO_LIBRARY_SUPPORTED
#define UART_LIBRARY_SUPPORTED
#define SPI_LIBRARY_SUPPORTED
#define I2C_LIBRARY_SUPPORTED
#define ADC_LIBRARY_SUPPORTED
//#define FLASH_LIBRARY_SUPPORTED

//******************************************************************************
// Module Settings
//******************************************************************************

//******************************************
//                IO
//******************************************
#define NUMBER_OF_IO_PORTS 3


//******************************************
//                UART
//******************************************
#define UART1_AVAILABLE
//#define UART2_AVAILABLE
//#define UART3_AVAILABLE
//#define UART4_AVAILABLE

#define UART1_RX_IDLE_STATE U1MODEbits.URXINV
#define UART2_RX_IDLE_STATE U2MODEbits.URXINV
#define UART3_RX_IDLE_STATE U3MODEbits.URXINV
#define UART4_RX_IDLE_STATE U4MODEbits.URXINV

#define UART1_RX_EN U1STAbits.URXEN
#define UART2_RX_EN U2STAbits.URXEN
#define UART3_RX_EN U3STAbits.URXEN
#define UART4_RX_EN U4STAbits.URXEN


//UART 1 pin mux 
#ifndef UART1_TX_LINE
    #define    UART1_TX_LINE RPOR5bits.RP11R = 0x0003;  //RB11     
#endif        

#ifndef UART1_RX_LINE
    #define    UART1_RX_LINE RPINR18bits.U1RXR = 0x0009;  //RB10
#endif        


//UART 2 pin mux 
#ifndef UART2_TX_LINE
    #define    UART2_TX_LINE RPOR5bits.RP11R = 0x0003;  //RB11   
#endif 

#ifndef UART2_RX_LINE
    #define    UART2_RX_LINE RPINR18bits.U1RXR;  //RB10
#endif    

//UART 3 pin mux 
#ifndef UART3_TX_LINE
    #define    UART3_TX_LINE RPOR5bits.RP11R = 0x0003;  //RB11   
#endif 

#ifndef UART3_RX_LINE
    #define    UART3_RX_LINE RPINR18bits.U1RXR;  //RB10
#endif   

//UART 4 pin mux 
#ifndef UART4_TX_LINE
    #define    UART4_TX_LINE RPOR5bits.RP11R = 0x0003;  //RB11   
#endif 

#ifndef UART4_RX_LINE
    #define    UART4_RX_LINE RPINR18bits.U1RXR;  //RB10
#endif  

//******************************************
//                SPI
//******************************************
#define SPI1_AVAILABLE
#define SPI2_AVAILABLE
#define SPI3_AVAILABLE

#define SPI_BAUDRATE_BRG
#define SPI1_BAUDRATE_REG SPI1BRGL
#define SPI2_BAUDRATE_REG SPI2BRGL
#define SPI2_BAUDRATE_REG SPI3BRGL

//SPI 1 pin mux 
#define SPI1_SCK_LINE	RPOR10bits.RP20R = 0x0008; 	//RC4	         
#define SPI1_SDI_LINE	RPINR20bits.SDI1R = 0x0008;	//RB8		 
#define SPI1_SDO_LINE	RPOR4bits.RP9R = 0x0007;	//RB9	
#define SPI1_SS_LINE	RPOR9bits.RP19R = 0x0009;	//RC3	 

//SPI 2 pin mux 
#define SPI2_SCK_LINE	RPOR8bits.RP16R = 0x000B;   //RC0            
#define SPI2_SDI_LINE	RPINR22bits.SDI2R = 0x0011;	//RC1    
#define SPI2_SDO_LINE	RPOR9bits.RP18R = 0x000A;	//RC2
#define SPI2_SS_LINE	RPOR10bits.RP21R = 0x000C;	//RC5  

//SPI 3 pin mux 
#define SPI3_SCK_LINE	RPOR11bits.RP22R = 0x0018;  //RC6           
#define SPI3_SDI_LINE	RPINR28bits.SDI3R = 0x0017;	//RC7   
#define SPI3_SDO_LINE	RPOR12bits.RP24R = 0x0017;	//RC8
#define SPI3_SS_LINE	RPOR12bits.RP25R = 0x0019;	//RC9  


//******************************************
//                I2C
//******************************************
#define I2C1_AVAILABLE
#define I2C2_AVAILABLE

#define I2C1_CONFIG_REG I2C1CONLbits
#define I2C2_CONFIG_REG I2C2CONLbits

#define I2C1_CONFIG_REG_SIDL I2C1_CONFIG_REG.SIDL
#define I2C2_CONFIG_REG_SIDL I2C2_CONFIG_REG.SIDL
//
////There is no need for muxing since the open drain pins are fixed.
////Once the module is ON it overdrives the pin settings
////I2C 1 pin mux 
////SCL1  RB8
////SDA1  RB9
//
////SCL2  RB3
////SDA2  RB2 

//******************************************
//                ADC
//******************************************
//13 ADC channels are standard, the rest are special functions or not implemented
#define ADC_NUMBER_OF_CHANNELS 13

#define ADC_RESOLUTION_10_12_BITS

#define ADC_WITH_ANSEL_REG
#define ADC_PORT_CONFIGURATION_1 ANSA
#define ADC_PORT_CONFIGURATION_2 ANSB
#define ADC_PORT_CONFIGURATION_3 ANSC

#define ADC_ENABLE_BIT AD1CON1bits.ADON
#define ADC_START_CONVERSION_BIT AD1CON1bits.SAMP
#define ADC_CONVERSION_DONE IFS0bits.AD1IF
#define ADC_CHANNEL_SELECTION_REGISTER AD1CHS
#define ADC_BUFFER ADC1BUF0
#define ADC_SETTINGS_1 AD1CON1
#define ADC_SETTINGS_2 AD1CON2 
#define ADC_SETTINGS_3 AD1CON3 
#define ADC_SETTINGS_4 AD1CON4
#define ADC_SETTINGS_5 AD1CON5

////Note
////Not all the ADC channels may be implemented in the device
#define ADC_CH0  0x00000001 
#define ADC_CH1  0x00000002 
#define ADC_CH2  0x00000004
#define ADC_CH3  0x00000008
#define ADC_CH4  0x00000010
#define ADC_CH5  0x00000020
#define ADC_CH6  0x00000040
#define ADC_CH7  0x00000080
#define ADC_CH8  0x00000100
#define ADC_CH9  0x00000200
#define ADC_CH10 0x00000400
#define ADC_CH11 0x00000800
#define ADC_CH12 0x00001000
#define ADC_CH13 0x00002000
#define ADC_CH14 0x00004000
#define ADC_CH15 0x00008000
#define ADC_CH16 0x00010000
#define ADC_CH17 0x00020000
#define ADC_CH18 0x00040000
#define ADC_CH19 0x00080000
#define ADC_CH20 0x00100000
#define ADC_CH21 0x00200000
#define ADC_CH22 0x00400000
#define ADC_CH23 0x00800000
#define ADC_CH24 0x01000000
#define ADC_CH25 0x02000000
#define ADC_CH26 0x04000000
#define ADC_CH27 0x08000000
#define ADC_CH28 0x10000000
#define ADC_CH29 0x20000000
#define ADC_CH30 0x40000000
#define ADC_CH31 0x80000000

//******************************************
//                FLASH
//******************************************
//#define FLASH_ERASE_BLOCK 256  // 256 Instructions Words IW 1024 bytes        
//#define FLASH_WRITE_BLOCK 32   // 32 Instructions Words IW 128 bytes   
//
//#define DEVICE_ID_ADDRESS 0xBF80F220
//#define DEVICE_ID_WORDS 0x01
//
//#define DEVICE_REVISION_ADDRESS 0xBF80F220
//#define DEVICE_REVISION_WORDS 0x01
//
//#define FLASH_KEY_1 0xAA996655
//#define FLASH_KEY_2	0x556699AA
//
//#define DEVICE_ID 0x04A00053 


//******************************************************************************
// MCU configurations
//******************************************************************************

#ifndef MCC_USED

    // CONFIG4
    #pragma config DSWDTPS = DSWDTPS1F      //  (1:68719476736 (25.7 Days))
    #pragma config DSWDTOSC = LPRC          //  (DSWDT uses LPRC as reference clock)
    #pragma config DSBOREN = OFF            // Deep Sleep BOR Enable bit (DSBOR Disabled)
    #pragma config DSWDTEN = OFF            // Deep Sleep Watchdog Timer Enable (DSWDT Disabled)
    #pragma config DSSWEN = OFF             // DSEN Bit Enable (Deep Sleep operation is always disabled)
    #pragma config PLLDIV = PLL4X           // USB 96 MHz PLL Prescaler Select bits (4x PLL selected)
    #pragma config I2C1SEL = DISABLE        // Alternate I2C1 enable bit (I2C1 uses SCL1 and SDA1 pins)
    #pragma config IOL1WAY = OFF            // PPS IOLOCK Set Only Once Enable bit (The IOLOCK bit can be set and cleared using the unlock sequence)

    // CONFIG3
    #pragma config WPFP = WPFP127           // Write Protection Flash Page Segment Boundary (Page 127 (0x1FC00))
    #pragma config SOSCSEL = ON             // SOSC Selection bits (SOSC circuit selected)
    #pragma config WDTWIN = PS25_0          // Window Mode Watchdog Timer Window Width Select (Watch Dog Timer Window Width is 25 percent)
    #pragma config PLLSS = PLL_FRC          // PLL Secondary Selection Configuration bit (PLL is fed by the on-chip Fast RC (FRC) oscillator)
    #pragma config BOREN = ON               // Brown-out Reset Enable (Brown-out Reset Enable)
    #pragma config WPDIS = WPDIS            // Segment Write Protection Disable (Disabled)
    #pragma config WPCFG = WPCFGDIS         // Write Protect Configuration Page Select (Disabled)
    #pragma config WPEND = WPENDMEM         // Segment Write Protection End Page Select (Write Protect from WPFP to the last page of memory)

    // CONFIG2
    #pragma config POSCMD = NONE            // Primary Oscillator Select (Primary Oscillator Disabled)
    #pragma config WDTCLK = LPRC            // WDT Clock Source Select bits (WDT uses LPRC)
    #pragma config OSCIOFCN = OFF           // OSCO Pin Configuration (OSCO/CLKO/RA3 functions as CLKO (FOSC/2))
    #pragma config FCKSM = CSECMD           // Clock Switching and Fail-Safe Clock Monitor Configuration bits (Clock switching enabled, Fail-Safe Clock Monitor disabled)
    #pragma config FNOSC = FRCPLL           // Initial Oscillator Select (Fast RC Oscillator with PLL module (FRCPLL))
    #pragma config ALTCMPI = CxINC_RB       // Alternate Comparator Input bit (C1INC is on RB13, C2INC is on RB9 and C3INC is on RA0)
    #pragma config WDTCMX = WDTCLK          // WDT Clock Source Select bits (WDT clock source is determined by the WDTCLK Configuration bits)
    #pragma config IESO = OFF               // Internal External Switchover (Disabled)

    // CONFIG1
    #pragma config WDTPS = PS32768          // Watchdog Timer Postscaler Select (1:32,768)
    #pragma config FWPSA = PR128            // WDT Prescaler Ratio Select (1:128)
    #pragma config WINDIS = OFF             // Windowed WDT Disable (Standard Watchdog Timer)
    #pragma config FWDTEN = OFF             // Watchdog Timer Enable (WDT disabled in hardware; SWDTEN bit disabled)
    #pragma config ICS = PGx1               // Emulator Pin Placement Select bits (Emulator functions are shared with PGEC1/PGED1)
    #pragma config LPCFG = OFF              // Low power regulator control (Disabled - regardless of RETEN)
    #pragma config GWRP = OFF               // General Segment Write Protect (Write to program memory allowed)
    #pragma config GCP = OFF                // General Segment Code Protect (Code protection is disabled)
    #pragma config JTAGEN = OFF             // JTAG Port Enable (Disabled)
#endif

#endif
