/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0

Created on Date : 17/07/2023
Last update     : 17/07/2023

CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author)
is intended for use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable copyright laws.
All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws (Italian or International ones), as well as to
civil liability for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the following
software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This file contains all the configuration words needed for the PIC24FJ64GA104.
It can be easily adapted to other PICs according to the available configurations.

******  WARNING ******

The configurations must be changed to reflect the application needs!

*******************************************************************************/

#ifndef PIC24FJ64GA104_CONFIG_H
#define PIC24FJ64GA104_CONFIG_H

#ifdef COMPILER_XC16
	#include <xc.h>
#endif


//******************************************************************************
// LTlib configurations
//******************************************************************************
#define IO_LIBRARY_SUPPORTED
#define UART_LIBRARY_SUPPORTED
#define SPI_LIBRARY_SUPPORTED
#define I2C_LIBRARY_SUPPORTED
#define ADC_LIBRARY_SUPPORTED
//#define FLASH_LIBRARY_SUPPORTED

//******************************************************************************
// Module Settings
//******************************************************************************

//******************************************
//                IO
//******************************************
#define NUMBER_OF_IO_PORTS 3


//******************************************
//                UART
//******************************************
#define UART1_AVAILABLE
#define UART2_AVAILABLE

#define UART1_RX_IDLE_STATE U1MODEbits.RXINV
#define UART2_RX_IDLE_STATE U2MODEbits.RXINV

//The PIC24FJ64GA104 does not have the RX EN pin so it is linked to the EN pin
#define UART1_RX_EN U1MODEbits.UARTEN
#define UART2_RX_EN U2MODEbits.UARTEN

//UART 1 pin mux 
#ifndef UART1_TX_LINE
    #define    UART1_TX_LINE RPOR7bits.RP15R = 0x0003;     //RB15     
#endif        

#ifndef UART1_RX_LINE
    #define    UART1_RX_LINE RPINR18bits.U1RXR = 0x000D; //RB13 - RP13
#endif        


//UART 2 pin mux 
#ifndef UART2_TX_LINE
    #define    UART2_TX_LINE RPOR5bits.RP11R = 5;   //RB11   
#endif 

#ifndef UART2_RX_LINE
    #define    UART2_RX_LINE RPINR19bits.U2RXR = 10;  //RB10
#endif    


//******************************************
//                SPI
//******************************************
#define SPI1_AVAILABLE
#define SPI2_AVAILABLE

#define SPI1_CONFIG_REG1 SPI1CON1bits
#define SPI2_CONFIG_REG1 SPI2CON1bits

#define SPI1_ENABLE SPI1STATbits.SPIEN
#define SPI2_ENABLE SPI2STATbits.SPIEN

#define SPI1_BUFFER SPI1BUF
#define SPI2_BUFFER SPI2BUF

//bits which do not exist are duplicated with existing ones.
#define SPI1_FRMERR SPI1STATbits.SPIROV
#define SPI1_SPIROV SPI1STATbits.SPIROV
#define SPI1_SPITUR SPI1STATbits.SPIROV	

#define SPI1_IF IFS0bits.SPI1IF
#define SPI1_TXIF IFS0bits.SPI1IF 
#define SPI1_RXIF IFS0bits.SPI1IF

#define SPI2_FRMERR SPI2STATbits.SPIROV
#define SPI2_SPIROV SPI2STATbits.SPIROV
#define SPI2_SPITUR SPI2STATbits.SPIROV	

#define SPI2_IF IFS2bits.SPI2IF
#define SPI2_TXIF IFS2bits.SPI2IF
#define SPI2_RXIF IFS2bits.SPI2IF

#define SPI1_BUSY SPI1STATbits.SPITBF
#define SPI2_BUSY SPI2STATbits.SPITBF

#define SPI_BAUDRATE_PRI_SEC_PRESCALER
#define SPI1_BAUDRATE_REG SPI1CON1
#define SPI2_BAUDRATE_REG SPI2CON1

//SPI 1 pin mux 
#define SPI1_SCK_LINE	RPOR7bits.RP14R = 0x0008;   //RB14            
#define SPI1_SDI_LINE	RPINR20bits.SDI1R = 0x0008;	//RB8    
#define SPI1_SDO_LINE	RPOR5bits.RP11R = 0x0007; 	//RB11
#define SPI1_SS_LINE	RPOR3bits.RP7R = 0x0009;	//RB7  

//SPI 2 pin mux 
#define SPI2_SCK_LINE	RPOR10bits.RP20R = 0x0008; 	//RC4	         
#define SPI2_SDI_LINE	RPINR20bits.SDI1R = 0x0008;	//RB8		 
#define SPI2_SDO_LINE	RPOR4bits.RP9R = 0x0007;	//RB9	
#define SPI2_SS_LINE	RPOR9bits.RP19R = 0x0009;	//RC3	 


//******************************************
//                I2C
//******************************************
#define I2C1_AVAILABLE
#define I2C2_AVAILABLE

#define I2C1_CONFIG_REG I2C1CONbits
#define I2C2_CONFIG_REG I2C2CONbits

#define I2C1_CONFIG_REG_SIDL I2C1_CONFIG_REG.I2CSIDL
#define I2C2_CONFIG_REG_SIDL I2C2_CONFIG_REG.I2CSIDL

//
////There is no need for muxing since the open drain pins are fixed.
////Once the module is ON it overdrives the pin settings
////I2C 1 pin mux 
////SCL1  RB8
////SDA1  RB9
//
////SCL2  RB3
////SDA2  RB2 

//******************************************
//                ADC
//******************************************
//13 ADC channels are standard, the rest are special functions or not implemented
#define ADC_NUMBER_OF_CHANNELS 13

#define ADC_RESOLUTION_10BITS

//#define ADC_WITH_ANSEL_REG
#define ADC_WITH_ADxPCFG_REG
#define ADC_PORT_CONFIGURATION_1 AD1PCFGL

#define ADC_ENABLE_BIT AD1CON1bits.ADON
#define ADC_START_CONVERSION_BIT AD1CON1bits.SAMP
#define ADC_CONVERSION_DONE IFS0bits.AD1IF
#define ADC_CHANNEL_SELECTION_REGISTER AD1CHS
#define ADC_BUFFER ADC1BUF0
#define ADC_SETTINGS_1 AD1CON1
#define ADC_SETTINGS_2 AD1CON2 
#define ADC_SETTINGS_3 AD1CON3 


////Note
////Not all the ADC channels may be implemented in the device
#define ADC_CH0  0x00000001 
#define ADC_CH1  0x00000002 
#define ADC_CH2  0x00000004
#define ADC_CH3  0x00000008
#define ADC_CH4  0x00000010
#define ADC_CH5  0x00000020
#define ADC_CH6  0x00000040
#define ADC_CH7  0x00000080
#define ADC_CH8  0x00000100
#define ADC_CH9  0x00000200
#define ADC_CH10 0x00000400
#define ADC_CH11 0x00000800
#define ADC_CH12 0x00001000
#define ADC_CH13 0x00002000
#define ADC_CH14 0x00004000
#define ADC_CH15 0x00008000
#define ADC_CH16 0x00010000
#define ADC_CH17 0x00020000
#define ADC_CH18 0x00040000
#define ADC_CH19 0x00080000
#define ADC_CH20 0x00100000
#define ADC_CH21 0x00200000
#define ADC_CH22 0x00400000
#define ADC_CH23 0x00800000
#define ADC_CH24 0x01000000
#define ADC_CH25 0x02000000
#define ADC_CH26 0x04000000
#define ADC_CH27 0x08000000
#define ADC_CH28 0x10000000
#define ADC_CH29 0x20000000
#define ADC_CH30 0x40000000
#define ADC_CH31 0x80000000

//******************************************
//                FLASH
//******************************************
//#define FLASH_ERASE_BLOCK 256  // 256 Instructions Words IW 1024 bytes        
//#define FLASH_WRITE_BLOCK 32   // 32 Instructions Words IW 128 bytes   
//
//#define DEVICE_ID_ADDRESS 0xBF80F220
//#define DEVICE_ID_WORDS 0x01
//
//#define DEVICE_REVISION_ADDRESS 0xBF80F220
//#define DEVICE_REVISION_WORDS 0x01
//
//#define FLASH_KEY_1 0xAA996655
//#define FLASH_KEY_2	0x556699AA
//
//#define DEVICE_ID 0x04A00053 


//******************************************************************************
// MCU configurations
//******************************************************************************

#ifndef MCC_USED

// CONFIG4
#pragma config DSWDTPS = DSWDTPSF       // DSWDT Postscale Select (1:2,147,483,648 (25.7 days))
#pragma config DSWDTOSC = LPRC          // Deep Sleep Watchdog Timer Oscillator Select (DSWDT uses Low Power RC Oscillator (LPRC))
#pragma config RTCOSC = LPRC            // RTCC Reference Oscillator  Select (RTCC uses Low Power RC Oscillator (LPRC))
#pragma config DSBOREN = ON             // Deep Sleep BOR Enable bit (BOR enabled in Deep Sleep)
#pragma config DSWDTEN = OFF            // Deep Sleep Watchdog Timer (DSWDT disabled)

// CONFIG3
#pragma config WPFP = WPFP63            // Write Protection Flash Page Segment Boundary (Highest Page (same as page 42))
#pragma config SOSCSEL = IO             // Secondary Oscillator Pin Mode Select (SOSC pins have digital I/O functions (RA4, RB4))
#pragma config WUTSEL = LEG             // Voltage Regulator Wake-up Time Select (Default regulator start-up time used)
#pragma config WPDIS = WPDIS            // Segment Write Protection Disable (Segmented code protection disabled)
#pragma config WPCFG = WPCFGDIS         // Write Protect Configuration Page Select (Last page and Flash Configuration words are unprotected)
#pragma config WPEND = WPENDMEM         // Segment Write Protection End Page Select (Write Protect from WPFP to the last page of memory)

// CONFIG2
#pragma config POSCMOD = XT             // Primary Oscillator Select (XT Oscillator mode selected)
#pragma config I2C1SEL = PRI            // I2C1 Pin Select bit (Use default SCL1/SDA1 pins for I2C1 )
#pragma config IOL1WAY = OFF             // IOLOCK One-Way Set Enable (Once set, the IOLOCK bit cannot be cleared)
#pragma config OSCIOFNC = OFF           // OSCO Pin Configuration (OSCO pin functions as clock output (CLKO))
#pragma config FCKSM = CSDCMD           // Clock Switching and Fail-Safe Clock Monitor (Sw Disabled, Mon Disabled)
#pragma config FNOSC = PRI              // Initial Oscillator Select (Primary Oscillator (XT, HS, EC))
#pragma config IESO = OFF               // Internal External Switchover (IESO mode (Two-Speed Start-up) disabled)

// CONFIG1
#pragma config WDTPS = PS32768          // Watchdog Timer Postscaler (1:32,768)
#pragma config FWPSA = PR128            // WDT Prescaler (Prescaler ratio of 1:128)
#pragma config WINDIS = OFF             // Windowed WDT (Standard Watchdog Timer enabled,(Windowed-mode is disabled))
#pragma config FWDTEN = OFF             // Watchdog Timer (Watchdog Timer is disabled)
#pragma config ICS = PGx1               // Emulator Pin Placement Select bits (Emulator functions are shared with PGEC1/PGED1)
#pragma config GWRP = OFF               // General Segment Write Protect (Writes to program memory are allowed)
#pragma config GCP = OFF                // General Segment Code Protect (Code protection is disabled)
#pragma config JTAGEN = OFF             // JTAG Port Enable (JTAG port is disabled)

#endif

#endif
