/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0

Created on Date : 23/01/2019
Last update     : 23/01/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author)
is intended for use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable copyright laws.
All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws (Italian or International ones), as well as to
civil liability for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the following
software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This file contains all the configuration words needed for the PIC32MX150F128B.
It can be easily adapted to other PICs according to the available configurations.

******  WARNING ******

The configurations must be changed to reflect the application needs!

*******************************************************************************/

#ifndef PIC32MX150F128B_CONFIG_H
#define PIC32MX150F128B_CONFIG_H

#ifdef COMPILER_XC32
	#include <xc.h>
#endif


//******************************************************************************
// LTlib configurations
//******************************************************************************
#define IO_LIBRARY_SUPPORTED
#define UART_LIBRARY_SUPPORTED
#define SPI_LIBRARY_SUPPORTED
#define I2C_LIBRARY_SUPPORTED
#define ADC_LIBRARY_SUPPORTED
#define FLASH_LIBRARY_SUPPORTED

//******************************************************************************
// Module Settings
//******************************************************************************

//******************************************
//                IO
//******************************************
#define NUMBER_OF_IO_PORTS 2

#define PULL_UP_RESISTORS_AVAILABLE
#define PULL_DOWN_RESISTORS_AVAILABLE

#define PULL_UP_ENABLE_REGISTER_A CNPUA
#define PULL_UP_ENABLE_REGISTER_B CNPUB
#define PULL_DOWN_ENABLE_REGISTER_A CNPDA 
#define PULL_DOWN_ENABLE_REGISTER_B CNPDB

//******************************************
//                UART
//******************************************
#define UART1_AVAILABLE
#define UART2_AVAILABLE

//UART 1 pin mux 
#ifndef UART1_TX_LINE
    #define    UART1_TX_LINE RPB7R = 0b0001;  //RB7     
#endif        

#ifndef UART1_RX_LINE
    #define    UART1_RX_LINE U1RXR = 0b0100;  //RB2
#endif        


//UART 2 pin mux 
#ifndef UART2_TX_LINE
    #define    UART2_TX_LINE RPB9R = 0b0010;  //RB9 
#endif 

#ifndef UART2_RX_LINE
    #define    UART2_RX_LINE U2RXR = 0b0100;  //RB8
#endif     

//******************************************
//                SPI
//******************************************
#define SPI1_AVAILABLE
#define SPI2_AVAILABLE

//SPI 1 pin mux 
#define SPI1_SCK_LINE					//RB14 fixed pin		         
#define SPI1_SDI_LINE	SDI1R = 0b0000	//RA1		 
#define SPI1_SDO_LINE	RPB5R = 0b0011	//RB5	
#define SPI1_SS_LINE	SS1R = 0b0010	//RB4	 

//SPI 2 pin mux 
#define SPI2_SCK_LINE					//RB15 fixed pin             
#define SPI2_SDI_LINE	SDI2R = 0b0000	//RA2    
#define SPI2_SDO_LINE	RPA4R = 0b0100	//RA4
#define SPI2_SS_LINE	SS2R = 0b0010	//RB10  

//******************************************
//                I2C
//******************************************
#define I2C1_AVAILABLE
#define I2C2_AVAILABLE

//There is no need for muxing since the open drain pins are fixed.
//Once the module is ON it overdrives the pin settings
//I2C 1 pin mux 
//SCL1  RB8
//SDA1  RB9

//SCL2  RB3
//SDA2  RB2 

//******************************************
//                ADC
//******************************************
#define ADC_NUMBER_OF_CHANNELS 10

#define ADC_WITH_ANSEL_REG
#define ADC_PORT_CONFIGURATION_1 ANSELA
#define ADC_PORT_CONFIGURATION_2 ANSELB

#define ADC_ENABLE_BIT AD1CON1bits.ON
#define ADC_START_CONVERSION_BIT AD1CON1bits.SAMP
#define ADC_STOP_CONVERSION_BIT AD1CON1bits.CLRASAM
#define ADC_CONVERSION_DONE IFS0bits.AD1IF
#define ADC_CHANNEL_SELECTION_REGISTER AD1CHS
#define ADC_BUFFER ADC1BUF0
#define ADC_SETTINGS_1 AD1CON1
#define ADC_SETTINGS_2 AD1CON2 
#define ADC_SETTINGS_3 AD1CON3 

//Note
//Not all the ADC channels may be implemented in the device
#define ADC_CH0 0x00000001 
#define ADC_CH1 0x00000002 
#define ADC_CH2 0x00000004
#define ADC_CH3 0x00000008
#define ADC_CH4 0x00000010
#define ADC_CH5 0x00000020
#define ADC_CH6 0x00000040
#define ADC_CH7 0x00000080
#define ADC_CH8  0x00000100
#define ADC_CH9  0x00000200
#define ADC_CH10 0x00000400
#define ADC_CH11 0x00000800
#define ADC_CH12 0x00001000
#define ADC_CH13 0x00002000
#define ADC_CH14 0x00004000
#define ADC_CH15 0x00008000


//******************************************
//                FLASH
//******************************************
#define FLASH_ERASE_BLOCK 256  // 256 Instructions Words IW 1024 bytes        
#define FLASH_WRITE_BLOCK 32   // 32 Instructions Words IW 128 bytes   

#define DEVICE_ID_ADDRESS 0xBF80F220
#define DEVICE_ID_WORDS 0x01

#define DEVICE_REVISION_ADDRESS 0xBF80F220
#define DEVICE_REVISION_WORDS 0x01

#define FLASH_KEY_1 0xAA996655
#define FLASH_KEY_2	0x556699AA

#define DEVICE_ID 0x04D06053 

//******************************************************************************
// MCU configurations
//******************************************************************************

#ifndef MCC_USED

	// DEVCFG3
	#pragma config USERID = 0xFFFF          // Enter Hexadecimal value (Enter Hexadecimal value)
	#pragma config PMDL1WAY = ON            // Peripheral Module Disable Configuration (Allow only one reconfiguration)
	#pragma config IOL1WAY = ON             // Peripheral Pin Select Configuration (Allow only one reconfiguration)

	// DEVCFG2
	#pragma config FPLLIDIV = DIV_12        // PLL Input Divider (12x Divider)
	#pragma config FPLLMUL = MUL_24         // PLL Multiplier (24x Multiplier)
	#pragma config FPLLODIV = DIV_256       // System PLL Output Clock Divider (PLL Divide by 256)

	// DEVCFG1
	#pragma config FNOSC = PRI              // Oscillator Selection Bits (Primary Osc (XT,HS,EC))
	#pragma config FSOSCEN = OFF            // Secondary Oscillator Enable (Disabled)
	#pragma config IESO = ON                // Internal/External Switch Over (Enabled)
	#pragma config POSCMOD = HS             // Primary Oscillator Configuration (HS osc mode)
	#pragma config OSCIOFNC = OFF           // CLKO Output Signal Active on the OSCO Pin (Disabled)
	#pragma config FPBDIV = DIV_1           // Peripheral Clock Divisor (Pb_Clk is Sys_Clk/1)
	#pragma config FCKSM = CSDCMD           // Clock Switching and Monitor Selection (Clock Switch Disable, FSCM Disabled)
	#pragma config WDTPS = PS1048576        // Watchdog Timer Postscaler (1:1048576)
	#pragma config WINDIS = OFF             // Watchdog Timer Window Enable (Watchdog Timer is in Non-Window Mode)
	#pragma config FWDTEN = OFF             // Watchdog Timer Enable (WDT Disabled (SWDTEN Bit Controls))
	#pragma config FWDTWINSZ = WINSZ_25     // Watchdog Timer Window Size (Window Size is 25%)

	// DEVCFG0
	#pragma config JTAGEN = OFF             // JTAG Enable (JTAG Disabled)
	#pragma config ICESEL = ICS_PGx1        // ICE/ICD Comm Channel Select (Communicate on PGEC1/PGED1)
	#pragma config PWP = OFF                // Program Flash Write Protect (Disable)
	#pragma config BWP = OFF                // Boot Flash Write Protect bit (Protection Disabled)
	#pragma config CP = OFF                 // Code Protect (Protection Disabled)

#endif
#endif
