//******************************************************************************
// The software reads the pressure and temperature out of the BMP280 sensor
// and writes it to the LCD.
// The LCD must be connected as specified in the library included above.
// The BMP280 must be connected to the I2C Module 2 of the MCU.  
// The address bit (SDO) is connected to GND (Device address: 0x76)
//
//
// Tested Platform: Freedom III
// Tested MCU: PIC18F46K22
// Tested Compiler: XC8 2.05
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "module_IO.h"
#include "module_IO.c"

#include "module_I2C.h"
#include "module_I2C.c"

#include "MCP2300x.h"
#include "MCP2300x.c"

#include "LCD_44780_I2C.h"
#include "LCD_44780_I2C.c"

#define BMP280_I2C_MODULE_2 
#include "BMP280.h"
#include "BMP280.c"

#define DEVICE_ADDRESS 0x76

void main (void) {

    uint8_t device_ID = 0x00;
    int32_t temperature = 0;
    uint32_t pressure = 0;
    
    IO_set_all_ports_as_inputs();
    
    LCD_initialize(20);
    LCD_backlight(LCD_TURN_ON_LED);
    
    BMP280_initialize (DEVICE_ADDRESS, 100);

    LCD_home();
                   
    while(1){        

        BMP280_trigger_new_measure (DEVICE_ADDRESS);        
        while (BMP280_is_conversion_running(DEVICE_ADDRESS));

        //**********************************
        //         Pressure
        //**********************************
        LCD_goto_line(1);        
        LCD_write_message("P: ");
        BMP280_get_pressure (DEVICE_ADDRESS, &pressure);        
        //The LCD shows kPa value
        LCD_write_integer((int16_t) (pressure / 1000),3, LCD_ZERO_CLEANING_ON);
        LCD_write_message(" kPa");        
        
        //**********************************
        //         Temperature
        //**********************************
        LCD_goto_line(2);
        LCD_write_message("T: ");
        BMP280_get_temperature (DEVICE_ADDRESS, &temperature);
        //Write integer part only
        LCD_write_integer((int16_t) temperature/100 ,2, LCD_ZERO_CLEANING_ON);
        LCD_write_message(" ");
        LCD_write_char(0xDF);
        LCD_write_message("C");
        
        delay_ms(500);
    }
}

