//******************************************************************************
// The software reads the pressure and temperature out of the BMP280 sensor
// and writes it to the LCD.
// The sensor settings are optimzed to get the best resolution and reduce the noise.
// The sensor is set to trigger new measures automatically every 1s. 
// The LCD must be connected as specified in the library included above.
// The BMP280 must be connected to the I2C Module 2 of the MCU.  
// The address bit (SDO) is connected to GND (Device address: 0x76)
//
//
// Tested Platform: Freedom III
// Tested MCU: PIC18F46K22
// Tested Compiler: XC8 2.05
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "module_IO.h"
#include "module_IO.c"

#include "module_I2C.h"
#include "module_I2C.c"

#include "MCP2300x.h"
#include "MCP2300x.c"

#include "LCD_44780_I2C.h"
#include "LCD_44780_I2C.c"

#define BMP280_I2C_MODULE_2 
#include "BMP280.h"
#include "BMP280.c"

#define DEVICE_ADDRESS 0x76

void main (void) {

    uint8_t device_ID = 0x00;
    int32_t temperature = 0;
    uint32_t pressure = 0;
    
    IO_set_all_ports_as_inputs();
    
    LCD_initialize(20);
    LCD_backlight(LCD_TURN_ON_LED);
    
    BMP280_initialize (DEVICE_ADDRESS, 100);

    LCD_home();
    
    BMP280_set_time_interval (DEVICE_ADDRESS, BMP280_TIME_INTERVAL_1000_ms);
    BMP280_set_pressure_oversampling(DEVICE_ADDRESS, BMP280_OVERSAMPLING_16);
    BMP280_set_temperature_oversampling(DEVICE_ADDRESS, BMP280_OVERSAMPLING_16);
    BMP280_set_FIR_coefficients (DEVICE_ADDRESS, BMP280_FIR_COEFFICIENTS_16);
    
    //Trigger the new measures automatically every 1s
    BMP280_set_mode(DEVICE_ADDRESS, BMP280_MODE_NORMAL);
                
    while(1){        
        
        while (BMP280_is_conversion_running(DEVICE_ADDRESS));

        //**********************************
        //         Pressure
        //**********************************
        LCD_goto_line(1);        
        LCD_write_message("P: ");
        BMP280_get_pressure (DEVICE_ADDRESS, &pressure);
               
        
        //The LCD shows Pa value
        //Since the pressure is 32 bits integer, to properly write it on the LCD
        //it is divided in 2 16bits blocks        
        uint16_t thousands_10 = 0;
        thousands_10 = pressure / 10000;
        LCD_write_integer((int16_t) (thousands_10) ,2, LCD_ZERO_CLEANING_ON);
        LCD_write_integer((int16_t) (pressure - (thousands_10 * 10000)) ,4, LCD_ZERO_CLEANING_OFF);
        LCD_write_message(" Pa");
        
        //**********************************
        //         Temperature
        //**********************************
        LCD_goto_line(2);
        LCD_write_message("T: ");
        BMP280_get_temperature (DEVICE_ADDRESS, &temperature);
        //Write integer part
        LCD_write_integer((int16_t) temperature/100 ,2, LCD_ZERO_CLEANING_ON);
        LCD_write_message(".");
        //Write decimal part
        LCD_write_integer((int16_t) (temperature - ((int16_t)(temperature/100) * 100))  ,2, LCD_ZERO_CLEANING_OFF);
        
        LCD_write_message(" ");
        LCD_write_char(0xDF);
        LCD_write_message("C");        
        
        //**********************************
        //         Device Info
        //**********************************
//        LCD_goto_line(2);
//        LCD_write_message("Device : ");
//        BMP280_get_device_ID (DEVICE_ADDRESS, &device_ID);
//        
//        if (device_ID == BMP280_DEVICE_ID) {
//            LCD_write_message("BMP280");
//        } else {
//            LCD_write_message("Unkown");
//        }

        delay_ms(500);
    }
}

