//******************************************************************************
// The software reads the humidity and temperature out of the HDC2080. 
// The values are shown on the LCD display.
// The display must be connected as required by the LCD library. 
//
//
// Tested Platform: Freedom III
// Tested MCU: PIC18F46K22
// Tested Compiler: XC8 2.05
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "module_IO.h"
#include "module_IO.c"

#include "module_I2C.h"
#include "module_I2C.c"

#include "MCP2300x.h"
#include "MCP2300x.c"

#include "LCD_44780_I2C.h"
#include "LCD_44780_I2C.c"

#define HDC2080_I2C_MODULE_2 
#include "HDC2080.h"
#include "HDC2080.c"

#define DEVICE_ADDRESS 0x40 //A0=GND

void main (void){
    
    float humidity = 0;
    float temperature = 0;
    uint16_t device_ID = 0;
    
    IO_set_all_ports_as_inputs(); 
    
    LCD_initialize(20);
    
    HDC2080_initialize(DEVICE_ADDRESS, 400);   

    while (1){
        
        LCD_home ();
        
        HDC2080_trigger_new_measure (DEVICE_ADDRESS);
        delay_ms (10);
        
        LCD_write_message("Humidity: ");
        HDC2080_get_humidity (DEVICE_ADDRESS, &humidity);        
        LCD_write_integer((int16_t) humidity ,2, LCD_ZERO_CLEANING_ON);
        LCD_write_message(" %");
        
        LCD_goto_line(2);
        
        LCD_write_message("Temp. : ");
        HDC2080_get_temperature (DEVICE_ADDRESS, &temperature);        
        LCD_write_integer((int16_t) temperature ,2, LCD_ZERO_CLEANING_ON);
        LCD_write_char(0xDF);
        LCD_write_message("C");
        
        delay_ms(500);
    }    
}