//******************************************************************************
// The software reads the humidity and temperature out of the HDC2080. 
// The INT pin must be connected to a LED+R to Vcc. 
// The LED will be turned on once the temperature rises over the TEMPERATURE_LIMIT 
// and it will be turned off once it will fall below the TEMPERATURE_LIMIT. 
// The display must be connected as required by the LCD library. 
//
//
// Tested Platform: Freedom III
// Tested MCU: PIC18F46K22
// Tested Compiler: XC8 2.05
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "module_IO.h"
#include "module_IO.c"

#include "module_I2C.h"
#include "module_I2C.c"

#include "MCP2300x.h"
#include "MCP2300x.c"

#include "LCD_44780_I2C.h"
#include "LCD_44780_I2C.c"

#define HDC2080_I2C_MODULE_2 
#include "HDC2080.h"
#include "HDC2080.c"

#define DEVICE_ADDRESS 0x40 //A0=GND
#define TEMPERATURE_LIMIT 24

void main (void){
    
    float humidity = 0;
    float temperature = 0;
    
    IO_set_all_ports_as_inputs(); 
    
    LCD_initialize(20);
    
    HDC2080_initialize(DEVICE_ADDRESS, 400);
    
    HDC2080_cleaning_cycle (DEVICE_ADDRESS, 10);
    
    //Calibration value, if needed (You may use different values))
    HDC2080_set_humidity_offset (DEVICE_ADDRESS, HDC2080_CONF_HUM_ADJ_MINUS_25 + HDC2080_CONF_HUM_ADJ_PLUS_12_5 + 
                                HDC2080_CONF_HUM_ADJ_PLUS_3_1 + HDC2080_CONF_HUM_ADJ_PLUS_0_4);
    
    HDC2080_set_temperature_limit (DEVICE_ADDRESS, HDC2080_TEMPERATURE_HIGH, TEMPERATURE_LIMIT);
    
    //Comparator mode is used. If yo uuse Level mode, to reset the interrupt bit you need to read back 
    //The status register, or the interrupt will remain set. 
    HDC2080_set_interrupt_modes (DEVICE_ADDRESS, HDC2080_INT_ENABLED + HDC2080_INT_ACTIVE_HIGH + HDC2080_INT_COMPARATOR_MODE);
    
    //Enable the Interrupt on the high temperature level
    HDC2080_set_interrupt_enable (DEVICE_ADDRESS, HDC2080_INT_TH_ENABLE);
    
    HDC2080_set_sampling_modes (DEVICE_ADDRESS, HDC2080_SAMPLING_EVERY_5s);
    
    //It is required to start the auto measure sequence
    HDC2080_trigger_new_measure (DEVICE_ADDRESS);
    
    while (1){
        
        LCD_home ();
        
        LCD_write_message("Humidity: ");
        HDC2080_get_humidity (DEVICE_ADDRESS, &humidity);        
        LCD_write_integer((int16_t) humidity ,2, LCD_ZERO_CLEANING_ON);
        LCD_write_message("%");
        
        LCD_goto_line(2);
        
        LCD_write_message("T: ");
        HDC2080_get_temperature (DEVICE_ADDRESS, &temperature);        
        LCD_write_integer((int16_t) temperature ,2, LCD_ZERO_CLEANING_ON);
        LCD_write_char(0xDF);
        LCD_write_message("C");
        
        //Threshold/Limit is shown on the same line. 
        LCD_write_message(" Th. ");
        LCD_write_integer(TEMPERATURE_LIMIT, 2, LCD_ZERO_CLEANING_ON);
        LCD_write_char(0xDF);
        LCD_write_message("C");
        
        delay_ms(500);
    }    
}
