//******************************************************************************
// The software reads the accelaration on the axes X,Y,Z of the HMC5883 and
// writes the values to the LCD.
// The LCD must be connected as specified in the library included above.
// The HMC5883 must be connected to the I2C Module 2 of the MCU.
// Disconnect the LEDs jumper by Freedom III board. 
//
// Tested Platform: Freedom III
// Tested MCU: PIC18F46K22
// Tested Compiler: XC8 2.05
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "module_IO.h"
#include "module_IO.c"

#include "module_I2C.h"
#include "module_I2C.c"

#include "MCP2300x.h"
#include "MCP2300x.c"

#include "LCD_44780_I2C.h"
#include "LCD_44780_I2C.c"

#define HMC5883_I2C_MODULE_2
#include "HMC5883.h"
#include "HMC5883.c"

void main(void) {
    
    int16_t field_X = 0x00;
    int16_t field_Y = 0x00;
    int16_t field_Z = 0x00;
    
    IO_set_all_ports_as_inputs();
    
    LCD_initialize(20);
    LCD_backlight(LCD_TURN_ON_LED);
    
    HMC5883_initialize(HMC5883_100KHz);
    HMC5883_set_average (HMC5883_AVERAGE_8);
    HMC5883_set_sampling_rate (HMC5883_SAMPLING_75);
    
    while(1){
        
        delay_ms (100);
        
        LCD_home();
        
        LCD_write_message("X: ");        
        field_X = HMC5883_read_magnetic_field (HMC5883_FIELD_X);
        LCD_write_integer (field_X, 4, LCD_ZERO_CLEANING_ON);
        
        LCD_write_message(" Y: ");        
        field_Y = HMC5883_read_magnetic_field (HMC5883_FIELD_Y);
        LCD_write_integer (field_Y, 4, LCD_ZERO_CLEANING_ON);
        
        LCD_goto_line(2);
        
        LCD_write_message("Z: ");
        
        field_Z = HMC5883_read_magnetic_field (HMC5883_FIELD_Z);
        LCD_write_integer (field_Z, 4, LCD_ZERO_CLEANING_ON);         
        
    }
}
