//******************************************************************************
// The software reads the angular speed out of the L3GD20H sensor
// and writes it to the LCD.
// The LCD must be connected as specified in the library included above.
// The L3GD20H must be connected to the I2C Module 2 of the MCU.  
// The address bit is connected to GND (Device address: 0x6A)
//
//
// Tested Platform: Freedom III
// Tested MCU: PIC18F46K22
// Tested Compiler: XC8 2.05
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "module_IO.h"
#include "module_IO.c"

#include "module_I2C.h"
#include "module_I2C.c"

#define L3GD20H_I2C_MODULE_2 
#include "L3GD20H.h"
#include "L3GD20H.c"

#include "MCP2300x.h"
#include "MCP2300x.c"

#include "LCD_44780_I2C.h"
#include "LCD_44780_I2C.c"

#define DEVICE_ADDRESS 0x6A

void main (void) {

    int16_t d_speed_X = 0;
    int16_t d_speed_Y = 0;
    int16_t d_speed_Z = 0;
    
    IO_set_all_ports_as_inputs();
    
    LCD_initialize(20);
    LCD_backlight(LCD_TURN_ON_LED);
    
    L3GD20H_initialize (DEVICE_ADDRESS, 100);
    L3GD20H_set_full_scale (DEVICE_ADDRESS, L3GD20H_FULL_SCALE_245);
    
    LCD_home(); 
                   
    while(1){        

        L3GD20H_get_angular_speed (DEVICE_ADDRESS, &d_speed_X, &d_speed_Y, &d_speed_Z);  
        
        //**********************************
        //         Read Angular Speed 
        //**********************************
        LCD_goto_line(1);        
        LCD_write_message("dps_X:");
        LCD_write_integer(d_speed_X,5, LCD_ZERO_CLEANING_ON);   
        
        LCD_goto_line(2);        
        LCD_write_message("dps_Y:");
        LCD_write_integer(d_speed_Y,5, LCD_ZERO_CLEANING_ON);     
   
        //Change the line number to 2 and comment the other lines
        //If you use 3 lines LCD you can uncomment the code below as it is.
//        LCD_goto_line(3);        
//        LCD_write_message("dps_Z:");
//        LCD_write_integer(d_speed_Z,5, LCD_ZERO_CLEANING_ON);     
    }
}

