//******************************************************************************
// The software reads the L3GD20H ID number and temperature register,
// and writes it to the LCD.
// The LCD must be connected as specified in the library included above.
// The L3GD20H must be connected to the I2C Module 2 of the MCU.  
// The address bit is connected to GND (Device address: 0x6A)
//
//
// Tested Platform: Freedom III
// Tested MCU: PIC18F46K22
// Tested Compiler: XC8 2.05
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "module_IO.h"
#include "module_IO.c"

#include "module_I2C.h"
#include "module_I2C.c"

#define L3GD20H_I2C_MODULE_2 
#include "L3GD20H.h"
#include "L3GD20H.c"

#include "MCP2300x.h"
#include "MCP2300x.c"

#include "LCD_44780_I2C.h"
#include "LCD_44780_I2C.c"

#define DEVICE_ADDRESS 0x6A

void main (void) {

    uint8_t device_ID = 0x00;
    int8_t temperature = 0;
    
    IO_set_all_ports_as_inputs();
    
    LCD_initialize(20);
    LCD_backlight(LCD_TURN_ON_LED);
    
    L3GD20H_initialize (DEVICE_ADDRESS, 100);
    
    LCD_home(); 
                   
    while(1){     

        delay_ms(500);

        //**********************************
        //         Temperature
        //**********************************
        //This is not the ambient temperature.
        //No info on the datasheet. 
        //It can be used as relative temperature variation. 
        LCD_goto_line(1);
        LCD_write_message("Temp.: ");
        L3GD20H_get_temperature (DEVICE_ADDRESS, &temperature);
        LCD_write_integer((int16_t) temperature,2, LCD_ZERO_CLEANING_ON);
        LCD_write_char(0xDF);
        LCD_write_message("C");
        
        
        //**********************************
        //         Device Info
        //**********************************
        LCD_goto_line(2);
        LCD_write_message("Device : ");
        L3GD20H_get_device_ID (DEVICE_ADDRESS, &device_ID);
        
        if (device_ID == L3GD20H_DEVICE_ID) {
            LCD_write_message("L3GD20H");
        } else {
            LCD_write_message("Unkown");
        }
    }
}

