//******************************************************************************
// The software reads the angular speed out of the L3GD20H sensor
// and writes it to the LCD. The application also set the trheshold limits
// at which the interrupt is set upon passing it. 
// The LCD must be connected as specified in the library included above.
// The L3GD20H must be connected to the I2C Module 2 of the MCU.  
// The address bit is connected to GND (Device address: 0x6A)
//
//
// Tested Platform: Freedom III
// Tested MCU: PIC18F46K22
// Tested Compiler: XC8 2.05
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "module_IO.h"
#include "module_IO.c"

#include "module_I2C.h"
#include "module_I2C.c"

#define L3GD20H_I2C_MODULE_2 
#include "L3GD20H.h"
#include "L3GD20H.c"

#include "MCP2300x.h"
#include "MCP2300x.c"

#include "LCD_44780_I2C.h"
#include "LCD_44780_I2C.c"

#define DEVICE_ADDRESS 0x6A
#define X_LIMIT 200.0
#define Y_LIMIT 200.0
#define Z_LIMIT 200.0

void main (void) {

    int16_t d_speed_X = 0;
    int16_t d_speed_Y = 0;
    int16_t d_speed_Z = 0;
    
    IO_set_all_ports_as_inputs();
    
    LCD_initialize(20);
    LCD_backlight(LCD_TURN_ON_LED);
    
    L3GD20H_initialize (DEVICE_ADDRESS, 100);
    L3GD20H_set_full_scale (DEVICE_ADDRESS, L3GD20H_FULL_SCALE_245);
    
    //Interrupt settings
    L3GD20H_set_interrupt_threshold (DEVICE_ADDRESS, L3GD20H_THRESHOLD_X, X_LIMIT);
    L3GD20H_set_interrupt_threshold (DEVICE_ADDRESS, L3GD20H_THRESHOLD_Y, Y_LIMIT);
    L3GD20H_set_interrupt_threshold (DEVICE_ADDRESS, L3GD20H_THRESHOLD_Z, Z_LIMIT);
    
    L3GD20H_set_interrupt_mode (DEVICE_ADDRESS, L3GD20H_INT_OR_ON| L3GD20H_INT_LATCH_OFF);                  
    L3GD20H_set_interrupt_pin_mode(DEVICE_ADDRESS,L3GD20H_INT_PIN_ACTIVE_HIGH|L3GD20H_INT_PIN_PUSH_PULL);
    L3GD20H_set_interrupt_pin (DEVICE_ADDRESS, L3GD20H_INT1_PIN_IG_ENABLED | L3GD20H_INT2_PIN_DISABLED);
    L3GD20H_set_event_duration (DEVICE_ADDRESS, L3GD20H_WAIT_ON, 2);                                       
    
    L3GD20H_set_interrupt_enabled (DEVICE_ADDRESS, L3GD20H_INT_ENABLE_X_HIGH_EVENT_ON| 
                                                   L3GD20H_INT_ENABLE_Y_HIGH_EVENT_ON| 
                                                   L3GD20H_INT_ENABLE_Z_HIGH_EVENT_ON);
    
    LCD_home(); 
                   
    while(1){        

        L3GD20H_get_angular_speed (DEVICE_ADDRESS, &d_speed_X, &d_speed_Y, &d_speed_Z);  
        
        //**********************************
        //         Read Angular Speed 
        //**********************************
        LCD_goto_line(1);        
        LCD_write_message("dps_X:");
        LCD_write_integer(d_speed_X,4, LCD_ZERO_CLEANING_ON);   
        LCD_write_message(" L.");
        LCD_write_integer(X_LIMIT,3, LCD_ZERO_CLEANING_ON);  
        
        LCD_goto_line(2);        
        LCD_write_message("dps_Y:");
        LCD_write_integer(d_speed_Y,4, LCD_ZERO_CLEANING_ON);  
        LCD_write_message(" L.");
        LCD_write_integer(Y_LIMIT,3, LCD_ZERO_CLEANING_ON);          
    
        //Uncomment if you use 3 lines LCD
//        LCD_goto_line(3);        
//        LCD_write_message("dps_Z:");
//        LCD_write_integer(d_speed_Z,4, LCD_ZERO_CLEANING_ON);
//        LCD_write_message(" L.");
//        LCD_write_integer(Z_LIMIT,3, LCD_ZERO_CLEANING_ON);          
    }
}

