//******************************************************************************
// In this example the temperature is read out from the MAX6675.
// The read out is written on the LCD.
// The LCD must be connected as specified in the library included above.
// The MAX6675 is connected to the SPI module 2 but it can be connected also 
// to module 1 by changing the definition before the library. 
// Check the device configuration file for the peripheral pin out.
// The firmware can detect errors on the probe, showing the message ERROR in case
// something is detected. 
//
// Tested Platform: Freedom III
// Tested MCU: PIC18F46K22
// Tested Compiler: XC8 2.05
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "module_I2C.h"
#include "module_I2C.c"

#include "module_SPI.h"
#include "module_SPI.c"

#include "module_IO.h"
#include "module_IO.c"

#include "MCP2300x.h"
#include "MCP2300x.c"

#define MAX6675_SPI_MODULE_2
#include "MAX6675.h"
#include "MAX6675.c"

#include "LCD_44780_I2C.h"
#include "LCD_44780_I2C.c"

void main(void) {
    
    uint16_t read_value = 0;
    
    IO_set_all_ports_as_inputs();
    
    LCD_initialize(20);
    
    MAX6675_initialize (MAX6675_CLOCK_FOSC_64);
        
    while (1){
        
        delay_ms (500);        
        LCD_home();
        LCD_write_message("Temp.: "); 
        
        read_value = MAX6675_read_temperature ();
        
        //Check if the IC and the probe are properly connected
        if ((MAX6675_is_connected () == MAX6675_DEVICE_CONNECTED) && 
                (MAX6675_is_probe_connected () == MAX6675_PROBE_CONNECTED)){
            
            LCD_write_integer(read_value, 4, ZERO_CLEANING_ON);  
            LCD_write_char(0xDF);
            LCD_write_message("C"); 
        } else {
            LCD_write_message("ERROR ");
        }
    }
}
