//******************************************************************************
// In this example the temperature is read out from the MAX6675.
// The read out is sent to the UART set to 115200bit/s 8bits , 1 bit stop.
// The MAX6675 is connected to the SPI module 2 but it can be connected also 
// to module 1 by changing the definition before the library. 
// Check the device configuration file for the peripheral pin out.
// The firmware can detect errors on the probe, showing the message ERROR in case
// something is detected. 
//
//
// Tested Platform: Freedom III
// Tested MCU: PIC18F46K22
// Tested Compiler: XC8 2.05
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "module_IO.h"
#include "module_IO.c"

#include "module_SPI.h"
#include "module_SPI.c"

#include "module_UART.h"
#include "module_UART.c"

#define MAX6675_SPI_MODULE_2
#include "MAX6675.h"
#include "MAX6675.c"

void main(void) {
    
    uint16_t temperature_numeric = 0;
    uint8_t tempterature_string[5] ;
    
    IO_set_all_ports_as_inputs();
    
    UART1_open(UART_BAUDRATE_115200);
    
    MAX6675_initialize(MAX6675_CLOCK_FOSC_64);
                
    while (1){
        
        temperature_numeric = MAX6675_read_temperature();
        
        LTlib_int16_to_string(tempterature_string, temperature_numeric);
        
        //Send Delete character if the terminal supports 
        //Automatic monitor cleaning
        UART1_write_byte(24);
        
        //Write the temperature only in case there is no errror.
        if ((MAX6675_is_probe_connected() == MAX6675_PROBE_CONNECTED) && 
                (MAX6675_is_connected() == MAX6675_DEVICE_CONNECTED)) {
        
                    UART1_write_message("Temperature : ");
                    UART1_write_string(tempterature_string);   
        } else {
            
                    UART1_write_message("Error - Check the Hardware!");
        }
                
        //Send carriage return
        UART1_write_byte(13);
                        
        delay_ms(500);
    }
}
