//******************************************************************************
// The software reads the temperature out of the TMP1075. 
// The Alert pin must be connected to a LED+R to Vcc. 
// It will be turned on once the temperature rises 25C and is turned off once the 
// Remperature falls down to to 22C.
//
// Tested Platform: Freedom III
// Tested MCU: PIC18F46K22
// Tested Compiler: XC8 2.05
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "module_IO.h"
#include "module_IO.c"

#include "module_I2C.h"
#include "module_I2C.c"

#include "LCD_ST7036.h"
#include "LCD_ST7036.c"

#define TMP1075_I2C_MODULE_2 
#include "TMP1075.h"
#include "TMP1075.c"

#define DEVICE_ADDRESS 0x48 //A0=A1=A2=GND

void main (void){
    
    uint16_t data_readback = 0;
    int16_t temperature = 0;
    
    IO_set_all_ports_as_inputs ();
    
    LCD_initialize(20);
    
    TMP1075_initialize(400);
          
    TMP1075_configuration(DEVICE_ADDRESS, TMP1075_CONF_ALERT_ACTIVE_LOW | TMP1075_CONF_ALERT_AS_COMPARATOR);
    
    TMP1075_set_high_limit (DEVICE_ADDRESS, 25);    
    TMP1075_set_low_limit (DEVICE_ADDRESS, 22);
      
    while (1){
        
        LCD_home ();
        LCD_write_message("Temp. : ");
        TMP1075_read_temperature (DEVICE_ADDRESS, &temperature);
      
        LCD_write_integer(temperature ,3, LCD_ZERO_CLEANING_ON);
        
        LCD_goto_line(3);
        
        LCD_write_message("Device : ");
        TMP1075_read_address (DEVICE_ADDRESS, TMP1075_DEVICE_ID_REG, &data_readback);
        
        if (data_readback == TMP1075_DEVICE_ID) {
            LCD_write_message("TMP1075");
        } else {
            LCD_write_message("Unkown");
        }

        delay_ms(500);
    }    
}
