//******************************************************************************
// The software reads the voltage out of the Temperature and Light sensors 
// connected to the channels specified by the defines below. 
// The value are shown to the LCD that must be connected as specified in the 
// library included above. 
//
// Tested Platform: Freedom III
// Tested MCU: PIC18F46K22
// Tested Compiler: XC8 2.05
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "module_IO.h"
#include "module_IO.c"

#include "module_I2C.h"
#include "module_I2C.c"

#include "module_ADC.h"
#include "module_ADC.c"

#include "MCP2300x.h"
#include "MCP2300x.c"

#include "LCD_44780_I2C.h"
#include "LCD_44780_I2C.c"

//Temperature channel
#define TEMPERATURE_CH ADC_CH5

//Light channel
#define LIGHT_CH ADC_CH0

void main (void){
    
    uint16_t light_value = 0;
    uint16_t temperature_value = 0;
    
    IO_set_all_ports_as_inputs();
    
    LCD_initialize(20);
    
    ADC_open(LIGHT_CH + TEMPERATURE_CH);
    
    while(1){
        
        LCD_home();
        
        ADC_select_channel (TEMPERATURE_CH);
        ADC_start_conversion();
        while(ADC_is_converting());
        temperature_value = ADC_read_value()/2;
        
        ADC_select_channel (LIGHT_CH);
        ADC_start_conversion();
        while(ADC_is_converting());
        light_value = ADC_read_value();
        
        LCD_write_message("Temp  : ");
        LCD_write_integer (temperature_value, 4, LCD_ZERO_CLEANING_ON); 
        LCD_write_char (223);
        LCD_write_char ('C');
        
        LCD_goto_line(2);
        
        LCD_write_message("Light : ");
        LCD_write_integer (light_value, 4, LCD_ZERO_CLEANING_ON);          
    }
}
