//******************************************************************************
// The example must be loaded into two systems connected via the CAN bus.
// The systems should have 4 buttons connected between PORTB4-PORTB7.
// Pressing any button on one board will be shown on the second board LEDs.
// The LEDs are connected on PORTD4-PORTD7
//
//
// Tested Platform: Freedom II
// Tested MCU: PIC18F4580
// Tested Compiler: XC8 2.05
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "module_IO.h"
#include "module_IO.c"

#include "module_CAN.h"
#include "module_CAN.c"

#define BUTTON_PRESSED 0x00

#define BUTTON1 PORTBbits.RB4 
#define BUTTON2 PORTBbits.RB5 
#define BUTTON3 PORTBbits.RB6 
#define BUTTON4 PORTBbits.RB7 

void main (void) {
    
    BYTE info [8];
    CANmessage msg;
    
    IO_set_all_ports_as_inputs();
    
    IO_set_port_direction(IO_PORTD, IO_ALL_PORT_OUTPUT);

    IO_enable_pull_up_resistors(IO_PORTB, IO_BIT4 + IO_BIT5 + IO_BIT6 + IO_BIT7);
    
    //*********************************************************
    // CAN Library Test
    //*********************************************************
    //Initialize at 125Kbits/s 16xTq
    // 16MHz 125Kb/s --> 2,7,6,1,3
    // 20MHz 125Kb/s --> 2,7,6,1,4
    CAN_open (2,7,6,1,4, CAN_CONFIG_LINE_FILTER_OFF &
    CAN_CONFIG_SAMPLE_ONCE &
    CAN_CONFIG_ALL_VALID_MSG &
    CAN_CONFIG_DBL_BUFFER_ON);
    
        
    //Monitor all the buttons
    while (1){
        
        info[0] = 0x00;
        
        if (BUTTON1 == BUTTON_PRESSED){
           info[0] = 0x01; 
        }
        
        if (BUTTON2 == BUTTON_PRESSED){
           info[0] = 0x02; 
        }
        
        if (BUTTON3 == BUTTON_PRESSED){
           info[0] = 0x04; 
        }
        
        if (BUTTON4 == BUTTON_PRESSED){
           info[0] = 0x08; 
        }
           
       
        //Check if any button was pressed
        if (info[0] > 0) {    

            while (!CAN_is_TX_ready());

            CAN_write_message (0x0A005510, info,1, CAN_TX_XTD_FRAME &
            CAN_NORMAL_TX_FRAME &
            CAN_TX_PRIORITY_2);
        }

       //Check if any data was received
        if (CAN_is_RX_ready ()) {
            CAN_read_message (&msg);

            LATD = msg.data[0];
        }  
        
    }
}
