//******************************************************************************
// The software erase the flash memory block 0x2000. Afterward it writes the
// buffer with dummy data (0,1,2,3,4...) and writes it back to the flash.
// After the write process all the data is read back and written byte by byte
// to the PORTD where the 8x LED are connected. 
//
// Tested Platform: Freedom III
// Tested MCU: PIC18F46K22
// Tested Compiler: XC8 2.05
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "module_FLASH.h"
#include "module_FLASH.c"

#include "module_IO.h"
#include "module_IO.c"

void main(void) {
    
    uint8_t data_buffer[FLASH_WRITE_BLOCK];
    uint8_t index = 0;
    
    IO_set_all_ports_as_inputs();
    
    IO_set_port_direction(IO_PORTD, 0x00);
    
    //Erase the memory block
    FLASH_erase_memory(0x2000);
    
    //Fill the buffer with the index value.
    for (index=0; index < FLASH_WRITE_BLOCK; index++ ){
        data_buffer[index] = index;
    }
    
    //All LEDs are ON to signal the start of the write cycle
    IO_write_port(IO_PORTD,0xFF);
    
    FLASH_write_memory(0x2000,FLASH_WRITE_BLOCK, &data_buffer);
    
    //All LEDs are OFF to signal the end of the write cycle
    IO_write_port(IO_PORTD,0x00);
    delay_ms(500);
    
    //Write back the data to the LEDs
    
    //Clean the buffer
    for (index=0; index < FLASH_WRITE_BLOCK; index++ ){
        data_buffer[index] = 0x00;
    }
    
    FLASH_read_memory(0x2000,FLASH_WRITE_BLOCK, &data_buffer);
    
    for (index=0; index < FLASH_WRITE_BLOCK; index++ ){
       IO_write_port(IO_PORTD,data_buffer[index]);
       delay_ms(100);
    }
    
    while (1);
}
