//******************************************************************************
// The software is an I2C Slave. It receives the data from a Master.
// A second MCU must be programmed with the Master firmware to send the data. 
// The I2C module can be selected in the Firmware and it could be different from 
// the Master. The device address selected in the master must be the same one set
// within the Slave.
// 
//
// Tested Platform: Freedom III
// Tested MCU: PIC18F46K22
// Tested Compiler: XC8 2.05
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "module_IO.h"
#include "module_IO.c"

#include "module_I2C.h"
#include "module_I2C.c"

#define SLAVE_ADDRESS 0xA6

#define READ  0x01
#define WRITE 0x00

#define BYTE_TO_SEND 0x05


//*************************************
//     Module Switch
//*************************************

#define I2C_MODULE_1
//#define I2C_MODULE_2

#ifdef I2C_MODULE_1
    #define I2C_Module_open I2C1_open
    #define I2C_Module_baud_rate I2C1_baud_rate
    #define I2C_Module_wait_bus_IDLE I2C1_wait_bus_IDLE
    #define I2C_Module_start_bit I2C1_start_bit
    #define I2C_Module_read_byte I2C1_read_byte
    #define I2C_Module_reset_write_collision_flag I2C1_reset_write_collision_flag
    #define I2C_Module_write_byte I2C1_write_byte 
    #define I2C_Module_stop_bit I2C1_stop_bit
    #define I2C_Module_set_slave_address I2C1_set_slave_address
    #define I2C_Module_check_data_ready I2C1_check_data_ready
    #define I2C_Module_check_read_write_operation I2C1_check_read_write_operation
    #define I2C_Module_check_stop_bit I2C1_check_stop_bit
    #define I2C_Module_check_start_bit I2C1_check_start_bit
#endif

#ifdef I2C_MODULE_2 
    #define I2C_Module_open I2C2_open
    #define I2C_Module_baud_rate I2C2_baud_rate
    #define I2C_Module_wait_bus_IDLE I2C2_wait_bus_IDLE
    #define I2C_Module_start_bit I2C2_start_bit
    #define I2C_Module_read_byte I2C2_read_byte
    #define I2C_Module_reset_write_collision_flag I2C2_reset_write_collision_flag
    #define I2C_Module_write_byte I2C2_write_byte 
    #define I2C_Module_stop_bit I2C2_stop_bit
    #define I2C_Module_set_slave_address I2C2_set_slave_address
    #define I2C_Module_check_data_ready I2C2_check_data_ready
    #define I2C_Module_check_read_write_operation I2C2_check_read_write_operation
    #define I2C_Module_check_stop_bit I2C2_check_stop_bit
    #define I2C_Module_check_start_bit I2C2_check_start_bit
#endif

//*************************************
//     Prototipi delle funzioni
//*************************************
void board_initialization (void);

//*************************************
//            I2C Slave
//*************************************
void main(void) {

    uint8_t address;
    uint8_t data;
   
    IO_set_all_ports_as_inputs();
    
    IO_set_port_direction(IO_PORTD, IO_ALL_PORT_OUTPUT);

    // I2C initialization
	I2C_Module_set_slave_address (SLAVE_ADDRESS);
    I2C_Module_open(I2C_SLAVE_7_BIT_ADDRESS);

    while (1) {

        //Wait for the first byte (Address)
        while(I2C_Module_check_data_ready() == 0);
        
        address = I2C_Module_read_byte();

         //Check Read or Write operation
        if (I2C_Module_check_read_write_operation() == WRITE){

            //Wait for the second byte (Data)
            while(I2C_Module_check_data_ready()==0);
            data = I2C_Module_read_byte();

            //Print the data out
            IO_write_port(IO_PORTD, data);
 
        } 

        //wait for the stop bit 
        while(I2C_Module_check_start_bit() == 0);

    }
}
