//******************************************************************************
// The software is an SPI master. It sends the data reflecting the push button 
// values. The push buttons are connected to RB4-RB7. A second MCU must be 
// programmed with the slave firmware to receive the data. The SPI module is 1
// but it can be selected in the Firmware and it could be different from the Slave.
//
//
// Tested Platform: Freedom III
// Tested MCU: PIC18F46K22
// Tested Compiler: XC8 2.05
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "module_IO.h"
#include "module_IO.c"

#include "module_SPI.h"
#include "module_SPI.c"

#define BUTTON_MASK  0xF0

#define BUTTON_1  0x01
#define BUTTON_2  0x02
#define BUTTON_3  0x04
#define BUTTON_4  0x08

#define BUTTON_1_PRESSED   0xE0     //0b11100000
#define BUTTON_2_PRESSED   0xD0     //0b11010000
#define BUTTON_3_PRESSED   0xB0     //0b10110000
#define BUTTON_4_PRESSED   0x70     //0b01110000


//*************************************
//           SPI Master
//*************************************
void main(void) {

    IO_set_all_ports_as_inputs();
    IO_enable_pull_up_resistors(IO_PORTB, 0xFF);
        
    SPI1_baudrate(SPI_CLK_OSC_64);
    SPI1_mode (SPI_MODE_1);
    SPI1_open (SPI_MASTER_DEVICE);
   
    //check pressed buttons
    while (1) {
      
        switch (PORTB & BUTTON_MASK) {

            case BUTTON_1_PRESSED : SPI1_write_byte (BUTTON_1);
                                    break;
            case BUTTON_2_PRESSED : SPI1_write_byte (BUTTON_2);
                                    break;
            case BUTTON_3_PRESSED : SPI1_write_byte (BUTTON_3);
                                    break;
            case BUTTON_4_PRESSED : SPI1_write_byte (BUTTON_4);
                                    break;
           }
    }
}
