//******************************************************************************
// The software reads the keypad pressed button and shows it to the LCD.
// The LCD must be connected as specified in the library included above.
// The keypad is connected on PORTB.
// New port can be used by defining the following before the library include:
// 
// #define KEYPAD_PORT IO_PORTx   //x is the name of the port you want.
//
// The Port you select, should have pull up resistors or you have to add it
// externally. 
//
// Refer to the schematic folder for the connection details. 
//
// Tested Platform: Freedom III
// Tested MCU: PIC18F46K22
// Tested Compiler: XC8 2.05
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "module_IO.h"
#include "module_IO.c"

#include "module_I2C.h"
#include "module_I2C.c"

#include "keypad.h"
#include "keypad.c"

#include "MCP2300x.h"
#include "MCP2300x.c"

#include "LCD_44780_I2C.h"
#include "LCD_44780_I2C.c"

void main(void) {
    
    uint8_t pressed_button = 0;
    
    IO_set_all_ports_as_inputs();
    
    LCD_initialize(20);
     
    keypad_initialization();
            
    while (1){
        
        LCD_home ();
        
        LCD_write_message ("Button : "); 
        
        pressed_button = keypad_read(DEBAUNCE_ON);
        
        if (pressed_button == KEYPAD_KEY_NOT_PRESSED) {
           LCD_write_message ("-"); 
        } else {
            LCD_write_char(pressed_button);
        }        
    }
}
