/*******************************************************************************

Author : Mauro Laurenti
Version : 1.1
 
Created on Date : 08/12/2018
Last update     : 28/07/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This library supports the accelerometer ADXL345.
Only I2C format is currently supported 

*******************************************************************************/
#ifndef LTLIB_ADXL335_H
#define LTLIB_ADXL335_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif


//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef LTLIB_ADC_MODULE_H
    #error The ADXL335 Library requires the "module_ADC" library to be included.
#endif

#ifndef LTLIB_DELAY_H
    #error The ADXL335 Library requires the "LTlib_delay" library to be included.
#endif


//*****************************************
//   ADC Channal Selection
//*****************************************
//You can define new channels, simply by defining it before the call 
//of the library header file

#ifndef ADXL335_X_VALUE
    #define ADXL335_X_VALUE ADC_CH0
#endif

#ifndef ADXL335_Y_VALUE
    #define ADXL335_Y_VALUE ADC_CH1 
#endif

#ifndef ADXL335_Z_VALUE
    #define ADXL335_Z_VALUE ADC_CH2
#endif


//**************************************************
//               Standard Constants
//**************************************************

//The value must be scaled according the full scale and be written as integer
//Default value is 3.3V with a full scale 5V (10 bits ADC)
//3.3Vcc with 10bits Vref 5V --> ADXL335_VCC 676
//3.3Vcc with 10bits Vref 3.3V --> ADXL335_VCC 1024
#ifndef ADXL335_VCC
    #define ADXL335_VCC 676
#endif

//it is the 0g voltage level, defined as ADXL335_VCC/2
#define ADXL335_0g ADXL335_VCC/2

 
//**************************************************

/**
 * This function reads a generic acceleration component.
 * The acc. component can be specified using the constants 
 * defined within the library.
 *
 * @param acc_component Acceleration component to be read out [ADXL335_X_VALUE, ADXL335_Y_VALUE, ADXL335_Z_VALUE]
 *
 * @return Acceleration read out (signed int)
 *
 */
int16_t ADXL335_read_acceleration (uint8_t acc_component);

#endif

