/*******************************************************************************

Author : Mauro Laurenti
Version : 1.1
 
Created on Date : 08/12/2018
Last update     : 10/02/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This library supports the accelerometer ADXL345.
Only I2C format is currently supported 

*******************************************************************************/


#ifndef LTLIB_ADXL345_H
#define LTLIB_ADXL345_H


#ifdef COMPILER_XC
    #include <xc.h>
#endif


//*****************************************
//   Library Test
//*****************************************

#include "LTlib.h"

#ifndef LTLIB_I2C_MODULE_H
    #error The ADXL345 Library requires the "module_I2C" library to be included.
#endif

#ifndef LTLIB_DELAY_H
    #error The ADXL345 Library requires the "delay" library to be included.
#endif


//*****************************************
//   I2C Module Switch
//*****************************************
#if defined (ADXL345_I2C_MODULE_1) || defined (ADXL345_I2C_MODULE_2)

#else
    #define ADXL345_I2C_MODULE_1
#endif 


#ifdef ADXL345_I2C_MODULE_1
    #define ADXL345_I2C_open I2C1_open
    #define ADXL345_I2C_baud_rate I2C1_baud_rate  
    #define ADXL345_I2C_write_byte_to_external_device I2C1_write_byte_to_external_device
    #define ADXL345_I2C_read_byte_from_external_device I2C1_read_byte_from_external_device
#endif

#ifdef ADXL345_I2C_MODULE_2 
    #define ADXL345_I2C_open I2C2_open
    #define ADXL345_I2C_baud_rate I2C2_baud_rate 
    #define ADXL345_I2C_write_byte_to_external_device I2C2_write_byte_to_external_device
    #define ADXL345_I2C_read_byte_from_external_device I2C2_read_byte_from_external_device
#endif

//**************************************************
//           Device Address in 8 bit formats
//**************************************************
//A specifc write address can be selected before the library, by defining
// ADXL345_WRITE_ADD . The address is specified as control word (8 bits), 
// including the write bit.  
//#define ADXL345_WRITE_ADD 0x3A
//#define ADXL345_WRITE_ADD 0xA6

#ifndef ADXL345_WRITE_ADD
    #define ADXL345_WRITE_ADD 0xA6
#endif


//**************************************************
//           Address constants
//**************************************************
#define ADXL345_REG_DEVID           0x00
#define ADXL345_REG_THRESH_TAP      0x1D
#define ADXL345_REG_OFSX            0x1E
#define ADXL345_REG_OFSY            0x1F
#define ADXL345_REG_OFSZ            0x20
#define ADXL345_REG_DUR             0x21
#define ADXL345_REG_LATENT          0x22
#define ADXL345_REG_WINDOW          0x23
#define ADXL345_REG_THRESH_ACT      0x24
#define ADXL345_REG_THRESH_INACT    0x25
#define ADXL345_REG_TIME_INACT      0x26
#define ADXL345_REG_ACT_INACT_CTL   0x27
#define ADXL345_REG_THRESH_FF       0x28
#define ADXL345_REG_TIME_FF         0x29
#define ADXL345_REG_TAP_AXES        0x2A
#define ADXL345_REG_ACT_TAP_STATUS  0x2B
#define ADXL345_REG_BW_RATE         0x2C
#define ADXL345_REG_POWER_CTL       0x2D
#define ADXL345_REG_INT_ENABLE      0x2E
#define ADXL345_REG_INT_MAP         0x2F
#define ADXL345_REG_INT_SOURCE      0x30
#define ADXL345_REG_DATA_FORMAT     0x31
#define ADXL345_REG_DATA_X0         0x32
#define ADXL345_REG_DATA_X1         0x33
#define ADXL345_REG_DATA_Y0         0x34
#define ADXL345_REG_DATA_Y1         0x35
#define ADXL345_REG_DATA_Z0         0x36
#define ADXL345_REG_DATA_Z1         0x37
#define ADXL345_REG_FIFO_CTL        0x38
#define ADXL345_REG_FIFO_STATUS     0x39

//**************************************************
//            Sampling rate Constants
//**************************************************

#define ADXL345_SAMPLING_400    0x0C
#define ADXL345_SAMPLING_200    0x0B
#define ADXL345_SAMPLING_100    0x0A
#define ADXL345_SAMPLING_50     0x09
#define ADXL345_SAMPLING_25     0x08
#define ADXL345_SAMPLING_12_5   0x07
#define ADXL345_SAMPLING_6_25   0x06

//**************************************************
//               Standard Constants
//**************************************************
#define ADXL345_DEVID   0xE5

#define ADXL345_X_VALUE 0x01
#define ADXL345_Y_VALUE 0x02
#define ADXL345_Z_VALUE 0x03

#define ADXL345_RANGE_2g    0x00
#define ADXL345_RANGE_4g    0x01
#define ADXL345_RANGE_8g    0x10
#define ADXL345_RANGE_16g   0x11

#define ADXL345_MASK_SLEEP      0xFB
#define ADXL345_SLEEP_MODE_ON   0x04
#define ADXL345_SLEEP_MODE_OFF  0x00

//**************************************************
//               Functions prototypes
//**************************************************
/**
 * This function initialize the I2C module accordingly to the clock and baud rate.
 * The Clock is defined within the LTlib.h file.
 *
 *
 * @param baud_rate_KHz Baud rate expressed in KHz [min: 1, max:400]
 * 
 * @param range Full scale range [ADXL345_RANGE_2g, ADXL345_RANGE_4g, ADXL345_RANGE_8g, ADXL345_RANGE_16g]
 * 
 * @param sampling_rate Sampling rate in Hz, defined by the constants ADXL345_SAMPLING_xxx
 *
 * @return void
 *
 * @note  If other devices are connected on the I2C bus the baud rate would be changed by
 * the function. Among several calls from different device initializations, only the last one will be
 * reflected in the I2C module configuration. 
 * 
 * @note The current initialization supports I2C module only. SPI mode is not supported. 
 *
 */
void ADXL345_initialize (uint16_t baud_rate_KHz, uint8_t range, uint8_t sampling_rate);


/**
 * This function allows reducing the device current by setting the device in sleep mode.
 * Only the sleep bit is changed. The other register values remains unchanged.
 *
 * @param sleep_mode Change the sleep mode as needed [ADXL345_SLEEP_MODE_ON, ADXL345_SLEEP_MODE_OFF]
 *
 * @return 
 *
 */
void ADXL345_sleep_mode (uint8_t sleep_mode);
   

/**
 * This function writes a generic register within the ADXL345 device.
 * The register can be specified as hex value or using the constants 
 * defined within the library.
 *
 * @param address Address of the internal register to be written.
 *
 * @param value  Register content to be written.
 * 
 * @return status   0: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 */
int8_t ADXL345_write_address (uint8_t address, uint8_t value);


/**
 * This function reads a generic register within the ADXL345 device.
 * The register can be specified as hex value or using the constants 
 * defined within the library.
 *
 * @param address Address of the internal register to be read.
 *
 * @param value  Register content readback.
 * 
 * @return status   0: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 */
int8_t ADXL345_read_address (uint8_t address, uint8_t * value);



/**
 * This function reads a generic acceleration component.
 * The acc. component can be specified using the constants 
 * defined within the library.
 *
 * @param acc_component Acceleration component to be read out [ADXL345_X_VALUE, ADXL345_Y_VALUE, ADXL345_Z_VALUE]
 *
 * @param readback_value  Register content readback.
 * 
 * @return status   0: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 */
int8_t ADXL345_read_acceleration (uint8_t acc_component, int16_t * readback_value);


/**
 * This function reads the Device ID out of the ADXL345 device.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param device_ID Pointer to a uint8_t variable that will contain the Device ID. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t ADXL345_get_device_ID (uint8_t device_address, uint8_t * device_ID);

#endif

