/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
 
Created on Date : 23/11/2019
Last update     : 23/11/2019
 
CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

These functions allow the user to read the data out of the pressure sensor 
BMP280.

*******************************************************************************/
#ifndef LTLIB_BMP280_H
#define LTLIB_BMP280_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif


//*****************************************
//   Library Test
//*****************************************

#include "LTlib.h"

#ifndef LTLIB_I2C_MODULE_H
    #error The BMP280 Library requires the "module_I2C" library to be included.
#endif

#ifndef LTLIB_DELAY_H
    #error The BMP280 Library requires the "LTlib_delay" library to be included.
#endif


//*****************************************
//   I2C Module Switch
//*****************************************
#if defined (BMP280_I2C_MODULE_1) || defined (BMP280_I2C_MODULE_2)

#else
    #define BMP280_I2C_MODULE_1
#endif 

#ifdef BMP280_I2C_MODULE_1
    #define BMP280_I2C_open I2C1_open
    #define BMP280_I2C_baud_rate I2C1_baud_rate  
    #define BMP280_I2C_write_byte_to_external_device I2C1_write_byte_to_external_device
    #define BMP280_I2C_read_byte_from_external_device I2C1_read_byte_from_external_device
#endif

#ifdef BMP280_I2C_MODULE_2 
    #define BMP280_I2C_open I2C2_open
    #define BMP280_I2C_baud_rate I2C2_baud_rate 
    #define BMP280_I2C_write_byte_to_external_device I2C2_write_byte_to_external_device
    #define BMP280_I2C_read_byte_from_external_device I2C2_read_byte_from_external_device
#endif

//*****************************************
//   Device Address options
//*****************************************
#define BMP280_DEVICE_ADDRESS_0 0x76
#define BMP280_DEVICE_ADDRESS_1 0x77

//*****************************************
//   Device registers
//*****************************************
#define BMP280_REG_TEMP_XLSB    0xFC 
#define BMP280_REG_TEMP_LSB     0xFB 
#define BMP280_REG_TEMP_MSB     0xFA
#define BMP280_REG_PRESS_XLSB   0xF9 
#define BMP280_REG_PRESS_LSB    0xF8 
#define BMP280_REG_PRESS_MSB    0xF7
#define BMP280_REG_CONFIG       0xF5
#define BMP280_REG_CTRL_MES     0xF4
#define BMP280_REG_STATUS       0xF3
#define BMP280_REG_RESET        0xE0
#define BMP280_REG_ID           0xD0


//*****************************************
//   Constant Configigurations
//*****************************************
#define BMP280_DEVICE_ID 0x58

#define BMP280_OVERSAMPLING_0   0x00
#define BMP280_OVERSAMPLING_1   0x01
#define BMP280_OVERSAMPLING_2   0x02
#define BMP280_OVERSAMPLING_4   0x03
#define BMP280_OVERSAMPLING_8   0x04
#define BMP280_OVERSAMPLING_16  0x05
#define BMP280_OVERSAMPLING_TEMP_MASK   0x1F
#define BMP280_OVERSAMPLING_PRESS_MASK  0xE3

#define BMP280_MODE_SLEEP   0x00
#define BMP280_MODE_FORCED  0x01
#define BMP280_MODE_NORMAL  0x03
#define BMP280_MODE_MASK    0xFC

#define BMP280_CONVERSION_RUNNING    0x01
#define BMP280_CONVERSION_COMPLETED  0x00
#define BMP280_MEASURING_BIT_MASK    0x08

#define BMP280_TIME_INTERVAL_0_5_ms   0x00 << 5
#define BMP280_TIME_INTERVAL_62_5_ms  0x01 << 5
#define BMP280_TIME_INTERVAL_125_ms   0x02 << 5
#define BMP280_TIME_INTERVAL_250_ms   0x03 << 5
#define BMP280_TIME_INTERVAL_500_ms   0x04 << 5
#define BMP280_TIME_INTERVAL_1000_ms  0x05 << 5
#define BMP280_TIME_INTERVAL_2000_ms  0x06 << 5
#define BMP280_TIME_INTERVAL_4000_ms  0x07 << 5
#define BMP280_TIME_INTERVAL_MASK     0x1F  

#define BMP280_FIR_COEFFICIENTS_01    0x00 << 2
#define BMP280_FIR_COEFFICIENTS_02    0x01 << 2
#define BMP280_FIR_COEFFICIENTS_04    0x02 << 2
#define BMP280_FIR_COEFFICIENTS_08    0x03 << 2
#define BMP280_FIR_COEFFICIENTS_16    0x04 << 2
#define BMP280_FIR_COEFFICIENTS_MASK  0xE3

//*****************************************
//   Compensatation coefficients
//*****************************************
uint16_t BMP280_dig_T1 = 0x0000;
int16_t  BMP280_dig_T2 = 0x0000;
int16_t  BMP280_dig_T3 = 0x0000;

uint16_t BMP280_dig_P1 = 0x0000;
int16_t  BMP280_dig_P2 = 0x0000;
int16_t  BMP280_dig_P3 = 0x0000;
int16_t  BMP280_dig_P4 = 0x0000;
int16_t  BMP280_dig_P5 = 0x0000;
int16_t  BMP280_dig_P6 = 0x0000;
int16_t  BMP280_dig_P7 = 0x0000;
int16_t  BMP280_dig_P8 = 0x0000;
int16_t  BMP280_dig_P9 = 0x0000;

// This variable requires at least one temperature read.
// afterward it is used by the pressure read function to compensate the pressure
// ADC read out value. 
uint32_t BMP280_t_fine = 0;

//******************************************************************************


/**
 * This function initializes the I2C module accordingly to the clock and baud rate.
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param baud_rate_KHz Baud rate expressed in KHz
 *
 * @return 
 *
 * @note  If other devices are connected on the I2C bus the baud rate would be changed by
 * the function. Among several calls from different device initializations, only the last one will be
 * reflected in the I2C module configuration. 
 *
 */
void BMP280_initialize (uint8_t device_address, uint16_t baud_rate_KHz);

 
/**
 * This function writes into any register of the BMP280 device.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param register_add Address of the register that must be written. 
 *
 * @param data Data that must be written into the register. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 * @note    For the register address you must use the one specified in the header
 *          file. The names reflect the ones in the datasheet but 
 *           they have the BMP280_REG_ prefix.
 *
 */
int8_t BMP280_set_register (uint8_t device_address, uint8_t register_add,  uint8_t data );


/**
 * This function reads from any register of the BMP280 device.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param register_add Address of the register that must be written. 
 * 
 * @param readback_value Pointer to a u_int8 variable that will contain the register content. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 * @note    For the register address you must use the one specified in the header
 *          file. The names reflect the ones in the datasheet but 
 *          they have the BMP280_REG_ prefix.
 *
 */
int8_t BMP280_get_register (uint8_t device_address, uint8_t register_address, uint8_t * readback_value);


/**
 * This function reads the temperature out of the BMP280 device.
 * The returned value is an integer value, "2357" equal to 23.57C 
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param temperature Pointer to a int32_t variable that will contain the temperature. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 */
int8_t BMP280_get_temperature (uint8_t device_address, int32_t * temperature);


/**
 * This function reads the pressure out of the BMP280 device.
 * The returned value is Pa. 
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param pressure Pointer to a int32_t variable that will contain the pressure. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t BMP280_get_pressure (uint8_t device_address, uint32_t * pressure);


/**
 * This function reads the Device ID out of the BMP280 device.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param device_ID Pointer to a uint8_t variable that will contain the Device ID. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t BMP280_get_device_ID (uint8_t device_address, uint8_t * device_ID);


/**
 * This function reads the calibration data out of the BMP280 device.
 * This function is called automatically by the device initialization, thus 
 * there is no need to call at application level. Nevertheless if two sensors are 
 * used, it is required to call the function before any measure read out, to allow
 * using the right coefficients for each sensor. 
 *
 * @param device_address Device address in 7 bits format.
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t BMP280_download_calibration_data (uint8_t device_address);


/**
 * This function sets the oversampling associated with the temperature readout.
 * By setting oversampling to 0, you disable the measurement and only the pressure
 * is sampled and converted. 
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param oversampling Set the oversampling value for the measurement [use the oversampling constants].
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t BMP280_set_temperature_oversampling (uint8_t device_address, uint8_t oversampling);


/**
 * This function sets the oversampling associated with the pressure readout.
 * By setting oversampling to 0, you disable the measurement. 
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param oversampling Set the oversampling value for the measurement [use the oversampling constants].
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t BMP280_set_pressure_oversampling (uint8_t device_address, uint8_t oversampling);


/**
 * This function resets the device via a software command.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t BMP280_software_reset (uint8_t device_address);


/**
 * This function starts the pressure and temperature sampling. It must be called 
 * periodically for single shot measure (forced event mode). 
 * It must be called only once if the device is configured for auto-sampling. 
 * 
 * @param device_address Device address in 7 bits format.
 * 
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t BMP280_trigger_new_measure (uint8_t device_address);


/**
 * This function sets the BMP280, etiher Sleep, Forces or Normal.
 * If you set the mode to Forced, the fucntion is equivalent to 
 * BMP280_trigger_new_measure.
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param mode New operating mode [BMP280_MODE_SLEEP, BMP280_MODE_FORCED, BMP280_MODE_NORMAL]
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t BMP280_set_mode (uint8_t device_address, uint8_t mode);


/**
 * This function sets the BMP280 time interval among measures in Normal Mode.
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param time New interval [Use BMP280_TIME_INTERVAL_xxxx definitions]
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t BMP280_set_time_interval (uint8_t device_address, uint8_t time);

/**
 * This function sets the BMP280 coefficients for the FIR filter.
 * If the coefficients are set to 1, the filter is OFF.  
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param coefficients New FIR coefficients [Use BMP280_FIR_COEFFICIENTS_XX definitions]
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t BMP280_set_FIR_coefficients (uint8_t device_address, uint8_t coefficients);


/**
 * This function checks if the BMP280 is currently converting the ADC samples.
 * It can be used after a forced measure to make sure that the conversion is 
 * over and the data in the ADC registers are valid for the new measure. 
 *
 * @param device_address Device address in 7 bits format.
 *
 * @return status 0: Conversion is completed
 *       	      1: Conversion is running
 */
int8_t BMP280_is_conversion_running (uint8_t device_address);

#endif
