/*******************************************************************************

Author : Mauro Laurenti
Version : 1.4
Created on Date : 26/05/2012
Last update     : 25/17/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************


This library contains all the functions that are handy for controlling 			
a GLCD display based on KS0108B controller.						
 	
More details and pictures can be downloaded from the Brief Note BN0016(www.LaurTec.com)

*******************************************************************************/

#ifndef LTLIB_GLCD_K0108B_H
#define LTLIB_GLCD_K0108B_H

#ifdef COMPILER_XC
    #include <xc.h>
    #define NOP_DELAY _delay (5)
#endif


//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef LTLIB_DELAY_H
    #error The GLCD_K0108B Library requires the "delay" library to be included.
#endif

#ifndef LTLIB_UTILITIES_H
    #error The LCD_44780 Library requires the "LTlib_utilities" library to be included.
#endif

//**************************************************
// HW Initialization
// This section must be properly initialized according
// to the GLCD connections
//**************************************************

#ifdef _PIC18
    #define GLCD_DATA_WRITE         LATD
    #define GLCD_DATA_READ          PORTD
    #define GLCD_DATA_DIRECTION     TRISD

    //Control bit declaration
    #define GLCD_D_I LATAbits.LATA3
    #define GLCD_R_W LATAbits.LATA4
    #define GLCD_E   LATAbits.LATA5
    #define GLCD_CS1 LATEbits.LATE0
    #define GLCD_CS2 LATEbits.LATE1
    #define GLCD_RST LATEbits.LATE2
    #define GLCD_LED LATCbits.LATC1

    //Control bit direction
    #define GLCD_D_I_DIRECTION TRISAbits.RA3
    #define GLCD_R_W_DIRECTION TRISAbits.RA4
    #define GLCD_E_DIRECTION   TRISAbits.RA5
    #define GLCD_CS1_DIRECTION TRISEbits.RE0
    #define GLCD_CS2_DIRECTION TRISEbits.RE1
    #define GLCD_RST_DIRECTION TRISEbits.RE2
    #define GLCD_LED_DIRECTION TRISCbits.RC1
#endif


#ifndef _PIC18

    #define GLCD_DATA_WRITE         PORTD
    #define GLCD_DATA_READ          PORTD
    #define GLCD_DATA_DIRECTION     TRISD

    #define GLCD_D_I PORTAbits.RA3
    #define GLCD_R_W PORTAbits.RA4
    #define GLCD_E   PORTAbits.RA5
    #define GLCD_CS1 PORTEbits.RE0
    #define GLCD_CS2 PORTEbits.RE1
    #define GLCD_RST PORTEbits.RE2
    #define GLCD_LED PORTCbits.RC1

    #define GLCD_D_I_DIRECTION TRISAbits.TRISA3
    #define GLCD_R_W_DIRECTION TRISAbits.TRISA4
    #define GLCD_E_DIRECTION   TRISAbits.TRISA5
    #define GLCD_CS1_DIRECTION TRISEbits.TRISE0
    #define GLCD_CS2_DIRECTION TRISEbits.TRISE1
    #define GLCD_RST_DIRECTION TRISEbits.TRISE2
    #define GLCD_LED_DIRECTION TRISCbits.TRISC1

#endif


//**************************************************
// GLCD Controller constants
//**************************************************

#define GLCD_INPUT 0xFF
#define GLCD_OUTPUT 0x00

#define GLCD_NUMBER_OF_PAGES 8
#define GLCD_X_RESOLUTION 128
#define GLCD_Y_RESOLUTION 64

#define GLCD_CONTROLLER_MAX_X 64


//**************************************************
// Font Tables and Pictures 
//**************************************************

#define GLCD_ENABLE_FONT_5x7

#define GLCD_ENABLE_LOGO_1

//**************************************************
// GLCD Font Size
//**************************************************

#define GLCD_FONT_WIDTH_5   5
#define GLCD_FONT_HEIGTH_7  7

#define GLCD_FONT_SPACE 1


//**************************************************
// GLCD command constants 
//**************************************************

#define GLCD_COMM_DISPLAY_ON 	0x3F    //0b00111111
#define GLCD_COMM_DISPLAY_OFF	0x3E    //0b00111110

#define GLCD_COMM_PAGE_SELECT	0xB8    //0b10111000

#define GLCD_COMM_ADDRESS_SELECT 0x40   //0b01000000

#define GLCD_COMM_START_LINE 	0xC0    //0b11000000


//**************************************************
// GLCD Status Flag 
//**************************************************

#define GLCD_BUSY_FLAG 0x80

	
//**************************************************
// General Purpose constants 
//**************************************************

#define GLCD_TURN_ON_LED    0x01    //0b00000001
#define GLCD_TURN_OFF_LED   0x00    //0b00000000

#define GLCD_TURN_ON_DISPLAY     0x01    //0b00000001
#define GLCD_TURN_OFF_DISPLAY    0x00    //0b00000000

#define GLCD_HIGH           0x01    //0b00000001
#define GLCD_LOW            0x00    //0b00000000

#define GLCD_ENABLE         0x01    //0b00000001
#define GLCD_DISABLE        0x00    //0b00000000

#define GLCD_ENABLE_RESET   0x00    //0b00000000
#define GLCD_DISABLE_RESET  0x01    //0b00000001

#define GLCD_CHIP_SELECT_1 0x00
#define GLCD_CHIP_SELECT_2 0x01

#define GLCD_FILLING_WHITE 0x00
#define GLCD_FILLING_BLACK 0xFF

#define GLCD_ZERO_CLEANING_ON 0x01
#define GLCD_ZERO_CLEANING_OFF 0x00

//****************************************************************************************************
//    PUBLIC FUNCTIONS
//****************************************************************************************************

/**
 * GLCD Initialization. This function should be called before using the GLCD.
 * Controller are initialized, backlight is set to OFF, Display is cleared and font is set.
 *
 * @param void
 *
 * @return void
 *
 */
void  GLCD_initialize (void);


/**
 * This function clears the display either in black or white color. 
 *
 * @param filling_color Filling color allows to set the display either on white or black colour [GLCD_FILLING_WHITE, GLCD_FILLING_BLACK]
 *
 * @return void
 *
 */
void GLCD_clear (uint8_t filling_color);


/**
 * This function turns ON-OFF the GLCD back light.
 *
 * @param backlight Control the backlight LED.
 *         TURN_ON_LED  : Backlight is ON
 *         TURN_OFF_LED : Backlight is OFF
 *
 * @return void
 *
 */
void GLCD_backlight (uint8_t backlight);


/**
 * This function sets the display turning it ON or OFF.
 *
 * @param display_status Turn ON or OFF the display [GLCD_TURN_ON_DISPLAY, GLCD_TURN_OFF_DISPLAY]
 *
 * @return void
 *
 * @note If the display is turned OFF the RAM retains the data.
 */
void GLCD_set_display (uint8_t display_status);


/**
 * This function sets the vertical offset allowing a display shift.
 *
 * @param vertical_offset Vertical offset [0..63].  
 *
 * @return void
 *
 */
void GLCD_set_vertical_offset (uint8_t vertical_offset);


/**
 * This function plots a single dot either in black or white colour. 
 * Plot position is set accordingly to x,y.
 * 
 * @param x_pos Position X [0..127].
 *
 * @param y_pos Position Y [0..63].
 *
 * @param filling_color Filling colour allows to set the dot either on white or black colour [GLCD_FILLING_WHITE, GLCD_FILLING_BLACK]
 *
 * @return void
 *
 */
void GLCD_plot_xy (uint8_t x_pos,  uint8_t y_pos, uint8_t filling_color);


/**
 * This function plots an horizontal line either in black or white colour. 
 * Origin and length can be set by the user.
 * 
 * @param x_origin Line X origin [0..127].
 *
 * @param y_origin Line Y origin [0..63].
 *
 * @param length Line length [0..127].
 *
 * @param filling_color Filling colour allows to set the dot either on white or black colour [GLCD_FILLING_WHITE, GLCD_FILLING_BLACK]
 *
 * @return void
 *
 * @warning It is user responsibility to make sure that the line length is valid, depending on x_origin, y_origin.
 *
 */
void GLCD_draw_horizontal_line (uint8_t x_origin, uint8_t y_origin, uint8_t length, uint8_t filling_color);


/**
 * This function plots a vertical line either in black or white colour. 
 * Origin and length can be set by the user.
 * 
 * @param x_origin Line X origin [0..128].
 *
 * @param y_origin Line Y origin [0..63].
 *
 * @param length Line length [0..63].
 *
 * @param filling_color Filling colour allows to set the dot either on white or black colour [GLCD_FILLING_WHITE, GLCD_FILLING_BLACK]
 *
 * @return void
 *
 * @warning It is user responsibility to make sure that the line length is valid, depending on x_origin, y_origin.
 *
 */
void GLCD_draw_vertical_line (uint8_t x_origin, uint8_t y_origin, uint8_t length, uint8_t filling_color);


/**
 * This function plots an empty window either with black or white colour borders. 
 * Origin and length and height can be set by the user.
 * 
 * @param x_origin Window X origin [0..127].
 *
 * @param y_origin Window Y origin [0..63].
 *
 * @param length Window length [0..127].
 *
 * @param length Window height [0..63].
 *
 * @param filling_color Filling colour allows to set the dot either on white or black colour [GLCD_FILLING_WHITE, GLCD_FILLING_BLACK]
 *
 * @return void
 *
 * @warning It is user responsibility to make sure that the window length and height are valid, depending on x_origin, y_origin.
 *
 */
void GLCD_draw_window (uint8_t x_origin, uint8_t y_origin, uint8_t length, uint8_t height, uint8_t filling_color);


/**
 * This function plots a coloured box either in black or white colour. 
 * Origin and length and height can be set by the user.
 * 
 * @param x_origin Box X origin [0..127]
 *
 * @param y_origin Box Y origin [0..63]
 *
 * @param length Box length.
 *
 * @param length Box height.
 *
 * @param filling_color Filling colour allows to set the dot either on white or black colour [GLCD_FILLING_WHITE, GLCD_FILLING_BLACK]
 *
 * @return void
 *
 * @warning It is user responsibility to make sure that the window length and height are valid, depending on x_origin, y_origin.
 *
 */
void GLCD_draw_box (uint8_t x_origin, uint8_t y_origin, uint8_t length, uint8_t height, uint8_t filling_color);


/**
 * This function draws a picture with 128x64 resolution.
 *
 * @param picture_table Pointer to the table containing the picture (stored in flash). 
 *
 * @return void
 *
 */
void GLCD_draw_picture (const uint8_t * picture_table);


/**
 * This function writes an ASCII character.
 *
 * @param x_pos Position X [0..127].
 *
 * @param y_pos Position Y [0..63].
 *
 * @param character ASCII character either as number (unsigned char) or character. Example: 97, 'a'. 
 *
 * @return void
 *
 */
void GLCD_write_char (uint8_t x_pos, uint8_t y_pos, uint8_t character);


/**
 * This function writes an entire string of char. The string must be terminated with /0.
 *
 * @param x_pos Position X [0..127].
 *
 * @param y_pos Position Y [0..63].
 *
 * @param character_array Pointer to the array that contains the string. 
 *
 * @return void
 *
 * @warning The array of unsigned char must be terminated with the /0 character.
 *
 */
void GLCD_write_string (uint8_t x_pos, uint8_t y_pos, uint8_t * character_array);


/**
 * This function writes a constant string of char. 
 *
 * @param x_pos Position X [0..127].
 *
 * @param y_pos Position Y [0..63].
 *
 * @param character_array Pointer to the array that contains the string (stored in flash). 
 *
 * @return void
 *
 */
void GLCD_write_message (uint8_t x_pos, uint8_t y_pos, const char * character);


/**
 * This function writes variable that contains an integer number. 
 *
 * @param x_pos Position X [0..127].
 *
 * @param y_pos Position Y [0..63].
 *
 * @param value Integer value to be written (either positive or negative)
 *
 * @param number_of_digits The number is formatted with a fixed number of digits to ensure a constant string length.
 *
 * @return void
 *
 */
void GLCD_write_integer (uint8_t x_pos, uint8_t y_pos, int16_t value, uint8_t number_of_digits, uint8_t zero_cleaning);


/**
 * This function sets the font. Currently only one font is supported so is not yet that important.
 * The initialization function is calling it so there is no reason to call it till new fonts will be supported.
 *
 * @param width Font width.
 *
 * @param width Font height.
 *
 * @return void
 *
 */
void GLCD_set_font (uint8_t width, uint8_t height);


//****************************************************************************************************
//    PRIVATE FUNCTIONS (The user should not use it)
//****************************************************************************************************


/**
 * This function waits until the controller is not busy any longer.
 * It calls the private function GLCD_get_controller_status ().
 *
 * @param void  
 *
 * @return void
 *
 * @note The function is a blocking function. This is a private function, the user should not use it.
 */
void GLCD_wait_busy_status (void);


/**
 * This function gets the Controller Status, among which the status bit.
 *
 * @param void  
 *
 * @return status Controller status (refer to the datasheet)
 *
 * @note This is a private function. The user should not use it.
 */
uint8_t GLCD_get_controller_status (void);


/**
 * This function writes a command to the controller. 
 * The available commands are GLCD command constants. 
 * For each command there is a dedicated function. The User should use these rather than sending commands.
 *
 * @param command_to_send Command to be sent to the Controller.  
 *
 * @return void
 *
 * @note This is a private function. The user should not use it.
 */

void GLCD_write_command (uint8_t command_to_send);


/**
 * This function sets the controller page (0 to GLCD_NUMBER_OF_PAGES).
 *
 * @param controller_page Controller page [0..GLCD_NUMBER_OF_PAGES].
 *
 * @return void
 *
 * @note This is a private function. The user should not use it.
 */
void GLCD_set_page (uint8_t controller_page);


/**
 * This function sets the controller address (0 to 63). 
 *
 * @param controller_address Controller address [0..63]. 
 *
 * @return void
 *
 * @note This is a private function. The user should not use it.
 */
void GLCD_set_address (uint8_t controller_address);


/**
 * This function writes a data byte to the controller. 
 *
 * @param data_to_send Data byte to be sent out. 
 *
 * @return void
 *
 * @note This is a private function. The user should not use it.
 */
void GLCD_write_data (uint8_t data_to_send);


/**
 * This function reads a data byte from the controller. 
 * It is used to know the data byte value of a certain address before writing into it.
 *
 * @param void
 *
 * @return Data byte out of selected address.
 *
 * @note This is a private function. The user should not use it.
 */
uint8_t GLCD_read_data (void);



//************************************************************************************************
//                                       Font Tables
//************************************************************************************************

#ifdef GLCD_ENABLE_FONT_5x7

const uint8_t font_5x7[96][5] = {

	0x00,0x00,0x00,0x00,0x00, // empty space  
	0x00,0x00,0x5f,0x00,0x00, // ! 
 	0x00,0x03,0x00,0x03,0x00, // " 
	0x14,0x7f,0x14,0x7f,0x14, // # 
   	0x24,0x2a,0x7f,0x2a,0x12, // $
	0x23,0x13,0x08,0x64,0x62, // % 
	0x36,0x49,0x55,0x22,0x50, // & 
	0x00,0x05,0x03,0x00,0x00, // ' 
 	0x00,0x1c,0x22,0x41,0x00, // (
	0x00,0x41,0x22,0x1c,0x00, // ) 
    0x14,0x08,0x3e,0x08,0x14, // *
	0x08,0x08,0x3e,0x08,0x08, // +
    0x00,0x50,0x30,0x00,0x00, // ,
	0x08,0x08,0x08,0x08,0x08, // -
    0x00,0x60,0x60,0x00,0x00, // .
	0x20,0x10,0x08,0x04,0x02, // /
    0x3e,0x51,0x49,0x45,0x3e, // 0
	0x00,0x42,0x7f,0x40,0x00, // 1
    0x42,0x61,0x51,0x49,0x46, // 2
	0x21,0x41,0x45,0x4b,0x31, // 3
    0x18,0x14,0x12,0x7f,0x10, // 4
	0x27,0x45,0x45,0x45,0x39, // 5
    0x3c,0x4a,0x49,0x49,0x30, // 6
	0x01,0x71,0x09,0x05,0x03, // 7
    0x36,0x49,0x49,0x49,0x36, // 8
	0x06,0x49,0x49,0x29,0x1e, // 9
    0x00,0x36,0x36,0x00,0x00, // :
	0x00,0x56,0x36,0x00,0x00, // ;
    0x08,0x14,0x22,0x41,0x00, // <
	0x14,0x14,0x14,0x14,0x14, // =
    0x00,0x41,0x22,0x14,0x08, // >
	0x02,0x01,0x51,0x09,0x06, // ?
    0x32,0x49,0x79,0x41,0x3e, // @
	0x7e,0x11,0x11,0x11,0x7e, // A
    0x7f,0x49,0x49,0x49,0x36, // B
	0x3e,0x41,0x41,0x41,0x22, // C
    0x7f,0x41,0x41,0x22,0x1c, // D
	0x7f,0x49,0x49,0x49,0x41, // E
    0x7f,0x09,0x09,0x09,0x01, // F
	0x3e,0x41,0x49,0x49,0x7a, // G
    0x7f,0x08,0x08,0x08,0x7f, // H
	0x00,0x41,0x7f,0x41,0x00, // I
    0x20,0x40,0x41,0x3f,0x01, // J
	0x7f,0x08,0x14,0x22,0x41, // K
    0x7f,0x40,0x40,0x40,0x40, // L
	0x7f,0x02,0x0c,0x02,0x7f, // M
    0x7f,0x04,0x08,0x10,0x7f, // N
	0x3e,0x41,0x41,0x41,0x3e, // O
    0x7f,0x09,0x09,0x09,0x06, // P
	0x3e,0x41,0x51,0x21,0x5e, // Q
    0x7f,0x09,0x19,0x29,0x46, // R
	0x46,0x49,0x49,0x49,0x31, // S
    0x01,0x01,0x7f,0x01,0x01, // T
	0x3f,0x40,0x40,0x40,0x3f, // U
    0x1f,0x20,0x40,0x20,0x1f, // V
	0x3f,0x40,0x38,0x40,0x3f, // W
    0x63,0x14,0x08,0x14,0x63, // X
	0x07,0x08,0x70,0x08,0x07, // Y
    0x61,0x51,0x49,0x45,0x43, // Z
	0x00,0x7f,0x41,0x41,0x00, // [
    0x02,0x04,0x08,0x10,0x20, // \]
	0x00,0x41,0x41,0x7f,0x00, // ]
    0x04,0x02,0x01,0x02,0x04, // ^
	0x40,0x40,0x40,0x40,0x40, // _
    0x00,0x01,0x02,0x04,0x00, // `
	0x20,0x54,0x54,0x54,0x78, // a
    0x7f,0x48,0x44,0x44,0x38, // b
	0x38,0x44,0x44,0x44,0x00, // c
    0x38,0x44,0x44,0x48,0x7f, // d
	0x38,0x54,0x54,0x54,0x18, // e
    0x08,0x7e,0x09,0x01,0x02, // f
	0x0c,0x52,0x52,0x52,0x3e, // g
    0x7f,0x08,0x04,0x04,0x78, // h
	0x00,0x44,0x7d,0x40,0x00, // i
    0x20,0x40,0x44,0x3d,0x00, // j
	0x7f,0x10,0x28,0x44,0x00, // k
    0x00,0x41,0x7f,0x40,0x00, // l
	0x7c,0x04,0x18,0x04,0x78, // m
    0x7c,0x08,0x04,0x04,0x78, // n
	0x38,0x44,0x44,0x44,0x38, // o
    0x7c,0x14,0x14,0x14,0x08, // p
	0x08,0x14,0x14,0x18,0x7c, // q
    0x7c,0x08,0x04,0x04,0x08, // r
	0x48,0x54,0x54,0x54,0x20, // s
    0x04,0x3f,0x44,0x40,0x20, // t
	0x3c,0x40,0x40,0x20,0x7c, // u
    0x1c,0x20,0x40,0x20,0x1c, // v
	0x3c,0x40,0x30,0x40,0x3c, // w
    0x44,0x28,0x10,0x28,0x44, // x
	0x0c,0x50,0x50,0x50,0x3c, // y
    0x44,0x64,0x54,0x4c,0x44, // z
	0x00,0x08,0x36,0x41,0x00, // {
    0x00,0x00,0x7f,0x00,0x00, // |
	0x00,0x41,0x36,0x08,0x00, // }
    0x10,0x08,0x08,0x10,0x08, // ^
	0x00,0x00,0x02,0x05,0x02}; // degree 

#endif // ENABLE_FONT_5x7


//************************************************************************************************
//                                       Pictures and Logos
//************************************************************************************************

#ifdef GLCD_ENABLE_LOGO_1
    

const uint8_t logo_1 [] = {

0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x30, 0xF0, 0xF0, 0xF0, 0xF0, 0x30, 0x30, 0x30, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xE0, 0xF0, 0x70, 0x30, 0x30,
0xF0, 0xF0, 0xF0, 0xF0, 0x30, 0x30, 0x70, 0xF0, 0xF0, 0xE0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xC0,
0xFC, 0xFF, 0xFF, 0x3F, 0x03, 0x00, 0x00, 0x00, 0x00, 0x80, 0xC0, 0x40, 0x00, 0xE0, 0xF0, 0xF8,
0xFC, 0x3C, 0x0E, 0x06, 0x06, 0x86, 0xEE, 0xFC, 0xFE, 0xFE, 0x3E, 0x06, 0x08, 0x0C, 0xCC, 0xFC,
0xFC, 0xFE, 0x3E, 0x06, 0x00, 0xC0, 0xF8, 0xFE, 0xFE, 0x7E, 0x06, 0x00, 0x00, 0x0C, 0x8C, 0xFC,
0xFC, 0xFE, 0x3E, 0x0E, 0x04, 0x06, 0x1E, 0x1E, 0x0C, 0x00, 0x03, 0x03, 0x00, 0x00, 0xC0, 0xFC,
0xFF, 0xFF, 0x3F, 0x03, 0x00, 0x00, 0x00, 0x00, 0x03, 0x03, 0xE3, 0xF0, 0xF8, 0xFC, 0xFC, 0xDE,
0xC6, 0xE6, 0x7E, 0x7E, 0x7C, 0x3C, 0x00, 0xE0, 0xF0, 0xF8, 0xFC, 0x3C, 0x0E, 0x06, 0x06, 0x1E,
0x3E, 0x3C, 0x38, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0C, 0x0C, 0x0C, 0x0F,
0x0F, 0x0F, 0x0F, 0x0C, 0x0C, 0x0C, 0x0C, 0x0E, 0x0F, 0x07, 0x01, 0x00, 0x00, 0x03, 0x07, 0x0F,
0x0F, 0x0C, 0x0C, 0x0C, 0x06, 0x07, 0x0F, 0x0F, 0x0F, 0x0D, 0x0C, 0x06, 0x04, 0x02, 0x07, 0x0F,
0x0F, 0x0F, 0x0C, 0x04, 0x03, 0x07, 0x0F, 0x0F, 0x0F, 0x0C, 0x0C, 0x08, 0x00, 0x0C, 0x0F, 0x0F,
0x0F, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0C, 0x0C, 0x0C, 0x0F, 0x0F,
0x0F, 0x0F, 0x0C, 0x0C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x07, 0x07, 0x0F, 0x0E, 0x0C,
0x0C, 0x0C, 0x06, 0x06, 0x02, 0x00, 0x00, 0x03, 0x07, 0x07, 0x0F, 0x0E, 0x0C, 0x0C, 0x0C, 0x06,
0x06, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02,
0xFE, 0x02, 0x02, 0x02, 0x02, 0x84, 0x78, 0x00, 0xE0, 0x50, 0x50, 0x60, 0x00, 0x20, 0x50, 0x50,
0x90, 0x00, 0x10, 0xF4, 0x00, 0x60, 0xD0, 0x50, 0x20, 0x20, 0x00, 0x10, 0xF0, 0x20, 0x10, 0xF0,
0x00, 0x00, 0x00, 0x00, 0x10, 0xF0, 0x20, 0x10, 0xF0, 0x20, 0x10, 0xF0, 0x00, 0x00, 0xA0, 0x50,
0x50, 0xF0, 0x00, 0x00, 0xE0, 0x10, 0x10, 0x12, 0xFE, 0x00, 0x00, 0xE0, 0x50, 0x50, 0x60, 0x00,
0x00, 0x00, 0x00, 0xE0, 0x50, 0x50, 0x60, 0x00, 0xA0, 0x50, 0x50, 0xF0, 0x00, 0x00, 0x20, 0x50,
0x50, 0x90, 0x00, 0x10, 0xF0, 0x00, 0xF0, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01,
0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01,
0x00, 0x00, 0x01, 0x01, 0x01, 0x06, 0x09, 0x09, 0x09, 0x06, 0x00, 0x01, 0x01, 0x01, 0x00, 0x01,
0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x00, 0x01, 0x01, 0x00, 0x01, 0x01,
0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00,
0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01,
0x01, 0x00, 0x00, 0x08, 0x04, 0x03, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x01, 0x00, 0x00,
0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};

#endif // ENABLE_LOGO_1


#endif // GLCD_K0108B_H

