/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
Created on Date : 23/10/2019
Last update     : 23/10/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

These functions allow accessing the HDC2080 sensor

*******************************************************************************/

#ifndef LTLIB_HDC2080_H
#define LTLIB_HDC2080_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif

//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef LTLIB_I2C_MODULE_H
    #error The HDC2080 Library requires the "module_I2C" library to be included.
#endif

#ifndef LTLIB_DELAY_H
    #error The HDC2080 Library requires the "LTlib_delay" library to be included.
#endif

//*****************************************
//   I2C Module Switch
//*****************************************
#if defined (HDC2080_I2C_MODULE_1) || defined (HDC2080_I2C_MODULE_2)

#else
    #define HDC2080_I2C_MODULE_1
#endif 


#ifdef HDC2080_I2C_MODULE_1
    #define HDC2080_I2C_open I2C1_open
    #define HDC2080_I2C_baud_rate I2C1_baud_rate  
    #define HDC2080_I2C_write_byte_to_external_device I2C1_write_byte_to_external_device
    #define HDC2080_I2C_read_byte_from_external_device I2C1_read_byte_from_external_device
#endif

#ifdef HDC2080_I2C_MODULE_2 
    #define HDC2080_I2C_open I2C2_open
    #define HDC2080_I2C_baud_rate I2C2_baud_rate 
    #define HDC2080_I2C_write_byte_to_external_device I2C2_write_byte_to_external_device
    #define HDC2080_I2C_read_byte_from_external_device I2C2_read_byte_from_external_device
#endif

//*****************************************
//   Register mapping
//*****************************************

#define HDC2080_REG_TEMPERATURE_LOW         0x00
#define HDC2080_REG_TEMPERATURE_HIGH        0x01
#define HDC2080_REG_HUMIDITY_LOW            0x02
#define HDC2080_REG_HUMIDITY_HIGH           0x03
#define HDC2080_REG_INTERRUPT_DRDY          0x04
#define HDC2080_REG_TEMPERATURE_MAX         0x05
#define HDC2080_REG_HUMIDITY_MAX            0x06
#define HDC2080_REG_INTERRUPT_ENABLE        0x07
#define HDC2080_REG_TEMP_OFFSET_ADJUST      0x08
#define HDC2080_REG_HUM_OFFSET_ADJUST       0x09
#define HDC2080_REG_TEMP_THR_LOW            0x0A
#define HDC2080_REG_TEMP_THR_HIGH           0x0B
#define HDC2080_REG_RH_THR_LOW              0x0C
#define HDC2080_REG_RH_THR_HIGH             0x0D
#define HDC2080_REG_RESET_DRDY_INT_CONF     0x0E
#define HDC2080_REG_MEASUREMENT_CONF        0x0F
#define HDC2080_REG_MANUFACTURER_ID_LOW     0xFC
#define HDC2080_REG_MANUFACTURER_ID_HIGH    0xFD
#define HDC2080_REG_DEVICE_ID_LOW           0xFE
#define HDC2080_REG_DEVICE_ID_HIGH          0xFF 

//*****************************************
//   Constant Configigurations
//*****************************************
// Measurement Configuration Register constants
#define HDC2080_CONF_MEASUREMENT_TRIGGER_MASK      0xFE
#define HDC2080_CONF_MEASUREMENT_TRIGGER           0x01

//Measurement type constants
#define HDC2080_CONF_MEASUREMENT_MASK              0xF9 
#define HDC2080_CONF_HUMIDITY_TEMPERTAURE          0x00
#define HDC2080_CONF_TEMPERTAURE_ONLY              0x02

//Humidity resolution constants
#define HDC2080_CONF_HUMIDITY_RES_MASK             0xCF
#define HDC2080_CONF_HUMIDITY_RES_14_BITS          0x00
#define HDC2080_CONF_HUMIDITY_RES_11_BITS          0x10
#define HDC2080_CONF_HUMIDITY_RES_09_BITS          0x20

//Temperature resolution constants
#define HDC2080_CONF_TEMPERATURE_RES_MASK          0x3F
#define HDC2080_CONF_TEMPERATURE_RES_14_BITS       0x00
#define HDC2080_CONF_TEMPERATURE_RES_11_BITS       0x40
#define HDC2080_CONF_TEMPERATURE_RES_09_BITS       0x80

//Register Reset, Int, DRDY 

//Interrupt configuration constants
#define HDC2080_INT_DRDY_ENABLE     0x80
#define HDC2080_INT_TH_ENABLE       0x40 
#define HDC2080_INT_TL_ENABLE       0x20 
#define HDC2080_INT_HH_ENABLE       0x10 
#define HDC2080_INT_HL_ENABLE       0x08 

//Temperature offset Register constants
#define HDC2080_CONF_TEMP_ADJ_MINUS_20_62   0x80
#define HDC2080_CONF_TEMP_ADJ_PLUS_10_32    0x40
#define HDC2080_CONF_TEMP_ADJ_PLUS_5_16     0x20
#define HDC2080_CONF_TEMP_ADJ_PLUS_2_58     0x10
#define HDC2080_CONF_TEMP_ADJ_PLUS_1_28     0x08
#define HDC2080_CONF_TEMP_ADJ_PLUS_0_64     0x04
#define HDC2080_CONF_TEMP_ADJ_PLUS_0_32     0x02
#define HDC2080_CONF_TEMP_ADJ_PLUS_0_16     0x01

//Humidity offset Register constants
#define HDC2080_CONF_HUM_ADJ_MINUS_25     0x80
#define HDC2080_CONF_HUM_ADJ_PLUS_12_5    0x40
#define HDC2080_CONF_HUM_ADJ_PLUS_6_3     0x20
#define HDC2080_CONF_HUM_ADJ_PLUS_3_1     0x10
#define HDC2080_CONF_HUM_ADJ_PLUS_1_6     0x08
#define HDC2080_CONF_HUM_ADJ_PLUS_0_8     0x04
#define HDC2080_CONF_HUM_ADJ_PLUS_0_4     0x02
#define HDC2080_CONF_HUM_ADJ_PLUS_0_2     0x01

//*****************************************
//   Constant definitions
//*****************************************
#define HDC2080_DEVICE_ID           0x07D0
#define HDC2080_MANUFACTURER_ID     0x5449

#define HDC2080_HEATER_ON   0x08
#define HDC2080_HEATER_OFF  0x00
#define HDC2080_HEATER_MASK 0xF7

//Defined for the function humidity limit 
#define HDC2080_HUMIDITY_HIGH  HDC2080_REG_RH_THR_HIGH
#define HDC2080_HUMIDITY_LOW   HDC2080_REG_RH_THR_LOW

//Defined for the function temperature limit 
#define HDC2080_TEMPERATURE_HIGH  HDC2080_REG_TEMP_THR_HIGH
#define HDC2080_TEMPERATURE_LOW   HDC2080_REG_TEMP_THR_LOW

//Defined for the function get_interrupt_status (used for bit comparison)
//Interrupt status constants
#define HDC2080_INT_DRDY_ACTIVE     0x80
#define HDC2080_INT_TH_ACTIVE       0x40 
#define HDC2080_INT_TL_ACTIVE       0x20 
#define HDC2080_INT_HH_ACTIVE       0x10 
#define HDC2080_INT_HL_ACTIVE       0x08 


//Defined for the function set_interrupt_modes 
//Interrupt mode constants
#define HDC2080_INT_MODES_MASK          0xF8

#define HDC2080_INT_HIGH_Z              0x00
#define HDC2080_INT_ENABLED             0x04  
#define HDC2080_INT_ACTIVE_LOW          0x00
#define HDC2080_INT_ACTIVE_HIGH         0x02
#define HDC2080_INT_LEVEL_SENSITIVE     0x00
#define HDC2080_INT_COMPARATOR_MODE     0x01    


//Defined for the function set_sampling_modes 
//Sampling mode constants
#define HDC2080_SAMPLING_MODES_MASK     0x8F
#define HDC2080_SAMPLING_ON_TRIGGER     0x00               
#define HDC2080_SAMPLING_EVERY_120s     0x10   
#define HDC2080_SAMPLING_EVERY_60s      0x20
#define HDC2080_SAMPLING_EVERY_10s      0x30
#define HDC2080_SAMPLING_EVERY_5s       0x40
#define HDC2080_SAMPLING_EVERY_1s       0x50
#define HDC2080_SAMPLING_2_PER_s        0x60
#define HDC2080_SAMPLING_5_PER_s        0x70

//******************************************************************************


/**
 * This function initializes the I2C module accordingly to the clock and baud rate.
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param baud_rate_KHz Baud rate expressed in KHz
 *
 * @return 
 *
 * @note  If other devices are connected on the I2C bus the baud rate would be changed by
 * the function. Among several calls from different device initializations, only the last one will be
 * reflected in the I2C module configuration. 
 *
 */
void HDC2080_initialize (uint8_t device_address, uint16_t baud_rate_KHz);

 
/**
 * This function writes into any register of the HDC2080 device.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param register_add Address of the register that must be written. 
 *
 * @param data Data that must be written into the register. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 * @note    For the register address you must use the one specified in the header
 *          file (e.g. HDC2080_REG_TEMPERATURE_LOW). The names reflects the oned 
 *          in the datasheet but they have the HDC2080_REG_ prefix.
 *
 */
int8_t HDC2080_set_register (uint8_t device_address, uint8_t register_add,  uint8_t data );


/**
 * This function reads from any register of the HDC2080 device.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param register_add Address of the register that must be written. 
 * 
 * @param readback_value Pointer to a u_int8 variable that will contain the register content. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 * @note    For the register address you must use the one specified in the header
 *          file (e.g. HDC2080_REG_TEMPERATURE_LOW). The names reflects the oned 
 *          in the datasheet but they have the HDC2080_REG_ prefix.
 *
 */
int8_t HDC2080_get_register (uint8_t device_address, uint8_t register_address, uint8_t * readback_value);


/**
 * This function reads the temperature out of the HDC2080 device.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param temperature Pointer to a float variable that will contain the temperature. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 */
int8_t HDC2080_get_temperature (uint8_t device_address, float * temperature);


/**
 * This function reads the humidity out of the HDC2080 device.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param humidity Pointer to a float variable that will contain the humidity. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t HDC2080_get_humidity (uint8_t device_address, float * humidity);


/**
 * This function reads the Device ID out of the HDC2080 device.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param device_ID Pointer to a uint16_t variable that will contain the Device ID. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t HDC2080_get_device_ID (uint8_t device_address, uint16_t * device_ID);


/**
 * This function reads the Manufacturer ID out of the HDC2080 device.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param manufacturer_ID Pointer to a uint16_t variable that will contain the Manufacturer ID. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t HDC2080_get_manufacturer_ID (uint8_t device_address, uint16_t * manufacturer_ID);


/**
 * This function either enables or disables the integrated heater.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param setting Value to enable or disable the Heater [values: HDC2080_HEATER_ON, HDC2080_HEATER_OFF]. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t HDC2080_heater (uint8_t device_address, uint8_t setting);


/**
 * This function sets the temperature offset. The value must be determined by running 
 * a device calibration at system level, comparing the read out with a re "real value".
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param adjustment Value to adjust the offset [values: Temperature offset Register constants]. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t HDC2080_set_temperature_offset (uint8_t device_address, uint8_t adjustment);


/**
 * This function sets the humidity offset. The value must be determined by running 
 * a device calibration at system level, comparing the read out with a re "real value".
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param adjustment Value to adjust the offset [values: Humidity offset Register constants]. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t HDC2080_set_humidity_offset (uint8_t device_address, uint8_t adjustment);


/**
 * This function sets device temperature resolution.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param resolution Device resolution [values: Temperature resolution constants]. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t HDC2080_set_temperature_resolution (uint8_t device_address, uint8_t resolution);


/**
 * This function sets device humidity resolution.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param resolution Device resolution [values: Humidity resolution constants]. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t HDC2080_set_humidity_resolution (uint8_t device_address, uint8_t resolution);


/**
 * This function sets the measurament that will be performed at each sample phase.
 * Either temperature only or Temperature and Humidity. 
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param measurement Measurement type [values: Measurement type constants]. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t HDC2080_set_measurement (uint8_t device_address, uint8_t measurement);


/**
 * This function uses the heating element to clean the device from eccessive moisture.
 * It could be run upon system power up. Any meadure taken right after the cleaning
 * process will not reflect the environment. The sensor will take several seconds to 
 * settle again to the environment temperature and humidity. 
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param cycles Number of time that the cleaning will be run. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 * 
 */
int8_t HDC2080_cleaning_cycle (uint8_t device_address, uint8_t cycles);


/**
 * This function sets the humidity limits, etiher the low or the high one, depending on 
 * the selection. The value can be written as float, and it will be converted by the
 * function to the right settings. 
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param limit_type Limit that will be set [values: HDC2080_HUMIDITY_HIGH, HDC2080_HUMIDITY_LOW].  
 * 
 * @param humidity Value to be set as limit. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t HDC2080_set_humidity_limit (uint8_t device_address, uint8_t limit_type, float humidity);


/**
 * This function sets the temperature limits, etiher the low or the high one, depending on 
 * the selection. The value can be written as float, and it will be converted by the
 * function to the right settings. 
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param limit_type Limit that will be set [values: HDC2080_TEMPERATURE_HIGH, HDC2080_TEMPERATURE_LOW].  
 * 
 * @param humidity Value to be set as limit. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t HDC2080_set_temperature_limit (uint8_t device_address, uint8_t limit_type, float temperature);
 

/**
 * This function enables the interrupt of the specific event. You can enable either 
 * the humidity or temperature events (High and Low thresholds).
 * constants must be ORed in one single funcion call.
 * Each function call overrites the previos one.  
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param interrupt_enable Enable the interrupt according to the constant [values: Interrupt configuration constants].  
 * 
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t HDC2080_set_interrupt_enable (uint8_t device_address, uint8_t interrupt_enable);


/**
 * This function sets the interrupt mode for the specific enabled interrupt events.
 * In particular it also configures the Interrupt output pin mode.
 * The constants must be ORed in one single funcion call.
 * Each function call overrites the previos one.  
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param interrupt_modes Enable the interrupt according to the constant [values: Interrupt mode constants].  
 * 
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t HDC2080_set_interrupt_modes (uint8_t device_address, uint8_t interrupt_modes);


/**
 * This function reads the status register that contains the interrupt events. 
 * It must be read to clean the interrupt pin upon an interrupt. 
 * To check which event has generated the interrupt you must mask the output 
 * with the "Interrupt status constants".
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param interrupt_status Pointer to the unit8_t variable that will contain the status.  
 * 
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t HDC2080_get_interrupt_status (uint8_t device_address, uint8_t * interrupt_status);


/**
 * This function sets the sampling mode either on Trigger event or auto run mode.
 * In auto run mode you have to define the sampling period by using the  
 * "Sampling mode constants".
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param sampling_modes Configure the sampling mode [values: Sampling mode constants].  
 * 
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t HDC2080_set_sampling_modes (uint8_t device_address, uint8_t sampling_modes);


/**
 * This function starts the humidity and temperature sampling. It must be called 
 * periodically for single shot measure (trigger event mode). 
 * It must be called only once if the device is configured for auto-sampling. 
 * 
 * @param device_address Device address in 7 bits format.
 * 
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t HDC2080_trigger_new_measure (uint8_t device_address);


/**
 * This function resets the device via software command.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t HDC2080_software_reset (uint8_t device_address);


#endif
