/*******************************************************************************

Author : Mauro Laurenti
Version : 1.1
Created on Date : 21/10/2017
Last update     : 27/07/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This library supports the HMC5883L magnetic sensor from Honeywell

*******************************************************************************/

#ifndef LTLIB_HMC5883_H
#define LTLIB_HMC5883_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif

//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef LTLIB_I2C_MODULE_H
    #error The HMC5883 Library requires the "module_I2C" library to be included.
#endif

#ifndef LTLIB_DELAY_H
    #error The HMC5883 Library requires the "delay" library to be included.
#endif

//*****************************************
//   I2C Module Switch
//*****************************************
#if defined (HMC5883_I2C_MODULE_1) || defined (HMC5883_I2C_MODULE_2)

#else
    #define HMC5883_I2C_MODULE_1
#endif 

#ifdef HMC5883_I2C_MODULE_1
    #define HMC5883_I2C_open I2C1_open
    #define HMC5883_I2C_bit_rate I2C1_baud_rate  
    #define HMC5883_I2C_write_byte_to_external_device I2C1_write_byte_to_external_device
    #define HMC5883_I2C_read_byte_from_external_device I2C1_read_byte_from_external_device
#endif

#ifdef HMC5883_I2C_MODULE_2 
    #define HMC5883_I2C_open I2C2_open
    #define HMC5883_I2C_bit_rate I2C2_baud_rate 
    #define HMC5883_I2C_write_byte_to_external_device I2C2_write_byte_to_external_device
    #define HMC5883_I2C_read_byte_from_external_device I2C2_read_byte_from_external_device
#endif

//*****************************************
//   Constants
//*****************************************

#define HMC5883_ADDRESS 0x1E << 1

#define HMC5883_100KHz 100
#define HMC5883_400KHz 400

#define HMC5883_MODE_SINGLE     0x01 
#define HMC5883_MODE_CONTINUOUS 0x00
#define HMC5883_MODE_IDLE       0x11

#define HMC5883_AVERAGE_1   0b00000000
#define HMC5883_AVERAGE_2   0b00100000
#define HMC5883_AVERAGE_4   0b01000000
#define HMC5883_AVERAGE_8   0b01100000
#define HMC5883_AVERAGE_AND_MASK 0b10011111 

//the _ represents a comma
#define HMC5883_SAMPLING_0_75   0b00000000
#define HMC5883_SAMPLING_1_5    0b00000100
#define HMC5883_SAMPLING_3      0b00001000
#define HMC5883_SAMPLING_7_5    0b00001100
#define HMC5883_SAMPLING_15     0b00010000
#define HMC5883_SAMPLING_30     0b00010100   
#define HMC5883_SAMPLING_75     0b00011000
#define HMC5883_SAMPLING_AND_MASK 0b11100011 

#define HMC5883_NOT_CONNECTED   0x00
#define HMC5883_CONNECTED       0x01

#define HMC5883_DATA_NOT_READY      0x00
#define HMC5883_DATA_READY          0x01
#define HMC5883_DATA_READY_BIT_MASK 0x01

#define ID_REG_A_VALUE 0x48
#define ID_REG_B_VALUE 0x34
#define ID_REG_C_VALUE 0x33

#define HMC5883_FIELD_X 0x00
#define HMC5883_FIELD_Y 0x01
#define HMC5883_FIELD_Z 0x02

//*****************************************
//   Gain Tabble
//*****************************************

const float HMC5883_resolution_table[] = {0.73, 0.92, 1.22, 1.52, 2.27, 2.56, 3.03, 4.35};

#define HMC5883_GAIN_1370   0x00 << 1
#define HMC5883_GAIN_1090   0x10 << 1    
#define HMC5883_GAIN_820    0x20 << 1
#define HMC5883_GAIN_660    0x30 << 1
#define HMC5883_GAIN_440    0x40 << 1
#define HMC5883_GAIN_390    0x50 << 1
#define HMC5883_GAIN_330    0x60 << 1
#define HMC5883_GAIN_230    0x70 << 1

uint8_t HMC5883_gain_value = 0;

//*****************************************
//   Register mapping
//*****************************************

#define HMC5883_CONFIGURATION_REG_A 0x00
#define HMC5883_CONFIGURATION_REG_B 0x01
#define HMC5883_MODE_REGISTER		0x02
#define HMC5883_DATA_OUT_X_MSB		0x03
#define HMC5883_DATA_OUT_X_LSB		0x04
#define HMC5883_DATA_OUT_Z_MSB		0x05
#define HMC5883_DATA_OUT_Z_LSB		0x06
#define HMC5883_DATA_OUT_Y_MSB		0x07
#define HMC5883_DATA_OUT_Y_LSB		0x08
#define HMC5883_STATUS_REGISTER		0x09
#define HMC5883_ID_REGISTER_A		0x0A
#define HMC5883_ID_REGISTER_B		0x0B
#define HMC5883_ID_REGISTER_C		0x0C


/**
 * This function initialize the I2C bus and HMC5883L sensor.
 * Some default values are also set:
 * 
 * HMC5883_AVERAGE_1, HMC5883_MODE_CONTINUOUS, HMC5883_SAMPLING_15, HMC5883_GAIN_1090
 *
 * @param bit_rate_KHZ Bit rate for the I2C bus used for the HMC5883. 
 *                     The value is the frequency in KHz [range: 100, 400]
 *
 * @return none
 *
 */
void HMC5883_initialize (uint8_t bit_rate_KHZ);


/**
 * This function read the magneti field. The axis is specified as parameter. 
 *
 * @param field_axis Axis to be read out of the sensor [value:HMC5883_FIELD_X, HMC5883_FIELD_Y, HMC5883_FIELD_Z] 
 *
 * @return vale   signed int value out of the sensor [range: -2048, 2047]
 *
 * @warning An overflow is represented by the value -4096.
 */
int16_t HMC5883_read_magnetic_field (uint8_t field_axis);


/**
 * This function reads a register within the HMC5883. Random access is allowed,
 * bypassing the internal auto-increment function. 
 *
 * @param register_address Register value that you want to access. [range: 0x00...0x0B].
 *
 * @return value   Register value.
 *
 * @info The user should normally not use this function, unless you want to access a specific register.
 * @info Use the register name mapping defined in the library, for a clear use of the function.
 */
uint8_t HMC5883_read_register (uint8_t register_address);


/**
 * This function writes a new value within a specifc internal register.
 *
 * @param register_address Register value that you want to access. [range: 0x00...0x0B].
 * @param register_value Value you want to write within the register [range: 0x00-0xFF].
 *
 * @return status   0: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @info The user should normally not use this function, unless you want to access a specific register.
 * @info Use the register name mapping defined in the library, for a clear use of the function.
 */
int8_t HMC5883_write_register (uint8_t register_address, uint8_t register_value);


/**
 * This function checks the status register and monitor if the data is ready to be read out.
 *
 * @param void.
 *
 * @return value   Data ready status:
 * 
 *                 HMC5883_DATA_NOT_READY : The data is not ready
 *                 HMC5883_DATA_READY: The data is read to be read out.
 *
 * @info Use the defined constants to properly monitor the status.
 */
uint8_t HMC5883_is_data_ready (void);


/**
 * This function checks if the device is properly connected. 
 * The check is not performed on the error during the I2C communication but 
 * monitoring the internal ID register. 
 *
 * @param void.
 *
 * @return value   Device connected status:
 * 
 *                 HMC5883_NOT_CONNECTED: device is not connected.
 *                 HMC5883_CONNECTED: Device is connected.
 *
 * @info Use the defined constants to properly monitor the status.
 */
uint8_t HMC5883_is_device_connected (void);


/**
 * This function sets the sampling rate used for monitoring the sample. 
 * The default value set during the initialization is 15S/s.
 *
 * @param sampling_rate Sampling frequency. [value: use the defined constants]
 *
 * @return void
 *
 * @warning The defined constants uses the _ after the digit, as comma. 
 *          0_75 represents 0.75Hz. 
 * 
 * @warning Use the defined constants: HMC5883_SAMPLING_0_75, HMC5883_SAMPLING_1_5,
 * HMC5883_SAMPLING_3, HMC5883_SAMPLING_7_5, HMC5883_SAMPLING_15, HMC5883_SAMPLING_30,
 * HMC5883_SAMPLING_75
 */
void HMC5883_set_sampling_rate (uint8_t sampling_rate);


/**
 * This function sets the internal gain used for the amplifier. 
 * Depending on the field strenght, different values can be set. The default value
 * is suitable for earth field.
 *
 * @param amplifier_gain Gain to set for the amplifier [value: use defined constants].
 *
 * @return void
 *
 * @warning The new gain setting take place starting from the second measurement.
 * 
 * @warning Use the defined constants: HMC5883_GAIN_1370, HMC5883_GAIN_1090,
 * HMC5883_GAIN_820, HMC5883_GAIN_660, HMC5883_GAIN_440, HMC5883_GAIN_390,
 * HMC5883_GAIN_330, HMC5883_GAIN_230.   
 */
void HMC5883_set_gain (uint8_t amplifier_gain);


/**
 * This function writes the seconds in the RTC.
 *
 * @param mode Set the new device mode. [value: use defined values].
 *
 * @return void.
 *
* @warning Use the defined constants: HMC5883_MODE_SINGLE, HMC5883_MODE_CONTINUOUS, HMC5883_MODE_IDLE
 */
void HMC5883_set_measurement_mode (uint8_t mode);


/**
 * This function sets the avaraging used before each measure can be read out.
 * The default value is HMC5883_AVERAGE_1. 
 *
 * @param averaging_samples Value for the avaraging [value: used the defined constnats].
 *
 * @return void.
 *
 * @warning Use the defined constants: HMC5883_AVERAGE_1, HMC5883_AVERAGE_2, 
 * HMC5883_AVERAGE_4, HMC5883_AVERAGE_8.
 */
void HMC5883_set_average (uint8_t averaging_samples);

#endif

