/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
 
Created on Date : 01/12/2019
Last update     : 01/12/2019
 
CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

These functions allow the user to read the data out of the sensor L3GD20H.

*******************************************************************************/
#ifndef LTLIB_L3GD20H_H
#define LTLIB_L3GD20H_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif


//*****************************************
//   Library Test
//*****************************************

#include "LTlib.h"

#ifndef LTLIB_I2C_MODULE_H
    #error The L3GD20H Library requires the "module_I2C" library to be included.
#endif

#ifndef LTLIB_DELAY_H
    #error The L3GD20H Library requires the "LTlib_delay" library to be included.
#endif


//*****************************************
//   I2C Module Switch
//*****************************************
#if defined (L3GD20H_I2C_MODULE_1) || defined (L3GD20H_I2C_MODULE_2)

#else
    #define L3GD20H_I2C_MODULE_1
#endif 

#ifdef L3GD20H_I2C_MODULE_1
    #define L3GD20H_I2C_open I2C1_open
    #define L3GD20H_I2C_baud_rate I2C1_baud_rate  
    #define L3GD20H_I2C_write_byte_to_external_device I2C1_write_byte_to_external_device
    #define L3GD20H_I2C_read_byte_from_external_device I2C1_read_byte_from_external_device
#endif

#ifdef L3GD20H_I2C_MODULE_2 
    #define L3GD20H_I2C_open I2C2_open
    #define L3GD20H_I2C_baud_rate I2C2_baud_rate 
    #define L3GD20H_I2C_write_byte_to_external_device I2C2_write_byte_to_external_device
    #define L3GD20H_I2C_read_byte_from_external_device I2C2_read_byte_from_external_device
#endif

//*****************************************
//   Device Address options
//*****************************************
#define L3GD20H_DEVICE_ADDRESS_0 0x6A
#define L3GD20H_DEVICE_ADDRESS_1 0x6B

//*****************************************
//   Device registers
//*****************************************
#define L3GD20H_REG_WHO_AM_I    0x0F 
#define L3GD20H_REG_CTRL1       0x20 
#define L3GD20H_REG_CTRL2       0x21 
#define L3GD20H_REG_CTRL3       0x22 
#define L3GD20H_REG_CTRL4       0x23 
#define L3GD20H_REG_CTRL5       0x24 
#define L3GD20H_REG_REFERENCE   0x25
#define L3GD20H_REG_OUT_TEMP    0x26
#define L3GD20H_REG_STATUS      0x27 
#define L3GD20H_REG_OUT_X_L     0x28 
#define L3GD20H_REG_OUT_X_H     0x29 
#define L3GD20H_REG_OUT_Y_L     0x2A 
#define L3GD20H_REG_OUT_Y_H     0x2B 
#define L3GD20H_REG_OUT_Z_L     0x2C 
#define L3GD20H_REG_OUT_Z_H     0x2D 
#define L3GD20H_REG_FIFO_CTRL   0x2E 
#define L3GD20H_REG_FIFO_SRC    0x2F 
#define L3GD20H_REG_IG_CFG      0x30
#define L3GD20H_REG_IG_SRC      0x31
#define L3GD20H_REG_IG_THS_XH   0x32
#define L3GD20H_REG_IG_THS_XL   0x33
#define L3GD20H_REG_IG_THS_YH   0x34
#define L3GD20H_REG_IG_THS_YL   0x35
#define L3GD20H_REG_IG_THS_ZH   0x36
#define L3GD20H_REG_IG_THS_ZL   0x37
#define L3GD20H_REG_IG_DURATION 0x38
#define L3GD20H_REG_LOW_ODR     0x39


//*****************************************
//   Constant Configigurations
//*****************************************
#define L3GD20H_DEVICE_ID 0xD7

#define L3GD20H_MODE_MASK       0xF7
#define L3GD20H_MODE_POWER_DOWN 0x00
#define L3GD20H_MODE_NORMAL     0x0F

#define L3GD20H_RESET_MASK      0xFB
#define L3GD20H_SOFTWARE_RESET  0x04

#define L3GD20H_FULL_SCALE_MASK 0xCF
#define L3GD20H_FULL_SCALE_245  0x00
#define L3GD20H_FULL_SCALE_500  0x10
#define L3GD20H_FULL_SCALE_2000 0x20

#define L3GD20H_WAIT_ON		0x80
#define L3GD20H_WAIT_OFF	0x00

#define L3GD20H_THRESHOLD_X	0x01
#define L3GD20H_THRESHOLD_Y	0x02
#define L3GD20H_THRESHOLD_Z	0x03

#define L3GD20H_INT_ENABLE_MASK             0xC0
#define L3GD20H_INT_ENABLE_Z_HIGH_EVENT_ON  0x20
#define L3GD20H_INT_ENABLE_Z_LOW_EVENT_ON   0x10
#define L3GD20H_INT_ENABLE_Y_HIGH_EVENT_ON  0x08
#define L3GD20H_INT_ENABLE_Y_LOW_EVENT_ON   0x04
#define L3GD20H_INT_ENABLE_X_HIGH_EVENT_ON  0x02
#define L3GD20H_INT_ENABLE_X_LOW_EVENT_ON   0x01

#define L3GD20H_INT_MODE_MASK         0x3F
#define L3GD20H_INT_AND_ON            0x80
#define L3GD20H_INT_OR_ON             0x00
#define L3GD20H_INT_LATCH_ON          0x40
#define L3GD20H_INT_LATCH_OFF         0x00

#define L3GD20H_INT_PIN_MODE_MASK       0xCF
#define L3GD20H_INT_PIN_ACTIVE_HIGH     0x00
#define L3GD20H_INT_PIN_ACTIVE_LOW      0x20
#define L3GD20H_INT_PIN_OPEN_DRAIN      0x10
#define L3GD20H_INT_PIN_PUSH_PULL       0x00

#define L3GD20H_INT_PIN_MASK            0x30
#define L3GD20H_INT1_PIN_DISABLED       0x00
#define L3GD20H_INT1_PIN_IG_ENABLED     0x80
#define L3GD20H_INT1_PIN_BOOT_STATUS    0x40
#define L3GD20H_INT2_PIN_DISABLED       0x00
#define L3GD20H_INT2_PIN_DRDY           0x08
#define L3GD20H_INT2_PIN_FIFO_TH        0x04
#define L3GD20H_INT2_PIN_FIFO_OVERRUN   0x02
#define L3GD20H_INT2_PIN_FIFO_EMPTY     0x01

//*****************************************
//         Global const/variables
//*****************************************
//The constants are used to properly set the full scale factors and determine 
//the proper register values.
const float L3GD20H_sensitivity_245 = 8.75;
const float L3GD20H_sensitivity_500 = 17.50;
const float L3GD20H_sensitivity_2000 = 70.0;
float L3GD20H_active_full_scale = 0;

//The variables related to the thresolds, help to keep track of the values and 
//automatically update the internal registers upon changing the ADC full scale
float L3GD20H_active_threshold_X = 0;
float L3GD20H_active_threshold_Y = 0;
float L3GD20H_active_threshold_Z = 0;

//******************************************************************************


/**
 * This function initializes the I2C module accordingly to the clock and baud rate.
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param baud_rate_KHz Baud rate expressed in KHz
 *
 * @return 
 *
 * @note  If other devices are connected on the I2C bus the baud rate would be changed by
 * the function. Among several calls from different device initializations, only the last one will be
 * reflected in the I2C module configuration. 
 *
 */
void L3GD20H_initialize (uint8_t device_address, uint16_t baud_rate_KHz);

 
/**
 * This function writes into any register of the L3GD20H device.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param register_add Address of the register that must be written. 
 *
 * @param data Data that must be written into the register. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 * @note    For the register address you must use the ones specified in the header
 *          file. The names reflect the ones in the datasheet but 
 *          they have the L3GD20H_REG_ prefix.
 *
 */
int8_t L3GD20H_set_register (uint8_t device_address, uint8_t register_add,  uint8_t data );


/**
 * This function reads from any register of the L3GD20H device.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param register_add Address of the register that must be written. 
 * 
 * @param readback_value Pointer to a u_int8 variable that will contain the register content. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 * @note    For the register address you must use the ones specified in the header
 *          file. The names reflect the ones in the datasheet but 
 *          they have the L3GD20H_REG_ prefix.
 *
 */
int8_t L3GD20H_get_register (uint8_t device_address, uint8_t register_address, uint8_t * readback_value);


/**
 * This function update a register value. 
 * It should be considered a private function that the user should not use. 
 * The bits that will be updated will depend on the mask that will be provided.
 * The bits that must remain unchanged must be set to 1 into the mask. 
 * The new_value must have a valid value out of the defined constansts 
 * to avoid unwanted behaviours. 
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param register_address Address to change.
 * 
 * @param register_mask Register mask. Bit set to 1 will remain unchanged.
 *                                          
 * 
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t L3GD20H_update_register (uint8_t device_address, uint8_t register_address, uint8_t register_mask, uint8_t new_value);


/**
 * This function reads the temperature out of the L3GD20H device.
 * The returned value is 2 complement 8 bits format. This value is close too be useless
 * since it is not properly documented in the datasheet and it is not the environment
 * temerparature. 
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param temperature Pointer to a int8_t variable that will contain the temperature. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 */
int8_t L3GD20H_get_temperature (uint8_t device_address, int8_t * temperature);


/**
 * This function reads the angular speed from the L3GD20H device.
 * The value are given back as dps (degree per second) as integer format. 
 * The values are computed against the sensitivity settings (gain and full scale). 
 * All 3 axes are read together.  
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param dps_X Pointer to a int16_t variable that will contain the X dps (degree per second). 
 * 
 * @param dps_Y Pointer to a int16_t variable that will contain the Y dps (degree per second). 
 * 
 * @param dps_Z Pointer to a int16_t variable that will contain the Z dps (degree per second). 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t L3GD20H_get_angular_speed (uint8_t device_address, int16_t *dps_X, int16_t *dps_Y, int16_t *dps_Z);


/**
 * This function reads the angular speed from the L3GD20H device.
 * The value are given back as dps (degree per second) as float format. 
 * The values are computed against the sensitivity settings (gain and full scale). 
 * All 3 axes are read together.  
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param dps_X Pointer to a float variable that will contain the X dps (degree per second). 
 * 
 * @param dps_Y Pointer to a float variable that will contain the Y dps (degree per second). 
 * 
 * @param dps_Z Pointer to a float variable that will contain the Z dps (degree per second). 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t L3GD20H_get_angular_speed_float (uint8_t device_address, float *dps_X, float *dps_Y, float *dps_Z);


/**
 * This function reads the Device ID out of the L3GD20H device.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param device_ID Pointer to a uint8_t variable that will contain the Device ID. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t L3GD20H_get_device_ID (uint8_t device_address, uint8_t * device_ID);


/**
 * This function resets the device via a software command.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t L3GD20H_software_reset (uint8_t device_address);


/**
 * This function sets the L3GD20H, etiher in Power Down or Normal mode.
 * By power on the device goes is poewr down mode. 
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param mode New operating mode [L3GD20H_MODE_POWER_DOWN, L3GD20H_MODE_NORMAL]
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t L3GD20H_set_mode (uint8_t device_address, uint8_t mode);


/**
 * This function sets the L3GD20H full scale. As per datasheet the following values
 * are supported 245, 500 and 2000 dps. Proper constants must be used. 
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param full_scale New full scale value [L3GD20H_FULL_SCALE_245, L3GD20H_FULL_SCALE_500, L3GD20H_FULL_SCALE_2000]
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t L3GD20H_set_full_scale (uint8_t device_address, uint8_t full_scale);


/**
 * This function enables the wait states for the interrupt events. 
 * If enabled, an interrupt will occur only after the event count.
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param wait_enable Turn on or off the wait state [L3GD20H_WAIT_ON, L3GD20H_WAIT_OFF]
 * 
 * @param duration Number of events beyond the threshold before an interrupt occurs [0...127]
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t L3GD20H_set_event_duration (uint8_t device_address, uint8_t wait_enable, uint8_t duration);


/**
 * This function sets the threshold for each component. You have to select the 
 * component for which it is required to set a new threshold. The threshold accuracy is
 * till the decimal point. The value is written in degrees per second such as 34.7.
 * The threshold register values are automatically set considering the active Full scale setting. 
 * If the full scale setting is changed the thresold registers are automatically updated (scaled) 
 * to the new full scale setting. 
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param component Component for which the value will be updated. [L3GD20H_THRESHOLD_X, L3GD20H_THRESHOLD_Y, L3GD20H_THRESHOLD_Z]
 * 
 * @param threshold_value as dps as float format (decimal point is accepted). 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t L3GD20H_set_interrupt_threshold (uint8_t device_address, uint8_t component, float threshold_value);


/**
 * This function sets interrupt enables for each component. The enables must be set
 * all together. Each time the funtions is called it cleans the registers and sets
 * the new interrupts enable bits. Each component may either have an high or low 
 * envent. The event is the relation between the current value and the threshold value.
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param interrupt_enable Interrupt enable [L3GD20H_INT_ENABLE_Z_HIGH_EVENT_ON, L3GD20H_INT_ENABLE_Z_LOW_EVENT_ON, 
 *                                           L3GD20H_INT_ENABLE_Y_HIGH_EVENT_ON, L3GD20H_INT_ENABLE_Y_LOW_EVENT_ON
 *                                           L3GD20H_INT_ENABLE_X_HIGH_EVENT_ON, L3GD20H_INT_ENABLE_X_LOW_EVENT_ON]
 * 
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t L3GD20H_set_interrupt_enabled (uint8_t device_address, uint8_t interrupt_enable);


/**
 * This function sets the interrupt mode as either OR or AND among the enabled interrupts.
 * It also sets the latch option for the interrupt. If the latch option is enabled
 * the interrupt remains activ until the interrupt status register is read out.
 * The constants must be ORed together. 
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param mode Interrupt mode [L3GD20H_INT_AND_ON, L3GD20H_INT_OR_ON, L3GD20H_INT_LATCH_ON, L3GD20H_INT_LATCH_OFF]
 *                                          
 * 
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t L3GD20H_set_interrupt_mode (uint8_t device_address, uint8_t mode);


/**
 * This function defines how the interrupt pin will operate (INT 1 and INT 2).
 * In particular it will define which event will drive the pin. 
 * The constants must be ORed. Each INT pin can support one function. 
 * If the pin is not used you have to used the DISABLED option. 
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param pin_function Select the function for each INT pin [L3GD20H_INT1_PIN_DISABLED,L3GD20H_INT1_PIN_IG_ENABLED
 *                                                           L3GD20H_INT1_PIN_BOOT_STATUS, L3GD20H_INT2_PIN_DISABLED
 *                                                           L3GD20H_INT2_PIN_DRDY, L3GD20H_INT2_PIN_FIFO_TH
 *                                                           L3GD20H_INT2_PIN_FIFO_OVERRUN, L3GD20H_INT2_PIN_FIFO_EMPTY ]
 * 
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t L3GD20H_set_interrupt_pin (uint8_t device_address, uint8_t pin_function);


/**
 * This function sets the characteristic of both interrupt pins (INT 1 and INT 2).
 * In particular it is possible to set those pin as open drain or push-pull.
 * Also the active logic level can be set.  
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param pin_mode Select the function for each INT pin [L3GD20H_INT_PIN_ACTIVE_HIGH, L3GD20H_INT_PIN_ACTIVE_LOW, 
 *                                                       L3GD20H_INT_PIN_OPEN_DRAIN, L3GD20H_INT_PIN_PUSH_PULL]
 *                                          
 * 
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t L3GD20H_set_interrupt_pin_mode (uint8_t device_address, uint8_t pin_mode);


/**
 * This function gets the interrupt status register value. 
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param interrupt_status Value of the interrupt status.
 *                                          
 * 
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t L3GD20H_get_interrupt_status (uint8_t device_address, uint8_t * interrupt_status);

#endif

