/*******************************************************************************

Author : Mauro Laurenti
Version : 2.0

Created on Date : 04/09/2006
Last update     : 28/03/2023

CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.


********************************************************
PURPOSES
********************************************************


 This library contains all the functions that are handy for controlling 			
 an LCD with a 44780 Hitachi controller.							
 	
 To use these functions for your purpose you must set up properly 
 the LCD constants and the pins of the PORT/s that you will use.
 You can use any pin you want!

*******************************************************************************/

#ifndef LTLIB_LCD_44780_H
#define LTLIB_LCD_44780_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif

//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef LTLIB_DELAY_H
    #error The LCD_44780 Library requires the "LTlib_delay" library to be included.
#endif

#ifndef LTLIB_UTILITIES_H
    #error The LCD_44780 Library requires the "LTlib_utilities" library to be included.
#endif


//**************************************************
// HW Initialization
// This section must be properly initialized according
// to the LCD connections
//**************************************************

#ifdef COMPILER_XC8 
    #ifdef _PIC18

        // LAT Register settings
        #define LCD_D0 LATDbits.LATD4   
        #define LCD_D1 LATDbits.LATD5
        #define LCD_D2 LATDbits.LATD6
        #define LCD_D3 LATDbits.LATD7
        #define LCD_RS LATDbits.LATD2
        #define LCD_E  LATDbits.LATD3
        #define LCD_RW LATDbits.LATD1
        #define LCD_LED LATCbits.LATC1

        // TRIS Register settings
        #define LCD_D0_TRIS TRISDbits.RD4   
        #define LCD_D1_TRIS TRISDbits.RD5
        #define LCD_D2_TRIS TRISDbits.RD6
        #define LCD_D3_TRIS TRISDbits.RD7
        #define LCD_RS_TRIS TRISDbits.RD2
        #define LCD_E_TRIS TRISDbits.RD3
        #define LCD_RW_TRIS TRISDbits.RD1
        #define LCD_LED_TRIS TRISCbits.RC1

    #endif

    #ifndef _PIC18
        // PORT Register settings
        #define LCD_D0 PORTDbits.RD4
        #define LCD_D1 PORTDbits.RD5
        #define LCD_D2 PORTDbits.RD6
        #define LCD_D3 PORTDbits.RD7
        #define LCD_RS PORTDbits.RD2
        #define LCD_E  PORTDbits.RD3
        #define LCD_RW PORTDbits.RD1
        #define LCD_LED PORTCbits.RC1

    //  TRIS Register settings
        #define LCD_D0_TRIS TRISDbits.TRISD4
        #define LCD_D1_TRIS TRISDbits.TRISD5
        #define LCD_D2_TRIS TRISDbits.TRISD6
        #define LCD_D3_TRIS TRISDbits.TRISD7
        #define LCD_RS_TRIS TRISDbits.TRISD2
        #define LCD_E_TRIS TRISDbits.TRISD3
        #define LCD_RW_TRIS TRISDbits.TRISD1
        #define LCD_LED_TRIS TRISCbits.TRISC1
    #endif
#endif

#ifdef COMPILER_XC16
    
    // LAT Register settings
    #define LCD_D0 LATBbits.LATB11  
    #define LCD_D1 LATBbits.LATB12 
    #define LCD_D2 LATBbits.LATB13 
    #define LCD_D3 LATBbits.LATB14 
    #define LCD_RS LATBbits.LATB15 
    #define LCD_E  LATAbits.LATA7 
    #define LCD_RW LATAbits.LATA8   
    #define LCD_LED LATBbits.LATB10

    // TRIS Register settings
    #define LCD_D0_TRIS TRISBbits.TRISB11  
    #define LCD_D1_TRIS TRISBbits.TRISB12 
    #define LCD_D2_TRIS TRISBbits.TRISB13 
    #define LCD_D3_TRIS TRISBbits.TRISB14 
    #define LCD_RS_TRIS TRISBbits.TRISB15 
    #define LCD_E_TRIS TRISAbits.TRISA7 
    #define LCD_RW_TRIS TRISAbits.TRISA8 
    #define LCD_LED_TRIS TRISBbits.TRISB10 

#endif


#ifdef COMPILER_XC32
    
    // LAT Register settings
    #define LCD_D0 LATBbits.LATB10  
    #define LCD_D1 LATBbits.LATB11 
    #define LCD_D2 LATBbits.LATB13 
    #define LCD_D3 LATBbits.LATB15 
    #define LCD_RS LATBbits.LATB9 
    #define LCD_E  LATBbits.LATB7 
    #define LCD_RW LATBbits.LATB8  
    #define LCD_LED LATAbits.LATA0

    // TRIS Register settings
    #define LCD_D0_TRIS TRISBbits.TRISB10  
    #define LCD_D1_TRIS TRISBbits.TRISB11 
    #define LCD_D2_TRIS TRISBbits.TRISB13 
    #define LCD_D3_TRIS TRISBbits.TRISB15 
    #define LCD_RS_TRIS TRISBbits.TRISB9 
    #define LCD_E_TRIS TRISBbits.TRISB7 
    #define LCD_RW_TRIS TRISBbits.TRISB8 
    #define LCD_LED_TRIS TRISAbits.TRISA0

#endif


//**************************************************
//               Constant Definitions
//**************************************************
#define LCD_LEFT 0
#define LCD_RIGHT 1

#define LCD_TURN_ON_LED 1
#define LCD_TURN_OFF_LED 0

#define LCD_TURN_ON_LED 1
#define LCD_TURN_OFF_LED 0

#define LCD_TURN_ON_CURSOR 1
#define LCD_TURN_OFF_CURSOR 0

#define LCD_BLINKING_ON 1
#define LCD_BLINKING_OFF 0

#define LCD_ZERO_CLEANING_ON 0x01
#define LCD_ZERO_CLEANING_OFF 0x00

//**************************************************
//               SHIFT Definitions
//**************************************************
//This number of shift definition, is just to avoid magic
//numbers in the code and avoid warnings 

#define LCD_SHIFT_01 1
#define LCD_SHIFT_02 2
#define LCD_SHIFT_03 3
#define LCD_SHIFT_04 4
#define LCD_SHIFT_05 5
#define LCD_SHIFT_06 6
#define LCD_SHIFT_07 7

#define LCD_BITMASK_BIT_0 1
#define LCD_BITMASK_BIT_1 2
#define LCD_BITMASK_BIT_2 4
#define LCD_BITMASK_BIT_3 8
#define LCD_BITMASK_BIT_4 16
#define LCD_BITMASK_BIT_5 32
#define LCD_BITMASK_BIT_6 64
#define LCD_BITMASK_BIT_7 128

//**************************************************


/**
 * This function generates the Enable pulse.
 *
 *
 * @warning This function is a Private one. It should not be used by the user.
 */
void LCD_enable_pulse (void);


/**
 * This function generates the Enable pulse.
 *
 *
 * @param bit_0 bit 0 of the data bus (4 bit modality)
 *
 * @param bit_1 bit 1 of the data bus (4 bit modality)
 *
 * @param bit_2 bit 2 of the data bus (4 bit modality)
 *
 * @param bit_3 bit 3 of the data bus (4 bit modality)
 *
 * @warning This function is a Private one. It should not be used by the user.
 */
void LCD_send_command (uint8_t bit_3, uint8_t bit_2, uint8_t bit_1, uint8_t bit_0);


/**
 * This function locates the cursor at home location. First line first character.
 *
 */
void LCD_home(void);


/**
 * This function shifts the LCD screen on the left or right.
 *
 * @param shift Specify where the shift should be [LCD_LEFT, LCD_RIGHT].
 *
 * @param number_of_shift Specify the number of times the shift is executed.
 *
 * @note Use the constants LEFT, RIGHT to specify the direction
 *       to ensure the compatibility with LCD_44780_I2C library.
 */
void LCD_shift(uint8_t shift, uint8_t number_of_shift);


/**
 * This function shifts the LCD cursor on the left or right. The position of the cursor is where the
 * next writing will be performed.
 *
 * @param shift Specify where the shift should be [LCD_LEFT, LCD_RIGHT].
 *
 * @param number_of_shift Specify the number of times the shift is executed.
 *
 * @note Use the constants LEFT, RIGHT to specify the direction
 *       to ensure the compatibility with LCD_44780_I2C library.
 */
void LCD_shift_cursor(uint8_t shift,uint8_t number_of_shift);


/**
 * This function locates the LCD cursor on the selected line. 
 *
 * @param line Specify the number of the line [1,2,3,4]
 *
 * @warning It might not work with all the LCD Displays. Tested on 20x4 16x2 LCD displays.
 */
void LCD_goto_line (uint8_t line);


/**
 * This function locates the LCD cursor to an arbitrary X Y location.
 * Y represents the line number from top.
 *
 * @param x It Specifies horizontal position
 *
 * @param y It Specifies vertical position (line number)
 *
 * @warning It might not work with all the LCD Displays. Tested on 20x4 16x2 LCD displays.
 */
void LCD_goto_xy (uint8_t x, uint8_t y);


/**
 * This function writes a char to the LCD display. (e.g LCD_write_char ('a'); )
 *
 * @param value Specify the character to be sent [0-128 ASCII code].
 *
 * @note The function accept ASCII integer or constants 'a'..'z'.
 */
void LCD_write_char (uint8_t value);


/**
 * This function writes a const string to the LCD display, e.g LCD_write_message("Hello"); .
 *
 * @param buffer Is a const string written in rom (is not an array).
 *
 * @note The function accepts strings written in rom (constant), it does not accept array.
 *       To write arrays refers the LCD_write_string () function.
 *
 */
void LCD_write_message(const char *buffer);


/**
 * This function writes an array of char to the LCD display, terminated with /0 .
 *
 * @param buffer It is an array of char terminated with /0 .
 *
 * @note The function accepts strings  within an array. To use const array within flash use LCD_write_message ().
 *
 */
void LCD_write_string(uint8_t *buffer);


/**
 * This function writes a byte in the CGRAM. This allows setting up a custom font. 
 *
 * @param address CGRAM address to write.
 * 
 * @param data Data byte to write
 *
 * @note The font that is supported in 2x Lines is 5x8 only.
 * @note The function returns home (LCD_home) at the end. This resets the Address 
 *       counter and enables the new fonts to be requested from the DDRAM.
 * 
 */
void LCD_write_data_CGRAM(uint8_t address, uint8_t data);


/**
 * This function writes an integer to the LCD display. The integer is converted to string.
 *
 * @param value It is integer that must be written to the LCD display.
 *
 * @param numeber_of_digits It specifies the number of shown digit [0-6].
 *                    0: Left Justified
 *                    1-6: n digits
 *
 *  @param zero_cleaning It specifies whether the left zero must be removed [LCD_ZERO_CLEANING_ON, LCD_ZERO_CLEANING_OFF].
 *
 * @note If you set a number of digits less than the required the digits will be lost starting from the most
 *       significant digit. Minus is like a digit and it is always kept. So if you need 3 digits for a negative number
 *       you have to set 4 digits.
 *
 */
void LCD_write_integer(int16_t value, uint8_t number_of_digits, uint8_t zero_cleaning);


/**
 * This function writes a variable that contains a real number. 
 *
 * @param value Float value to be written (either positive or negative)
 *
 * @param number_of_digits The number is formatted with a fixed number of digits to ensure a constant string length.
 * 
 * @param number_of_fractional The number of digits representing fractional. No approximation is done.
 *
 * @return void
 *
 */
void LCD_write_float (float value, uint8_t number_of_digits, uint8_t number_of_fractional_digits, uint8_t zero_cleaning);


/**
 * This Function cleans the LCD display.
 *
 */
void LCD_clear (void);


/**
 * This function controls the cursor option (blinking, active)
 *
 * @param active Activate the cursor, showing it or not [LCD_TURN_ON_CURSOR, LCD_TURN_OFF_CURSOR]
 *
 * @param blinking  Let the cursor blink [LCD_BLINKING_ON, LCD_BLINKING_OFF]
 *
 * @note You must use the constant LCD_TURN_ON_CURSOR, LCD_TURN_OFF_CURSOR, LCD_BLINKING_ON, LCD_BLINKING_OFF
 *       to ensure the compatibility with LCD_44780_I2C library.
 *
 */
void LCD_cursor(uint8_t active, uint8_t blinking);


/**
 * This function controls the backligth LED.
 *
 * @param active Activate the LED backlight [LCD_TURN_ON_LED, LCD_TURN_OFF_LED]
 *
 * @param blinking  Let the cursor blink [LCD_BLINKING_ON, LCD_BLINKING_OFF]
 *
 * @note You must use the constant LCD_TURN_ON_LED, LCD_TURN_OFF_LED
 *       to ensure the compatibility with LCD_44780_I2C library.
 */
void LCD_backlight(uint8_t active);


/**
 * This function initializes the LCD to work in 4 bit modality.
 *
 * @param quartz_frequency Quartz freq. expressed in MHz used to run the CPU.
 *
 *
 * @note You must properly set the microcontroller pins using the TRISx registers
 *       The library requires the delay library to be included.
 */
void  LCD_initialize(uint8_t quartz_frequency);


#endif
