/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
 
Created on Date : 19/07/2023
Last update     : 19/07/2023
 
CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

These functions allow the user to read the data out of the sensor LSM6DSM.

*******************************************************************************/
#ifndef LTLIB_LSM6DSM_H
#define LTLIB_LSM6DSM_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif


//*****************************************
//   Library Test
//*****************************************

#include "LTlib.h"

#ifndef LTLIB_I2C_MODULE_H
    #error The LSM6DSM Library requires the "module_I2C" library to be included.
#endif

#ifndef LTLIB_DELAY_H
    #error The LSM6DSM Library requires the "LTlib_delay" library to be included.
#endif


//*****************************************
//   I2C Module Switch
//*****************************************
#if defined (LSM6DSM_I2C_MODULE_1) || defined (LSM6DSM_I2C_MODULE_2)

#else
    #define LSM6DSM_I2C_MODULE_1
#endif 

#ifdef LSM6DSM_I2C_MODULE_1
    #define LSM6DSM_I2C_open I2C1_open
    #define LSM6DSM_I2C_baud_rate I2C1_baud_rate  
    #define LSM6DSM_I2C_write_byte_to_external_device I2C1_write_byte_to_external_device
    #define LSM6DSM_I2C_read_byte_from_external_device I2C1_read_byte_from_external_device
#endif

#ifdef LSM6DSM_I2C_MODULE_2 
    #define LSM6DSM_I2C_open I2C2_open
    #define LSM6DSM_I2C_baud_rate I2C2_baud_rate 
    #define LSM6DSM_I2C_write_byte_to_external_device I2C2_write_byte_to_external_device
    #define LSM6DSM_I2C_read_byte_from_external_device I2C2_read_byte_from_external_device
#endif

//*****************************************
//   Device Address options
//*****************************************
#define LSM6DSM_DEVICE_ADDRESS_0 0x6A
#define LSM6DSM_DEVICE_ADDRESS_1 0x6B

//*****************************************
//   Device registers
//*****************************************

#define LSM6DSM_REG_FUNC_CFG_ACCESS     0x01
#define LSM6DSM_REG_SENSOR_SYNC_TIME    0x04
#define LSM6DSM_REG_SENSOR_SYNC_RES     0x05
#define LSM6DSM_REG_FIFO_CTRL1          0x06
#define LSM6DSM_REG_FIFO_CTRL2          0x07
#define LSM6DSM_REG_FIFO_CTRL3          0x08
#define LSM6DSM_REG_FIFO_CTRL4          0x09
#define LSM6DSM_REG_FIFO_CTRL5          0x0A
#define LSM6DSM_REG_DRDY_PULSE_CFG      0x0B
#define LSM6DSM_REG_INT1_CTRL           0x0D
#define LSM6DSM_REG_INT2_CTRL           0x0E
#define LSM6DSM_REG_WHO_AM_I            0x0F 
#define LSM6DSM_REG_CTRL1_XL            0x10
#define LSM6DSM_REG_CTRL2_G             0x11
#define LSM6DSM_REG_CTRL3_C             0x12
#define LSM6DSM_REG_CTRL4_C             0x13
#define LSM6DSM_REG_CTRL5_C             0x14
#define LSM6DSM_REG_CTRL6_C             0x15
#define LSM6DSM_REG_CTRL7_G             0x16
#define LSM6DSM_REG_CTRL8_XL            0x17
#define LSM6DSM_REG_CTRL9_XL            0x18
#define LSM6DSM_REG_CTRL10_C            0x19
#define LSM6DSM_REG_MASTER_CONFIG       0x1A
#define LSM6DSM_REG_WAKE_UP_SRC         0x1B
#define LSM6DSM_REG_TAP_SRC             0x1C
#define LSM6DSM_REG_D6D_SRC             0x1D
#define LSM6DSM_REG_STATUS              0x1E

#define LSM6DSM_REG_OUT_TEMP_L          0x20
#define LSM6DSM_REG_OUT_TEMP_H          0x21

#define LSM6DSM_REG_OUTX_L_G            0x22
#define LSM6DSM_REG_OUTX_H_G            0x23
#define LSM6DSM_REG_OUTY_L_G            0x24
#define LSM6DSM_REG_OUTY_H_G            0x25
#define LSM6DSM_REG_OUTZ_L_G            0x26
#define LSM6DSM_REG_OUTZ_H_G            0x27

#define LSM6DSM_REG_OUTX_L_XL           0x28
#define LSM6DSM_REG_OUTX_H_XL           0x29
#define LSM6DSM_REG_OUTY_L_XL           0x2A
#define LSM6DSM_REG_OUTY_H_XL           0x2B
#define LSM6DSM_REG_OUTZ_L_XL           0x2C
#define LSM6DSM_REG_OUTZ_H_XL           0x2D

#define LSM6DSM_REG_FIFO_STATTUS1       0x3A
#define LSM6DSM_REG_FIFO_STATTUS2       0x3B
#define LSM6DSM_REG_FIFO_STATTUS3       0x3C
#define LSM6DSM_REG_FIFO_STATTUS4       0x3D
#define LSM6DSM_REG_FIFO_DATA_OUT_L     0x3E
#define LSM6DSM_REG_FIFO_DATA_OUT_H     0x3F

#define LSM6DSM_REG_TIMESTAMP0          0x40
#define LSM6DSM_REG_TIMESTAMP1          0x41
#define LSM6DSM_REG_TIMESTAMP2          0x42
#define LSM6DSM_REG_STEP_TIMESTAMP_L    0x49
#define LSM6DSM_REG_STEP_TIMESTAMP_H    0x4A
#define LSM6DSM_REG_STEP_COUNTER_L      0x4B
#define LSM6DSM_REG_STEP_COUNTER_H      0x4C

#define LSM6DSM_REG_FUNC_SRC1           0x53
#define LSM6DSM_REG_FUNC_SRC2           0x54
#define LSM6DSM_REG_WIRST_TILT_IA       0x55

#define LSM6DSM_REG_TAP_CFG             0x58
#define LSM6DSM_REG_TAP_THS_6D          0x59

#define LSM6DSM_REG_INT_DUR2            0x5A
#define LSM6DSM_REG_WAKE_UP_THS         0x5B
#define LSM6DSM_REG_WAKE_UP_DUR         0x5C
#define LSM6DSM_REG_FREE_FALL           0x5D
#define LSM6DSM_REG_MD1_CFG             0x5E
#define LSM6DSM_REG_MD2_CFG             0x5F

#define LSM6DSM_REG_INT_OIS             0x6F
#define LSM6DSM_REG_CTRL1_OIS           0x70
#define LSM6DSM_REG_CTRL2_OIS           0x71
#define LSM6DSM_REG_CTRL3_OIS           0x72

#define LSM6DSM_REG_X_OFS_USR           0x73
#define LSM6DSM_REG_Y_OFS_USR           0x74
#define LSM6DSM_REG_Z_OFS_USR           0x75


//********************************************
//   Constant Configigurations
//********************************************
#define LSM6DSM_DEVICE_ID 0x6A

//********************************************
//   Common
//********************************************
#define LSM6DSM_ACCELEROMETER   0x01
#define LSM6DSM_GYROSCOPE       0x02

#define LSM6DSM_RESET_MASK      0xFE
#define LSM6DSM_SOFTWARE_RESET  0x01

//Bit status can be monitored doing the logic & 
//with the constant and Status register value
#define LSM6DSM_STATUS_TDA      0x04
#define LSM6DSM_STATUS_GDA      0x02
#define LSM6DSM_STATUS_XLDA     0x01


#define LSM6DSM_INT1_PIN        0x01
#define LSM6DSM_INT2_PIN        0x02

#define LSM6DSM_INT1_RESET_ALL_SIGNALS  0xAA
#define LSM6DSM_INT2_RESET_ALL_SIGNALS  0x55

//INT 1 signals
#define LSM6DSM_INT1_PEDOMETER_STEP     0x80
#define LSM6DSM_INT1_SIGNIFICANT_MOTION 0x40
#define LSM6DSM_INT1_FIFO_FULL          0x20
#define LSM6DSM_INT1_FIFO_OVERUN        0x10
#define LSM6DSM_INT1_FIFO_THRESHOLD     0x08
#define LSM6DSM_INT1_BOOT_STATUS        0x04
#define LSM6DSM_INT1_GYRO_DATA_READY    0x02
#define LSM6DSM_INT1_ACC_DATA_READY     0x01

//INT 2 signals
#define LSM6DSM_INT2_PEDOMETER_STEP_DELTA   0x80
#define LSM6DSM_INT2_STEP_COUNTER_OVERFLOW  0x40
#define LSM6DSM_INT2_FIFO_FULL              0x20
#define LSM6DSM_INT2_FIFO_OVERUN            0x10
#define LSM6DSM_INT2_FIFO_THRESHOLD         0x08
#define LSM6DSM_INT2_TEMP_DATA_READY        0x04
#define LSM6DSM_INT2_GYRO_DATA_READY        0x02
#define LSM6DSM_INT2_ACC_DATA_READY         0x01


#define LSM6DSM_INT_MERGE_RESET_MASK            0xCF
#define LSM6DSM_INT_MODE_MERGE_INT1_INT2        0x20
#define LSM6DSM_INT_MODE_INDIPENDENT_INT1_INT2  0x00

#define LSM6DSM_INT_MODE_RESET_MASK         0xDF
#define LSM6DSM_INT_MODE_OPEN_DRAIN         0x10
#define LSM6DSM_INT_MODE_PUSH_PULL          0x00
#define LSM6DSM_INT_MODE_ACTIVE_HIGH        0x00
#define LSM6DSM_INT_MODE_ACTIVE_LOW         0x20


//********************************************
//   Accelerometer
//********************************************
#define LSM6DSM_ACC_POWER_DOWN_MASK     0x0F
#define LSM6DSM_GYRO_POWER_DOWN_MASK    0x0F
#define LSM6DSM_POWER_DOWN_ALL          0x01
#define LSM6DSM_POWER_DOWN_ACC          0x02
#define LSM6DSM_POWER_DOWN_GYRO         0x03
#define LSM6DSM_POWER_ON_ALL            0x04  //Set 12.5Hz
#define LSM6DSM_POWER_ON_ACC            0x05  //Set 12.5Hz
#define LSM6DSM_POWER_ON_GYRO           0x06  //Set 12.5Hz

#define LSM6DSM_ACC_SAMPLING_MASK       0x0F
#define LSM6DSM_ACC_SAMPLING_1_6_HZ     0xB0
#define LSM6DSM_ACC_SAMPLING_12_5_HZ    0x10
#define LSM6DSM_ACC_SAMPLING_26_HZ      0x20
#define LSM6DSM_ACC_SAMPLING_52_HZ      0x30
#define LSM6DSM_ACC_SAMPLING_104_HZ     0x40
#define LSM6DSM_ACC_SAMPLING_208_HZ     0x50
#define LSM6DSM_ACC_SAMPLING_416_HZ     0x60
#define LSM6DSM_ACC_SAMPLING_833_HZ     0x70
#define LSM6DSM_ACC_SAMPLING_1660_HZ    0x80
#define LSM6DSM_ACC_SAMPLING_3330_HZ    0x90
#define LSM6DSM_ACC_SAMPLING_6660_HZ    0xA0

#define LSM6DSM_ACC_FULLSCALE_MASK      0xF3
#define LSM6DSM_ACC_FULLSCALE_2g        0x00
#define LSM6DSM_ACC_FULLSCALE_4g        0x08
#define LSM6DSM_ACC_FULLSCALE_8g        0x0C
#define LSM6DSM_ACC_FULLSCALE_16g       0x04

//********************************************
//   Gyroscope 
//********************************************

#define LSM6DSM_GYRO_SAMPLING_MASK 0x0F
#define LSM6DSM_GYRO_SAMPLING_12_5_HZ    0x10
#define LSM6DSM_GYRO_SAMPLING_26_HZ      0x20
#define LSM6DSM_GYRO_SAMPLING_52_HZ      0x30
#define LSM6DSM_GYRO_SAMPLING_104_HZ     0x40
#define LSM6DSM_GYRO_SAMPLING_208_HZ     0x50
#define LSM6DSM_GYRO_SAMPLING_416_HZ     0x60
#define LSM6DSM_GYRO_SAMPLING_833_HZ     0x70
#define LSM6DSM_GYRO_SAMPLING_1660_HZ    0x80
#define LSM6DSM_GYRO_SAMPLING_3330_HZ    0x90
#define LSM6DSM_GYRO_SAMPLING_6660_HZ    0xA0

#define LSM6DSM_GYRO_FULLSCALE_MASK      0xF1
#define LSM6DSM_GYRO_FULLSCALE_125_dps   0x02
#define LSM6DSM_GYRO_FULLSCALE_250_dps   0x00
#define LSM6DSM_GYRO_FULLSCALE_500_dps   0x04
#define LSM6DSM_GYRO_FULLSCALE_1000_dps  0x08
#define LSM6DSM_GYRO_FULLSCALE_2000_dps  0x0C


//*****************************************
//         Global const/variables
//*****************************************
//The constants are used to properly set the full scale factors and determine 
//the proper register values.
const float LSM6DSM_GYRO_sensitivity_125 = 4.375;
const float LSM6DSM_GYRO_sensitivity_250 = 8.75;
const float LSM6DSM_GYRO_sensitivity_500 = 17.50;
const float LSM6DSM_GYRO_sensitivity_1000 = 35;
const float LSM6DSM_GYRO_sensitivity_2000 = 70.0;
float LSM6DSM_GYRO_active_full_scale = 0;

const float LSM6DSM_ACC_sensitivity_2g = 0.061;
const float LSM6DSM_ACC_sensitivity_4g = 0.122;
const float LSM6DSM_ACC_sensitivity_8g = 0.244;
const float LSM6DSM_ACC_sensitivity_16g = 0.488;
float LSM6DSM_ACC_active_full_scale = 0;


//******************************************************************************


/**
 * This function initializes the I2C module accordingly to the clock and baud rate.
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param baud_rate_KHz Baud rate expressed in KHz
 *
 * @return 
 *
 * @note  If other devices are connected on the I2C bus the baud rate would be changed by
 * the function. Among several calls from different device initializations, only the last one will be
 * reflected in the I2C module configuration. 
 *
 */
void LSM6DSM_initialize (uint8_t device_address, uint16_t baud_rate_KHz);

 
/**
 * This function writes into any register of the L3GD20H device.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param register_add Address of the register that must be written. 
 *
 * @param data Data that must be written into the register. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 * @note    For the register address you must use the ones specified in the header
 *          file. The names reflect the ones in the datasheet but 
 *          they have the L3GD20H_REG_ prefix.
 *
 */
int8_t LSM6DSM_set_register (uint8_t device_address, uint8_t register_add,  uint8_t data );


/**
 * This function reads from any register of the L3GD20H device.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param register_add Address of the register that must be written. 
 * 
 * @param readback_value Pointer to a u_int8 variable that will contain the register content. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 * @note    For the register address you must use the ones specified in the header
 *          file. The names reflect the ones in the datasheet but 
 *          they have the L3GD20H_REG_ prefix.
 *
 */
int8_t LSM6DSM_get_register (uint8_t device_address, uint8_t register_address, uint8_t * readback_value);


/**
 * This function update a register value. 
 * It should be considered a private function that the user should not use. 
 * The bits that will be updated will depend on the mask that will be provided.
 * The bits that must remain unchanged must be set to 1 into the mask. 
 * The new_value must have a valid value out of the defined constansts 
 * to avoid unwanted behaviours. 
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param register_address Address to change.
 * 
 * @param register_mask Register mask. Bit set to 1 will remain unchanged.
 *                                          
 * 
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t LSM6DSM_update_register (uint8_t device_address, uint8_t register_address, uint8_t register_mask, uint8_t new_value);


/**
 * This function reads the temperature out of the L3GD20H device.
 * The returned value is 2 complement 8 bits format. This value is close too be useless
 * since it is not properly documented in the datasheet and it is not the environment
 * temperature. 
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param temperature Pointer to a int16_t variable that will contain the temperature. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 */
int8_t LSM6DSM_get_temperature (uint8_t device_address, int16_t * temperature);


/**
 * This function reads a generic acceleration component.
 * The acc. component can be specified using the constants 
 * defined within the library.
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param Acc_X Pointer to the variable that will contatain the X component [m/s^2]
 * 
 * @param Acc_Y Pointer to the variable that will contatain the Y component [m/s^2]
 * 
 * @param Acc_Z Pointer to the variable that will contatain the Z component [m/s^2]
 *
 * @return status   0: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 */
int8_t LSM6DSM_get_acceleration (uint8_t device_address, float * Acc_X, float * Acc_Y, float * Acc_Z );


/**
 * This function reads the angular speed from the L3GD20H device.
 * The value are given back as dps (degree per second) as float format. 
 * The values are computed against the sensitivity settings (gain and full scale). 
 * All 3 axes are read together.  
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param dps_X Pointer to a float variable that will contain the X dps (degree per second). 
 * 
 * @param dps_Y Pointer to a float variable that will contain the Y dps (degree per second). 
 * 
 * @param dps_Z Pointer to a float variable that will contain the Z dps (degree per second). 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t LSM6DSM_get_angular_speed (uint8_t device_address, float *dps_X, float *dps_Y, float *dps_Z);


/**
 * This function reads the Device ID out of the L3GD20H device.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param device_ID Pointer to a uint8_t variable that will contain the Device ID. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t LSM6DSM_get_device_ID (uint8_t device_address, uint8_t * device_ID);


/**
 * This function resets the device via a software command.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t LSM6DSM_software_reset (uint8_t device_address);


/**
 * This function sets the LSM6DSM, etiher in Power Down or Normal mode.
 * By power on the device goes is poewr down mode. 
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param mode New operating mode [use the constants LSM6DSM_POWER_DOWN_xxx and LSM6DSM_POWER_ON_xxx]
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t LSM6DSM_set_power_mode (uint8_t device_address, uint8_t mode);


/**
 * This function sets the LSM6DSM full scale. YOu can select both the Gyroscope
 * and the Accelerometer. 
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param sub_system Select the accelerometer or Gyrosccope [LSM6DSM_ACCELEROMETER, LSM6DSM_GYROSCOPE]
 * 
 * @param full_scale New full scale value [use the constants L3GD20H_GYRO_FULLSCALE_xxxx, L3GD20H_ACC_FULLSCALE_xxxx]
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t LSM6DSM_set_full_scale (uint8_t device_address, uint8_t sub_system, uint8_t full_scale);


/**
 * This function sets the sampling frequency (ODS Output Data Sampling)
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param sub_system Select the accelerometer or Gyrosccope [LSM6DSM_ACCELEROMETER, LSM6DSM_GYROSCOPE]
 * 
 * @param sampling_frequency Select the sampling frequency [use the constants for each sub-system]
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t LSM6DSM_set_sampling_frequency (uint8_t device_address, uint8_t sub_system, uint8_t sampling_frequency);


/**
 * This function defines how the interrupt pin will operate (INT 1 and INT 2).
 * In particular it will define which event will drive the pin. 
 * The constants must be ORed. Each INT pin can support one function. 
 * If the pin is not used you have to used the DISABLED option. 
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param int_pin Select to which pin apply the signal outputs [LSM6DSM_INT1_PIN, LSM6DSM_INT2_PIN]
 * 
 * @param signal Select the signal to be routed to the INTx pin [LSM6DSM_INT1_xxx , LSM6DSM_INT2_xxx ]
 * 
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 * 
 * * @note Any time you call the functions, all the previous settings remains. 
 *         You can call the function multiple times to set all the signals. 
 *         if you want to remove the signals as signal select either LSM6DSM_INT1_RESET_ALL_SIGNALS LSM6DSM_INT2_RESET_ALL_SIGNALS  
 */
int8_t LSM6DSM_set_interrupt_pin_signals (uint8_t device_address, uint8_t int_pin, uint8_t signal);


/**
 * This function sets the characteristic of both interrupt pins (INT 1 and INT 2).
 * In particular it is possible to set those pin as open drain or push-pull.
 * Also the active logic level can be set.  
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param pin_mode defines the propriety of the INTx pins.
 *                 the required settings should be ORed in a signle call
 *                                                          [LSM6DSM_INT_MODE_OPEN_DRAIN
 *                                                          LSM6DSM_INT_MODE_PUSH_PULL
 *                                                          LSM6DSM_INT_MODE_ACTIVE_HIGH
 *                                                          LSM6DSM_INT_MODE_ACTIVE_LOW]
 *                                          
 * @param merge defines if the IN1 and INT2 signals are routed together (OR) [LSM6DSM_INT_MODE_INDIPENDENT_INT1_INT2 
 *                                                                           LSM6DSM_INT_MODE_MERGE_INT1_INT2]
 * 
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 * 
 * @note Any time you call the functions, all the previous settings are set to 0. 
 *       Only the new ones apply. So you have to OR all together.  
 */
int8_t LSM6DSM_set_interrupt_pin_mode (uint8_t device_address, uint8_t pin_mode, uint8_t merge);

/**
 * This function gets the interrupt status register value. 
 *
 * @param device_address Device address in 7 bits format.
 * 
 * @param interrupt_status Value of the interrupt status.
 *                                          
 * 
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t LSM6DSM_get_interrupt_status (uint8_t device_address, uint8_t * interrupt_status);

#endif

