/*******************************************************************************
 
Author : Mauro Laurenti
Version : 1.5
 
Created on Date : 25/10/2015
Last update     : 03/04/2023

CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 
*******************************************************************************/
#ifndef LTLIB_SETTINGS_H
#define LTLIB_SETTINGS_H

//***************************************************
//          COMPILER AND MCU INFO
//***************************************************

#if defined (_XC_H_) || defined (__XC_H) || defined (__XC__)
    #define COMPILER_XC
#endif

#ifdef __XC8
    #define COMPILER_XC8
#endif

#if defined (__XC16) || defined (__XC16__)
    #define COMPILER_XC16
#endif

#if defined (__PIC32C) || defined (__PIC32M) || defined (__XC32__)
    #define COMPILER_XC32
#endif

#ifdef COMPILER_XC
    #include <xc.h>
#endif

//***************************************************
//            LIBRARY TYPE DEFINITIONS 
//***************************************************

#include "LTlib_types.h" 

//***************************************************
//            SYSTEM & MODULE CLOCKS
//***************************************************

#ifdef MCC_USED 
    #define SYSTEM_CLOCK _XTAL_FREQ
#endif

#ifndef SYSTEM_CLOCK
    #define SYSTEM_CLOCK 20000000UL
#endif

#ifndef I2C_CLOCK
    #define I2C_CLOCK SYSTEM_CLOCK
#endif

#ifndef UART_CLOCK
    #define UART_CLOCK SYSTEM_CLOCK
#endif

#ifndef SPI_CLOCK
    #define SPI_CLOCK SYSTEM_CLOCK
#endif

//***************************************************
//             DEVICE CONFIGURATION
//***************************************************
#ifdef CHIP_SUPPORTED
    #undef CHIP_SUPPORTED
#endif
//******************************************************************************
//                                  8 bits PIC
//******************************************************************************
#ifdef COMPILER_XC8

    #ifdef _16F877
        #include "PIC16F877_config.h"
        #define CHIP_SUPPORTED
    #endif

    #ifdef _16F877A
        #include "PIC16F877A_config.h"
        #define CHIP_SUPPORTED
    #endif

    #ifdef _18F45K22
        #include "PIC18F45K22_config.h"
        #define CHIP_SUPPORTED
    #endif


    #ifdef _18F46K22
        #include "PIC18F46K22_config.h"
        #define CHIP_SUPPORTED
    #endif

    #ifdef _18LF4525
        #include "PIC18LF4525_config.h"
        #define CHIP_SUPPORTED
    #endif

    #ifdef _18F4550
        #include "PIC18F4550_config.h"
        #define CHIP_SUPPORTED
    #endif


    #ifdef _18F14K50
        #include "PIC18F14K50_config.h"
        #define CHIP_SUPPORTED
    #endif

    #ifdef _18F4580
        #include "PIC18F4580_config.h"
        #define CHIP_SUPPORTED
    #endif

#endif


//******************************************************************************
//                                  16 bits PIC
//******************************************************************************

#ifdef COMPILER_XC16

    #ifdef __PIC24FJ128GA204__
        #include "PIC24FJ128GA204_config.h"
        #define CHIP_SUPPORTED
    #endif

    #ifdef __PIC24FJ64GA104__
        #include "PIC24FJ64GA104_config.h"
        #define CHIP_SUPPORTED
    #endif

#endif


//******************************************************************************
//                                  32 bits PIC
//******************************************************************************
#ifdef COMPILER_XC32

    #ifdef __32MX150F128B__
        #include "PIC32MX150F128B_config.h"
        #define CHIP_SUPPORTED
    #endif

	#ifdef __32MX150F128D__
        #include "PIC32MX150F128D_config.h" 
        #define CHIP_SUPPORTED
    #endif

	#ifdef __32MX220F032B__
        #include "PIC32MX220F032B_config.h" 
        #define CHIP_SUPPORTED
    #endif

#endif



//******************************************************************************
//                                  Device End
//******************************************************************************
#ifndef MCC_USED
    #ifndef CHIP_SUPPORTED
        #error (LTlib) LTlib is not tested on the microcontroller you have selected
    #endif
#endif

#ifdef MCC_USED
    #ifndef CHIP_SUPPORTED
        #warning (LTlib) LTlib is not tested on the microcontroller you have selected
        #warning (LTlib) LTlib has loaded the MCC default config file. 
        #include "MCC_default_config.h" 
    #endif
#endif

#endif

