/*******************************************************************************
 
Autore : Mauro Laurenti
Versione : 1.1
 
Created on Date : 21/07/2019
Last update     : 16/03/2023

CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 
*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#ifndef LTLIB_UTILITIES_H
#define LTLIB_UTILITIES_H

//It sets how big a float conversion could be 
#define MAX_NUM_DIGITS 12

const uint8_t hex_table [] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};


/**
 * This function converts signed integer to a string of chars.
 * The returned string is always the shortest possible one containing the value.
 * Digit 0 contains - sign if the number is negative. The sign + is not shown.
 *
 * @param buffer Pointer to the array that will contain the string.
 *
 * @param value Integer value to convert
 * 
 * @return error 0: Conversion OK
 *              -1: Overflow - The number is too big  
 */
int8_t LTlib_int16_to_string (uint8_t * buffer, int16_t value);


/**
 * This function converts a float number to a string of chars.
 *
 * @param buffer buffer Pointer to the array that will contain the string.
 *
 * @param value float value to convert (float)
 * 
 * @param fractional_digits it specified the number of fractional digits 
 * 
 * @return error 0: Conversion OK
 *              -1: Overflow - The number is too big.  
 * 
 * @warning The buffer must be at least of size 8 to avoid overflow. 
 *          The function generates an internal buffer of MAX_NUM_DIGITS size.
 */
int8_t LTlib_float_to_string (uint8_t * buffer, float value,uint8_t fractional_digits );


/**
 * This function converts unsigned integer (HEX 8) to a string of chars made of ASCII digits.
 *
 * @param buffer buffer Pointer to the array that will contain the string.
 *
 * @param value Integer value to convert (uint8_t)
 * 
 * @return error 0: Conversion OK
 *              -1: Overflow - The number is too big.  
 * 
 * @warning The buffer must be at least of size 5 to avoid overflow.
 */
int8_t LTlib_Hex8_to_string (uint8_t * buffer, uint8_t value);


/**
 * This function converts unsigned integer (HEX 16) to a string of chars made of ASCII digits.
 *
 * @param buffer buffer Pointer to the array that will contain the string.
 *
 * @param value Integer value to convert (uint16_t)
 * 
 * @return error 0: Conversion OK
 *              -1: Overflow - The number is too big.  
 * 
 * @warning The buffer must be at least of size 5 to avoid overflow.
 */
int8_t LTlib_Hex16_to_string (uint8_t * buffer, uint16_t value);


/**
 * This function converts unsigned integer (HEX 32) to a string of chars made of HEX digits.
 *
 * @param buffer buffer Pointer to the array that will contain the string.
 *
 * @param value Integer value to convert (uint32_t)
 * 
 * @return error 0: Conversion OK
 *              -1: Overflow - The number is too big.  
 * 
 * @warning The buffer must be at least of size 8 to avoid overflow.
 */
int8_t LTlib_Hex32_to_string (uint8_t * buffer, uint32_t value);


/**
 * This function returns the string length, marked with the '\0' character
 * The \0 character is not counted.
 *
 * @param buffer Pointer to the string that contains the char array ending with '\0'
 * 
 * @return value String length 
 *      
 */
uint8_t LTlib_string_length (uint8_t * buffer);

#endif

 
