/*******************************************************************************

Author : Mauro Laurenti
Version : 1.5
 
Created on Date : 19/03/2011
Last update     : 29/07/2019
 
CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

These functions allow the user to read/write to the data
port of the I2C I/O extender PCF8574. 
Either PCF8574 or PCF8574A can be used. The user must set the
proper internal address (see the define below).

PCF8574  Address : 0100xxx0
PCF8574A Address : 0111xxx0

xxx = A2 A1 A0 (External Address Pins)

*******************************************************************************/
#ifndef LTLIB_PCF8574_H
#define LTLIB_PCF8574_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif


//*****************************************
//   Library Test
//*****************************************

#include "LTlib.h"

#ifndef LTLIB_I2C_MODULE_H
    #error The PCF8574 Library requires the "module_I2C" library to be included.
#endif

//*****************************************
//   I2C Module Switch
//*****************************************
#if defined (PCF8574_I2C_MODULE_1) || defined (PCF8574_I2C_MODULE_2)

#else
    #define PCF8574_I2C_MODULE_1
#endif 

#ifdef PCF8574_I2C_MODULE_1
    #define PCF8574_I2C_open I2C1_open
    #define PCF8574_I2C_baud_rate I2C1_baud_rate  
    #define PCF8574_I2C_wait_bus_IDLE  I2C1_wait_bus_IDLE
    #define PCF8574_I2C_start_bit I2C1_start_bit
    #define PCF8574_I2C_check_bus_collision I2C1_check_bus_collision
    #define PCF8574_I2C_write_byte I2C1_write_byte
    #define PCF8574_I2C_stop_bit I2C1_stop_bit
    #define PCF8574_I2C_check_ACK I2C1_check_ACK
    #define PCF8574_I2C_negative_ACK I2C1_negative_ACK
    #define PCF8574_I2C_read_byte I2C1_read_byte
    #define PCF8574_I2C_set_master_as_receiver I2C1_set_master_as_receiver

#endif

#ifdef PCF8574_I2C_MODULE_2 
    #define PCF8574_I2C_open I2C2_open
    #define PCF8574_I2C_baud_rate I2C2_baud_rate 
    #define PCF8574_I2C_wait_bus_IDLE I2C2_wait_bus_IDLE
    #define PCF8574_I2C_start_bit I2C2_start_bit
    #define PCF8574_I2C_check_bus_collision I2C2_check_bus_collision
    #define PCF8574_I2C_write_byte I2C2_write_byte
    #define PCF8574_I2C_stop_bit I2C2_stop_bit
    #define PCF8574_I2C_check_ACK I2C2_check_ACK
    #define PCF8574_I2C_negative_ACK I2C2_negative_ACK
    #define PCF8574_I2C_read_byte I2C2_read_byte
    #define PCF8574_I2C_set_master_as_receiver I2C2_set_master_as_receiver
#endif


//*****************************************
//   Device Settings
//*****************************************

#define USED_DEVICE_PCF8574 0x40
#define USED_DEVICE_PCF8574A  0x70

#ifndef PCF8574_INTERNAL_ADDRESS
    #define PCF8574_INTERNAL_ADDRESS USED_DEVICE_PCF8574
#endif


//*****************************************
/**
 * This function initializes the I2C module accordingly to the clock and baud rate.
 *
 * @param baud_rate_KHz Baud rate expressed in KHz
 *
 * @return 
 *
 * @note  If other devices are connected on the I2C bus the baud rate would be changed by
 * the function. Among several calls from different device initializations, only the last one will be
 * reflected in the I2C module configuration. 
 *
 */
void PCF8574_initialize (uint16_t baud_rate_KHz);


/**
 * This function writes to the data Port of the I2C I/O expander.
 *
 * @param control Is the address of the PCF8574 (Internal Address + Pin address)
 *
 * @param data Byte to write [min: 0, max: 255]
 *
 * @return status 0: The byte has been properly written
 *               -1: Communication Error
 *
 * @note    PCF8574  Address : 0100xxx0
 *          PCF8574A Address : 0111xxx0
 *
 *          xxx = A2 A1 A0 (External Address Pins)
 */
int8_t PCF8574_write_data (uint8_t device_address, uint8_t data );


/**
 * This function reads to the data Port of the I2C I/O expander.
 *
 * @param control Is the address of the PCF8574 (Internal Address + Pin address
 *
 * @param data Address where the data is written into.
 *
 * @return status 0: The byte has been properly written
 *               -1: Communication Error
 *
 * @note    PCF8574  Address : 0100xxx0
 *          PCF8574A Address : 0111xxx0
 *
 *          xxx = A2 A1 A0 (External Address Pins) 
 */
int8_t PCF8574_read_data(uint8_t device_address, uint8_t *data);


#endif
