/*******************************************************************************
 
Autore : Mauro Laurenti
Versione : 1.1
 
Created on Date : 16/03/2016
Last update     : 31/01/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 
*******************************************************************************/

#ifndef LTLIB_ADC_MODULE_H
#define LTLIB_ADC_MODULE_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif

//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef ADC_LIBRARY_SUPPORTED
    #error (LTlib) The MCU you have selected is not supported by module_ADC library.
#endif

#ifdef COMPILER_XC32
    #ifndef LTLIB_DELAY_H
        #error The module_ADC Library requires the "LTlib_delay" library to be included.
    #endif
#endif

//****************************************************

#ifdef COMPILER_XC8 
    #define ADC_JUSTIFICATION_LEFT 0
    #define ADC_JUSTIFICATION_RIGHT 1


    #if defined (ADC_MODULE_TYPE_1) || defined (ADC_MODULE_TYPE_2)|| defined (ADC_MODULE_TYPE_3)
        #define ADC_ACQUISITION_TIME_0_TAD  0x00
        #define ADC_ACQUISITION_TIME_2_TAD  0x08
        #define ADC_ACQUISITION_TIME_4_TAD  0x10
        #define ADC_ACQUISITION_TIME_6_TAD  0x18
        #define ADC_ACQUISITION_TIME_8_TAD  0x20
        #define ADC_ACQUISITION_TIME_12_TAD 0x28
        #define ADC_ACQUISITION_TIME_16_TAD 0x30
        #define ADC_ACQUISITION_TIME_20_TAD 0x38

        #define ADC_CLOCK_FOSC_2    0x00
        #define ADC_CLOCK_FOSC_8    0x01
        #define ADC_CLOCK_FOSC_32   0x02
        #define ADC_CLOCK_FOSC_RC   0x03
        #define ADC_CLOCK_FOSC_4    0x04
        #define ADC_CLOCK_FOSC_16   0x05
        #define ADC_CLOCK_FOSC_64   0x06
    #endif

    #ifdef ADC_MODULE_TYPE_4 
        #define ADC_CLOCK_FOSC_RC   0xC0
        #define ADC_CLOCK_FOSC_4    0x00
        #define ADC_CLOCK_FOSC_16   0x40
        #define ADC_CLOCK_FOSC_64   0x80
    #endif

    //**************************************************
    #ifdef ADC_MODULE_TYPE_1 
        #define ADC_VREF_P_AVDD 0x00
        #define ADC_VREF_P_EXT  0x10

        #define ADC_VREF_N_AVSS 0x00
        #define ADC_VREF_N_EXT  0x20
    #endif

    //**************************************************
    #if defined (ADC_MODULE_TYPE_2)|| defined (ADC_MODULE_TYPE_3)
        #define ADC_VREF_P_AVDD 0x00
        #define ADC_VREF_P_EXT  0x04
        #define ADC_VREF_P_BUF  0x08

        #define ADC_VREF_N_AVSS 0x00
        #define ADC_VREF_N_EXT  0x01
    #endif
#endif

#ifdef COMPILER_XC16
        #define ADC_VREF_P_AVDD_N_AVSS 0x0000
        #define ADC_VREF_P_EXT_N_AVSS  0x4000
        #define ADC_VREF_P_AVDD_N_EXT  0x2000
        #define ADC_VREF_EXT_N_EXT     0x6000

        #define ADC_CLOCK_RESET_MASK   0x8000
        #define ADC_CLOCK_INTERNAL RC  0x8000
        #define ADC_CLOCK_PBCLK        0x0000
#endif

#ifdef COMPILER_XC32
        #define ADC_VREF_P_AVDD_N_AVSS 0x00000000
        #define ADC_VREF_P_EXT_N_AVSS  0x00002000
        #define ADC_VREF_P_AVDD_N_EXT  0x00004000
        #define ADC_VREF_EXT_N_EXT     0x00006000

        #define ADC_CLOCK_RESET_MASK   0x00008000
        #define ADC_CLOCK_INTERNAL RC  0x00008000
        #define ADC_CLOCK_PBCLK        0x00000000
#endif

//****************************************************

 /**
 * This function enables the ADC and sets some default values. 
 *
 * @param ADC_enable_channel Or of the channels to enable. You must use the channel numbers declared in the header file (ADC_CH0, ADC_CH1...) 
 *
 * @return none
 * 
 * @warning For the ADC_MODULE_TYPE_4, openening the ADC enable all the Analog channels. 
 * This limitation is due to the ADC itself that does not allow single channel settings.
 * To make it simple the library enables all the channels at once. 
 *
 */ 
void ADC_open (uint32_t ADC_enable_channel);


 /**
 * This function disables the ADC. Disabling the ADC sets all the analog channels back to Digital.
 *
 * @param none
 *
 * @return none
 *
 */ 
void ADC_close (void);


 /**
 * This function selects the ADC channel that will be converted upon the next 
 * Start of conversion. You must use the channel numbers declared in the header file (ADC_CH0, ADC_CH1...) 
 *
 * @param ADC_channel Channel that will be converted
 *
 * @return none
 *
 */ 
void ADC_select_channel (uint32_t ADC_channel);


 /**
 * This function starts the ADC conversion.
 *
 * @param none
 *
 * @return none
 *
 */ 
void ADC_start_conversion (void);


 /**
 * This function aborts the ADC conversion that is eventually running.
 *
 * @param none
 *
 * @return none
 *
 */ 
void ADC_abort_conversion (void);


/**
 * This function returns the ADC conversion results merging the High and Low bytes 
 * used by the MCU as conversion buffers. The function requires that the ADC is properly opened 
 * and configured accordingly to the application.
 *
 * @param none
 *
 * @return results Conversion result.
 *
 */ 
uint16_t ADC_read_value (void);


/**
 * This function returns the ADC conversion results merging the High and Low bytes 
 * used by the MCU as conversion buffers. The function is a stand alone function that initializes the ADC
 * and returns the ADC conversion of the selected ADC channel. Afterward closes the ADC. 
 * Only one Channel can be selected at a time.
 *
 * @param ADC_channel Channel to convert. You must use the channel numbers declared in the header file (ADC_CH0, ADC_CH1...) 
 *
 * @return results Conversion result.
 *
 * @warning This function is a blocking one, waiting for the end of conversion.
 */ 
uint16_t ADC_read_channel (uint32_t ADC_channel);


/**
 * This function returns the ADC running status.
 *
 * @param none
 *
 * @return results 0: Conversion is over.
 *                 1: Conversion is running.
 *
 */ 
uint8_t ADC_is_converting (void);


 /**
 * This function sets the acquisition time as multiple of TADs. Refer to the datasheet
 * for more information on TAD time base. 
 *
 * @param acq_time Acquisition time. For 8 bits PIC you must use the constants ADC_ACQUISITION_TIME_xx_TAD to properly set the time. 
 *                                xx can be 0,2,4,6,8,12,16,20. For PIC32 you can write the time in us directly. 
 *
 * @return none
 *
 */ 
void ADC_set_acquisition_time (register_size acq_time);


/**
 * This function sets the clock frequency used by the ADC. 
 *
 * @param ADC_clock ADC Clock. You must use the constants ADC_CLOCK_xxx to properly set the clock. 
 *                             Depending on the MCU architecture, different configuration may apply. 
 *
 * @return none
 *
 */ 
void ADC_set_clock (register_size ADC_clock);


#ifdef COMPILER_XC8 
/**
 * This function sets the ADC justification. Since the data is stored into 2 registers. 
 * The justification allows storing the data either starting from the left (high side) or the right (low side).
 * Using left justification and read only the high side register is like having 8 bit conversion instead of 10 bits.
 *
 * @param ADC_justification ADC read buffer justification. You must use either the constant ADC_JUSTIFICATION_LEFT or ADC_JUSTIFICATION_RIGHT.   
 *
 * @return none
 *
 */ 
void ADC_set_justification (uint8_t ADC_justification);
#endif


/**
 * This function sets the ADC references for both the positive and negative one.
 *
 * @param ADC_reference_V_P ADC Positive reference V+. You must use either the constant ADC_VREF_P_AVDD or ADC_VREF_P_EXT or ADC_VREF_P_BUF. 
 * @param ADC_reference_V_N ADC Positive reference V+. You must use either the constant ADC_VREF_N_AVSS or ADC_VREF_N_EXT. 
 *
 * @return none
 * 
 * @warning This function does not support ADC_MODULE_TYPE_4
 *
 */
#ifdef COMPILER_XC8
void ADC_set_reference (register_size ADC_reference_V_P, register_size ADC_reference_V_N);
#endif 

#ifdef COMPILER_XC16
void ADC_set_reference (register_size ADC_reference);
#endif 

#ifdef COMPILER_XC32
void ADC_set_reference (register_size ADC_reference);
#endif 

 
/**
 * Private function that converts the channel mask to the ADC channel that will be actually converted.
 * This function should not be used by the user. It is intended for internal use only.
 *
 * @param channel_mask ADC channel mask. 
 *
 * @return ADC channel to be loaded within the internal register. 
 *
 */ 
uint8_t ADC_channel_converter (uint32_t channel_mask);
    

#endif
