/*******************************************************************************
 
Author : Mauro Laurenti
Versione : 1.3
 
Created on Date : 02/10/2010
Last update     : 03/02/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This library is written to support reading and writing the internal EEPROM
that many PIC microcontrollers have.
 
*******************************************************************************/

#ifndef LTLIB_EEPROM_MODULE_H
#define LTLIB_EEPROM_MODULE_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif

//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef EEPROM_LIBRARY_SUPPORTED
    #error (LTlib) The MCU you have selected is not supported by module_EEPROM library.
#endif


/**
 * This function writes a byte inside the internal EEPROM. 
 *
 * @param data Byte to write [min: 0, max: 255]
 *
 * @param address Address where the byte must be written [min: 0, max: 64K]
 *
 * @return status 1: The byte has been properly written
 *                0: The byte has not been properly written
 *
 * @warning During the writing process the Interrupts are disabled. The old status is restored afterward.
 *
 */
int8_t EEPROM_write_byte (uint16_t address, uint8_t data);


/**
 * This function reads a byte from the internal EEPROM. 
 *
 *
 * @param address Address where the byte must be read [min: 0, max: 255]
 *
 * @return data Value read from the internal EEPROM.
 *
 */
uint8_t EEPROM_read_byte (uint16_t address);


#endif
