/*******************************************************************************
 
Autore : Mauro Laurenti
Versione : 1.3
 
Created on Date : 15/01/2016
Last update     : 10/11/2022

CopyRight 2006-2022 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ALTLIB_IO_MODULE_H
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 
*******************************************************************************/

#ifndef LTLIB_IO_MODULE_H
#define LTLIB_IO_MODULE_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif

//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef IO_LIBRARY_SUPPORTED
    #error (LTlib) The MCU you have selected is not supported by module_IO library.
#endif

//*****************************************
//   Library constants
//*****************************************

#define IO_PORTA 0x01
#define IO_PORTB 0x02 
#define IO_PORTC 0x03
#define IO_PORTD 0x04
#define IO_PORTE 0x05
#define IO_PORTF 0x05

#define IO_BIT0  0x0001
#define IO_BIT1  0x0002
#define IO_BIT2  0x0004
#define IO_BIT3  0x0008
#define IO_BIT4  0x0010
#define IO_BIT5  0x0020
#define IO_BIT6  0x0040
#define IO_BIT7  0x0080


#if defined (COMPILER_XC32) || defined (COMPILER_XC16) 
    #define IO_BIT8  0x0100
    #define IO_BIT9  0x0200
    #define IO_BIT10 0x0400
    #define IO_BIT11 0x0800
    #define IO_BIT12 0x1000
    #define IO_BIT13 0x2000
    #define IO_BIT14 0x4000
    #define IO_BIT15 0x8000
#endif

#ifdef COMPILER_XC8
    #define IO_ALL_PORT_OUTPUT 0x00
    #define IO_ALL_PORT_INPUT 0xFF
#endif

#if defined (COMPILER_XC32) || defined (COMPILER_XC16) 
    #define IO_ALL_PORT_OUTPUT 0x0000
    #define IO_ALL_PORT_INPUT 0xFFFF
#endif


#define IO_BIT0_IN 0x0001
#define IO_BIT1_IN 0x0002
#define IO_BIT2_IN 0x0004
#define IO_BIT3_IN 0x0008
#define IO_BIT4_IN 0x0010
#define IO_BIT5_IN 0x0020
#define IO_BIT6_IN 0x0040
#define IO_BIT7_IN 0x0080

#if defined (COMPILER_XC32) || defined (COMPILER_XC16) 
    #define IO_BIT8_IN 0x0100
    #define IO_BIT9_IN 0x0200
    #define IO_BIT10_IN 0x0400
    #define IO_BIT11_IN 0x0800
    #define IO_BIT12_IN 0x1000
    #define IO_BIT13_IN 0x2000
    #define IO_BIT14_IN 0x4000
    #define IO_BIT15_IN 0x8000
#endif

#ifdef COMPILER_XC8
    #define IO_BIT0_OUT 0b11111110
    #define IO_BIT1_OUT 0b11111101
    #define IO_BIT2_OUT 0b11111011
    #define IO_BIT3_OUT 0b11110111
    #define IO_BIT4_OUT 0b11101111
    #define IO_BIT5_OUT 0b11011111
    #define IO_BIT6_OUT 0b10111111
    #define IO_BIT7_OUT 0b01111111
#endif

#if defined (COMPILER_XC32) || defined (COMPILER_XC16) 
    #define IO_BIT0_OUT 0b1111111111111110
    #define IO_BIT1_OUT 0b1111111111111101
    #define IO_BIT2_OUT 0b1111111111111011
    #define IO_BIT3_OUT 0b1111111111110111
    #define IO_BIT4_OUT 0b1111111111101111
    #define IO_BIT5_OUT 0b1111111111011111
    #define IO_BIT6_OUT 0b1111111110111111
    #define IO_BIT7_OUT 0b1111111101111111
    #define IO_BIT8_OUT 0b1111111011111111
    #define IO_BIT9_OUT 0b1111110111111111
    #define IO_BIT10_OUT 0b1111101111111111
    #define IO_BIT11_OUT 0b1111011111111111
    #define IO_BIT12_OUT 0b1110111111111111
    #define IO_BIT13_OUT 0b1101111111111111
    #define IO_BIT14_OUT 0b1011111111111111
    #define IO_BIT15_OUT 0b0111111111111111
#endif

//******************************************************************************

/**
 * This function initializes all the PORTs as inputs
 *
 * @param void
 *
 * @return void
 *
 */
void IO_set_all_ports_as_inputs (void);


/**
 * This function sets the MCU port direction.
 *
 * @param mcu_port port for which you want to enable the pull-up resistors
 * @param port_direction Each bit set the proper bit direction
 *
 * @return void
 *
 * @note For the port name use the defined constants to allow future improvements. 
 * @note To properly use the function you need to call the IO_set_all_ports_as_inputs first.
 *       Indeed this function handles all the Analog inputs.  
 */
void IO_set_port_direction (uint8_t mcu_port, port_size port_direction);


/**
 * This function writes the data to the port.
 *
 * @param mcu_port Port to which you want to write the data.
 * @param port_data Data to be sent out
 *
 * @return void
 *
 * @note For the port name use the defined constants to allow future improvements. 
 * @note To properly use the function you need to call the IO_set_all_ports_as_inputs first.
 *       Indeed this function handles all the Analog inputs.  
 */
void IO_write_port (uint8_t mcu_port, port_size port_data);


/**
 * This function reads the data from the port.
 *
 * @param mcu_port Port from which you want to read the data.
 *
 * @return Data from the selected port
 *
 * @note For the port name use the defined constants to allow future improvements. 
 * @note To properly use the function you need to call the IO_set_all_ports_as_inputs first.
 *       Indeed this function handles all the Analog inputs.  
 */
port_size IO_read_port (uint8_t mcu_port);


/**
 * This function reads bit from the selected port.
 *
 * @param mcu_port Port from which you want to read the data.
 * @param bit_mask Selected bit to be read out of the port.
 *
 * @return Bit data from the selected port
 *
 * @note For the port name use the defined constants to allow future improvements. 
 * @note To properly use the function you need to call the IO_set_all_ports_as_inputs first.
 *       Indeed this function handles all the Analog inputs.  
 */
port_size IO_read_port_bit (uint8_t mcu_port, port_size bit_mask );


#ifdef PULL_UP_RESISTORS_AVAILABLE
/**
 * This function enables the pull-up resistors on the selected port (if available). 
 *
 * @param mcu_port port for which you want to enable the pull-up resistors
 * @param resistors_to_enable Each bit enable single resistor (if the PIC supports it) otherwise is ignored
 *
 * @return void
 *
 * @note For the port name use the defined constants to allow future improvements.  
 */
    void IO_enable_pull_up_resistors (uint8_t mcu_port, port_size resistors_to_enable);
#endif    
 

#ifdef PULL_DOWN_RESISTORS_AVAILABLE    
    /**
     * This function enables the pull-down resistors on the selected port (if available). 
     *
     * @param mcu_port port for which you want to enable the pull-down resistors
     * @param resistors_to_enable Each bit enable single resistor 
     * 
     * @return void
     *
     * @note For the port name use the defined constants to allow future improvements.  
     */
        void IO_enable_pull_down_resistors (uint8_t mcu_port, port_size resistors_to_enable);
    #endif  
#endif  

