/*******************************************************************************

Author : Mauro Laurenti
Version : 1.3
 
Created on Date : 09/03/2016
Last update     : 24/02/2023

CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This library contains all the functions that are handy for controlling 			
the SPI module
*******************************************************************************/

#ifndef LTLIB_SPI_MODULE_H
#define LTLIB_SPI_MODULE_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif

//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef SPI_LIBRARY_SUPPORTED
    #error (LTlib) The MCU you have selected is not supported by module_SPI library.
#endif


//*****************************************
//       SPI module initialization
//*****************************************

#define SPI_SS_ENABLED 0x04     
#define SPI_SS_DISABLED 0x05    
#define SPI1_SS_PIN SPI_SS_DISABLED
#define SPI2_SS_PIN SPI_SS_DISABLED
#define SPI3_SS_PIN SPI_SS_DISABLED


//*****************************************
//          SPI module constants
//*****************************************

#define SPI_MASTER_DEVICE 0x01
#define SPI_SLAVE_DEVICE 0x02

#define SPI_MODE_0 0x00
#define SPI_MODE_1 0x01
#define SPI_MODE_2 0x02
#define SPI_MODE_3 0x03

#define SPI_TIMER_OUT_CLK   0x03    //0b00000011
#define SPI_CLK_OSC_64      0x02    //0b00000010
#define SPI_CLK_OSC_16      0x01    //0b00000001
#define SPI_CLK_OSC_4       0x00    //0b00000000

#ifdef COMPILER_XC8
    #if (SPI_CLOCK == 20000000)
        #define SPI_CLK_312_KHZ 0x02    //0b00000010
        #define SPI_CLK_1250_KHZ 0x01   //0b00000001
        #define SPI_CLK_5000_KHZ 0x00   //0b00000000
    #endif
#endif


#ifdef COMPILER_XC16

    #ifdef SPI_BAUDRATE_BRG
        #if (SPI_CLOCK == 32000000)
            #define SPI_CLK_50_KHZ  159    
            #define SPI_CLK_100_KHZ 79   
            #define SPI_CLK_150_KHZ 53   
            #define SPI_CLK_250_KHZ 31   
            #define SPI_CLK_500_KHZ 15   
            #define SPI_CLK_1000_KHZ 7  
            #define SPI_CLK_2000_KHZ 3  
            #define SPI_CLK_4000_KHZ 1
        #endif

        #if (SPI_CLOCK == 20000000)
            #define SPI_CLK_50_KHZ  99    
            #define SPI_CLK_100_KHZ 49   
            #define SPI_CLK_150_KHZ 33   
            #define SPI_CLK_250_KHZ 19   
            #define SPI_CLK_500_KHZ 9   
            #define SPI_CLK_1000_KHZ 4 
            #define SPI_CLK_2000_KHZ 2  
            #define SPI_CLK_4000_KHZ 0 
        #endif

        #if (SPI_CLOCK == 16000000)
            #define SPI_CLK_50_KHZ  79    
            #define SPI_CLK_100_KHZ 39   
            #define SPI_CLK_150_KHZ 26   
            #define SPI_CLK_250_KHZ 15   
            #define SPI_CLK_500_KHZ 7   
            #define SPI_CLK_1000_KHZ 3  
            #define SPI_CLK_2000_KHZ 1  
            #define SPI_CLK_4000_KHZ 0 
        #endif

        #if (SPI_CLOCK == 8000000)
            #define SPI_CLK_50_KHZ  40    
            #define SPI_CLK_100_KHZ 20   
            #define SPI_CLK_150_KHZ 13   
            #define SPI_CLK_250_KHZ 8  
            #define SPI_CLK_500_KHZ 4   
            #define SPI_CLK_1000_KHZ 2  
            #define SPI_CLK_2000_KHZ 1  
            #define SPI_CLK_4000_KHZ 0  // Not supported
        #endif

    #endif

    #ifdef SPI_BAUDRATE_PRI_SEC_PRESCALER

        #if (SPI_CLOCK == 16000000)
            #define SPI_CLK_50_KHZ  40    
            #define SPI_CLK_100_KHZ 20   
            #define SPI_CLK_150_KHZ 13   
            #define SPI_CLK_250_KHZ 8  
            #define SPI_CLK_500_KHZ 4   
            #define SPI_CLK_1000_KHZ 2  
            #define SPI_CLK_2000_KHZ 0b00011010 
            #define SPI_CLK_4000_KHZ 0b00011110  
        #endif
    #endif 

#endif


#ifdef COMPILER_XC32

        #if (SPI_CLOCK == 32000000)
            #define SPI_CLK_50_KHZ  318    
            #define SPI_CLK_100_KHZ 158   
            #define SPI_CLK_150_KHZ 106   
            #define SPI_CLK_250_KHZ 62   
            #define SPI_CLK_500_KHZ 30   
            #define SPI_CLK_1000_KHZ 14  
            #define SPI_CLK_2000_KHZ 6  
            #define SPI_CLK_4000_KHZ 2
        #endif


    #if (SPI_CLOCK == 20000000)
        #define SPI_CLK_50_KHZ  199    
        #define SPI_CLK_100_KHZ 99   
        #define SPI_CLK_150_KHZ 66   
        #define SPI_CLK_250_KHZ 39   
        #define SPI_CLK_500_KHZ 19   
        #define SPI_CLK_1000_KHZ 9  
        #define SPI_CLK_2000_KHZ 4
        #define SPI_CLK_4000_KHZ 2 
        #define SPI_CLK_5000_KHZ 1   
    #endif

    #if (SPI_CLOCK == 16000000)
        #define SPI_CLK_50_KHZ  79    
        #define SPI_CLK_100_KHZ 39   
        #define SPI_CLK_150_KHZ 26   
        #define SPI_CLK_250_KHZ 15   
        #define SPI_CLK_500_KHZ 7   
        #define SPI_CLK_1000_KHZ 3  
        #define SPI_CLK_2000_KHZ 1  
        #define SPI_CLK_4000_KHZ 0 
    #endif
#endif


/**
 * This function opens the SPI module for communication.
 * In particular it sets the device type either to Master or Slave.
 *
 * @param device_type Device Type [SPI_MASTER_DEVICE, SPI_SLAVE_DEVICE]
 *
 * @return void
 *
 * @warning This function should be called only after having set the module with the right 
 * baudrate and mode.
 * 
 */
void SPI1_open (uint8_t device_type);

#ifdef SPI2_AVAILABLE
    void SPI2_open (uint8_t device_type);
#endif
    
#ifdef SPI3_AVAILABLE
    void SPI3_open (uint8_t device_type);
#endif

    
/**
 * This function closes the SPI module.
 *
 * @param void
 *
 * @return void
 * 
 */
void SPI1_close (void);

#ifdef SPI2_AVAILABLE
    void SPI2_close (void);
#endif
    
#ifdef SPI3_AVAILABLE
    void SPI3_close (void);
#endif

    
/**
 * This function sets the communication baudrate.
 * It should be called before SPI_open function.
 *
 * @param baudrate Baudrate [SPI_CLK_OSC_64, SPI_CLK_OSC_16, SPI_CLK_OSC_4, SPI_TIMER_OUT_CLK]
 *                           SPI_CLK_xxxx for KHz use cases
 * @return void
 *
 * @warning This function should be called before SPI_open and for Master device Only. 
 * 
 */
void SPI1_baudrate (uint8_t baudrate);

#ifdef SPI2_AVAILABLE
    void SPI2_baudrate (uint8_t baudrate);
#endif
    
#ifdef SPI3_AVAILABLE
    void SPI3_baudrate (uint8_t baudrate);
#endif


/**
 * This function sets the communication mode.
 * It should be called before SPI_open function.
 *
 * @param communication_mode Operational mode [SPI_MODE_0, SPI_MODE_1, SPI_MODE_2, SPI_MODE_3]
 *
 * @return void
 *
 * @warning This function should be called before SPI_open for both MAster and Slave devices. 
 * 
 */
void SPI1_mode (uint8_t communication_mode);

#ifdef SPI2_AVAILABLE
    void SPI2_mode (uint8_t communication_mode);
#endif

#ifdef SPI3_AVAILABLE
    void SPI3_mode (uint8_t communication_mode);
#endif

    
/**
 * This function writes a single byte on the bus.
 *
 * @param byte_to_send Byte to send [0..255]
 *
 * @return error Communication error 
 *               0: The byte has been properly written
 *               1: Communication error
 * 
 */
int8_t SPI1_write_byte (uint8_t byte_to_send);

#ifdef SPI2_AVAILABLE
    int8_t SPI2_write_byte (uint8_t byte_to_send);
#endif

#ifdef SPI3_AVAILABLE
    int8_t SPI3_write_byte (uint8_t byte_to_send);
#endif

     
/**
 * This function reads a single byte from the bus
 *
 * @param void
 *
 * @return byte Value read from the input buffer. 
 * 
 * @warning The function is a blocking one. It waits for 8 bits to be sent out.
 * 
 */
uint8_t SPI1_read_byte (void);

#ifdef SPI2_AVAILABLE
    uint8_t SPI2_read_byte (void);
#endif

#ifdef SPI3_AVAILABLE
    uint8_t SPI3_read_byte (void);
#endif
    
    
/**
 * This function reads a single byte while it is writing one on the bus
 *
 * @param write_byte Byte to be written on the bus, while reading the byte coming out
 *
 * @return byte Value read from the input buffer. 
 * 
 * @warning The function is a blocking one. It waits for 8 bits to be sent out.
 * 
 */
uint8_t SPI1_write_read_byte (uint8_t write_byte);

#ifdef SPI2_AVAILABLE
    uint8_t SPI2_write_read_byte (uint8_t write_byte);
#endif

#ifdef SPI3_AVAILABLE
    uint8_t SPI3_write_read_byte (uint8_t write_byte);
#endif    


/**
 * This macro checks if the data is ready. It can be used for TX mode only.
 *
 * @param void
 *
 * @return Bit BF 0: Receive buffer is not full
 *                1: Receive buffer is full
 */
#ifdef COMPILER_XC8    
    #define SPI1_check_data_ready() (SSPSTATbits.BF)
    #ifdef SPI2_AVAILABLE
        #define SPI2_check_data_ready() (SSP2STATbits.BF)
    #endif
#endif

#ifdef COMPILER_XC16  
    #define SPI1_check_data_ready() (SPI1STATLbits.SPIRBF)
    #ifdef SPI2_AVAILABLE
        #define SPI2_check_data_ready() (SPI2STATLbits.SPIRBF)
    #endif
    #ifdef SPI3_AVAILABLE
        #define SPI3_check_data_ready() (SPI3STATLbits.SPIRBF)
    #endif
#endif    
    
#ifdef COMPILER_XC32  
    #define SPI1_check_data_ready() (SPI1STATbits.SPIRBF)
    #ifdef SPI2_AVAILABLE
        #define SPI2_check_data_ready() (SPI2STATbits.SPIRBF)
    #endif
#endif

    
/**
 * This macro checks if any communication .
 *
 * @param void
 *
 * @return Bit WCOL 0: No communication collision. 
 *                  1: Communication collision dedected.
 */
#ifdef COMPILER_XC8 
    #define SPI1_check_bus_collision() (SSPCON1bits.WCOL)
    #ifdef SPI2_AVAILABLE
        #define SPI2_check_bus_collision() (SSP2CON1bits.WCOL)
    #endif
#endif

#ifdef COMPILER_XC16  
    //Not available
#endif    
    
#ifdef COMPILER_XC32  
    //Not available
#endif
    
    
/**
 * This macro checks if any buffer overflow occurred.
 *
 * @param void
 *
 * @return Bit WCOL 0: No overflow. 
 *                  1: Buffer Overflow dedected.
 */
#ifdef COMPILER_XC8 
    #define SPI1_check_buffer_overflow() (SSPCON1bits.SSPOV)
    #ifdef SPI2_AVAILABLE
        #define SPI2_check_buffer_overflow() (SSP2CON1bits.SSPOV)
    #endif
#endif

#ifdef COMPILER_XC16  
    #define SPI1_check_buffer_overflow() (SPI1STATLbits.SPIROV)
    #ifdef SPI2_AVAILABLE
        #define SPI2_check_buffer_overflow() (SPI2STATLbits.SPIROV)
    #endif
    #ifdef SPI3_AVAILABLE
        #define SPI3_check_buffer_overflow() (SPI3STATLbits.SPIROV)
    #endif
#endif    
    
    
#ifdef COMPILER_XC32  
    #define SPI1_check_buffer_overflow() (SPI1STATbits.SPIROV)
    #ifdef SPI2_AVAILABLE
        #define SPI2_check_buffer_overflow() (SPI2STATbits.SPIROV)
    #endif
    #ifdef SPI2_AVAILABLE
        #define SPI2_check_buffer_overflow() (SPI2STATbits.SPIROV)
    #endif
#endif

    
/**
 * This macro checks TX transition is over. It check the interrupt flag as reference.
 *
 * @param void
 *
 * @return Bit Interrupt Flag   0: Communication in progress
 *                              1: Communication completed
 */
#ifdef COMPILER_XC8 
    #define SPI1_is_TX_over() (PIR1bits.SSPIF)
    #ifdef SPI2_AVAILABLE
        #define SPI2_is_TX_over() (PIR3bits.SSP2IF)
    #endif
#endif

    
#ifdef COMPILER_XC16      
    #define SPI1_is_TX_over() (SPI1_BUSY)
    #ifdef SPI2_AVAILABLE
        #define SPI2_is_TX_over() (SPI2_BUSY)
    #endif
    #ifdef SPI3_AVAILABLE
        #define SPI3_is_TX_over() (SPI3_BUSY)
    #endif
#endif    
    
    
#ifdef COMPILER_XC32      
    #define SPI1_is_TX_over() (SPI1STATbits.SPIBUSY)
    #ifdef SPI2_AVAILABLE
        #define SPI2_is_TX_over() (SPI2STATbits.SPIBUSY)
    #endif
#endif

#endif
