/*******************************************************************************
 
Author : Mauro Laurenti
Version : 1.2
 
Created on Date : 25/11/2015
Last update     : 15/01/2023

CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 
*******************************************************************************/

#ifndef LTLIB_UART_MODULE_H
#define LTLIB_UART_MODULE_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif

//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef UART_LIBRARY_SUPPORTED
    #error (LTlib) The MCU you have selected is not supported by module_UART library.
#endif

//****************************************************
//                 BAUDRATE DEFINITION 
//****************************************************

#ifdef COMPILER_XC8
    #if (UART_CLOCK == 64000000)
        #define UART_BAUDRATE_9600 0    // Not supported
        #define BAUDRATE_19200 207
        #define UART_BAUDRATE_38400 103
        #define BAUDRATE_57600 68
        #define BAUDRATE_115200 34
    #endif

    #if (UART_CLOCK == 48000000)
        #define UART_BAUDRATE_9600 0    // Not supported
        #define UART_BAUDRATE_19200 155
        #define UART_BAUDRATE_38400 77
        #define UART_BAUDRATE_57600 51
        #define UART_BAUDRATE_115200 25
    #endif


    #if (UART_CLOCK == 40000000)
        #define UART_BAUDRATE_9600 64
        #define UART_BAUDRATE_19200 31
        #define UART_BAUDRATE_38400 15
        #define UART_BAUDRATE_57600 10
        #define UART_BAUDRATE_115200 4
    #endif

    #if (UART_CLOCK == 20000000)
        #define UART_BAUDRATE_9600 129
        #define UART_BAUDRATE_19200 64
        #define UART_BAUDRATE_38400 31
        #define UART_BAUDRATE_57600 21
        #define UART_BAUDRATE_115200 10
    #endif

    #if (UART_CLOCK == 16000000)
        #define UART_BAUDRATE_9600 103
        #define UART_BAUDRATE_19200 51
        #define UART_BAUDRATE_38400 25
        #define UART_BAUDRATE_57600 16
        #define UART_BAUDRATE_115200 8
    #endif
#endif

#ifdef COMPILER_XC16

    #if (UART_CLOCK == 8000000)
        #define UART_BAUDRATE_9600 51
        #define UART_BAUDRATE_19200 25
        #define UART_BAUDRATE_38400 12
        #define UART_BAUDRATE_57600 8
        #define UART_BAUDRATE_115200 3
        #define UART_BAUDRATE_250000 1
        #define UART_BAUDRATE_500000 0  //Not supported
    #endif

    #if (UART_CLOCK == 16000000)
        #define UART_BAUDRATE_9600 103
        #define UART_BAUDRATE_19200 51
        #define UART_BAUDRATE_38400 25
        #define UART_BAUDRATE_57600 16
        #define UART_BAUDRATE_115200 8
        #define UART_BAUDRATE_250000 3
        #define UART_BAUDRATE_500000 1  
    #endif

    #if (UART_CLOCK == 24000000)
        #define UART_BAUDRATE_9600 155
        #define UART_BAUDRATE_19200 77
        #define UART_BAUDRATE_38400 38
        #define UART_BAUDRATE_57600 25
        #define UART_BAUDRATE_115200 12
        #define UART_BAUDRATE_250000 5
        #define UART_BAUDRATE_500000 2  
    #endif

    #if (UART_CLOCK == 32000000)
        #define UART_BAUDRATE_9600 207
        #define UART_BAUDRATE_19200 103
        #define UART_BAUDRATE_38400 51
        #define UART_BAUDRATE_57600 34
        #define UART_BAUDRATE_115200 17
        #define UART_BAUDRATE_250000 7
        #define UART_BAUDRATE_500000 3  
    #endif

#endif

#ifdef COMPILER_XC32

    #if (UART_CLOCK == 40000000)
        #define UART_BAUDRATE_9600 259
        #define UART_BAUDRATE_19200 129
        #define UART_BAUDRATE_38400 64
        #define UART_BAUDRATE_57600 44
        #define UART_BAUDRATE_115200 21
        #define UART_BAUDRATE_250000 9
        #define UART_BAUDRATE_500000 4
    #endif

    #if (UART_CLOCK == 32000000)
        #define UART_BAUDRATE_9600 207
        #define UART_BAUDRATE_19200 103
        #define UART_BAUDRATE_38400 51
        #define UART_BAUDRATE_57600 34
        #define UART_BAUDRATE_115200 17
        #define UART_BAUDRATE_250000 7
        #define UART_BAUDRATE_500000 3  
    #endif

    #if (UART_CLOCK == 24000000)
        #define UART_BAUDRATE_9600 155
        #define UART_BAUDRATE_19200 77
        #define UART_BAUDRATE_38400 38
        #define UART_BAUDRATE_57600 25
        #define UART_BAUDRATE_115200 12
        #define UART_BAUDRATE_250000 5
        #define UART_BAUDRATE_500000 2  
    #endif

    #if (UART_CLOCK == 20000000)
        #define UART_BAUDRATE_9600 129
        #define UART_BAUDRATE_19200 64
        #define UART_BAUDRATE_38400 32
        #define UART_BAUDRATE_57600 21
        #define UART_BAUDRATE_115200 10
        #define UART_BAUDRATE_250000 4
        #define UART_BAUDRATE_500000 0  // Not supported
    #endif

    #if (UART_CLOCK == 16000000)
        #define UART_BAUDRATE_9600 103
        #define UART_BAUDRATE_19200 51
        #define UART_BAUDRATE_38400 25
        #define UART_BAUDRATE_57600 16
        #define UART_BAUDRATE_115200 8
        #define UART_BAUDRATE_250000 3
        #define UART_BAUDRATE_500000 1  
    #endif

    #if (UART_CLOCK == 8000000)
        #define UART_BAUDRATE_9600 51
        #define UART_BAUDRATE_19200 25
        #define UART_BAUDRATE_38400 12
        #define UART_BAUDRATE_57600 8
        #define UART_BAUDRATE_115200 3
        #define UART_BAUDRATE_250000 1
        #define UART_BAUDRATE_500000 0  //Not supported
    #endif

#endif

//****************************************************
//                   UART 1
//****************************************************

#ifdef UART1_AVAILABLE
    /**
     * This macro is used to check if the USART has some data in the RX buffer.
     *
     * @param none

     * @return status 1: The USART has some data in the RX buffer.
     *                0: The RX buffer is empty. 
     *
     */

    #ifdef COMPILER_XC8
        #define UART1_data_ready( ) (PIR1bits.RCIF)
    #endif

    #ifdef COMPILER_XC16
        #define UART1_data_ready( ) (U1STAbits.URXDA)
    #endif

    #ifdef COMPILER_XC32
        #define UART1_data_ready( ) (U1STAbits.URXDA)
    #endif


    /**
     * This macro close the USART disabling RX and TX buffer.
     *
     * @param none

     * @return none
     *
     */
    #ifdef COMPILER_XC8
        #define UART1_close( ) (RCSTA&=0b01001111,TXSTAbits.TXEN=0)
    #endif

    #ifdef COMPILER_XC16
        #define UART1_close( ) (U1MODEbits.UARTEN = 0,U1STAbits.UTXEN = 0)
    #endif

    #ifdef COMPILER_XC32
        #define UART1_close( ) (U1STA = 0x00000000, U1MODE =0x00000000)
    #endif


    /**
     * This macro is used to check if the USART is busy sending other data out.
     *
     * @param none

     * @return status 1: The USART module is busy
     *                0: The USART module can send other data out.
     *
     * @note  You are supposed to check the busy flag before sending data out.
     *
     */
    #ifdef COMPILER_XC8
        #define UART1_TX_busy( ) (!TXSTAbits.TRMT)
    #endif 

    #ifdef COMPILER_XC16
        #define UART1_TX_busy( ) (!U1STAbits.TRMT)
    #endif 

    #ifdef COMPILER_XC32
        #define UART1_TX_busy( ) (!U1STAbits.TRMT)
    #endif 


    /**
     * This function Opens the UART module enabling TX and RX channels. 
     *
     * @param data Byte to write [min: UART_BAUDRATE_9600, max: UART_BAUDRATE_115200]
     *
     *
     * @return void
     *
     * @Note
     * You must use the constants UART_BAUDRATE_9600, UART_BAUDRATE_19200,
     * UART_BAUDRATE_57600, UART_BAUDRATE_115200 to set the baudrate
     *
     */
    void UART1_open (uint16_t baud_rate);


    /**
     * This function writes a byte to the UART module. 
     *
     * @param data Byte to write [min: 0, max: 255]
     *
     * @return void
     *
     * @warning this function is a blocking one since it waits for the byte to be sent out.
     *
     */
    void UART1_write_byte (uint8_t data);


    /**
     * This function reads a byte from the RX buffer
     *
     * @param none
     *
     * @return byte Value out of the RX buffer. 
     *
     */
    uint8_t UART1_read_byte(void);


    /**
     * This function writes a constant message to the UART.
     *
     * @param *data Constant string pointer
     *
     * @return none
     * 
     * @note A constant message is a string within "" like this "Hello World".
     *
     */
    void UART1_write_message(const char * data);


    /**
     * This function writes a string to the UART.
     *
     * @param *data Pointer to the array of char (string)
     *
     * @return none
     * 
     * @note A string is an array of chars.
     *
     */
    void UART1_write_string (uint8_t *data);

#endif

    
//****************************************************
//                   UART 2
//****************************************************

#ifdef UART2_AVAILABLE

     /**
     * This macro is used to check if the USART has some data in the RX buffer.
     *
     * @param none

     * @return status 1: The USART has some data in the RX buffer.
     *                0: The RX buffer is empty. 
     *
     */
    #ifdef COMPILER_XC8
            #define UART2_data_ready( ) (PIR3bits.RC2IF)
    #endif

    #ifdef COMPILER_XC16
        #define UART2_data_ready( ) (U2STAbits.URXDA)
    #endif

    #ifdef COMPILER_XC32
        #define UART2_data_ready( ) (U2STAbits.URXDA)
    #endif
    
    
     /**
     * This macro close the USART disabling RX and TX buffer.
     *
     * @param none

     * @return none
     *
     */
    #ifdef COMPILER_XC8
        #define UART2_close( ) (RCSTA2&=0b01001111,TXSTA2bits.TXEN=0)
    #endif

    #ifdef COMPILER_XC16
        #define UART2_close( ) (U2MODEbits.UARTEN = 0,U2STAbits.UTXEN = 0)
    #endif

    #ifdef COMPILER_XC32
        #define UART2_close( ) (U2STA = 0x00000000, U2MODE =0x00000000)
    #endif
    
     /**
     * This macro is used to check if the USART is busy sending other data out.
     *
     * @param none

     * @return status 1: The USART module is busy
     *                0: The USART module can send other data out.
     *
     * @note  You are supposed to check the busy flag before sending data out.
     *
     */
    #ifdef COMPILER_XC8
        #define UART2_is_busy( ) (!TXSTA2bits.TRMT)
    #endif 

    #ifdef COMPILER_XC16
        #define UART2_TX_busy( ) (!U2STAbits.TRMT)
    #endif 

    #ifdef COMPILER_XC32
        #define UART2_TX_busy( ) (!U2STAbits.TRMT)
    #endif 
   

    /**
     * This function Opens the UART module enabling TX and RX channels. 
     *
     * @param data Byte to write [min: UART_BAUDRATE_9600, max: UART_BAUDRATE_115200]
     *
     *
     * @return void
     *
     * @Note
     * You must use the constants UART_BAUDRATE_9600, UART_BAUDRATE_19200,
     * UART_BAUDRATE_57600, UART_BAUDRATE_115200 to set the baudrate
     *
     */
    void UART2_open (uint16_t baud_rate);


    /**
     * This function writes a byte to the UART module. 
     *
     * @param data Byte to write [min: 0, max: 255]
     *
     * @return void
     *
     * @warning this function is a blocking one since it waits for the byte to be sent out.
     *
     */
    void UART2_write_byte (uint8_t data);


    /**
     * This function reads a byte from the RX buffer
     *
     * @param none
     *
     * @return byte Value out of the RX buffer. 
     *
     */
    uint8_t UART2_read_byte(void);


    /**
     * This function writes a constant message to the UART.
     *
     * @param *data Constant string pointer
     *
     * @return none
     * 
     * @note A constant message is a string within "" like this "Hello World".
     *
     */
    void UART2_write_message(const char *data);


    /**
     * This function writes a string to the UART.
     *
     * @param *data Pointer to the array of char (string)
     *
     * @return none
     * 
     * @note A string is an array of chars.
     *
     */
    void UART2_write_string (uint8_t *data);

    
#endif
    
        
//****************************************************
//                   UART 3
//****************************************************

#ifdef UART3_AVAILABLE

     /**
     * This macro is used to check if the USART has some data in the RX buffer.
     *
     * @param none

     * @return status 1: The USART has some data in the RX buffer.
     *                0: The RX buffer is empty. 
     *
     */
    #ifdef COMPILER_XC16
        #define UART3_data_ready( ) (U3STAbits.URXDA)
    #endif
    
     /**
     * This macro close the USART disabling RX and TX buffer.
     *
     * @param none

     * @return none
     *
     */
    #ifdef COMPILER_XC16
        #define UART3_close( ) (U3MODEbits.UARTEN = 0,U3STAbits.UTXEN = 0)
    #endif

    
     /**
     * This macro is used to check if the USART is busy sending other data out.
     *
     * @param none

     * @return status 1: The USART module is busy
     *                0: The USART module can send other data out.
     *
     * @note  You are supposed to check the busy flag before sending data out.
     *
     */
    #ifdef COMPILER_XC16
        #define UART3_TX_busy( ) (!U3STAbits.TRMT)
    #endif 
   

    /**
     * This function Opens the UART module enabling TX and RX channels. 
     *
     * @param data Byte to write [min: UART_BAUDRATE_9600, max: UART_BAUDRATE_115200]
     *
     *
     * @return void
     *
     * @Note
     * You must use the constants UART_BAUDRATE_9600, UART_BAUDRATE_19200,
     * UART_BAUDRATE_57600, UART_BAUDRATE_115200 to set the baudrate
     *
     */
    void UART3_open (uint16_t baud_rate);


    /**
     * This function writes a byte to the UART module. 
     *
     * @param data Byte to write [min: 0, max: 255]
     *
     * @return void
     *
     * @warning this function is a blocking one since it waits for the byte to be sent out.
     *
     */
    void UART3_write_byte (uint8_t data);


    /**
     * This function reads a byte from the RX buffer
     *
     * @param none
     *
     * @return byte Value out of the RX buffer. 
     *
     */
    uint8_t UART3_read_byte(void);


    /**
     * This function writes a constant message to the UART.
     *
     * @param *data Constant string pointer
     *
     * @return none
     * 
     * @note A constant message is a string within "" like this "Hello World".
     *
     */
    void UART3_write_message(const char *data);


    /**
     * This function writes a string to the UART.
     *
     * @param *data Pointer to the array of char (string)
     *
     * @return none
     * 
     * @note A string is an array of chars.
     *
     */
    void UART3_write_string (uint8_t *data);

    
#endif
    
    
//****************************************************
//                   UART 4
//****************************************************

#ifdef UART4_AVAILABLE

     /**
     * This macro is used to check if the USART has some data in the RX buffer.
     *
     * @param none

     * @return status 1: The USART has some data in the RX buffer.
     *                0: The RX buffer is empty. 
     *
     */
    #ifdef COMPILER_XC16
        #define UART4_data_ready( ) (U4STAbits.URXDA)
    #endif

    
     /**
     * This macro close the USART disabling RX and TX buffer.
     *
     * @param none

     * @return none
     *
     */
    #ifdef COMPILER_XC16
        #define UART4_close( ) (U4MODEbits.UARTEN = 0,U4STAbits.UTXEN = 0)
    #endif

    
     /**
     * This macro is used to check if the USART is busy sending other data out.
     *
     * @param none

     * @return status 1: The USART module is busy
     *                0: The USART module can send other data out.
     *
     * @note  You are supposed to check the busy flag before sending data out.
     *
     */
    #ifdef COMPILER_XC16
        #define UART4_TX_busy( ) (!U4STAbits.TRMT)
    #endif 
   

    /**
     * This function Opens the UART module enabling TX and RX channels. 
     *
     * @param data Byte to write [min: UART_BAUDRATE_9600, max: UART_BAUDRATE_115200]
     *
     *
     * @return void
     *
     * @Note
     * You must use the constants UART_BAUDRATE_9600, UART_BAUDRATE_19200,
     * UART_BAUDRATE_57600, UART_BAUDRATE_115200 to set the baudrate
     *
     */
    void UART4_open (uint16_t baud_rate);


    /**
     * This function writes a byte to the UART module. 
     *
     * @param data Byte to write [min: 0, max: 255]
     *
     * @return void
     *
     * @warning this function is a blocking one since it waits for the byte to be sent out.
     *
     */
    void UART4_write_byte (uint8_t data);


    /**
     * This function reads a byte from the RX buffer
     *
     * @param none
     *
     * @return byte Value out of the RX buffer. 
     *
     */
    uint8_t UART4_read_byte(void);


    /**
     * This function writes a constant message to the UART.
     *
     * @param *data Constant string pointer
     *
     * @return none
     * 
     * @note A constant message is a string within "" like this "Hello World".
     *
     */
    void UART4_write_message(const char *data);


    /**
     * This function writes a string to the UART.
     *
     * @param *data Pointer to the array of char (string)
     *
     * @return none
     * 
     * @note A string is an array of chars.
     *
     */
    void UART4_write_string (uint8_t *data);
    
#endif    
    
#endif

