/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0

Created on Date : 25/11/2023
Last update     : 25/11/2023

CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.


********************************************************
PURPOSES
********************************************************

 This library enable easy of use for the nRF24L01 radio transceiver.

*******************************************************************************/

#ifndef LTLIB_NRF24L01_H
#define LTLIB_NRF24L01_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif

//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef LTLIB_SPI_MODULE_H
    #error The nRF24L01 Library requires the "module_SPI" library to be included.
#endif

#ifndef LTLIB_DELAY_H
    #error The nRF24L01 Library requires the "LTlib_delay" library to be included.
#endif


//*****************************************
//   SPI Module Switch
//*****************************************
#if defined (NRF24L01_SPI_MODULE_1) || defined (NRF24L01_SPI_MODULE_2)

#else
    #define NRF24L01_SPI_MODULE_1
#endif 

#ifndef NRF24L01_SPI_BAUDRATE
    #define NRF24L01_SPI_BAUDRATE SPI_CLK_100_KHZ
#endif 

#ifdef NRF24L01_SPI_MODULE_1
    #define NRF24L01_SPI_open SPI1_open
    #define NRF24L01_SPI_baudrate SPI1_baudrate
    #define NRF24L01_SPI_mode SPI1_mode 
    #define NRF24L01_SPI_write_byte SPI1_write_byte
    #define NRF24L01_SPI_read_byte SPI1_read_byte
    #define NRF24L01_SPI_write_read_byte SPI1_write_read_byte
#endif

#ifdef NRF24L01_SPI_MODULE_2 
    #define NRF24L01_SPI_open SPI2_open
    #define NRF24L01_SPI_baudrate SPI2_baudrate
    #define NRF24L01_SPI_mode SPI2_mode
    #define NRF24L01_SPI_write_byte SPI2_write_byte
    #define NRF24L01_SPI_read_byte SPI2_read_byte
    #define NRF24L01_SPI_write_read_byte SPI2_write_read_byte
#endif

//**************************************************
// Device pin Settings CS, LAT
//**************************************************

#ifdef COMPILER_XC8
    #ifdef _PIC18
        //CS pin
        #define NRF24L01_CS_TRIS   TRISAbits.TRISA4
        #define NRF24L01_CS_PORT   LATAbits.LATA4

        //Enable pin
        #define NRF24L01_CE_TRIS   TRISAbits.TRISA10
        #define NRF24L01_CE_PORT   LATAbits.LATA10

        //IRQ pin
        #define NRF24L01_IRQ_TRIS  TRISCbits.TRISC6
        #define NRF24L01_IRQ_PORT  PORTCbits.RC6
    #endif

    #ifndef _PIC18
        //CS pin
        #define NRF24L01_CS_TRIS   TRISAbits.TRISA4
        #define NRF24L01_CS_PORT   PORTAbits.RA4

        //Enable pin
        #define NRF24L01_CE_TRIS   TRISAbits.TRISA10
        #define NRF24L01_CE_PORT   PORTAbits.RA10

        //IRQ pin
        #define NRF24L01_IRQ_TRIS  TRISCbits.TRISC6
        #define NRF24L01_IRQ_PORT  PORTCbits.RC6
    #endif
#endif

#ifdef COMPILER_XC16
        //CS pin
        #define NRF24L01_CS_TRIS   TRISAbits.TRISA4
        #define NRF24L01_CS_PORT   LATAbits.LATA4

        //Enable pin
        #define NRF24L01_CE_TRIS   TRISAbits.TRISA10
        #define NRF24L01_CE_PORT   LATAbits.LATA10

        //IRQ pin
        #define NRF24L01_IRQ_TRIS  TRISCbits.TRISC6
        #define NRF24L01_IRQ_PORT  PORTCbits.RC6
#endif


#ifdef COMPILER_XC32
        //CS pin
        #define NRF24L01_CS_TRIS   TRISAbits.TRISA4
        #define NRF24L01_CS_PORT   LATAbits.LATA4

        //Enable pin
        #define NRF24L01_CE_TRIS   TRISAbits.TRISA10
        #define NRF24L01_CE_PORT   LATAbits.LATA10

        //IRQ pin
        #define NRF24L01_IRQ_TRIS  TRISCbits.TRISC6
        #define NRF24L01_IRQ_PORT  PORTCbits.RC6
#endif


//*****************************************
//   Device registers
//*****************************************
#define NRF24L01_REG_CONFIG         0x00
#define NRF24L01_REG_EN_AA          0x01
#define NRF24L01_REG_EN_RXADDR      0x02
#define NRF24L01_REG_SETUP_AW       0x03
#define NRF24L01_REG_SETUP_RETR     0x04
#define NRF24L01_REG_RF_CH          0x05
#define NRF24L01_REG_RF_SETUP       0x06
#define NRF24L01_REG_STATUS         0x07
#define NRF24L01_REG_OBSERVE_TX     0x08
#define NRF24L01_REG_CD             0x09    //nRF24L01 register name
#define NRF24L01_REG_RPD            0x09    //nRF24L01+ register name
#define NRF24L01_REG_RX_ADDR_P0     0x0A
#define NRF24L01_REG_RX_ADDR_P1     0x0B
#define NRF24L01_REG_RX_ADDR_P2     0x0C
#define NRF24L01_REG_RX_ADDR_P3     0x0D
#define NRF24L01_REG_RX_ADDR_P4     0x0E
#define NRF24L01_REG_RX_ADDR_P5     0x0F
#define NRF24L01_REG_TX_ADDR        0x10
#define NRF24L01_REG_RX_PW_P0       0x11
#define NRF24L01_REG_RX_PW_P1       0x12
#define NRF24L01_REG_RX_PW_P2       0x13
#define NRF24L01_REG_RX_PW_P3       0x14
#define NRF24L01_REG_RX_PW_P4       0x15
#define NRF24L01_REG_RX_PW_P5       0x16
#define NRF24L01_REG_FIFO_STATUS    0x17
#define NRF24L01_REG_DYNPD          0x1C
#define NRF24L01_REG_FEATURE        0x1D

//*****************************************
//   Device commands 
//*****************************************
#define NRF24L01_COM_R_REGISTER          0x00
#define NRF24L01_COM_W_REGISTER          0x20
#define NRF24L01_COM_R_RX_PAYLOAD        0x61
#define NRF24L01_COM_W_TX_PAYLOAD        0xA0
#define NRF24L01_COM_FLUSH_TX            0xE1
#define NRF24L01_COM_FLUSH_RX            0xE2
#define NRF24L01_COM_REUSE_TX_PL         0xE3
#define NRF24L01_COM_ACTIVATE            0x50
#define NRF24L01_COM_R_RX_PL_WID         0x60
#define NRF24L01_COM_W_ACK_PAYLOAD       0xA8
#define NRF24L01_COM_W_TX_PAYLOAD_NOACK  0xB0
#define NRF24L01_COM_NOP                 0xFF

//**************************************************
// Constants
//**************************************************

//When shutdown is ON the device is powered down.
#define NRF24L01_TURN_ON  0x00
#define NRF24L01_TURN_OFF 0x10

#define NRF24L01_CS_DISABLED 0x01
#define NRF24L01_CS_ENABLED 0x00

#define NRF24L01_CE_DISABLED 0x00
#define NRF24L01_CE_ENABLED 0x01

#define NRF24L01_IRQ_ACTIVATED 0x00
#define NRF24L01_IRQ_DEACTIVATED 0x01

#define NRF24L01_FLUSH_TX_BUFFER 0x01
#define NRF24L01_FLUSH_RX_BUFFER 0x02


#define NRF24L01_ADDRESS_RX_P0 0x00
#define NRF24L01_ADDRESS_RX_P1 0x01
#define NRF24L01_ADDRESS_RX_P2 0x02
#define NRF24L01_ADDRESS_RX_P3 0x03
#define NRF24L01_ADDRESS_RX_P4 0x04
#define NRF24L01_ADDRESS_RX_P5 0x05
#define NRF24L01_ADDRESS_TX    0xF0

#define NRF24L01_ADDRESS_3_BYTES 0x03
#define NRF24L01_ADDRESS_4_BYTES 0x04
#define NRF24L01_ADDRESS_5_BYTES 0x05
#define NRF24L01_ADDRESS_MAX_SIZE 0x05

#define NRF24L01_CRC_ENABLED 0x08
#define NRF24L01_CRC_DISABLED 0xF7
#define NRF24L01_CRC_SIZE_1_BYTE 0xFB
#define NRF24L01_CRC_SIZE_2_BYTE 0x04

#define NRF24L01_PIPE_ENABLED  0xF1
#define NRF24L01_PIPE_DISABLED 0xF0
#define NRF24L01_PIPE_ALL      0x3F
#define NRF24L01_PIPE_0_EN     0x01
#define NRF24L01_PIPE_1_EN     0x02
#define NRF24L01_PIPE_2_EN     0x04
#define NRF24L01_PIPE_3_EN     0x08
#define NRF24L01_PIPE_4_EN     0x10
#define NRF24L01_PIPE_5_EN     0x20

#define NRF24L01_RF_POWER_MASK      0xF9
#define NRF24L01_RF_POWER_0dBm      0x06
#define NRF24L01_RF_POWER_m6dBm     0x04
#define NRF24L01_RF_POWER_m12dBm    0x02
#define NRF24L01_RF_POWER_m18dBm    0x00

#define NRF24L01_BIT_RATE_MASK      0xF7
#define NRF24L01_BIT_RATE_250KHZ    0x20    //Supported only by the nRF24L01+ not the nRF24L01
#define NRF24L01_BIT_RATE_1MHZ      0x00
#define NRF24L01_BIT_RATE_2MHZ      0x08

#define NRF24L01_STATUS_RX_P_NO_MASK    0x0E
#define NRF24L01_STATUS_RX_DR_MASK      0x40
#define NRF24L01_PIPE_0_READY 0x00
#define NRF24L01_PIPE_1_READY 0x01
#define NRF24L01_PIPE_2_READY 0x02
#define NRF24L01_PIPE_3_READY 0x03
#define NRF24L01_PIPE_4_READY 0x04
#define NRF24L01_PIPE_5_READY 0x05
#define NRF24L01_PIPES_EMPTY  0x07

#define NRF24L01_CHANGE_BIT_0    0xFE
#define NRF24L01_CHANGE_BIT_1    0xFD
#define NRF24L01_CHANGE_BIT_2    0xFB
#define NRF24L01_CHANGE_BIT_3    0xF7
#define NRF24L01_CHANGE_BIT_4    0xEF
#define NRF24L01_CHANGE_BIT_5    0xDF
#define NRF24L01_CHANGE_BIT_6    0xBF
#define NRF24L01_CHANGE_BIT_7    0x7F

#define NRF24L01_CONFIG_BIT_PRIM_RX        NRF24L01_CHANGE_BIT_0
#define NRF24L01_CONFIG_BIT_PWR_UP         NRF24L01_CHANGE_BIT_1
#define NRF24L01_CONFIG_BIT_CRCO           NRF24L01_CHANGE_BIT_2
#define NRF24L01_CONFIG_BIT_EN_CRC         NRF24L01_CHANGE_BIT_3
#define NRF24L01_CONFIG_BIT_MASK_MAX_RT    NRF24L01_CHANGE_BIT_4
#define NRF24L01_CONFIG_BIT_MASK_TX_DS     NRF24L01_CHANGE_BIT_5
#define NRF24L01_CONFIG_BIT_MASK_RX_DR     NRF24L01_CHANGE_BIT_6

#define NRF24L01_P0_SIZE    NRF24L01_REG_RX_PW_P0
#define NRF24L01_P1_SIZE    NRF24L01_REG_RX_PW_P1       
#define NRF24L01_P2_SIZE    NRF24L01_REG_RX_PW_P2       
#define NRF24L01_P3_SIZE    NRF24L01_REG_RX_PW_P3       
#define NRF24L01_P4_SIZE    NRF24L01_REG_RX_PW_P4       
#define NRF24L01_P5_SIZE    NRF24L01_REG_RX_PW_P5       

#define NRF24L01_STATUS_BIT_TX_DS  NRF24L01_CHANGE_BIT_5

#define NRF24L01_CARRIER_ON     0x01
#define NRF24L01_CARRIER_OFF    0x00

#define NRF24L01_TX_BUSY     0x01
#define NRF24L01_TX_OVER     0x00

#define NRF24L01_CARRIER_DETECTED         0x01
#define NRF24L01_CARRIER_NOT_DETECTED     0x00

#define NRF24L01_RX_MODE_OFF    0x00
#define NRF24L01_RX_MODE_ON     0x01


//**************************************************
// Data Type
//**************************************************
typedef struct{ 
    
	uint8_t payload[32];
    uint8_t size;
    uint8_t pipe;
    uint8_t valid;

} NRF24L01_packet;


/**
 * This function initializes the NRF24L01+ and PIC signals. 
 * The SPI data lines and clock are automatically set.
 *
 * @param TX_channel Number of channel for the TX function [ Fch[MHz] = 2400 + RF_CH ].
 * @param RF_power Power at which the TX amplifier will work at [NRF24L01_RF_POWER_0dBm, 
 *                                                               NRF24L01_RF_POWER_m6dBm,
 *                                                               NRF24L01_RF_POWER_m12dBm, 
 *                                                               NRF24L01_RF_POWER_m18dBm]
 * 
 * @param bit_rate Bit rate at which the TX channel will work at [NRF24L01_BIT_RATE_1MHZ, 
 *                                                                NRF24L01_BIT_RATE_2MHZ,
 *                                                                NRF24L01_BIT_RATE_250KHZ].
 *
 * @return void
 *
 * @note By default the SPI will work at 100Kbits/s.
 */
void NRF24L01_initialize (uint8_t TX_channel, uint8_t RF_power, uint8_t bit_rate);


/**
 * This function send a command to the NRF24L01+. 
 * The entire packet must be sent out even if the payload is not used.
 *
 * @param command Command to be sent [ Use the Device commands  list ].
 *
 * @return void
 *
 */
void NRF24L01_send_command (uint8_t command, NRF24L01_packet packet);


/**
 * This function set the NRF24L01+ power mode ON or OFF. 
 * The 250KHz is supported only by the NRF24L01+ not the NRF24L01.
 *
 * @param status Status for the device[ NRF24L01_TURN_ON, NRF24L01_TURN_OFF ].
 *
 * @return void
 * 
 */
void NRF24L01_set_power_mode (uint8_t status);


/**
 * This function flushes the selected NRF24L01+ buffer. 
 *
 * @param buffer Status for the device[ NRF24L01_FLUSH_TX_BUFFER, NRF24L01_FLUSH_RX_BUFFER ].
 *
 * @return void
 * 
 */
void NRF24L01_flush_buffer (uint8_t buffer);


/**
 * This function sets the RF channel at which the NRF24L01+ operates.
 * It is initially set during the initialization, but it can be changed on the fly.  
 *
 * @param channel Number of channel for the TX function [ Fch[MHz] = 2400 + RF_CH ].
 *
 * @return void
 * 
 */
uint8_t NRF24L01_set_channel (uint8_t channel);


/**
 * This function sets the packet format.
 * The format must be the same for the TX and RX devices, otherwise the data cannot be 
 * properly received. 
 *
 * @param ADDR_size Size of the device address [NRF24L01_ADDRESS_3_BYTES, NRF24L01_ADDRESS_4_BYTES, NRF24L01_ADDRESS_5_BYTES].
 * @param CRC_enable Enable or disable the CRC code [NRF24L01_CRC_ENABLED, NRF24L01_CRC_DISABLED].
 * @param CRC_size Number of CRC bytes [NRF24L01_CRC_SIZE_1_BYTE, NRF24L01_CRC_SIZE_2_BYTE].
 *
 * @return void
 * 
 * @note The CRC_size must be given even if the CRC code is not enabled. 
 * The value will be ignored.  
 * 
 */
void NRF24L01_packet_format (uint8_t ADDR_size, uint8_t CRC_enable, uint8_t CRC_size);


/**
 * This function sets the address for the selected submodule or pipe (TX, RX, Pipe).
 *
 * @param address_type Select the address of interest [NRF24L01_ADDRESS_RX_P0, NRF24L01_ADDRESS_RX_P1
 *                                                     NRF24L01_ADDRESS_RX_P2, NRF24L01_ADDRESS_RX_P3,
 *                                                     NRF24L01_ADDRESS_RX_P4, NRF24L01_ADDRESS_RX_P5,
 *                                                     NRF24L01_ADDRESS_TX].
 * @param address Value for the address. Pointer to the array that contains the bytes, 1 per location.
 *                LSByte is written first in location 0 of the array.
 * 
 * @param size Address size, it is the number of bytes that will be used [3,4,5]
 *
 * @return void
 * 
 */
void NRF24L01_set_address (uint8_t address_type, uint8_t* address, uint8_t size);


/**
 * This function enables the selected pipe. Multiple pipes can be ORed on the same 
 * function call.
 *
 * @param pipe Select a specific RX pipe [NRF24L01_PIPE_ALL, NRF24L01_PIPE_0_EN
 *                                        NRF24L01_PIPE_1_EN, NRF24L01_PIPE_2_EN
 *                                        NRF24L01_PIPE_3_EN, NRF24L01_PIPE_4_EN
 *                                        NRF24L01_PIPE_5_EN].
 *
 * @param pipe_enable enable or disable the RX pipe [NRF24L01_PIPE_ENABLED, NRF24L01_PIPE_DISABLED].
 *
 * @return void
 * 
 */
void NRF24L01_pipe_configuration (uint8_t pipe, uint8_t pipe_enable);


/**
 * This function configures the payload for the selected RX pipe.
 *
 * @param pipe Select a specific RX pipe [NRF24L01_PIPE_ALL, NRF24L01_PIPE_0_EN
 *                                        NRF24L01_PIPE_1_EN, NRF24L01_PIPE_2_EN
 *                                        NRF24L01_PIPE_3_EN, NRF24L01_PIPE_4_EN
 *                                        NRF24L01_PIPE_5_EN].
 *
 * @param payload_byte Nnmber of bytes that the pipe will receive [1...32].
 *
 * @return void
 * 
 */
uint8_t NRF24L01_set_pipe (uint8_t pipe, uint8_t payload_byte);


/**
 * This function reads a register content. You can use it only for registers that 
 * are 1 byte content. 
 *
 * @param register_address Select the register to read out [Use the Device registers list]
 *
 * @return Register content
 * 
 */
uint8_t NRF24L01_get_register (uint8_t register_address);


/**
 * This function reads a register content. This fucntion is used for the register 
 * locations that are returning more bytes, such has address, payload.
 *
 * @param register_address Select the register to read out [Use the Device registers list]
 *
 * @return Register content as char pointer. The array reflects the register content.
 * 
 */
uint8_t* NRF24L01_get_address (uint8_t register_address, uint8_t size);


/**
 * This function writes to a register a new content. You can use it only for registers that 
 * are 1 byte content. 
 *
 * @param register_address Select the register to read out [Use the Device registers list]
 * 
 * @param data Data to be written [0-255].
 *
 * @return NRF24L01+ status register
 * 
 */
uint8_t NRF24L01_set_register (uint8_t register_address, uint8_t data);


/**
 * This function sets the NRF24L01+ RF power level. 
 *
 * @param RF_power Set the power level [ NRF24L01_RF_POWER_0dBm, NRF24L01_RF_POWER_m6dBm,
 *                                      NRF24L01_RF_POWER_m12dBm, NRF24L01_RF_POWER_m18dBm].
 *
 * @return void
 * 
 */
void NRF24L01_set_RF_power (uint8_t RF_power);


/**
 * This function sets the TX bit rate. 
 * It can be changed over time but the RX module must have the same value.
 *
 * @param bit_rate Bit rate at which the TX channel will work at [NRF24L01_BIT_RATE_1MHZ, 
 *                                                                NRF24L01_BIT_RATE_2MHZ,
 *                                                                NRF24L01_BIT_RATE_250KHZ].
 *
 * @return void
 * 
 */
void NRF24L01_set_bit_rate (uint8_t bit_rate);


/**
 * This function checks if any of the RX pipes has receive any data.
 *
 * @param void
 *
 * @return Data received [NRF24L01_PIPES_EMPTY, RX Pipe line as the note below]
 * 
 * @Note: If there is data, the RX pipe that has data is returned NRF24L01_PIPE_0_READY, 
 * NRF24L01_PIPE_1_READY, NRF24L01_PIPE_2_READY, NRF24L01_PIPE_3_READY, NRF24L01_PIPE_4_READY, NRF24L01_PIPE_5_READY.
 * 
 */
uint8_t NRF24L01_is_data_received (void);


/**
 * This function checks if there is any TX operation ongoing.
 *
 * @param void
 *
 * @return TX busy status [NRF24L01_TX_OVER, NRF24L01_TX_BUSY]
 * 
 */
uint8_t NRF24L01_is_TX_busy (void);


/**
 * This function changes a single bit of any register that has a single byte content.
 *
 * @param register_address Select the register to read out [Use the Device registers list]
 * 
 * @param bit_to_change Select the bit to change [NRF24L01_CHANGE_BIT_0 ...NRF24L01_CHANGE_BIT_7]
 * 
 * @param value Select the value [0 or 1]
 *
 * @return NRF24L01+ status register
 * 
 */
uint8_t NRF24L01_update_register_bit (int8_t register_address, uint8_t bit_to_change, uint8_t value);


/**
 * This function transmits a carrier (no modulation) to the specific channel. It is used for test purposes.
 *
 * @param status Turn ON or OFF the carrier [NRF24L01_CARRIER_ON, NRF24L01_CARRIER_OFF]
 * 
 * @param carrier_CH Set the channel for the carrier [Fch[MHz] = 2400 + RF_CH ]
 * 
 * @return void
 * 
 * @note Only one carrier at the time is ON. 
 * 
 */
void NRF24L01_generate_carrier (uint8_t status, uint8_t carrier_CH );


/**
 * This function transmits a data packet to the specific channel. 
 *
 * @param packet Packet content to be sent out. 
 * 
 * @return void
 * 
 * @note You must create a packet type variable in the program that calls that function
 * 
 */
void NRF24L01_send_data (NRF24L01_packet packet);


/**
 * This function reads the available data out of the received packet. 
 *
 * @param packet Pointer to the Packet variable where the data will be stored. 
 * 
 * @return Status register. 
 * 
 * @note You must create a packet type variable in the program that calls that function
 * 
 */
uint8_t NRF24L01_read_data (NRF24L01_packet *packet);


/**
 * This function returns the NRF24L01 status register value.
 * Other functions, by default also return the status register value. 
 *
 * @param void 
 * 
 * @return Status register
 * 
 */
uint8_t NRF24L01_get_status (void);


/**
 * This function returns the NRF24L01 TX packet lost.
 * Once the TX send the data out, they may not be received. The counter keeps that count. 
 *
 * @param void 
 * 
 * @return Lost counter
 * 
 */
uint8_t NRF24L01_get_lost_packets (void);


/**
 * This function returns the NRF24L01 TX packet retransmit.
 * Once the TX send the data out, they may not be received. 
 * The TX module keep trying to send the data out. The counter keeps that count. 
 *
 * @param void 
 * 
 * @return Retry counter
 * 
 */
uint8_t NRF24L01_get_retransmitted_packets (void);


/**
 * This function returns the channel availability. 
 * Once the channel is selected, this function enable monitoring if any other carrier 
 * is detected. If a carrier is detected, the channel may be used by other transmitters.
 * It may be used by a frequency hopping scheme.   
 *
 * @param void 
 * 
 * @return Carrier detected [NRF24L01_CARRIER_DETECTED, NRF24L01_CARRIER_NOT_DETECTED].
 * 
 */
uint8_t NRF24L01_is_carrier_detected (void);


/**
 * This function sets the NRF24L01 in receiving mode.
 *
 * @param RX_mode Set the RX module ON or OFF [NRF24L01_RX_MODE_ON, NRF24L01_RX_MODE_OFF]
 * 
 * @return Status register
 * 
 */
uint8_t NRF24L01_set_RX_mode (uint8_t RX_mode);


#endif

