/*******************************************************************************

Author : Dario Di Turi
Version : 1.4
Date : 08/12/2012

Last Update: Mauro Laurenti
Last Update: 28/07/2019
 
CopyRight 2012-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Dario Di Turi (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "AD9833.h"

uint16_t  AD9833_K;
uint16_t  AD9833_register_word;
uint16_t  AD9833_tuning_freq;


//*********************************************
//             write_register
//*********************************************	
void AD9833_write_data(uint16_t data){

	//FSYNC low, load word into the device 
	FSYNC=0; 

	//MSB data
	while(AD9833_SPI_write_byte(data>>8));
    
	//LSB data
	while(AD9833_SPI_write_byte(data));
	
	//FSYNC high, load word completed 
	FSYNC=1; 
}


//*********************************************
//         compose_AD9833_control_word
//*********************************************	
uint16_t AD9833_compose_control_word(uint8_t function_type){


	// Write frequency register
	if(function_type==0){
		AD9833_register_word=AD9833_register_word | 0b0010000000000000; 
		return(AD9833_register_word);
	}
	
	// Sinusoidal waveform
	if(function_type==1){
		AD9833_register_word=AD9833_register_word & 0b1111111111000000;	
		return(AD9833_register_word);
	}
	
	// Triangular waveform
	if(function_type==2){
		AD9833_register_word=AD9833_register_word & 0b1111111111000000;
		AD9833_register_word=AD9833_register_word | 0b0000000000000010;
		return(AD9833_register_word);
	}
	
	// Square waveform
	if(function_type==3){
		AD9833_register_word=AD9833_register_word & 0b1111111111000000;	
		AD9833_register_word=AD9833_register_word | 0b0000000000101000;
		return(AD9833_register_word);
		
	}
	// FREQ0 out
	if(function_type==4){
		AD9833_register_word=AD9833_register_word & 0b1111011111111111;	
		return(AD9833_register_word);
		
	}
	// FREQ1 out
	if(function_type==5){
		AD9833_register_word=AD9833_register_word & 0b1111011111111111;	
		AD9833_register_word=AD9833_register_word | 0b0000100000000000;
		return(AD9833_register_word);
		
	}	
	// PHASE0 out
	if(function_type==6){
		AD9833_register_word=AD9833_register_word & 0b1111101111111111;	
		return(AD9833_register_word);
		
	}
	// PHASE1 out
	if(function_type==7){
		AD9833_register_word=AD9833_register_word & 0b1111101111111111;	
		AD9833_register_word=AD9833_register_word | 0b0000010000000000;
		return(AD9833_register_word);
		
	}
	// Write LSB frequency register
	if(function_type==8){
		AD9833_register_word=AD9833_register_word & 0b0000111111111111;	
		return(AD9833_register_word);
		
	}
	// Sleep mode
	if(function_type==9){
		AD9833_register_word=AD9833_register_word & 0b1111111100111111;	
		AD9833_register_word=AD9833_register_word | 0b0000010011000000;
		return(AD9833_register_word);
		
	}
	// Power ON
	if(function_type==10){
		AD9833_register_word=AD9833_register_word & 0b1111111100111111;	
		return(AD9833_register_word);
		
	}

        // This return is never executed but it removes a compiler warning.
        return (0);
}


//*********************************************
//        AD9833_initialize
//*********************************************	
void AD9833_initialize (uint8_t fmclk){

    FSYNC_DIRECTION = PIN_AS_OUTPUT;    
        
    AD9833_SPI_baudrate(SPI_CLK_OSC_64);
    AD9833_SPI_mode(SPI_MODE_2);
    AD9833_SPI_open(SPI_MASTER_DEVICE);
    
	AD9833_register_word=0x0000;
	AD9833_tuning_freq=0x0000;
	
	AD9833_K=268435456/(fmclk*100000); //2^28/(fmclk*100000)
	
	AD9833_write_data (AD9833_RESET);
	AD9833_set_frequency (0, AD9833_REG0);
	AD9833_set_frequency (0, AD9833_REG1);
	AD9833_set_phase (0,AD9833_REG0);
	AD9833_set_phase (0,AD9833_REG1);
 
 
	AD9833_set_function (AD9833_FREQUENCY0_OUT);
	AD9833_set_function (AD9833_PHASE0_OUT);
	AD9833_set_function (AD9833_SIN);

}


//*********************************************
//           AD9833_set_frequency
//*********************************************	
void AD9833_set_frequency (uint32_t freq_value, uint8_t frequency_register){
 
	uint32_t freq_reg_value;
	uint16_t freq_reg_value_l,freq_reg_value_h;
	uint16_t control_word;

	freq_reg_value = (freq_value/10) * AD9833_K; //2^28/(fmclk)
	
	freq_reg_value = freq_reg_value << 2;
	freq_reg_value_l = freq_reg_value;
	freq_reg_value_h = freq_reg_value >> 16;
 	freq_reg_value_l = freq_reg_value_l >> 2;
	
	// Save the LSB frequency register for the tuning function
	AD9833_tuning_freq=freq_reg_value_l;
 
	// Write into FREQ0 register
	if(frequency_register==0){
 
		freq_reg_value_l = freq_reg_value_l & 0x7FFF; //0b0111,1111,1111,1111
		freq_reg_value_l = freq_reg_value_l | 0x4000; //0b0100,0000,0000,0000

		freq_reg_value_h = freq_reg_value_h & 0x7FFF;
		freq_reg_value_h = freq_reg_value_h | 0x4000;
	}
	
	// Write into FREQ1 register
	if(frequency_register==1){
		
		freq_reg_value_l = freq_reg_value_l & 0xBFFF; //0b1011,1111,1111,1111
		freq_reg_value_l = freq_reg_value_l | 0x8000; //0b1000,0000,0000,0000

		freq_reg_value_h = freq_reg_value_h & 0xBFFF;
		freq_reg_value_h = freq_reg_value_h | 0x8000;
	}
 
	control_word = AD9833_compose_control_word(0);
	AD9833_write_data (control_word);
	AD9833_write_data (freq_reg_value_l);
	AD9833_write_data (freq_reg_value_h);
 
}


//*********************************************
//           AD9833_tuning_frequency
//*********************************************	
void AD9833_tuning_frequency (uint8_t tuning_step, uint8_t direction, uint8_t frequency_register ){

	uint16_t control_word;
	
	//tuning DOWN
	if(direction==0){		
		AD9833_tuning_freq=AD9833_tuning_freq-tuning_step;
	}
    
	//tuning UP	
	if(direction==1){		
		AD9833_tuning_freq=AD9833_tuning_freq+tuning_step;
	}
    
	// Write into FREQ0 register
	if(frequency_register==0){ 
		AD9833_tuning_freq = AD9833_tuning_freq & 0x7FFF; //0b0111,1111,1111,1111
		AD9833_tuning_freq = AD9833_tuning_freq | 0x4000; //0b0100,0000,0000,0000
	}
	
	// Write into FREQ1 register
	if(frequency_register==1){		
		AD9833_tuning_freq = AD9833_tuning_freq & 0xBFFF; //0b1011,1111,1111,1111
		AD9833_tuning_freq = AD9833_tuning_freq | 0x8000; //0b1000,0000,0000,0000
	}
	
	control_word = AD9833_compose_control_word(8);
	AD9833_write_data (control_word);
	AD9833_write_data (AD9833_tuning_freq);
}


//*********************************************
//           AD9833_set_phase
//*********************************************	
void AD9833_set_phase (uint32_t phase_value, uint8_t phase_register){

	uint16_t phase_reg;
	
	phase_value=(phase_value*1137)/1000;
	
	phase_reg=phase_value;
	
	// Write into PHASE0 register
	if(phase_register==0){		
		phase_reg=phase_reg & 0xFFF;  //0b0000111111111111
		phase_reg=phase_reg | 0xC000; //0b1100000000000000
	}
	
	// Write into PHASE1 register
	if(phase_register==1){		
		phase_reg=phase_reg & 0xFFF;  //0b0000111111111111
		phase_reg=phase_reg | 0xE000; //0b1110000000000000
	}	
 
	AD9833_write_data (phase_reg); 
}


//*********************************************
//            AD9833_set_function
//*********************************************	
void AD9833_set_function (uint8_t function_type){

	uint16_t function;

	switch(function_type){
	
		case AD9833_SIN:
            function = AD9833_compose_control_word(1);
            AD9833_write_data (function);
		break;
		
		case AD9833_TRIANGLE:
            function = AD9833_compose_control_word(2);
            AD9833_write_data (function);
		break;
		
		case AD9833_SQUARE:
            function = AD9833_compose_control_word(3);
            AD9833_write_data (function);
		break;
		
		case AD9833_FREQUENCY0_OUT:
            function = AD9833_compose_control_word(4);
            AD9833_write_data (function);
		break;
		
		case AD9833_FREQUENCY1_OUT:
            function = AD9833_compose_control_word(5);
            AD9833_write_data (function);
		break;
		
		case AD9833_PHASE0_OUT:
            function = AD9833_compose_control_word(6);
            AD9833_write_data (function);
		break;

		case AD9833_PHASE1_OUT:
            function = AD9833_compose_control_word(7);
            AD9833_write_data (function);
		break;
		
		case AD9833_SLEEP_MODE:
            function = AD9833_compose_control_word(9);
            AD9833_write_data (function);
		break;
		
		case AD9833_POWER_ON:
            function = AD9833_compose_control_word(10);
            AD9833_write_data (function);
		break;
	}
}
