/*******************************************************************************

Author : Mauro Laurenti
Version : 1.1
 
Created on Date : 08/12/2018
Last update     : 10/02/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.


*******************************************************************************/


#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "ADXL345.h"

//************************************************************
//         ADXL345_initialize function implementation
//************************************************************
void ADXL345_initialize (uint16_t baud_rate_KHz, uint8_t range, uint8_t sampling_rate){
    
    ADXL345_I2C_open (I2C_MASTER);
    ADXL345_I2C_baud_rate(baud_rate_KHz);
   
    //Low Power Mode and sampling rate as defined by by passed value
    ADXL345_write_address (ADXL345_REG_BW_RATE, 0x10 | sampling_rate);
    
    //Right justified, full resolution disabled, range is defined by passed value
    ADXL345_write_address (ADXL345_REG_DATA_FORMAT, 0x00 | range);
    
    //Enable the Measure mode
    ADXL345_write_address (ADXL345_REG_POWER_CTL, 0x08);   
}


//************************************************************
//         ADXL345_sleep_mode function implementation
//************************************************************
void ADXL345_sleep_mode (uint8_t sleep_mode){
    uint8_t dump_byte = 0x00;
    
    ADXL345_read_address (ADXL345_REG_POWER_CTL, &dump_byte);
    
    //Set to 0 the sleep bit
    dump_byte &= ADXL345_MASK_SLEEP;
    
    //Set the Sleep bit with the new configuration
    dump_byte |= sleep_mode | 0x03;
}

//************************************************************
//         ADXL345_write_address function implementation
//************************************************************
int8_t ADXL345_write_address (uint8_t address, uint8_t value){
    
    return (ADXL345_I2C_write_byte_to_external_device (ADXL345_WRITE_ADD,address,value));
}


//************************************************************
//         ADXL345_write_address function implementation
//************************************************************
int8_t ADXL345_read_address (uint8_t address, uint8_t * readback_value){
    
    uint8_t value = 0x00;
    int8_t code_error = 0x00;
    
    code_error = ADXL345_I2C_read_byte_from_external_device (ADXL345_WRITE_ADD,address, &value);
    *readback_value = value;
    
    return (code_error);
}


//************************************************************
//     ADXL345_read_acceleration function implementation
//************************************************************
int8_t ADXL345_read_acceleration (uint8_t acc_component, int16_t * readback_value){
   
    uint8_t reg_0 = 0x00;
    uint8_t reg_1 = 0x00;
    int8_t error_code = 0x00;
    
    uint8_t component_0 = 0x00;
    uint8_t component_1 = 0x00;
    uint16_t value = 0x0000;
    
    if (acc_component == ADXL345_X_VALUE){
        reg_0= ADXL345_REG_DATA_X0;
        reg_1= ADXL345_REG_DATA_X1;
    }
    
    if (acc_component == ADXL345_Y_VALUE){
        reg_0= ADXL345_REG_DATA_Y0;
        reg_1= ADXL345_REG_DATA_Y1;
    }
    
    if (acc_component == ADXL345_Z_VALUE){
        reg_0= ADXL345_REG_DATA_Z0;
        reg_1= ADXL345_REG_DATA_Z1;
    }
    
    error_code = ADXL345_read_address (reg_0, &component_0);
    error_code = ADXL345_read_address (reg_1, &component_1);
    
    value |= ((uint16_t) component_1) << 8;
    value |= ((uint16_t) component_0)  ;
       
    *readback_value = (int16_t) value;
   
    return (error_code);
}

//************************************************************
//           ADXL345_get_device_ID 
//************************************************************
int8_t ADXL345_get_device_ID (uint8_t device_address, uint8_t * device_ID){
   
    uint8_t value = 0x00;
    int8_t code_error = 0x00;
   
    code_error = ADXL345_read_address (ADXL345_REG_DEVID, &value);
    *device_ID = value;
        
    return (code_error);
}

