/*******************************************************************************

Author : Mauro Laurenti
Version : 1.4
Created on Date : 26/05/2012
Last update     : 25/17/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "GLCD_KS0108B.h"


uint8_t font_width = 0;
uint8_t font_height = 0;


//*********************************************
//             GLCD_wait_busy_status
//*********************************************
void GLCD_wait_busy_status (void){

	uint8_t busy_flag = 1;

	while (busy_flag) {

		if (GLCD_get_controller_status () & GLCD_BUSY_FLAG) 
			busy_flag = 1;
		else
			busy_flag = 0;
	}
}


//*********************************************
//             GLCD_get_controller_status
//*********************************************
uint8_t GLCD_get_controller_status (void) {

	uint8_t controller_status; 

	GLCD_DATA_DIRECTION = GLCD_INPUT;

	GLCD_E = GLCD_LOW;
	NOP_DELAY;
	GLCD_R_W = GLCD_HIGH;
	GLCD_D_I = GLCD_LOW;
	NOP_DELAY;
	GLCD_E = GLCD_HIGH;
	NOP_DELAY;
	controller_status = GLCD_DATA_READ;
	
	GLCD_DATA_DIRECTION = GLCD_OUTPUT;

	return (controller_status);
}


//*********************************************
//             GLCD_write_command
//*********************************************
void GLCD_write_command (uint8_t command_to_send) {

	GLCD_E = GLCD_LOW;
	NOP_DELAY;
	NOP_DELAY;
	GLCD_R_W = GLCD_LOW;
	GLCD_D_I = GLCD_LOW;
	NOP_DELAY;

	GLCD_E = GLCD_HIGH;
	GLCD_DATA_WRITE = command_to_send;
	NOP_DELAY;
    GLCD_E = GLCD_LOW;

	NOP_DELAY;
	GLCD_R_W = GLCD_HIGH;
	GLCD_D_I = GLCD_LOW;

	NOP_DELAY;

	GLCD_E = GLCD_HIGH;
}


//*********************************************
//             GLCD_set_display
//*********************************************
void GLCD_set_display (uint8_t display_status){
	
	GLCD_CS1 = GLCD_HIGH;
	GLCD_CS2 = GLCD_LOW;

	GLCD_wait_busy_status ();
	if (display_status)
		GLCD_write_command (GLCD_COMM_DISPLAY_ON);
	else 
		GLCD_write_command (GLCD_COMM_DISPLAY_OFF);

	GLCD_wait_busy_status ();

	GLCD_CS1 = GLCD_LOW;
	GLCD_CS2 = GLCD_HIGH;

	GLCD_wait_busy_status ();
	if (display_status)
		GLCD_write_command (GLCD_COMM_DISPLAY_ON);
	else 
		GLCD_write_command (GLCD_COMM_DISPLAY_OFF);

	GLCD_wait_busy_status ();

	GLCD_CS1 = GLCD_LOW;
	GLCD_CS2 = GLCD_LOW;
}

//*********************************************
//             GLCD_set_page
//*********************************************
void GLCD_set_page (uint8_t controller_page) {

	GLCD_wait_busy_status ();
	GLCD_write_command (GLCD_COMM_PAGE_SELECT | controller_page);
	GLCD_wait_busy_status ();
}


//*********************************************
//             GLCD_set_address
//*********************************************
void GLCD_set_address (uint8_t controller_address){

	GLCD_wait_busy_status ();
	GLCD_write_command (GLCD_COMM_ADDRESS_SELECT | controller_address);
	GLCD_wait_busy_status ();
}


//*********************************************
//             GLCD_set_vertical_offset
//*********************************************
void GLCD_set_vertical_offset (uint8_t vertical_offset){

	GLCD_CS1 = GLCD_HIGH;
	GLCD_CS2 = GLCD_LOW;

//	GLCD_wait_busy_status ();
	GLCD_write_command (GLCD_COMM_START_LINE | vertical_offset);
//	GLCD_wait_busy_status ();

	GLCD_CS1 = GLCD_LOW;
	GLCD_CS2 = GLCD_HIGH;

//	GLCD_wait_busy_status ();
	GLCD_write_command (GLCD_COMM_START_LINE | vertical_offset);
//	GLCD_wait_busy_status ();

	GLCD_CS1 = GLCD_LOW;
	GLCD_CS2 = GLCD_LOW;
}


//*********************************************
//             GLCD_write_data
//*********************************************
void GLCD_write_data (uint8_t data_to_send) {

	GLCD_E = GLCD_LOW;
	NOP_DELAY;
	NOP_DELAY;
	GLCD_R_W = GLCD_LOW;
	GLCD_D_I = GLCD_HIGH;
	NOP_DELAY;
	NOP_DELAY;
	GLCD_E = GLCD_HIGH;
	GLCD_DATA_WRITE = data_to_send;
	NOP_DELAY;
	NOP_DELAY;
	NOP_DELAY;
	GLCD_E = GLCD_LOW;

	NOP_DELAY;
	GLCD_R_W = GLCD_HIGH;
	GLCD_D_I = GLCD_LOW;

	NOP_DELAY;

	GLCD_E = GLCD_HIGH;

}


//*********************************************
//             GLCD_clear
//*********************************************
void GLCD_clear (uint8_t filling_color) {
	
	uint8_t page = 0;
	uint8_t address = 0;

	GLCD_CS1 = GLCD_HIGH;
	GLCD_CS2 = GLCD_LOW;
	
	GLCD_set_address (address);

	for (page = 0; page < GLCD_NUMBER_OF_PAGES; page++) {
		GLCD_set_page (page);
		for (address = 0; address < GLCD_Y_RESOLUTION; address++) {

		//	wait_busy_status_GLCD ();
			GLCD_write_data (filling_color);
		//	wait_busy_status_GLCD ();
		}
	}

	GLCD_CS1 = GLCD_LOW;
	GLCD_CS2 = GLCD_HIGH;


	address = 0;
	GLCD_set_address (address);

	for (page = 0; page < GLCD_NUMBER_OF_PAGES; page++) {
		GLCD_set_page (page);
		
		for (address = 0; address < GLCD_Y_RESOLUTION; address++) {
		//	GLCD_wait_busy_status ();
			GLCD_write_data (filling_color);
		//	GLCD_wait_busy_status ();
		}
	}

	GLCD_CS1 = GLCD_LOW;
	GLCD_CS2 = GLCD_LOW;
}


//*********************************************
//             GLCD_read_data
//*********************************************
uint8_t GLCD_read_data (void) {

	uint8_t data_out; 

	GLCD_DATA_DIRECTION = GLCD_INPUT;

	GLCD_E = GLCD_LOW;
	NOP_DELAY;
	NOP_DELAY;
	GLCD_R_W = GLCD_HIGH;
	GLCD_D_I = GLCD_HIGH;
	NOP_DELAY;
	GLCD_E = GLCD_HIGH;
	NOP_DELAY;
	data_out = GLCD_DATA_READ;
	NOP_DELAY;
	NOP_DELAY;
	GLCD_E = GLCD_LOW;
	NOP_DELAY;
	GLCD_R_W = GLCD_LOW;
	GLCD_D_I = GLCD_LOW;
	
	GLCD_DATA_DIRECTION = GLCD_OUTPUT;

	return (data_out);
}


//*********************************************
//             GLCD_plot_xy
//*********************************************
void GLCD_plot_xy (uint8_t x_pos,  uint8_t y_pos, uint8_t filling_color){

	uint8_t data_byte;
	uint8_t data_mask;

	y_pos = GLCD_Y_RESOLUTION-y_pos -1;

	if (x_pos < GLCD_CONTROLLER_MAX_X) {

		GLCD_CS1 = GLCD_HIGH;
		GLCD_CS2 = GLCD_LOW;
		GLCD_set_address (x_pos);
		GLCD_set_page (y_pos >> 3);	
		//dummy read
		data_byte = GLCD_read_data ();
		data_byte = GLCD_read_data ();

		GLCD_set_address (x_pos);

	} else {
		
		GLCD_CS1 = GLCD_LOW;
		GLCD_CS2 = GLCD_HIGH;
		GLCD_set_address (x_pos - GLCD_CONTROLLER_MAX_X);
		GLCD_set_page (y_pos >> 3);	
		//dummy read
		data_byte = GLCD_read_data ();
		data_byte = GLCD_read_data ();

		GLCD_set_address (x_pos - GLCD_CONTROLLER_MAX_X);
	}

	data_mask = 0x01 << (y_pos & 0x07);	

	if (filling_color == GLCD_FILLING_BLACK)
		data_byte = data_byte | data_mask;
	else
		data_byte = data_byte & ~data_mask;
	
//	GLCD_wait_busy_status ();
	GLCD_write_data (data_byte);
//	GLCD_wait_busy_status ();
	

	GLCD_CS1 = GLCD_LOW;
	GLCD_CS2 = GLCD_LOW;
}


//*********************************************
//           GLCD_draw_vertical_line
//*********************************************
void GLCD_draw_vertical_line (uint8_t x_origin, uint8_t y_origin, uint8_t length, uint8_t filling_color){

	uint8_t i;

	for (i = 0 ; i<length; i++) {
		GLCD_plot_xy (x_origin, y_origin + i, filling_color);
	}
}


//*********************************************
//          GLCD_draw_horizontal_line
//*********************************************
void GLCD_draw_horizontal_line (uint8_t x_origin, uint8_t y_origin, uint8_t length, uint8_t filling_color){

	uint8_t i;

	for (i = 0 ; i<length; i++) {
		GLCD_plot_xy (x_origin + i, y_origin, filling_color);
	}
}


//*********************************************
//             GLCD_draw_window
//*********************************************
void GLCD_draw_window (uint8_t x_origin, uint8_t y_origin, uint8_t width, uint8_t height, uint8_t filling_color){

	GLCD_draw_horizontal_line (x_origin, y_origin, width, filling_color);
	// + 1 is required to close the window
	GLCD_draw_horizontal_line (x_origin, y_origin + height, width+1, filling_color);

	GLCD_draw_vertical_line (x_origin, y_origin, height, filling_color);
	GLCD_draw_vertical_line (x_origin + width, y_origin, height, filling_color);

}


//*********************************************
//             GLCD_draw_box
//*********************************************

void GLCD_draw_box (uint8_t x_origin, uint8_t y_origin, uint8_t width, uint8_t height, uint8_t filling_color){

	uint8_t i;

	for (i=0; i < width; i++)
		GLCD_draw_vertical_line (x_origin +i, y_origin, height, filling_color);

}

//*********************************************
//             GLCD_write_char
//*********************************************
void GLCD_write_char (uint8_t x_pos, uint8_t y_pos, uint8_t character){

	uint8_t i;
	uint8_t data;
	uint8_t x;
	uint8_t bit_value;

	for (i = 0 ; i<font_width; i++){

		data = font_5x7[character - 32][i];

		for (x = 0 ; x<= font_height; x++) {
			
			if (data & (0x80 >> x))
				bit_value = GLCD_FILLING_BLACK;
			else 
				bit_value = GLCD_FILLING_WHITE;

			GLCD_plot_xy (x_pos + i, y_pos + x, bit_value);
			
		}			
	}	
}

//*********************************************
//             GLCD_write_string
//*********************************************
void GLCD_write_string (uint8_t x_pos, uint8_t y_pos, uint8_t * character_array){

uint8_t i = 0;

	while (character_array[i]) {
		
		GLCD_write_char (x_pos + (i* (font_width+ GLCD_FONT_SPACE)) , y_pos,character_array[i]);
	
		i++;
	}
}


//*********************************************
//             GLCD_write_message
//*********************************************
void GLCD_write_message (uint8_t x_pos, uint8_t y_pos, const char * character) {

    uint8_t i = 0;

    while (character[i]) {

        GLCD_write_char (x_pos + (i*(font_width + GLCD_FONT_SPACE)), y_pos,character[i]);

        i++;
    }
}


//*********************************************
//             GLCD_draw_picture
//*********************************************
void GLCD_draw_picture (const uint8_t  * picture_table){

	uint8_t page = 0;
	uint8_t address = 0;
	
	for (page = 0; page < GLCD_NUMBER_OF_PAGES; page++) {
            GLCD_CS1 = GLCD_HIGH;
            GLCD_CS2 = GLCD_LOW;

            GLCD_set_address (address);
            GLCD_set_page (page);

            for (address = 0; address < GLCD_Y_RESOLUTION; address++) {

                    GLCD_write_data (* picture_table);
                    picture_table++;
            }

            GLCD_CS1 = GLCD_LOW;
            GLCD_CS2 = GLCD_HIGH;

            address = 0;
            GLCD_set_address (address);
            GLCD_set_page (page);

            for (address = 0; address < GLCD_Y_RESOLUTION; address++) {

                    GLCD_write_data (* picture_table);
                    picture_table++;
            }
	}

	GLCD_CS1 = GLCD_LOW;
	GLCD_CS2 = GLCD_LOW;
}


//*********************************************
//             GLCD_write_integer
//*********************************************
void GLCD_write_integer (uint8_t x_pos, uint8_t y_pos, int16_t value, uint8_t number_of_digits, uint8_t zero_cleaning){

	// The array size is 6, plus end of string '\0'
	uint8_t convertedInt [7];

	uint8_t string_length;
    uint8_t counter;
    uint8_t shift_counter;
	
	// Integer is converted to string
    LTlib_int16_to_string (&convertedInt[0], value);
    
    string_length = LTlib_string_length (&convertedInt[0]);

    //Make the string as long as needed by appending 0 on the left
    for (counter = 0; counter < (number_of_digits - string_length); counter++){
        //shift right as needed by adding 0
        for (shift_counter = number_of_digits; shift_counter > 0; shift_counter--){
            convertedInt[shift_counter] = convertedInt[shift_counter-1];
        }   
        
        //After each shift a 0 is inserted for postive numbers
        //For negative numbers and empty space is added
        if (convertedInt[0] == '-' || convertedInt[0] == ' '){
            convertedInt[0] = ' ';
        } else {
            convertedInt[0] = '0';
        }        
    }    
    
    string_length = LTlib_string_length (&convertedInt[0]);
    
    //remove the 0 on the left, if required
    if (zero_cleaning == GLCD_ZERO_CLEANING_ON) {       
        //I keep at least one 0, thus I do not chek the entire string length
        for (counter = 0; counter < string_length; counter++){
            if (convertedInt[counter] == '0'){
                convertedInt[counter] = ' ';
            } else {
                //seeting the cunter to this value will terminate the for.
                counter = string_length;
            }   
        }  
    }  

	GLCD_write_string (x_pos, y_pos, convertedInt);
}


//*********************************************
//             GLCD_set_font
//*********************************************
void GLCD_set_font (uint8_t width, uint8_t height){

	font_width = width;
	font_height = height;
}


//*********************************************
//             GLCD_backlight
//*********************************************
void GLCD_backlight(uint8_t backlight) {

	GLCD_LED = backlight;
}


//*********************************************
//             GLCD_initialize
//*********************************************
void  GLCD_initialize (void) {

    //Pin initialization
    GLCD_DATA_DIRECTION = GLCD_OUTPUT;
    
    GLCD_D_I_DIRECTION = PIN_AS_OUTPUT;
    GLCD_R_W_DIRECTION = PIN_AS_OUTPUT;
    GLCD_E_DIRECTION = PIN_AS_OUTPUT;
    GLCD_CS1_DIRECTION = PIN_AS_OUTPUT;
    GLCD_CS2_DIRECTION = PIN_AS_OUTPUT;
    GLCD_RST_DIRECTION = PIN_AS_OUTPUT;
    GLCD_LED_DIRECTION = PIN_AS_OUTPUT;
      
	GLCD_RST = GLCD_LOW;
	GLCD_LED = GLCD_LOW;
	GLCD_E = GLCD_HIGH;
	GLCD_R_W = GLCD_HIGH;
	GLCD_D_I = GLCD_LOW;
	GLCD_CS1 = GLCD_LOW;
	GLCD_CS2 = GLCD_LOW;
	
	delay_ms (100);

	GLCD_RST = GLCD_HIGH;
	
	GLCD_backlight (GLCD_TURN_OFF_LED);
    GLCD_set_vertical_offset (0);
	GLCD_clear (GLCD_FILLING_WHITE);
	GLCD_set_display (GLCD_TURN_ON_DISPLAY);
	GLCD_set_font (GLCD_FONT_WIDTH_5, GLCD_FONT_HEIGTH_7);

}
