/*******************************************************************************

Author : Mauro Laurenti
Version : 2.0

Created on Date : 04/09/2006
Last update     : 28/03/2023

CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "LCD_44780.h"


//************************************************************
//             enable_pulse_LCD  Implementation
//************************************************************
void LCD_enable_pulse (void) {
	
	LCD_E = 1;
	delay_ms (1);
	LCD_E = 0;
	delay_ms (1);
}


//************************************************************
//             send_command_LCD  Implementation
//************************************************************
void LCD_send_command (uint8_t D3, uint8_t D2, uint8_t D1, uint8_t D0) {

	LCD_D0 = D0;
	LCD_D1 = D1;
	LCD_D2 = D2;
	LCD_D3 = D3;
	LCD_enable_pulse ();
}


//************************************************************
//              home_LCD  Implementation
//************************************************************
void LCD_home(void) {
	
	LCD_send_command (0,0,0,0);
	LCD_send_command (0,0,1,0);
}


//************************************************************
//               shift_LCD  Implementation
//************************************************************
void LCD_shift (uint8_t shift, uint8_t number_of_shift) {
	
	uint8_t i;
	
	for (i=0; i < number_of_shift; i++) {	
		LCD_send_command (0,0,0,1);
		LCD_send_command (1,shift,0,0);
	}	
}


//************************************************************
//              shift_cursor_LCD  Implementation
//************************************************************
void LCD_shift_cursor (uint8_t shift, uint8_t number_of_shift){
	
	uint8_t i;
	
	for (i=0; i < number_of_shift; i++) {
		LCD_send_command (0,0,0,1);
		LCD_send_command (0,shift,0,0);
	} 		
}


//************************************************************
//            LCD_goto_line  Implementation
//************************************************************
void LCD_goto_line (uint8_t line) {

switch(line) {

	case 1: LCD_send_command(1,0,0,0);
		    LCD_send_command(0,0,0,0);
			break;

	case 2: LCD_send_command(1,1,0,0);
		    LCD_send_command(0,0,0,0);
			break;

	case 3: LCD_send_command(1,0,0,1);
		    LCD_send_command(0,1,0,0);
			break;

	case 4: LCD_send_command(1,1,0,1);
            LCD_send_command(0,1,0,0);
	}
}


//************************************************************
//             LCD_goto_xy  Implementation
//************************************************************
void LCD_goto_xy (uint8_t x, uint8_t y){

    LCD_goto_line (y);
    LCD_shift_cursor (LCD_RIGHT, x-1);
}


//************************************************************
//           write_char_LCD  Implementation
//************************************************************
void LCD_write_char (uint8_t value) {
	
	uint8_t D3,D2,D1,D0;
	
	LCD_RS = 1;
		
	// Splitting of the first nibble				
	D3 = (value & LCD_BITMASK_BIT_7) >> LCD_SHIFT_07;	
	D2 = (value & LCD_BITMASK_BIT_6) >> LCD_SHIFT_06;
	D1 = (value & LCD_BITMASK_BIT_5) >> LCD_SHIFT_05;
	D0 = (value & LCD_BITMASK_BIT_4) >> LCD_SHIFT_04;
		
	LCD_send_command (D3,D2,D1,D0);
		
	// Splitting of the second nibble
	D3 = (value & LCD_BITMASK_BIT_3) >> LCD_SHIFT_03;	
	D2 = (value & LCD_BITMASK_BIT_2) >> LCD_SHIFT_02;
	D1 = (value & LCD_BITMASK_BIT_1) >> LCD_SHIFT_01;
	D0 = (value & LCD_BITMASK_BIT_0);
			
	LCD_send_command (D3,D2,D1,D0);
		
	LCD_RS = 0;
}


//************************************************************
//         LCD_write_data_CGRAM  Implementation
//************************************************************
void LCD_write_data_CGRAM(uint8_t address, uint8_t data)
{
    uint8_t D3,D2,D1,D0;
	
	LCD_RS = 0;
		
	// Splitting of the first nibble				
	D3 = 0;
	D2 = 1;
	D1 = (address & LCD_BITMASK_BIT_5) >> LCD_SHIFT_05;
	D0 = (address & LCD_BITMASK_BIT_4) >> LCD_SHIFT_04;
		
	LCD_send_command (D3,D2,D1,D0);
		
	// Splitting of the second nibble
	D3 = (address & LCD_BITMASK_BIT_3) >> LCD_SHIFT_03;	
	D2 = (address & LCD_BITMASK_BIT_2) >> LCD_SHIFT_02;
	D1 = (address & LCD_BITMASK_BIT_1) >> LCD_SHIFT_01;
	D0 = (address & LCD_BITMASK_BIT_0);
			
	LCD_send_command (D3,D2,D1,D0);
		
	LCD_RS = 0;
    
    delay_ms(1);
    
    LCD_RS = 1;
		
	// Splitting of the first nibble				
	D3 = (data & LCD_BITMASK_BIT_7) >> LCD_SHIFT_07;	
	D2 = (data & LCD_BITMASK_BIT_6) >> LCD_SHIFT_06;
	D1 = (data & LCD_BITMASK_BIT_5) >> LCD_SHIFT_05;
	D0 = (data & LCD_BITMASK_BIT_4) >> LCD_SHIFT_04;
	LCD_send_command (D3,D2,D1,D0);
		
	// Splitting of the second nibble
	D3 = (data & LCD_BITMASK_BIT_3) >> LCD_SHIFT_03;	
	D2 = (data & LCD_BITMASK_BIT_2) >> LCD_SHIFT_02;
	D1 = (data & LCD_BITMASK_BIT_1) >> LCD_SHIFT_01;
	D0 = (data & LCD_BITMASK_BIT_0);
			
	LCD_send_command (D3,D2,D1,D0);
		
	LCD_RS = 0;
    
    LCD_home ();
}


//************************************************************
//            write_message_LCD  Implementation
//************************************************************
void LCD_write_message (const char *buffer) {
	// Write data to LCD up to null
    while(*buffer) {
	    
	    // Write character to LCD
	    LCD_write_char(*buffer);
	    // Increment buffer
	    buffer++;              
   }
}


//************************************************************
//           write_string_LCD  Implementation
//************************************************************
void LCD_write_string (uint8_t *buffer) {
	
	// Write data to LCD up to null
	while(*buffer){
		
		// Write character to LCD
		LCD_write_char(*buffer);
		// Increment buffer
		buffer++;               
	}
}


//************************************************************
//           write_integer_LCD  Implementation
//************************************************************
void LCD_write_integer (int16_t value, uint8_t number_of_digits, uint8_t zero_cleaning){
	
    // The array size is 6, plus end of string '\0'
	uint8_t convertedInt [7];

	uint8_t string_length;
    uint8_t counter;
    uint8_t shift_counter;
	
	// Integer is converted to string
    LTlib_int16_to_string (&convertedInt[0], value);
    
    string_length = LTlib_string_length (&convertedInt[0]);

    //Make the string as long as needed by appending 0 on the left
    for (counter = 0; counter < (number_of_digits - string_length); counter++){
        //shift right as needed by adding 0
        for (shift_counter = number_of_digits; shift_counter > 0; shift_counter--){
            convertedInt[shift_counter] = convertedInt[shift_counter-1];
        }   
        
        //After each shift a 0 is inserted for postive numbers
        //For negative numbers and empty space is added
        if (convertedInt[0] == '-' || convertedInt[0] == ' '){
            convertedInt[0] = ' ';
        } else {
            convertedInt[0] = '0';
        }        
    }    
    
    string_length = LTlib_string_length (&convertedInt[0]);
    
    //remove the 0 on the left, if required
    if (zero_cleaning == LCD_ZERO_CLEANING_ON) {       
        //I keep at least one 0, thus I do not chek the entire string length
        for (counter = 0; counter < string_length; counter++){
            if (convertedInt[counter] == '0'){
                convertedInt[counter] = ' ';
            } else {
                //seeting the cunter to this value will terminate the for.
                counter = string_length;
            }   
        }  
    }  

	LCD_write_string (convertedInt);		
}	


//*********************************************
//             LCD_write_float
//*********************************************
void LCD_write_float (float value, uint8_t number_of_digits, uint8_t number_of_fractional_digits, uint8_t zero_cleaning){
  
    uint8_t string_length;
    uint8_t counter;
    uint8_t shift_counter;
    
    uint8_t string_result[MAX_NUM_DIGITS];
    
    LTlib_float_to_string (string_result, value, number_of_fractional_digits);
    
    string_length = LTlib_string_length (&string_result[0]);
    
    //Make the string as long as needed by appending 0 on the left
    for (counter = 0; counter < (number_of_digits - string_length); counter++){
        //shift right as needed by adding 0
        for (shift_counter = number_of_digits; shift_counter > 0; shift_counter--){
            string_result[shift_counter] = string_result[shift_counter-1];
        }   
        
        //After each shift a 0 is inserted for positive numbers
        //For negative numbers and empty space is added
        if (string_result[0] == '-' || string_result[0] == ' '){
            string_result[0] = ' ';
        } else {
            string_result[0] = '0';
        }        
    }    
    
    string_length = LTlib_string_length (&string_result[0]);
    
    //remove the 0 on the left, if required
    if (zero_cleaning == LCD_ZERO_CLEANING_ON) {       
        //I keep at least one 0, thus I do not check the entire string length
        for (counter = 0; counter < string_length; counter++){
            
            if (string_result[counter] != '.'){
                
                if (string_result[counter] == '0'){
                    string_result[counter] = ' ';
                } else {
                    //setting the counter to this value will terminate the for.
                    counter = string_length;
                }   
            }
        }  
    }  

    
    LCD_write_string (string_result);
}


//************************************************************
//              clear_LCD  Implementation
//************************************************************
void LCD_clear (void){
	
	LCD_send_command (0,0,0,0);
	LCD_send_command (0,0,0,1);
}


//************************************************************
//                  cursor_LCD  Implementation
//************************************************************
void LCD_cursor (uint8_t active, uint8_t blinking) {
	
	LCD_send_command (0,0,0,0);
	LCD_send_command (1,1,active,blinking);
}


//************************************************************
//                  back_light_LCD  Implementation
//************************************************************
void LCD_backlight (uint8_t active) {

	LCD_LED = active;	
}


//************************************************************
//                  initialize_LCD  Implementation
//************************************************************
void  LCD_initialize (uint8_t quartz_frequency) {
	
    //IO pin initialization    
    LCD_D0_TRIS = PIN_AS_OUTPUT;   
    LCD_D1_TRIS = PIN_AS_OUTPUT;  
    LCD_D2_TRIS = PIN_AS_OUTPUT;  
    LCD_D3_TRIS = PIN_AS_OUTPUT; 
    LCD_RS_TRIS = PIN_AS_OUTPUT;  
    LCD_E_TRIS = PIN_AS_OUTPUT; 
    LCD_RW_TRIS = PIN_AS_OUTPUT; 
    LCD_LED_TRIS = PIN_AS_OUTPUT; 
        
	delay_set_quartz (quartz_frequency);
	
	LCD_RS = 0x00;
	LCD_E = 0x00;
	LCD_RW = 0x00;

	delay_ms (100);
	LCD_send_command (0,0,1,1);
	delay_ms (100);
	LCD_send_command (0,0,1,1);
	delay_ms (10);
	LCD_send_command (0,0,1,1);

	LCD_send_command (0,0,1,0);

    LCD_send_command (0,0,1,0);
	LCD_send_command (1,0,0,0);

	LCD_send_command (0,0,0,0);
    LCD_send_command (1,0,0,0);

	LCD_send_command (0,0,0,0);
	LCD_send_command (0,0,0,1);
        
    LCD_send_command (0,0,0,0);
	LCD_send_command (0,1,1,0);

	LCD_clear ();
       
    LCD_cursor (0,0);
}

