/*******************************************************************************
 
Autore : Mauro Laurenti
Versione : 1.1
 
Created on Date : 21/07/2019
Last update     : 16/03/2023

CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 
*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "LTlib_utilities.h"


//***************************************************
//              LTlib_int16_to_string
//***************************************************
int8_t LTlib_int16_to_string (uint8_t * buffer, int16_t value){

    uint8_t new_digit = 0;
    uint8_t digit_counter = 0;
    //The flag is set if you should not ignore the following digits
    //0 ignore the digit
    //1 do not ignore the digit
    uint8_t flag = 0;
    
    if (value < 0) {
        buffer[digit_counter]= '-'; 
        value = value * (-1);
        digit_counter++;
    } else {
//        buffer[digit_counter]= '+'; 
//        digit_counter++;
    }
    
    //MSB
    new_digit = (uint8_t) (value / 10000);
    // adding 48 it will convert the integer in char
    if (new_digit != 0){
        buffer[digit_counter]= new_digit + 48; 
        value = value - (new_digit * 10000);
        flag = 1;
        digit_counter++;
    }  
    
    new_digit = (uint8_t) (value / 1000);
    if (new_digit != 0 || flag == 1){
        buffer[digit_counter]= new_digit + 48;
        value = value - (new_digit * 1000);
        flag = 1;
        digit_counter++;
    }

    new_digit = (uint8_t) (value / 100);
    if (new_digit != 0 || flag == 1){
        buffer[digit_counter]= new_digit + 48;
        value = value - (new_digit * 100);
        flag = 1;
        digit_counter++;
    }
        
    new_digit = (uint8_t) (value / 10);
    if (new_digit != 0 || flag == 1){
        buffer[digit_counter]= new_digit + 48;
        value = value - (new_digit * 10);
        flag = 1;
        digit_counter++;
    }
    
    //LSB
    new_digit = (uint8_t) value;
    buffer[digit_counter]= new_digit + 48;
    digit_counter++;
    
	buffer[digit_counter]= '\0';
    
	return (0); 
}

//***************************************************
//              LTlib_Hex16_to_string
//***************************************************
int8_t LTlib_float_to_string (uint8_t * buffer, float value, uint8_t fractional_digits ){
    uint16_t Integeger_part = 0;
    uint16_t Fractional_part = 0;
    float Fractional_part_real = 0;
    
    uint8_t signed_flag = 0;
    
    uint8_t counter = 0;
    uint8_t counter_main = 0;
    
    uint8_t fractional_string [MAX_NUM_DIGITS];
    uint8_t integer_string [MAX_NUM_DIGITS];
    
    uint8_t missing_0 = 0 ; 
    
    if (value < 0){
        signed_flag = 1;
        value = value * (-1);
    }       
    
    Integeger_part = (uint16_t) value;    
    Fractional_part_real = value - Integeger_part;
    
            
    for (counter=0; counter< fractional_digits; counter++){
        Fractional_part_real = Fractional_part_real * 10;
    }
    
    Fractional_part = (uint16_t) Fractional_part_real;
    
    LTlib_int16_to_string (integer_string, (int16_t) Integeger_part);
      
    LTlib_int16_to_string (fractional_string, (int16_t) Fractional_part);
    
    counter = 0;
    counter_main = 0;
    
    if (signed_flag == 1){
        buffer[counter_main] = '-';
        counter_main++;
    }
    
    while (integer_string[counter] != '\0'){      

        buffer[counter_main] = integer_string[counter];
        counter++;
        counter_main++;
    }
    
    buffer[counter_main] = '.';
    counter_main++;
    
    counter = 0;
    
    //add 0 that could have been lost during the fractional conversion
    missing_0 = fractional_digits - LTlib_string_length (fractional_string);
    for (counter = 0; counter < missing_0; counter++){         
        buffer[counter_main] = '0';
        counter_main++;
    }
    
    counter = 0;
    
    while (fractional_string[counter] != '\0'){        
        
        buffer[counter_main] = fractional_string[counter];
        counter++;
        counter_main++;
    }
    
    buffer[counter_main] = '\0';
    
    return (0); 
}


//***************************************************
//              LTlib_Hex8_to_string
//***************************************************
int8_t LTlib_Hex8_to_string (uint8_t * buffer, uint8_t value){
    
    uint8_t table_index = 0;
    
    table_index = (value & 0xF0) >> 4;
    buffer[0] = hex_table[table_index];
    
    table_index = (value & 0x0F);
    buffer[1] = hex_table[table_index];
    
	buffer[2] = '\0';
    
	return (0);
}


//***************************************************
//              LTlib_Hex16_to_string
//***************************************************
int8_t LTlib_Hex16_to_string (uint8_t * buffer, uint16_t value){

    uint8_t table_index = 0;
    
    table_index = (value & 0xF000) >> 12;
    buffer[0] = hex_table[table_index];
    
    table_index = (value & 0x0F00) >> 8;
    buffer[1] = hex_table[table_index];
    
    table_index = (value & 0x00F0) >> 4;
    buffer[2] = hex_table[table_index];
    
    table_index = (value & 0x000F);
    buffer[3] = hex_table[table_index];
    
	buffer[4] = '\0';
    
	return (0); 
}


//***************************************************
//              LTlib_Hex32_to_string
//***************************************************
int8_t LTlib_Hex32_to_string (uint8_t * buffer, uint32_t value){

    uint8_t table_index = 0;
    
    table_index = (value & 0xF0000000) >> 28;
    buffer[0] = hex_table[table_index];
    
    table_index = (value & 0x0F000000) >> 24;
    buffer[1] = hex_table[table_index];
    
    table_index = (value & 0x00F00000) >> 20;
    buffer[2] = hex_table[table_index];
    
    table_index = (value & 0x000F0000) >> 16;
    buffer[3] = hex_table[table_index];
    
    table_index = (value & 0x0000F000) >> 12;
    buffer[4] = hex_table[table_index];
    
    table_index = (value & 0x00000F00) >> 8;
    buffer[5] = hex_table[table_index];
    
    table_index = (value & 0x000000F0) >> 4;
    buffer[6] = hex_table[table_index];
    
    table_index = (value & 0x0000000F);
    buffer[7] = hex_table[table_index];
    
	buffer[8] = '\0';
    
	return (0); 
}


//***************************************************
//              LTlib_string_length
//***************************************************
uint8_t LTlib_string_length (uint8_t * buffer){
    uint8_t counter = 0;
    
    while (buffer[counter] != '\0'){        
        counter++;
    }
    
    return(counter); 
}


