/*******************************************************************************

Author : Mauro Laurenti
Version : 1.1
  
Created on Date : 02/10/2017
Last update     : 29/07/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.


********************************************************
PURPOSES
********************************************************

 This library allows the usage of the MAX6675 Temperature sensor.

*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "MAX6675.h"


//************************************************************
//                  MAX6675_initialize  Implementation
//************************************************************
void MAX6675_initialize (uint8_t clock){
      
    //Set the CS pin as Output
    MAX6675_CS_TRIS = 0x00;
    MAX6675_CS_PORT = MAX6675_CS_DISABLED;
    
    //SPI module initialization
    MAX6675_SPI_mode(SPI_MODE_1);
    MAX6675_SPI_baudrate(clock);
    MAX6675_SPI_open(SPI_MASTER_DEVICE);    
}


//************************************************************
//          MAX6675_read_temperature  Implementation
//************************************************************
uint16_t MAX6675_read_temperature (void){
    
    uint16_t register_16_bits = 0;
    
    register_16_bits = MAX6675_read_register ();
    register_16_bits = register_16_bits >> 3;
    
    //discard the decimal points
    register_16_bits = (uint16_t) ((float)register_16_bits * .25);
        
    return (register_16_bits);
}


//************************************************************
//             MAX6675_read_register  Implementation
//************************************************************
uint16_t MAX6675_read_register (void){
    
    uint8_t read_byte_1 = 0;
    uint8_t read_byte_2 = 0;
    uint16_t register_16_bits = 0;
    
    MAX6675_CS_PORT = MAX6675_CS_ENABLED;
    read_byte_1 = MAX6675_SPI_read_byte ();
    read_byte_2 = MAX6675_SPI_read_byte ();
    MAX6675_CS_PORT = MAX6675_CS_DISABLED;
    
    register_16_bits = (uint16_t) read_byte_1;
    register_16_bits = register_16_bits << 8;
    register_16_bits |= (uint16_t) read_byte_2;
     
    // The MAX6675 has a conversion time of max 22ms.
    // The delay ensure that among two readings the conversion time is elapsed
    delay_ms(50);
    
    return (register_16_bits);    
}


//************************************************************
//        MAX6675_is_connected  Implementation
//************************************************************
uint8_t MAX6675_is_connected (void){
    
    uint16_t read_value = 0xFFFF;
    
    read_value = MAX6675_read_register ();
    
    read_value &= MAX6675_CONNECTEDED_BIT_MASK; 
    
    if (read_value == 0) {
        return(MAX6675_DEVICE_CONNECTED);
    } else {
        return(MAX6675_DEVICE_DISCONNECTED);
    }
}


//************************************************************
//       MAX6675_is_probe_connected  Implementation
//************************************************************
uint8_t MAX6675_is_probe_connected (void){
    
    uint16_t read_value = 0xFFFF;
    
    read_value = MAX6675_read_register ();
    
    read_value &= MAX6675_PROBE_BIT_MASK; 
    
    if (read_value == 0) {
        return(MAX6675_PROBE_CONNECTED);
    } else {
        return(MAX6675_PROBE_DISCONNECTED);
    }    
}
