/*******************************************************************************

Author : Mauro Laurenti
Version : 1.3
Created on Date : 07/02/2014
Last update     : 31/01/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "MCP2300x.h"


//************************************************************
//         initialize_MC2300x implementation
//************************************************************
void MCP2300x_initialize (uint16_t baud_rate_KHz){

    MCP2300X_I2C_open (I2C_MASTER);
    MCP2300X_I2C_baud_rate (baud_rate_KHz);
}


//************************************************************
//         set_register_MCP2300x implementation
//************************************************************
int8_t MCP2300x_set_register (uint8_t device_address, uint8_t register_add,  uint8_t data ) {

    uint8_t control_byte = 0;

    control_byte = (uint8_t)(MCP2300X_INTERNAL_ADDRESS | (device_address <<1));

    return (MCP2300X_I2C_write_byte_to_external_device (control_byte,register_add,data));
}


//************************************************************
//           get_register_MCP2300x implementation
//************************************************************
uint8_t MCP2300x_get_register (uint8_t device_address, uint8_t register_add){
	
    uint8_t control_byte = 0;
    uint8_t data = 0;
    
    control_byte = (uint8_t) (MCP2300X_INTERNAL_ADDRESS | (device_address <<1));

    MCP2300X_I2C_read_byte_from_external_device (control_byte, register_add, &data);
            
    return (data);
}


//************************************************************
//         set_port_direction_MCP2300x implementation
//************************************************************
int8_t MCP2300x_set_port_direction (uint8_t device_address, uint8_t direction) {

    return (MCP2300x_set_register (device_address, MCP2300x_IODIR, direction));
}


//************************************************************
//         set_port_value_MCP2300x implementation
//************************************************************
int8_t MCP2300x_set_port_value (uint8_t device_address, uint8_t value){

    return (MCP2300x_set_register (device_address, MCP2300x_GPIO, value));
}


//************************************************************
//         set_port_polarity_MCP2300x implementation
//************************************************************
int8_t MCP2300x_set_port_polarity (uint8_t device_address, uint8_t value){

    return (MCP2300x_set_register (device_address, MCP2300x_IPOL, value));
}


//************************************************************
// set_port_pull_up_resistor_MCP2300x implementation
//************************************************************
int8_t MCP2300x_set_port_pull_up_resistor (uint8_t device_address, uint8_t value){

    return (MCP2300x_set_register (device_address, MCP2300x_GPPU, value));
}


//************************************************************
//         get_port_value_MCP2300x implementation
//************************************************************
uint8_t MCP2300x_get_port_value (uint8_t device_address){

    return (MCP2300x_get_register (device_address, MCP2300x_GPIO));
}



//************************************************************
//         set_port_interrupt_MCP2300x implementation
//************************************************************
int8_t MCP2300x_set_port_interrupt (uint8_t device_address, uint8_t value){

    int8_t return_value = 0;

    return_value =  MCP2300x_set_register (device_address, MCP2300x_GPINTEN, value);

    //This dummy read of the GPIO register is used to clean the interupt flags
    MCP2300x_get_register (device_address, MCP2300x_GPIO);

    return (return_value);

}

//************************************************************
//     set_port_interrupt_configuration_MCP2300x implementation
//************************************************************
int8_t MCP2300x_set_port_configuration (uint8_t device_address, uint8_t value){
    return (MCP2300x_set_register (device_address, MCP2300x_IOCON, value));
}

//************************************************************
// set_interrupt_compare_value_MCP2300x implementation
//************************************************************
int8_t MCP2300x_set_interrupt_compare_value (uint8_t device_address, uint8_t value){

    return (MCP2300x_set_register (device_address, MCP2300x_DEFVAL, value));
}


//************************************************************
// set_interrupt_compare_enable_MCP2300x implementation
//************************************************************
int8_t MCP2300x_set_interrupt_compare_enable (uint8_t device_address, uint8_t value){

    return (MCP2300x_set_register (device_address, MCP2300x_INTCON, value));
}


//************************************************************
// get_port_interrupt_flag_MCP2300x implementation
//************************************************************
uint8_t MCP2300x_get_port_interrupt_flag (uint8_t device_address){

    return (MCP2300x_get_register (device_address, MCP2300x_INTF));
}


//************************************************************
// get_port_interrupt_capture_MCP2300x implementation
//************************************************************
uint8_t MCP2300x_get_port_interrupt_capture (uint8_t device_address){
    return (MCP2300x_get_register (device_address, MCP2300x_INTCAP));
}

