/*******************************************************************************

Author : Mauro Laurenti
Version : 1.4

Created on Date : 08/09/2014
Last update     : 28/07/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.


********************************************************
PURPOSES
********************************************************

 This library allows the usage of the MCP4822 DAC.

*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "MCP4822.h"

uint8_t gain_value_DAC_A_MCP4822 = MCP4822_GAIN_2;
uint8_t power_mode_DAC_A_MCP4822 = MCP4822_SHUTDOWN_OFF;

uint8_t gain_value_DAC_B_MCP4822 = MCP4822_GAIN_2;
uint8_t power_mode_DAC_B_MCP4822 = MCP4822_SHUTDOWN_OFF;

//************************************************************
//                  MCP4822_initialize  Implementation
//************************************************************
void MCP4822_initialize (uint8_t clock){

    // Pin initialization
    MCP4822_CS_TRIS = PIN_AS_OUTPUT;
    MCP4822_CS_PORT = MCP4822_CS_DISABLED;

    MCP4822_LDAC_TRIS = PIN_AS_OUTPUT;
    MCP4822_LDAC_PORT = MCP4822_LDAC_DISABLED;

    MCP4822_SPI_mode(SPI_MODE_0);
    MCP4822_SPI_baudrate(clock);
    MCP4822_SPI_open(SPI_MASTER_DEVICE);

    MCP4822_set_options (MCP4822_GAIN_2, MCP4822_SHUTDOWN_ON, MCP4822_DAC_A);
    MCP4822_set_options (MCP4822_GAIN_2, MCP4822_SHUTDOWN_ON, MCP4822_DAC_B);

    MCP4822_set_amplitude (MCP4822_OUT_DEFAULT_VALUE, MCP4822_DAC_A);
    MCP4822_set_amplitude (MCP4822_OUT_DEFAULT_VALUE, MCP4822_DAC_B);
}


//************************************************************
//                  MCP4822_set_value  Implementation
//************************************************************
void MCP4822_set_value (uint16_t value, uint8_t channel){

    uint8_t byte_high = 0;
    uint8_t byte_low = 0;

    byte_high = (uint8_t)((value & 0xFF00) >> 8);
    byte_low = (uint8_t) (value & 0x00FF);

    //Clean the control bits
    byte_high &= 0x0F;

    // Set configurations
    if (channel == MCP4822_DAC_A){
        byte_high |= channel | gain_value_DAC_A_MCP4822 | power_mode_DAC_A_MCP4822;
    } else {
        byte_high |= channel | gain_value_DAC_B_MCP4822 | power_mode_DAC_B_MCP4822;
    }
    
    MCP4822_CS_PORT = MCP4822_CS_ENABLED;

    while(MCP4822_SPI_write_byte (byte_high));
    while(MCP4822_SPI_write_byte (byte_low));

    MCP4822_CS_PORT = MCP4822_CS_DISABLED;
}


//************************************************************
//              MCP4822_set_amplitude  Implementation
//************************************************************
void MCP4822_set_amplitude (uint16_t value, uint8_t channel){

    MCP4822_set_value (value, channel);      
    MCP4822_latch_pulse ();
}


//************************************************************
//              MCP4822_latch_pulse  Implementation
//************************************************************
void MCP4822_latch_pulse (void){

    MCP4822_LDAC_PORT = MCP4822_LDAC_ENABLED;
    delay_ms(1);
    MCP4822_LDAC_PORT = MCP4822_LDAC_DISABLED;
       
}


//************************************************************
//            MCP4822_set_options  Implementation
//************************************************************
void MCP4822_set_options (uint8_t gain, uint8_t power, uint8_t channel){

    if (channel == MCP4822_DAC_A) {
        gain_value_DAC_A_MCP4822 = gain;
        power_mode_DAC_A_MCP4822 = power;
    } else

    if (channel == MCP4822_DAC_B) {
        gain_value_DAC_B_MCP4822 = gain;
        power_mode_DAC_B_MCP4822 = power;
    }
}
