/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
 
Created on Date : 20/10/2019
Last update     : 20/10/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.


*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "OPT3001.h"


//************************************************************
//         OPT3001_initialize function implementation
//************************************************************
void OPT3001_initialize (uint16_t baud_rate_KHz){
    
    OPT3001_I2C_open (I2C_MASTER);
    OPT3001_I2C_baud_rate(baud_rate_KHz);    
}


//************************************************************
//         OPT3001_configuration function implementation
//************************************************************
int8_t OPT3001_configuration (uint8_t device_address, uint16_t configuration){

    return (OPT3001_set_register (device_address, OPT3001_REG_CONFIGURATION, configuration));
}


//************************************************************
//         OPT3001_set_register function implementation
//************************************************************
int8_t OPT3001_set_register (uint8_t device_address, uint8_t register_address, uint16_t value){
    
    uint8_t configuration_byte = device_address << 1;    
    return (OPT3001_I2C_write_word_to_external_device (configuration_byte, register_address, value));
}


//************************************************************
//         OPT3001_set_high_limit function implementation
//************************************************************
int8_t OPT3001_set_high_limit (uint8_t device_address, uint32_t lux){
 
    return (OPT3001_set_register(device_address, OPT3001_REG_HIGH_LIMIT, OPT3001_lux_converter(lux)));
}


//************************************************************
//         OPT3001_set_low_limit function implementation
//************************************************************
int8_t OPT3001_set_low_limit (uint8_t device_address, uint32_t lux){
            
    return (OPT3001_set_register(device_address, OPT3001_REG_LOW_LIMIT, OPT3001_lux_converter(lux)));
}


//************************************************************
//         OPT3001_get_register function implementation
//************************************************************
int8_t OPT3001_get_register (uint8_t device_address, uint8_t register_address, uint16_t * readback_value){
    
    uint16_t value = 0x00;
    int8_t code_error = 0x00;
    
    uint8_t configuration_byte = device_address << 1;
    
    code_error = OPT3001_I2C_read_word_from_external_device (configuration_byte, register_address, &value);
    
    *readback_value = value;
    
    return (code_error);
}


//************************************************************
//     OPT3001_get_device_ID function implementation
//************************************************************
int8_t OPT3001_get_device_ID (uint8_t device_address, uint16_t * device_ID){
    uint16_t value = 0x00;
    int8_t code_error = 0x00;
        
    code_error = OPT3001_get_register (device_address, OPT3001_REG_DEVICE_ID, &value);
    *device_ID = value;
    
    return (code_error);
}

//************************************************************
//     OPT3001_get_lux function implementation
//************************************************************
int8_t OPT3001_get_lux (uint8_t device_address, float * lux){
   
    uint16_t value = 0x00;
    int8_t code_error = 0x00;
    uint8_t exponent = 0;
    uint16_t mantissa = 0;
    float LSB_size = 0;
    uint16_t power_2 = 0;
    
    uint8_t configuration_byte = device_address << 1;
    
    code_error = OPT3001_I2C_read_word_from_external_device (configuration_byte, OPT3001_REG_RESULT, &value);
    
    exponent = (value & 0xF000) >> 12;
    mantissa = (value & 0x0FFF);
    
    // Computing the 2 power exponent oepration
    power_2 = 1 << exponent;

    LSB_size = .01 * power_2;
    
    *lux = LSB_size * mantissa;
    
    return (code_error);
}


//************************************************************
//     OPT3001_lux_converter function implementation
//************************************************************
uint16_t OPT3001_lux_converter (uint32_t lux){
    uint8_t counter = 0;
    uint16_t mantissa = 0;
    uint16_t exponent = 0;
    
    uint16_t register_value = 0;
    
    for (counter = 0; counter < OPT3001_FULL_SCALES_NR; counter++ ){
        if (lux < OPT3001_full_scales[counter]){
            exponent = counter;
            //I force the exit from the for
            counter = OPT3001_FULL_SCALES_NR;
        }
    }  
    
   mantissa = (uint16_t) (lux / (0.01 * (1 << exponent)));
          
   register_value = mantissa | (exponent << 12);
   
   return (register_value);        
}
