/*******************************************************************************

Author : Mauro Laurenti
Version : 1.5
 
Created on Date : 19/03/2011
Last update     : 29/07/2019
 
CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/
#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "PCF8574.h"


//************************************************************
//         PCF8574_initialize function implementation
//************************************************************
void PCF8574_initialize (uint16_t baud_rate_KHz){

    PCF8574_I2C_open(I2C_MASTER);
    PCF8574_I2C_baud_rate(baud_rate_KHz);
}


//************************************************************
//         PCF8574_write_data function implementation
//************************************************************
int8_t PCF8574_write_data (uint8_t device_address, uint8_t data ){
		
//*****************************
// Start Condition and control
// Byte are sent
//*****************************		

	PCF8574_I2C_wait_bus_IDLE();   
	             
	PCF8574_I2C_start_bit();                     
	
  	if (PCF8574_I2C_check_bus_collision()) {
        return (-1);                 
    }

	if (PCF8574_I2C_write_byte(PCF8574_INTERNAL_ADDRESS | (device_address << 1))){
        return (-1);             
    }
     
 
//*****************************
// Data Byte is sent
//*****************************   
 
    PCF8574_I2C_wait_bus_IDLE();  
      
    if (!PCF8574_I2C_check_ACK ()){
	    
        if (PCF8574_I2C_write_byte(data)) {
            return (-1);       
        }
    } else {
        return (-1);            
    }
    
//*****************************
// Stop command is sent
//*****************************

	PCF8574_I2C_wait_bus_IDLE() 
        ;   	
                   
  	if (!PCF8574_I2C_check_ACK ()) {
	
        PCF8574_I2C_stop_bit();    
  
    } else {
  		return (-1);           
    }
  
  if (PCF8574_I2C_check_bus_collision ()){
     return (-1);                
  }
  
  return (0);                   
}


//************************************************************
//           PCF8574_read_data function implementation
//************************************************************
int8_t PCF8574_read_data (uint8_t device_address, uint8_t *data){
	
    uint8_t control_byte;

    control_byte = PCF8574_INTERNAL_ADDRESS | (device_address << 1);
    
//*****************************
// Start Condition and control
// Byte are sent
//*****************************

	PCF8574_I2C_wait_bus_IDLE();    
              
	PCF8574_I2C_start_bit(); 
  
	if (PCF8574_I2C_check_bus_collision ()) {
		return (-1);               
	}
	
	if (PCF8574_I2C_write_byte(control_byte + 1)){
      return (-1);             
	} 

//*****************************
// Data is Read
//*****************************

	PCF8574_I2C_wait_bus_IDLE ()
        ;              
       
    if (!PCF8574_I2C_check_ACK ()){

        // 1 byte reception
        PCF8574_I2C_set_master_as_receiver ();

        // Send not ACK condition 
        PCF8574_I2C_negative_ACK(); 

        PCF8574_I2C_stop_bit ();

        if (PCF8574_I2C_check_bus_collision ()) {
            return (-1);        
        }         

    } else {
        return (-1);            
    }   
	
	*data = PCF8574_I2C_read_byte ();
 
	return (0);
}
