/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
 
Created on Date : 18/10/2019
Last update     : 18/10/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.


*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "TMP1075.h"

//************************************************************
//         TMP1075_initialize function implementation
//************************************************************
void TMP1075_initialize (uint16_t baud_rate_KHz){
    
    TMP1075_I2C_open (I2C_MASTER);
    TMP1075_I2C_baud_rate(baud_rate_KHz);    
}


//************************************************************
//         TMP1075_configuration function implementation
//************************************************************
int8_t TMP1075_configuration (uint8_t device_address, uint16_t configuration){

    return (TMP1075_set_register (device_address, TMP1075_REG_CONFIGURATION, configuration));
}


//************************************************************
//         TMP1075_set_register function implementation
//************************************************************
int8_t TMP1075_set_register (uint8_t device_address, uint8_t register_address, uint16_t value){
    
    uint8_t configuration_byte = device_address << 1;    
    return (TMP1075_I2C_write_word_to_external_device (configuration_byte, register_address, value));
}


//************************************************************
//         TMP1075_set_high_limit function implementation
//************************************************************
int8_t TMP1075_set_high_limit (uint8_t device_address, int16_t temperature){
      
    //4 bits shift is required since the 12 bits value is left justified
    return (TMP1075_set_register(device_address, TMP1075_REG_HIGH_LIMIT, (uint16_t)(temperature << 8)));
}


//************************************************************
//         TMP1075_set_low_limit function implementation
//************************************************************
int8_t TMP1075_set_low_limit (uint8_t device_address, int16_t temperature){
            
    //4 bits shift is required since the 12 bits value is left justified
    return (TMP1075_set_register(device_address, TMP1075_REG_LOW_LIMIT, (uint16_t)(temperature << 8)));
}


//************************************************************
//         TMP1075_get_register function implementation
//************************************************************
int8_t TMP1075_get_register (uint8_t device_address, uint8_t register_address, uint16_t * readback_value){
    
    uint16_t value = 0x00;
    int8_t code_error = 0x00;
    
    uint8_t configuration_byte = device_address << 1;
    
    code_error = TMP1075_I2C_read_word_from_external_device (configuration_byte, register_address, &value);
    
    *readback_value = value;
    
    return (code_error);
}


//************************************************************
//     TMP1075_get_temperature function implementation
//************************************************************
int8_t TMP1075_get_temperature (uint8_t device_address, uint16_t * temperature){
   
    uint16_t value = 0x00;
    int8_t code_error = 0x00;
    
    uint8_t configuration_byte = device_address << 1;
    
    code_error = TMP1075_I2C_read_word_from_external_device (configuration_byte, TMP1075_REG_TEMPERATURE_RESULTS, &value);
    
    // 4 shift are becuase the read back has the 12bits left justified.
    // 4 shift are to ignore the decimal part of the temperature.
    *temperature = (int16_t) (value >> 8);
    
    return (code_error);
}

