/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
 
Created on Date : 14/04/2023
Last update     : 14/04/2023

CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This library contains all the functions that are handy to use a touch panel.
Currently 4 wires resestive is supported.				
 	

*******************************************************************************/
#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "Touch.h"

//************************************************************
//      variables
//************************************************************

point Touch_Top_Right;
point Touch_Top_Left;
point Touch_Bottom_Right;
point Touch_Bottom_Left;

calibration X_coordinate;
calibration Y_coordinate;

uint16_t Touch_average = TOUCH_AVERAGING_16;


//************************************************************
//      Touch_initialize implementation
//************************************************************
void Touch_initialize (uint16_t x_size, uint16_t y_size ){
    
    TOUCH_BUZZER_PIN_DIR = TOUCH_OUTPUT_PIN;   
    
    TOUCH_RIGHT_PULL_UP_R = TOUCH_PULL_UP_OFF;
    TOUCH_BOTTOM_PULL_UP_R = TOUCH_PULL_UP_OFF;
        
    //The direction of the following pins changes during
    //the touch detections
    Touch_reset_pins ();
    
    Touch_Top_Right.X = 0;
    Touch_Top_Right.Y = 0;
    
    Touch_Top_Left.X = 0;
    Touch_Top_Left.Y = 0;
    
    Touch_Bottom_Right.X = 0;
    Touch_Bottom_Right.Y = 0;
    
    Touch_Bottom_Left.X = 0;
    Touch_Bottom_Left.Y = 0;
    
    X_coordinate.gain = 0;
    X_coordinate.offset = 0;
    
    Y_coordinate.gain = 0;
    Y_coordinate.offset = 0;
    
}


//************************************************************
//      Touch_calibrate implementation
//************************************************************
void Touch_calibrate (void){
    
    Display_clear (DISPLAY_FILLING_WHITE);  

    Display_write_message (10,18,"Press Top - Right");
    Display_write_message (10,28,"     corner.");
    Display_write_char (114,5,'+');

    Touch_calibration_point (TOUCH_TOP_RIGHT);

    while (Touch_is_pressed() == TOUCH_IS_PRESSED);

    Display_clear (DISPLAY_FILLING_WHITE);     

    Display_write_message (8,28,"Press Bottom - Right");
    Display_write_message (8,38,"     corner.");
    Display_write_char (114,57,'+');

    Touch_calibration_point (TOUCH_BOTTOM_RIGHT);

    while (Touch_is_pressed() == TOUCH_IS_PRESSED);

    Display_clear (DISPLAY_FILLING_WHITE);

    Display_write_message (10,28,"Press Top - Left");
    Display_write_message (10,38,"     corner.");
    Display_write_char (12,5,'+');

    Touch_calibration_point (TOUCH_TOP_LEFT);

    while (Touch_is_pressed() == TOUCH_IS_PRESSED);

    Display_clear (DISPLAY_FILLING_WHITE);

    Display_write_message (8,28,"Press Bottom - Left");
    Display_write_message (8,38,"     corner.");
    Display_write_char (12,57,'+');

    Touch_calibration_point (TOUCH_BOTTOM_LEFT);

    while (Touch_is_pressed() == TOUCH_IS_PRESSED);
       
    Display_clear (DISPLAY_FILLING_WHITE);
    
    Touch_correction();

}

//************************************************************
//      Touch_is_pressesed implementation
//************************************************************
uint8_t Touch_is_pressed (void){
    
    uint32_t raw_data = 0;
    uint8_t counter = 0;
    
    TOUCH_RIGHT_PULL_UP_R = TOUCH_PULL_UP_ON;
    TOUCH_BOTTOM_PULL_UP_R = TOUCH_PULL_UP_OFF;
    
    TOUCH_LEFT_PIN_DIR = TOUCH_INPUT_PIN;
    TOUCH_RIGHT_PIN_DIR = TOUCH_INPUT_PIN;
    TOUCH_BOTTOM_PIN_DIR = TOUCH_OUTPUT_PIN;
    TOUCH_TOP_PIN_DIR = TOUCH_OUTPUT_PIN;  
       
    TOUCH_BOTTOM_PIN = TOUCH_PIN_TO_GND;
    TOUCH_TOP_PIN = TOUCH_PIN_TO_VCC;
    
    delay_ms(TOUCH_DEBOUNCE_TIME_ms);

    //Other positions could have been fine for the pressure detection.
    for (counter = 0; counter < Touch_average; counter++){
        raw_data = raw_data + ADC_read_channel(RIGHT_ADC_CH);
    }
    
    raw_data = raw_data / Touch_average;
    
    //Button is pressed if the signal amplitude is smaller than the Threshold
    if (raw_data > TOUCH_Y_THRESHOLD){
        return (TOUCH_IS_NOT_PRESSED);      
    }else {
//        Touch_beep (1);
        return (TOUCH_IS_PRESSED);
    }            
    
}



//************************************************************
//      Touch_measure_raw implementation
//************************************************************
point Touch_measure_raw (uint8_t averaging){
    
    uint32_t raw_sum = 0;
    uint8_t counter = 0;
    static point touch;
    
    touch.X = 0;
    touch.Y = 0;
    
    TOUCH_RIGHT_PULL_UP_R = TOUCH_PULL_UP_OFF;
    TOUCH_BOTTOM_PULL_UP_R = TOUCH_PULL_UP_ON;
    
    TOUCH_LEFT_PIN_DIR = TOUCH_OUTPUT_PIN; 
    TOUCH_RIGHT_PIN_DIR = TOUCH_OUTPUT_PIN;
    TOUCH_BOTTOM_PIN_DIR = TOUCH_INPUT_PIN;
    TOUCH_TOP_PIN_DIR = TOUCH_INPUT_PIN;   
    
    TOUCH_LEFT_PIN = TOUCH_PIN_TO_GND;
    TOUCH_RIGHT_PIN = TOUCH_PIN_TO_VCC;
    
    delay_ms(TOUCH_DEBOUNCE_TIME_ms);

    raw_sum = 0;
    
    for (counter=0; counter < averaging; counter++){
        raw_sum = raw_sum + ADC_read_channel(BOTTOM_ADC_CH);
    }
    
    touch.X = raw_sum / averaging;
    
    TOUCH_RIGHT_PULL_UP_R = TOUCH_PULL_UP_ON;
    TOUCH_BOTTOM_PULL_UP_R = TOUCH_PULL_UP_OFF;
    
    TOUCH_LEFT_PIN_DIR = TOUCH_INPUT_PIN;
    TOUCH_RIGHT_PIN_DIR = TOUCH_INPUT_PIN;
    TOUCH_BOTTOM_PIN_DIR = TOUCH_OUTPUT_PIN;
    TOUCH_TOP_PIN_DIR = TOUCH_OUTPUT_PIN; 
    
    TOUCH_BOTTOM_PIN = TOUCH_PIN_TO_GND;
    TOUCH_TOP_PIN = TOUCH_PIN_TO_VCC;

    delay_ms(TOUCH_DEBOUNCE_TIME_ms);
    
    raw_sum = 0;
    
    for (counter=0; counter < averaging; counter++){
        raw_sum = raw_sum + ADC_read_channel(RIGHT_ADC_CH);
    }
    
    touch.Y = raw_sum / averaging;
    
    return (touch);  

}


//************************************************************
//      Touch_calibration_point implementation
//************************************************************
uint8_t Touch_calibration_point (uint8_t point_to_calibrate){
    
    point touch;

    while (Touch_is_pressed() == TOUCH_IS_NOT_PRESSED);
    
    touch = Touch_measure_raw (TOUCH_AVERAGING_16);

    
    if (point_to_calibrate == TOUCH_TOP_RIGHT) {
        
        Touch_Top_Right.X = touch.X;
        Touch_Top_Right.Y = touch.Y;
        
        Touch_reset_pins ();
        return (1);
    }
    
    if (point_to_calibrate == TOUCH_TOP_LEFT) {
        
       Touch_Top_Left.X = touch.X;
       Touch_Top_Left.Y = touch.Y;
       Touch_reset_pins ();
       return (1); 
    }
    
    if (point_to_calibrate == TOUCH_BOTTOM_RIGHT) {
       
        Touch_Bottom_Right.X = touch.X;
        Touch_Bottom_Right.Y = touch.Y;
        Touch_reset_pins ();
        return (1); 
    }
    
    if (point_to_calibrate == TOUCH_BOTTOM_LEFT) {
        
        Touch_Bottom_Left.X = touch.X;
        Touch_Bottom_Left.Y = touch.Y;
        Touch_reset_pins ();
        return (1);
    }

    return (0);
}


//************************************************************
//      Touch_reset_pins implementation
//************************************************************
void Touch_reset_pins (void){
    TOUCH_TOP_PIN_DIR = TOUCH_INPUT_PIN;
    TOUCH_BOTTOM_PIN_DIR = TOUCH_INPUT_PIN;
    TOUCH_LEFT_PIN_DIR = TOUCH_INPUT_PIN;
    TOUCH_RIGHT_PIN_DIR = TOUCH_INPUT_PIN;
        
    delay_ms(TOUCH_DEBOUNCE_TIME_ms);
}


//************************************************************
//      Touch_beep implementation
//************************************************************
void Touch_beep (uint8_t count){
   
    uint8_t counter = 0; 
    uint8_t pulses = 0;
    
    for (counter= 0; counter < count; counter++){
     
        
        for (pulses=0; pulses < TOUCH_BUZZER_PULSES ; pulses++){
            TOUCH_BUZZER_PIN = 0x01;    
            delay_us (TOUCH_BUZZER_T_ON_us);
    
            TOUCH_BUZZER_PIN = 0x00;        
            delay_us (TOUCH_BUZZER_T_OFF_us);
        }
        
    }    
}


//************************************************************
//      Touch_measure implementation
//************************************************************
point Touch_measure (uint8_t averaging){
    uint32_t raw_sum = 0;
    uint8_t counter = 0;
    
    point touch_raw;
    static point touch_calibrated;
   
    touch_raw.X=0;
    touch_raw.Y=0;
    
    if (Touch_is_pressed() == TOUCH_IS_NOT_PRESSED){
        return(touch_raw);
    }
   
    TOUCH_RIGHT_PULL_UP_R = TOUCH_PULL_UP_OFF;
    TOUCH_BOTTOM_PULL_UP_R = TOUCH_PULL_UP_ON;
    
    TOUCH_LEFT_PIN_DIR = TOUCH_OUTPUT_PIN; 
    TOUCH_RIGHT_PIN_DIR = TOUCH_OUTPUT_PIN;
    TOUCH_BOTTOM_PIN_DIR = TOUCH_INPUT_PIN;
    TOUCH_TOP_PIN_DIR = TOUCH_INPUT_PIN;   
    
    TOUCH_LEFT_PIN = TOUCH_PIN_TO_GND;
    TOUCH_RIGHT_PIN = TOUCH_PIN_TO_VCC;
    

    for (counter=0; counter < averaging; counter++){
        raw_sum = raw_sum + ADC_read_channel(BOTTOM_ADC_CH);
    }
    
    touch_raw.X = raw_sum / averaging;
    
    TOUCH_RIGHT_PULL_UP_R = TOUCH_PULL_UP_ON;
    TOUCH_BOTTOM_PULL_UP_R = TOUCH_PULL_UP_OFF;
    
    TOUCH_LEFT_PIN_DIR = TOUCH_INPUT_PIN;
    TOUCH_RIGHT_PIN_DIR = TOUCH_INPUT_PIN;
    TOUCH_BOTTOM_PIN_DIR = TOUCH_OUTPUT_PIN;
    TOUCH_TOP_PIN_DIR = TOUCH_OUTPUT_PIN; 
    
    TOUCH_BOTTOM_PIN = TOUCH_PIN_TO_GND;
    TOUCH_TOP_PIN = TOUCH_PIN_TO_VCC;
   
    raw_sum = 0;
    
    for (counter=0; counter < averaging; counter++){
        raw_sum = raw_sum + ADC_read_channel(RIGHT_ADC_CH);
    }
    
    touch_raw.Y  = raw_sum / averaging;

    touch_calibrated.X = (uint16_t) ((touch_raw.X - X_coordinate.offset)/ X_coordinate.gain);
    touch_calibrated.Y = (uint16_t) ((touch_raw.Y - Y_coordinate.offset)/Y_coordinate.gain);
    
    if (touch_calibrated.X < 0) touch_calibrated.X = 0;
    if (touch_calibrated.X > TOUCH_X_MAX-1) touch_calibrated.X = TOUCH_X_MAX-1;
    
    if (touch_calibrated.Y < 0) touch_calibrated.Y = 0;
    if (touch_calibrated.Y > (TOUCH_Y_MAX -1)) touch_calibrated.Y = TOUCH_Y_MAX - 1;
    
    touch_calibrated.Y  = TOUCH_Y_MAX - touch_calibrated.Y;
    
    return (touch_calibrated);
}


//************************************************************
//      Touch_correction implementation
//************************************************************
uint8_t Touch_correction (void){
    
    point Tollerance;
    
    //The tollerance is compensating the margin that separate the x of the touch point 
    //from the display border
    Tollerance.X = (uint16_t) (TOUCH_X_MAX * (MARGIN/100));
    Tollerance.Y = (uint16_t) (TOUCH_Y_MAX * (MARGIN/100));
    
    X_coordinate.gain =  (Touch_Top_Right.X - Touch_Top_Left.X) / (TOUCH_X_MAX ) ;
    Y_coordinate.gain = (Touch_Top_Right.Y - Touch_Bottom_Right.Y) / (TOUCH_Y_MAX);
    
    X_coordinate.offset = Touch_Top_Left.X ;    
    Y_coordinate.offset = Touch_Bottom_Right.Y ;
    
    return (1);
}


//************************************************************
//      Touch_export_calibration implementation
//************************************************************
void Touch_export_calibration (point *Top_Right, point *Top_Left, point *Bottom_Right, point *Bottom_Left){
    
    Top_Right = &Touch_Top_Right;
    Top_Left = &Touch_Top_Left;
    Bottom_Right = &Touch_Bottom_Right;
    Bottom_Left = &Touch_Bottom_Left;
    
}


//************************************************************
//      Touch_import_calibration implementation
//************************************************************
void Touch_import_calibration (point Top_Right, point Top_Left, point Bottom_Right, point Bottom_Left){
    
    Touch_Top_Right.X = Top_Right.X;
    Touch_Top_Right.Y = Top_Right.Y;
    
    Touch_Top_Left.X = Top_Left.X;
    Touch_Top_Left.Y = Top_Left.Y;
    
    Touch_Bottom_Right.X = Bottom_Right.X;
    Touch_Bottom_Right.Y = Bottom_Right.Y;
    
    Touch_Bottom_Left.X = Bottom_Left.X;
    Touch_Bottom_Left.Y = Bottom_Left.Y;
    
    Touch_correction();
}

