/*******************************************************************************

Author : Mauro Laurenti
Version : 1.1

Created on Date : 15/11/2018
Last update     : 25/07/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/
									

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "WS2812B.h"


//************************************************************
//                  WS2812_initialize  Implementation
//************************************************************
void WS2812_initialize (void){

    LED_GROUP_TRIS = 0x00;
    
    //Reset all the groups
    WS2812_reset_LED(0xFF);
}


//************************************************************
//               WS2812_reset_LED  Implementation
//************************************************************
void WS2812_reset_LED (uint8_t group){
    
    LED_PORT &= ~group;
    
    //wait more than 50us
    delay_ms (1);

}


//************************************************************
//             WS2812_turn_on_LED  Implementation
//************************************************************
void WS2812_turn_on_LED (uint8_t group, uint8_t red_color, uint8_t green_color, uint8_t blue_color){
    uint8_t counter = 0;
    uint8_t colors = 0;
    uint8_t color_to_send = 0;
    uint8_t bit_value = 0;
    
    for (colors = 0; colors < COLOR_COUNT; colors++){
        
        LED_PORT = 0x00;
        
        if (colors==0) color_to_send = green_color;
        if (colors==1) color_to_send = red_color;
        if (colors==2) color_to_send = blue_color;
        
        bit_value =  color_to_send & 0x01;
        
        for (counter=0; counter<COLOR_DEPTH; counter++){

            if (bit_value == 1){
                //Set to 1 the group of interest
                LED_PORT = group;

                //wait 0.85us
                #ifdef COMPILER_XC8
                    __nop();
                    __nop();
                #endif 
                    
                #ifdef COMPILER_XC32
                    __asm("NOP");
                    __asm("NOP");
                    __asm("NOP");
                    __asm("NOP");
                    __asm("NOP");
                    __asm("NOP");
                    __asm("NOP");
                    __asm("NOP");
                    __asm("NOP");
                    __asm("NOP");
                    __asm("NOP");
                    __asm("NOP");
                #endif    

                //Set to 0 all the groups    
                LED_PORT = 0;  

                //wait 0.4us
                #ifdef COMPILER_XC8
                    __nop();
                #endif

                #ifdef COMPILER_XC32
                    __asm("NOP");
                #endif     
                
            }  else {
                //Set to 1 the group of interest
                LED_PORT = group;

                //wait 0.4us
                #ifdef COMPILER_XC32
                    __asm("NOP");
                    __asm("NOP");
                    __asm("NOP");
                    __asm("NOP");
                    __asm("NOP");
                    __asm("NOP");
                #endif    

                //Set to 0 all the groups 
                LED_PORT = 0; 

                //wait 0.85us
                #ifdef COMPILER_XC8
                    __nop();
                    __nop();
                #endif 

                #ifdef COMPILER_XC32
                    __asm("NOP");
                #endif 
            }
            
            bit_value =  (color_to_send >> counter) & 0x01;
        }
    }    
}


//************************************************************
//          WS2812_turn_on_group  Implementation
//************************************************************
void WS2812_turn_on_group (uint8_t group, uint8_t red_color, uint8_t green_color, uint8_t blue_color){
    
    uint8_t group_counter = 0;
    uint8_t group_selection = 0;
    
    WS2812_reset_LED (group);
    
    if (group == LED_GROUP_1) group_selection = 0;
    if (group == LED_GROUP_2) group_selection = 1;
    if (group == LED_GROUP_3) group_selection = 2;
    if (group == LED_GROUP_4) group_selection = 3;
    if (group == LED_GROUP_5) group_selection = 4;
    if (group == LED_GROUP_6) group_selection = 5;
    if (group == LED_GROUP_7) group_selection = 6;
    if (group == LED_GROUP_8) group_selection = 7;
    
    for (group_counter=0; group_counter < group_size[group_selection]; group_counter++){
        
        WS2812_turn_on_LED (group, red_color, green_color, blue_color);
    }    
}


//************************************************************
//            WS2812_turn_off_group  Implementation
//************************************************************
void WS2812_turn_off_group (uint8_t group){
    
    WS2812_turn_on_group (group,0,0,0);
}
