/*******************************************************************************

Author : Mauro Laurenti
Version : 1.1
Created on Date : 11/03/2023
Last update     : 23/02/2024

CopyRight 2006-2024 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "display_utilities.h"



//*********************************************
//        Display_draw_vertical_line
//*********************************************
void Display_draw_vertical_line (uint8_t x_origin, uint8_t y_origin, uint8_t length, uint8_t filling_color){

	uint8_t i;

	for (i = 0 ; i<length; i++) {
		Display_plot_xy (x_origin, y_origin + i, filling_color);
	}
}


//*********************************************
//       Display_draw_horizontal_line
//*********************************************
void Display_draw_horizontal_line (uint8_t x_origin, uint8_t y_origin, uint8_t length, uint8_t filling_color){

	uint8_t i;

	for (i = 0 ; i<length; i++) {
		Display_plot_xy (x_origin + i, y_origin, filling_color);
	}
}


//*********************************************
//         Display_draw_line
//*********************************************
void Display_draw_line (uint8_t x_origin, uint8_t y_origin, uint8_t x_end, uint8_t y_end, uint8_t filling_color){
    float m;
    float q;
    uint8_t y = 0;
    uint8_t x = 0;
    
    m = ((float)( y_end - y_origin)) / (x_end - x_origin);
            
    q = ((float)((x_end * y_origin)-(x_origin * y_end))) / (x_end - x_origin);
    
    for (x = x_origin; x <x_end; x++ )
    {
      y = (uint8_t) ((m * x)  + q);
      Display_plot_xy (x, y, filling_color);
    }
}


//*********************************************
//          Display_draw_window
//*********************************************
void Display_draw_window (uint8_t x_origin, uint8_t y_origin, uint8_t width, uint8_t height, uint8_t filling_color){

	Display_draw_horizontal_line (x_origin, y_origin, width, filling_color);
	// + 1 is required to close the window
	Display_draw_horizontal_line (x_origin, y_origin + height, width+1, filling_color);

	Display_draw_vertical_line (x_origin, y_origin, height, filling_color);
	Display_draw_vertical_line (x_origin + width, y_origin, height, filling_color);

}


//*********************************************
//          Display_draw_box
//*********************************************

void Display_draw_box (uint8_t x_origin, uint8_t y_origin, uint8_t length, uint8_t height, uint8_t filling_color){

	uint8_t i;

	for (i=0; i < length; i++)
		Display_draw_vertical_line (x_origin +i, y_origin, height, filling_color);

}

//*********************************************
//          Display_draw_button
//*********************************************
void Display_draw_button (uint8_t x_origin, uint8_t y_origin, uint8_t length, uint8_t height, const char *text, uint8_t filling_color){
    
    uint8_t i;
    uint8_t text_length = 0;
    uint8_t X_text_offset = 0;
    uint8_t Y_text_offset = 0;
    uint8_t font_color_holder;
    

	for (i=0; i < length; i++)
		Display_draw_vertical_line (x_origin +i, y_origin, height, filling_color);
    
    //remove corners
    
    //Left upper corner
    Display_plot_xy(x_origin,y_origin, ~filling_color);
    Display_plot_xy(x_origin +1 ,y_origin, ~filling_color);
    Display_plot_xy(x_origin,y_origin + 1, ~filling_color);
    
    //Right upper corner
    Display_plot_xy(x_origin + length -1,y_origin, ~filling_color);
    Display_plot_xy(x_origin + length -2 ,y_origin, ~filling_color);
    Display_plot_xy(x_origin + length -1, y_origin + 1, ~filling_color);
    
    //Left bottom corner
    Display_plot_xy(x_origin,y_origin + height -1, ~filling_color);
    Display_plot_xy(x_origin,y_origin + height -2, ~filling_color);
    Display_plot_xy(x_origin +1,y_origin + height - 1, ~filling_color);
    
    //Right bottom corner
    Display_plot_xy(x_origin + length -1,y_origin + height -1, ~filling_color);
    Display_plot_xy(x_origin + length -1,y_origin + height -2, ~filling_color);
    Display_plot_xy(x_origin + length -2,y_origin + height - 1, ~filling_color);
    
    text_length = LTlib_string_length ((uint8_t *)text);
    X_text_offset =  (uint8_t )((length - (text_length * Display_font_width ) - (text_length -1))/ 2) ;
    Y_text_offset = (uint8_t )((height - Display_font_height)/2) ;
    
    font_color_holder = Display_font_color;
    
    Display_set_font_color(~filling_color);
    
    Display_write_message(x_origin + X_text_offset,y_origin + Y_text_offset, text);
    
    Display_set_font_color(font_color_holder);
}


//*********************************************
//           Display_write_char
//*********************************************
void Display_write_char (uint8_t x_pos, uint8_t y_pos, uint8_t character){

	uint8_t i;
	uint8_t data = 0;
	uint8_t x;
	uint8_t bit_value;

	for (i = 0 ; i<Display_font_width; i++){

        //Select the font table
        if ((Display_font_width == DISPLAY_FONT_WIDTH_4)&& (Display_font_height == DISPLAY_FONT_HEIGTH_5 )){
            data = font_4x5[character - 32][i];
        }
        
        if ((Display_font_width == DISPLAY_FONT_WIDTH_5)&& (Display_font_height == DISPLAY_FONT_HEIGTH_7 )){
            data = font_5x7[character - 32][i];
        }        
                
        if ((Display_font_width == DISPLAY_FONT_WIDTH_7)&& (Display_font_height == DISPLAY_FONT_HEIGTH_7 )){
            data = font_7x7[character - 32][i];
        }
        
        //*********************************************************************

        if (Display_font_color == DISPLAY_FILLING_WHITE){
            data = ~data;
        }
            
        
		for (x = 0 ; x<= Display_font_height; x++) {
			
			if (data & (0x01 << x))
				bit_value = DISPLAY_FILLING_BLACK;
			else 
				bit_value = DISPLAY_FILLING_WHITE;

			Display_plot_xy (x_pos + i, y_pos + x, bit_value);
			
		}			
	}	
}

//*********************************************
//          Display_write_string
//*********************************************
void Display_write_string (uint8_t x_pos, uint8_t y_pos, uint8_t * character_array){

uint8_t i = 0;

	while (character_array[i]) {
		
		Display_write_char (x_pos + (i* (Display_font_width+ DISPLAY_FONT_SPACE)) , y_pos,character_array[i]);
	
		i++;
	}
}


//*********************************************
//           Display_write_message
//*********************************************
void Display_write_message (uint8_t x_pos, uint8_t y_pos, const char *character) {

    uint8_t i = 0;

    while (character[i]) {

        Display_write_char (x_pos + (i*(Display_font_width + DISPLAY_FONT_SPACE)), y_pos,character[i]);

        i++;
    }
}


//*********************************************
//          Display_draw_picture
//*********************************************
void Display_draw_picture (const uint8_t  * picture_table){

	uint8_t page = 0;
	uint8_t column = 0;
	
	for (page = 0; page < DISPLAY_NUMBER_OF_PAGES; page++) {

            Display_set_column (0);
            Display_set_page (page);

            for (column = 0; column < DISPLAY_X_RESOLUTION; column++) {

                    Display_write_data (* picture_table);
                    picture_table++;
            }
    }
}


//*********************************************
//             Display_draw_icon
//*********************************************
void Display_draw_icon (uint8_t x_pos, uint8_t y_pos, uint8_t x_size, uint8_t y_size, const uint8_t * icon_table){

    uint8_t x_scan = 0;
    uint8_t y_scan = 0;
    uint8_t byte_scan = 0;
    uint8_t byte_to_write = 0;
    
    uint8_t y_offset = 0;
    
    for (y_scan = 0; y_scan < (y_size >> 3) ; y_scan++) {
        
        y_offset = y_scan*8;
        
        for (x_scan = 0; x_scan < x_size; x_scan++) {

            byte_to_write = * icon_table;
            
            for (byte_scan = 0; byte_scan < 8; byte_scan++){
                
                if (byte_to_write & (0x01 << byte_scan)) {
                    Display_plot_xy(x_pos + x_scan, y_pos + y_offset + byte_scan, DISPLAY_FILLING_BLACK);
                }               
            }

            icon_table++;    
        }
    }      
}


//*********************************************
//          Display_write_integer
//*********************************************
void Display_write_integer (uint8_t x_pos, uint8_t y_pos, int16_t value, uint8_t number_of_digits, uint8_t zero_cleaning){

	// The array size is 6, plus end of string '\0'
	uint8_t convertedInt [7];

	uint8_t string_length;
    uint8_t counter;
    uint8_t shift_counter;
	
	// Integer is converted to string
    LTlib_int16_to_string (&convertedInt[0], value);
    
    string_length = LTlib_string_length (&convertedInt[0]);

    //Make the string as long as needed by appending 0 on the left
    for (counter = 0; counter < (number_of_digits - string_length); counter++){
        //shift right as needed by adding 0
        for (shift_counter = number_of_digits; shift_counter > 0; shift_counter--){
            convertedInt[shift_counter] = convertedInt[shift_counter-1];
        }   
        
        //After each shift a 0 is inserted for positive numbers
        //For negative numbers and empty space is added
        if (convertedInt[0] == '-' || convertedInt[0] == ' '){
            convertedInt[0] = ' ';
        } else {
            convertedInt[0] = '0';
        }        
    }    
    
    string_length = LTlib_string_length (&convertedInt[0]);
    
    //remove the 0 on the left, if required
    if (zero_cleaning == DISPLAY_ZERO_CLEANING_ON) {       
        //I keep at least one 0, thus I do not check the entire string length
        for (counter = 0; counter < string_length; counter++){
            if (convertedInt[counter] == '0'){
                convertedInt[counter] = ' ';
            } else {
                //setting the counter to this value will terminate the for.
                counter = string_length;
            }   
        }  
    }  

	Display_write_string (x_pos, y_pos, convertedInt);
}


//*********************************************
//          Display_write_float
//*********************************************
void Display_write_float (uint8_t x_pos, uint8_t y_pos, float value, uint8_t number_of_digits, uint8_t number_of_fractional_digits, uint8_t zero_cleaning){
  
    uint8_t string_length;
    uint8_t counter;
    uint8_t shift_counter;
    
    uint8_t string_result[MAX_NUM_DIGITS];
    
    LTlib_float_to_string (string_result, value, number_of_fractional_digits);
    
    string_length = LTlib_string_length (&string_result[0]);
    
    //Make the string as long as needed by appending 0 on the left
    for (counter = 0; counter < (number_of_digits - string_length); counter++){
        //shift right as needed by adding 0
        for (shift_counter = number_of_digits; shift_counter > 0; shift_counter--){
            string_result[shift_counter] = string_result[shift_counter-1];
        }   
        
        //After each shift a 0 is inserted for positive numbers
        //For negative numbers and empty space is added
        if (string_result[0] == '-' || string_result[0] == ' '){
            string_result[0] = ' ';
        } else {
            string_result[0] = '0';
        }        
    }    
    
    string_length = LTlib_string_length (&string_result[0]);
    
    //remove the 0 on the left, if required
    if (zero_cleaning == DISPLAY_ZERO_CLEANING_ON) {       
        //I keep at least one 0, thus I do not check the entire string length
        for (counter = 0; counter < string_length; counter++){
            
            if (string_result[counter+1] == '.'){
                //setting the counter to this value will terminate the for.
                counter = string_length;
            } else {
            
                if (string_result[counter] != '.'){

                    if (string_result[counter] == '0'){
                        string_result[counter] = ' ';
                    } else {
                        //setting the counter to this value will terminate the for.
                        counter = string_length;
                    }   
                }
            }
        }  
    }  
    
    Display_write_string ( x_pos,y_pos, string_result);
}

