/*******************************************************************************

Author : Mauro Laurenti
Version : 1.1
Created on Date : 06/10/2017
Last update     : 28/07/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This library contains all the functions that are handy for controlling 			
matrix keypads with either 4x3 or 4x4 formats. 						
 	

*******************************************************************************/
#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "keypad.h"


//************************************************************
//      keypad_initialization function implementation
//************************************************************
void keypad_initialization (){
    
    #ifdef KEYPAD_4x3
        IO_set_port_direction(KEYPAD_PORT, 0xF1);
    #endif  

    #ifdef KEYPAD_4x4
        IO_set_port_direction(KEYPAD_PORT, 0xF0);
    #endif
    
    #ifdef PULL_UP_RESISTOR_ON 
        IO_enable_pull_up_resistors(KEYPAD_PORT, 0xFF);
    #endif

    #ifdef PULL_UP_RESISTOR_OFF 
        IO_enable_pull_up_resistors(KEYPAD_PORT, 0x00);
    #endif
}


//************************************************************
//      keypad_read implementation
//************************************************************
uint8_t keypad_read (uint8_t debaunce){
    uint8_t counter = 0x00;
    uint8_t selected_column = 0b11110111;
    uint8_t raw_data = KEYPAD_KEY_NOT_PRESSED;
    uint8_t raw_button = KEYPAD_KEY_NOT_PRESSED;
    
    //Monitor all the columns for a pressed button
    for (counter=0; counter < KEYPAD_COLUMNS ; counter ++){
        IO_write_port(KEYPAD_PORT, selected_column);
        
        raw_data = IO_read_port(KEYPAD_PORT);
        raw_data = raw_data & 0xF0;
        
        if (raw_data != 0xF0){
            raw_button = raw_data | (selected_column & 0x0F);               
        } else {
            selected_column = selected_column >> 1;
        }
    }
    
    //Filter out the spikes, if required
    if (debaunce == DEBAUNCE_ON){
        delay_ms(10);
        IO_write_port(KEYPAD_PORT, raw_button); 
        raw_data = IO_read_port(KEYPAD_PORT);
        if ( raw_button != raw_data){
            raw_button = KEYPAD_KEY_NOT_PRESSED;
        }
    }    
    
    //Check the button against the simbols
    if (raw_button != KEYPAD_KEY_NOT_PRESSED){
        for (counter= 0; counter < KEYPAD_KEYS; counter++){
            if (keypad_raw_values[counter] == raw_button)
                return (keypad_values[counter]);
        }  
    }
    
    return (KEYPAD_KEY_NOT_PRESSED);    
}

