/*******************************************************************************
 
Author : Mauro Laurenti
Version : 1.3
 
Created on Date : 4/11/2007
Last update     : 26/07/2019

CopyRight 2006-2015 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 
********************************************************
PURPOSES
********************************************************

This library is supposed to be used for CAN bus  
applications. All the basic functions are included.
Standard and Extended messages are supported

*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "module_CAN.h"


//*********************************************
// Initialize the CAN bus
//*********************************************

void CAN_open (BYTE propSeg, BYTE phaseSeg1, BYTE phaseSeg2,BYTE SJW, BYTE BRP, enum CAN_CONFIG_FLAGS flags) {
	
	BYTE FilterConfig1;
    BYTE FilterConfig2;

    //I/O pin configuraton
    #ifndef MCC_USED
		CAN1_TX_LINE_TRIS = 0x00;          
		CAN1_RX_LINE_TRIS = 0x01; 
	#endif
    
	//set configuration mode
	CAN_operation_mode(CAN_OP_MODE_CONFIG);	

	//cleaning the value
	BRGCON1 = 0x00;		
	BRGCON2 = 0x00;		
	BRGCON3 = 0x00;		

	SJW= (SJW -1) << 6;
	BRGCON1 |= SJW;
	BRGCON1 |= BRP;

	BRGCON2 |= propSeg -1;
	phaseSeg1 = (phaseSeg1 -1) <<3;
	BRGCON2 |= phaseSeg1;

	//SAM 
	if ( !(flags & CAN_CONFIG_SAMPLE_BIT) )	
        BRGCON2bits.SAM= 1;

    if ( flags & CAN_CONFIG_PHSEG2_PRG_BIT )
        BRGCON2bits.SEG2PHTS = 1;

	BRGCON3 |= phaseSeg2 -1;

	if ( flags & CAN_CONFIG_LINE_FILTER_BIT )
        BRGCON3bits.WAKFIL = 1;

	
	//Receiver Settings
	RXB0CON = flags & CAN_CONFIG_MSG_BITS;
    if ( (flags & CAN_CONFIG_DBL_BUFFER_BIT)== CAN_CONFIG_DBL_BUFFER_ON )
        RXB0CONbits.RXB0DBEN = 1;
	
    RXB1CON = RXB0CON;

	//Setting the mask to receive all the messages	
	CAN_set_mask (CAN_MASK_B1, 0x00000000, CAN_CONFIG_XTD_MSG);
	CAN_set_mask (CAN_MASK_B2, 0x00000000, CAN_CONFIG_XTD_MSG);


	switch( (flags & CAN_CONFIG_MSG_BITS) | ~CAN_CONFIG_MSG_BITS ) {
		
    case CAN_CONFIG_VALID_XTD_MSG:
        FilterConfig1 = CAN_CONFIG_XTD_MSG;
        FilterConfig2 = CAN_CONFIG_XTD_MSG;
        break;

    case CAN_CONFIG_VALID_STD_MSG:
        FilterConfig1 = CAN_CONFIG_STD_MSG;
        FilterConfig2 = CAN_CONFIG_STD_MSG;
        break;
        
    default:
		//Buffer 1 will receive the Standard messages
        FilterConfig1 = CAN_CONFIG_STD_MSG;	
		//Buffer 2 will receive the Extended messages
        FilterConfig2 = CAN_CONFIG_XTD_MSG; 
        break;
    }

    CAN_set_filter (CAN_FILTER_B1_F1, 0, FilterConfig1);
    CAN_set_filter (CAN_FILTER_B1_F2, 0, FilterConfig1);
    CAN_set_filter (CAN_FILTER_B2_F1, 0, FilterConfig2);
    CAN_set_filter (CAN_FILTER_B2_F2, 0, FilterConfig2);
    CAN_set_filter (CAN_FILTER_B2_F3, 0, FilterConfig2);
    CAN_set_filter (CAN_FILTER_B2_F4, 0, FilterConfig2);

	//set normal mode
	CAN_operation_mode(CAN_OP_MODE_NORMAL);		
}


//*********************************************
// CAN_close
//*********************************************
void CAN_close (void){
    //set SLEEP mode
	CAN_operation_mode(CAN_OP_MODE_SLEEP);	
}


//*********************************************
// Send a message
//*********************************************
void CAN_write_message (uint32_t identifier, BYTE *data, BYTE dataLength, enum CAN_TX_MSG_FLAGS flags) {	

	//used to compute the identifier
	uint32_t identifier_math;	

	if (TXB0CONbits.TXREQ == 0) {

		// set the data length
		TXB0DLC = dataLength;
	
			//set the RTR bit	
			 if (0b01000000 & flags)			
				TXB0DLCbits.TXRTR = 0x01;
			 else 
				TXB0DLCbits.TXRTR = 0x00;

			if (CAN_TX_FRAME_MASK & flags) {
							
				//EID0 - EID7 setting			
				identifier_math = identifier & 0x000000FF;		
				TXB0EIDL = (uint8_t) identifier_math;

				//EID8 - EID15 setting
				identifier_math = identifier & 0x0000FF00;		
				identifier_math = identifier_math >> 8;	
				TXB0EIDH = (uint8_t) identifier_math;
				
				TXB0SIDL = 0x00;
				TXB0SIDLbits.EXIDE = 0x01;

				if (identifier & 0x00010000)
				TXB0SIDLbits.EID16 = 0x01;
				if (identifier & 0x00020000)
				TXB0SIDLbits.EID17 = 0x01;
				if (identifier &  0x00040000)
				TXB0SIDLbits.SID0 = 0x01;
				if (identifier &  0x00080000)
				TXB0SIDLbits.SID1 = 0x01;
				if (identifier &  0x00100000)
				TXB0SIDLbits.SID2 = 0x01;

				identifier_math = (identifier >> 21);
				identifier_math = identifier_math & 0x000000FF;				
				TXB0SIDH = (uint8_t) identifier_math;				
			}

			else {
				
				//enable standard messages
				TXB0SIDLbits.EXIDE = 0x00;	

				identifier_math = (identifier >> 3);
				identifier_math = identifier_math & 0x000000FF;				
				TXB0SIDH = (uint8_t) identifier_math;
				
				identifier_math = identifier & 0x00000007;
				identifier_math = identifier_math << 5;
				TXB0SIDL = TXB0SIDL & 0b00011111;
				TXB0SIDL = (uint8_t) identifier_math;
			}

			//set transmission priority
			if (0b00000001 & flags)		
				TXB0CONbits.TXPRI0 = 0x01;
			else	
				TXB0CONbits.TXPRI0 = 0x00;
			if (0b00000010 & flags)
				TXB0CONbits.TXPRI1 = 0x01;	
			else
				TXB0CONbits.TXPRI1 = 0x00;		

			TXB0D0 = data[0];
			TXB0D1 = data[1];
			TXB0D2 = data[2];
			TXB0D3 = data[3];
			TXB0D4 = data[4];
			TXB0D5 = data[5];
			TXB0D6 = data[6];
			TXB0D7 = data[7];

			//enable trasmission 
			TXB0CONbits.TXREQ = 0x01;	
			return;
		}

	
	// Second TX Buffer setting 
	if (TXB1CONbits.TXREQ == 0) {
		
		// set the data length
		TXB1DLC = dataLength;	

			//set the RTR bit		
			if (0b01000000 & flags)		
				TXB1DLCbits.TXRTR = 0x01;
			 else 
				TXB1DLCbits.TXRTR = 0x00;			
			
			if (CAN_TX_FRAME_MASK & flags) {
				
				//EID0 - EID7 setting
				identifier_math = identifier & 0x000000FF;		
				TXB1EIDL = (uint8_t) identifier_math;

				//EID8 - EID15 setting
				identifier_math = identifier & 0x0000FF00;		
				identifier_math = identifier_math >> 8;	
				TXB1EIDH = (uint8_t) identifier_math;

				TXB1SIDL = 0x00;
				TXB1SIDLbits.EXIDE = 0x01;

				if (identifier & 0x00010000)
				TXB1SIDLbits.EID16 = 0x01;
				if (identifier & 0x00020000)
				TXB1SIDLbits.EID17 = 0x01;
				if (identifier &  0x00040000)
				TXB1SIDLbits.SID0 = 0x01;
				if (identifier &  0x00080000)
				TXB1SIDLbits.SID1 = 0x01;
				if (identifier &  0x00100000)
				TXB1SIDLbits.SID2 = 0x01;

				identifier_math = (identifier >> 21);
				identifier_math = identifier_math & 0x000000FF;				
				TXB1SIDH = (uint8_t) identifier_math;				
			}

			else {
				//enable standard messages
				TXB1SIDLbits.EXIDE = 0x00;	
				
				identifier_math = (identifier >> 3);
				identifier_math = identifier_math & 0x000000FF;				
				TXB1SIDH = (uint8_t) identifier_math;
				
				identifier_math = identifier & 0x00000007;
				identifier_math = identifier_math << 5;
				TXB1SIDL = TXB1SIDL & 0b00011111;
				TXB1SIDL = (uint8_t) identifier_math;
			}

			//set transmission priority
			if (0b00000001 & flags)		
				TXB1CONbits.TXPRI0 = 0x01;
			else	
				TXB1CONbits.TXPRI0 = 0x00;
			if (0b00000010 & flags)
				TXB1CONbits.TXPRI1 = 0x01;	
			else
				TXB1CONbits.TXPRI1 = 0x00;				

			TXB1D0 = data[0];
			TXB1D1 = data[1];
			TXB1D2 = data[2];
			TXB1D3 = data[3];
			TXB1D4 = data[4];
			TXB1D5 = data[5];
			TXB1D6 = data[6];
			TXB1D7 = data[7];

			//enable trasmission
			TXB1CONbits.TXREQ = 0x01;	
			
			return;
		}


	// Third TX Buffer setting 
	if (TXB2CONbits.TXREQ == 0) {	
	
		// set the data length
		TXB2DLC = dataLength;	
			
			//set the RTR bit
			if (0b01000000 & flags)				
				TXB2DLCbits.TXRTR = 0x01;
			 else 
				TXB2DLCbits.TXRTR = 0x00;		
			
			if (CAN_TX_FRAME_MASK & flags) {
				
				//EID0 - EID7 setting
				identifier_math = identifier & 0x000000FF;		
				TXB2EIDL = (uint8_t) identifier_math;

				//EID8 - EID15 setting
				identifier_math = identifier & 0x0000FF00;		
				identifier_math = identifier_math >> 8;	
				TXB2EIDH = (uint8_t) identifier_math;

				TXB2SIDL = 0x00;
				TXB2SIDLbits.EXIDE = 0x01;

				if (identifier & 0x00010000)
				TXB2SIDLbits.EID16 = 0x01;
				if (identifier & 0x00020000)
				TXB2SIDLbits.EID17 = 0x01;
				if (identifier &  0x00040000)
				TXB2SIDLbits.SID0 = 0x01;
				if (identifier &  0x00080000)
				TXB2SIDLbits.SID1 = 0x01;
				if (identifier &  0x00100000)
				TXB2SIDLbits.SID2 = 0x01;

				identifier_math = (identifier >> 21);
				identifier_math = identifier_math & 0x000000FF;				
				TXB2SIDH = (uint8_t) identifier_math;				
			}

			else {
				
				//enable standard messages
				TXB2SIDLbits.EXIDE = 0x00;	

				identifier_math = (identifier >> 3);
				identifier_math = identifier_math & 0x000000FF;				
				TXB2SIDH = (uint8_t) identifier_math;
				
				identifier_math = identifier & 0x00000007;
				identifier_math = identifier_math << 5;
				TXB2SIDL = TXB2SIDL & 0b00011111;
				TXB2SIDL = (uint8_t) identifier_math;
			}

			//set transmission priority
			if (0b00000001 & flags)		
				TXB2CONbits.TXPRI0 = 0x01;
			else	
				TXB2CONbits.TXPRI0 = 0x00;
			if (0b00000010 & flags)
				TXB2CONbits.TXPRI1 = 0x01;	
			else
				TXB2CONbits.TXPRI1 = 0x00;				

			TXB2D0 = data[0];
			TXB2D1 = data[1];
			TXB2D2 = data[2];
			TXB2D3 = data[3];
			TXB2D4 = data[4];
			TXB2D5 = data[5];
			TXB2D6 = data[6];
			TXB2D7 = data[7];

			//enable transmission 
			TXB2CONbits.TXREQ = 0x01;	
			
			return;
		}
}


//*********************************************
// Read the message from the RX buffer
//*********************************************
uint8_t CAN_read_message (CANmessage *msg) {	

	BYTE tamp;
	uint8_t error = 0;

	//check for buffers overflows
	if (COMSTATbits.RXB0OVFL == 0x01)	
		error |= CAN_RX_BUFFER_1_OVFL;
	
	if (COMSTATbits.RXB1OVFL == 0x01)
		error |= CAN_RX_BUFFER_2_OVFL;

	//RX Buffer 1 is read
	if (RXB1CONbits.RXFUL ==0x01) {		
		
		msg->identifier = 0;
		
		//retrieve the data
		msg->data[0] =RXB1D0;	
		msg->data[1] =RXB1D1;
		msg->data[2] =RXB1D2;
		msg->data[3] =RXB1D3;
		msg->data[4] =RXB1D4;
		msg->data[5] =RXB1D5;
		msg->data[6] =RXB1D6;
		msg->data[7] =RXB1D7;

		//retrieve the RTR bit
		msg->RTR = RXB1DLCbits.RXRTR;	
		
		//retrieve the length
		msg->length = RXB1DLC & 0x0F;	

		//retrieve the format (standard or extended)
		msg->type = RXB1SIDLbits.EXID;	
		
		//reading the identifier standard format
		if (RXB1SIDLbits.EXID == 0)	{	
			msg->identifier = ((uint32_t)RXB1SIDH)<< 3;
			tamp = (RXB1SIDL >> 5 ) & 0x07;
			msg->identifier = msg->identifier + tamp;
		}
		
		// reading the identifier extended format
		else {
			
			//retrieve EID0-EID7
			msg->identifier = (uint32_t) RXB1EIDL;		
			//retrieve EID8-EID15			
			msg->identifier += ((uint32_t) RXB1EIDH) << 8;	
			
			if (RXB1SIDLbits.EID16) 
				msg->identifier |= 0x00010000;
			if (RXB1SIDLbits.EID17) 
				msg->identifier |= 0x00020000;
			if (RXB1SIDLbits.SID0) 
				msg->identifier |= 0x00040000;
			if (RXB1SIDLbits.SID1) 
				msg->identifier |= 0x00080000;
			if (RXB1SIDLbits.SID2) 
				msg->identifier |= 0x00100000;

			msg->identifier |= ((uint32_t) RXB1SIDH) << 21; 
		}
		
		//release the RX buffer for new messages		
		RXB1CONbits.RXFUL = 0x00;	
		return (error);
	}

	//RX Buffer 0 is read
	if (RXB0CONbits.RXFUL ==0x01) {	
	
		msg->identifier = 0;
		
		//retrieve the data
		msg->data[0] =RXB0D0;	
		msg->data[1] =RXB0D1;
		msg->data[2] =RXB0D2;
		msg->data[3] =RXB0D3;
		msg->data[4] =RXB0D4;
		msg->data[5] =RXB0D5;
		msg->data[6] =RXB0D6;
		msg->data[7] =RXB0D7;

		//retrieve the RTR bit
		msg->RTR = RXB0DLCbits.RXRTR;	
		
		//retrieve the length
		msg->length = RXB0DLC & 0x0F;	

		//retrieve the format (standard or extended)
		msg->type = RXB0SIDLbits.EXID;	
		
		//reading the identifier standard format
		if (RXB0SIDLbits.EXID == 0)		
		{	
			msg->identifier = ((uint32_t)RXB0SIDH)<< 3;
			tamp = (RXB0SIDL >> 5 ) & 0x07;
			msg->identifier = msg->identifier + tamp;
		}
		else {
			
			// reading the identifier extended format
			//retrieve EID0-EID7
			msg->identifier = (uint32_t) RXB0EIDL;	
			//retrieve EID8-EID15			
			msg->identifier += ((uint32_t) RXB0EIDH) << 8;	
			
			if (RXB0SIDLbits.EID16) 
				msg->identifier |= 0x00010000;
			if (RXB0SIDLbits.EID17) 
				msg->identifier |= 0x00020000;
			if (RXB0SIDLbits.SID0) 
				msg->identifier |= 0x00040000;
			if (RXB0SIDLbits.SID1) 
				msg->identifier |= 0x00080000;
			if (RXB0SIDLbits.SID2) 
				msg->identifier |= 0x00100000;

			msg->identifier |= ((uint32_t) RXB0SIDH) << 21;
 
		}		
		
		//release the RX buffer for new messages
		RXB0CONbits.RXFUL = 0x00;	
		
		return (error);
	}    
    return (error);
}

//*********************************************
// Set the RX MASK
//*********************************************

void CAN_set_mask (enum CAN_MASK mask_number, uint32_t mask, enum CAN_CONFIG_FLAGS type) {	

    //used to compute the identifier
    uint32_t identifier_math;	

	//Standard MASK RX BUFFER 1
	if ((mask_number == CAN_MASK_B1) && (type == CAN_CONFIG_STD_MSG)) {
		identifier_math = (mask >> 3);
		identifier_math = identifier_math & 0x000000FF;				
		RXM0SIDH = (uint8_t) identifier_math;
				
		identifier_math = mask & 0x00000007;
		identifier_math = identifier_math << 5;
		RXM0SIDL = (uint8_t) identifier_math;
	}

	//Standard MASK RX BUFFER2
	if ((mask_number == CAN_MASK_B2) && (type == CAN_CONFIG_STD_MSG)) {
		identifier_math = (mask >> 3);
		identifier_math = identifier_math & 0x000000FF;				
		RXM1SIDH = (uint8_t) identifier_math;
				
		identifier_math = mask & 0x00000007;
		identifier_math = identifier_math << 5;
		RXM1SIDL = (uint8_t) identifier_math;
	}	

	//Extended MASK RX BUFFER 1
	if ((mask_number == CAN_MASK_B1) && (type == CAN_CONFIG_XTD_MSG)) {
		//EID0 - EID7 setting
		identifier_math = mask & 0x000000FF;		
		RXM0EIDL = (uint8_t) identifier_math;

		//EID8 - EID15 setting
		identifier_math = mask & 0x0000FF00;		
		identifier_math = identifier_math >> 8;	
		RXM0EIDH = (uint8_t) identifier_math;

		if (mask & 0x00010000) 
			RXM0SIDLbits.EID16 = 0x01; 
		else 
			RXM0SIDLbits.EID16 = 0x00; 

		if (mask & 0x00020000)
			RXM0SIDLbits.EID17 = 0x01;
		else
			RXM0SIDLbits.EID17 = 0x00;

		if (mask &  0x00040000)
			RXM0SIDLbits.SID0 = 0x01;
		else
		    RXM0SIDLbits.SID0 = 0x00;

		if (mask &  0x00080000)
			RXM0SIDLbits.SID1 = 0x01;
		else
			RXM0SIDLbits.SID1 = 0x00;
		
		if (mask &  0x00100000)
			RXM0SIDLbits.SID2 = 0x01 ;
		else
			RXM0SIDLbits.SID2 = 0x00 ;

		identifier_math = (mask >> 21);
		identifier_math = identifier_math & 0x000000FF;				
		RXM0SIDH = (uint8_t) identifier_math;
	}

	//Extended MASK BUFFER 2
	if ((mask_number == CAN_MASK_B2) && (type == CAN_CONFIG_XTD_MSG)) {
		//EID0 - EID7 setting
		identifier_math = mask & 0x000000FF;		
		RXM1EIDL = (uint8_t) identifier_math;

		//EID8 - EID15 setting
		identifier_math = mask & 0x0000FF00;		
		identifier_math = identifier_math >> 8;	
		RXM1EIDH = (uint8_t) identifier_math;

		if (mask & 0x00010000) 
			RXM1SIDLbits.EID16 = 0x01; 
		else 
			RXM1SIDLbits.EID16 = 0x00; 

		if (mask & 0x00020000)
			RXM1SIDLbits.EID17 = 0x01;
		else
			RXM1SIDLbits.EID17 = 0x00;

		if (mask &  0x00040000)
			RXM1SIDLbits.SID0 = 0x01;
		else
		    RXM1SIDLbits.SID0 = 0x00;

		if (mask &  0x00080000)
			RXM1SIDLbits.SID1 = 0x01;
		else
			RXM1SIDLbits.SID1 = 0x00;
		
		if (mask &  0x00100000)
			RXM1SIDLbits.SID2 = 0x01 ;
		else
			RXM1SIDLbits.SID2 = 0x00 ;

		identifier_math = (mask >> 21);
		identifier_math = identifier_math & 0x000000FF;				
		RXM1SIDH = (uint8_t) identifier_math;
	}
}

//*********************************************
// Set the RX Filters
//*********************************************

void CAN_set_filter (enum CAN_FILTER filter_number, uint32_t filter, enum CAN_CONFIG_FLAGS type) {
	
    uint32_t tamp; 

	// STANDARD FILTER 1 BUFFER 1 
	if ((filter_number == CAN_FILTER_B1_F1) && (type == CAN_CONFIG_STD_MSG)) {	
	
		//standard filter
		RXF0SIDLbits.EXIDEN = 0x00;	

		tamp = (filter >> 3);
		tamp = tamp & 0x000000FF;				
		RXF0SIDH = (uint8_t) tamp;
				
		tamp = filter & 0x00000007;
		tamp = tamp << 5;
		RXF0SIDL = (uint8_t) tamp;
	}

	// STANDARD FILTER 2 BUFFER 1
	if ((filter_number == CAN_FILTER_B1_F2) && (type == CAN_CONFIG_STD_MSG)) {	
	
		//standard filter
		RXF1SIDLbits.EXIDEN = 0x00;	

		tamp = (filter >> 3);
		tamp = tamp & 0x000000FF;				
		RXF1SIDH = (uint8_t) tamp;
				
		tamp = filter & 0x00000007;
		tamp = tamp << 5;
		RXF1SIDL = (uint8_t) tamp;
	}

	// STANDARD FILTER 1 BUFFER 2
	if ((filter_number == CAN_FILTER_B2_F1) && (type == CAN_CONFIG_STD_MSG)) {
		
		//standard filter
		RXF2SIDLbits.EXIDEN = 0x00;	

		tamp = (filter >> 3);
		tamp = tamp & 0x000000FF;				
		RXF2SIDH = (uint8_t) tamp;
				
		tamp = filter & 0x00000007;
		tamp = tamp << 5;
		RXF2SIDL = (uint8_t) tamp;
	}

	// STANDARD FILTER 2 BUFFER 2
	if ((filter_number == CAN_FILTER_B2_F2) && (type == CAN_CONFIG_STD_MSG))	{	
	
		//standard filter
		RXF3SIDLbits.EXIDEN = 0x00;	

		tamp = (filter >> 3);
		tamp = tamp & 0x000000FF;				
		RXF3SIDH = (uint8_t) tamp;
				
		tamp = filter & 0x00000007;
		tamp = tamp << 5;
		RXF3SIDL = (uint8_t) tamp;
	}

	// STANDARD FILTER 3 BUFFER 2
	if ((filter_number == CAN_FILTER_B2_F3) && (type == CAN_CONFIG_STD_MSG)) {	
	
		//standard filter
		RXF4SIDLbits.EXIDEN = 0x00;	

		tamp = (filter >> 3);
		tamp = tamp & 0x000000FF;				
		RXF4SIDH = (uint8_t) tamp;
				
		tamp = filter & 0x00000007;
		tamp = tamp << 5;
		RXF4SIDL = (uint8_t) tamp;
	}

	// STANDARD FILTER 4 BUFFER 2
	if ((filter_number == CAN_FILTER_B2_F4) && (type == CAN_CONFIG_STD_MSG))	{	
	
		//standard filter
		RXF5SIDLbits.EXIDEN = 0x00;	

		tamp = (filter >> 3);
		tamp = tamp & 0x000000FF;				
		RXF5SIDH = (uint8_t) tamp;
				
		tamp = filter & 0x00000007;
		tamp = tamp << 5;
		RXF5SIDL = (uint8_t) tamp;
	}

	// EXTENDED FILTER 1 BUFFER 1
	if ((filter_number == CAN_FILTER_B1_F1) && (type == CAN_CONFIG_XTD_MSG)) {	
	
		//standard filter
		RXF0SIDLbits.EXIDEN = 0x01;	

		//EID0 - EID7 setting
		tamp = filter & 0x000000FF;		
		RXF0EIDL = (uint8_t) tamp;

		//EID8 - EID15 setting
		tamp = filter & 0x0000FF00;		
		tamp = tamp >> 8;	
		RXF0EIDH = (uint8_t) tamp;

		if (filter & 0x00010000) 
			RXF0SIDLbits.EID16 = 0x01; 
		else 
			RXF0SIDLbits.EID16 = 0x00; 

		if (filter & 0x00020000)
			RXF0SIDLbits.EID17 = 0x01;
		else
			RXF0SIDLbits.EID17 = 0x00;

		if (filter &  0x00040000)
			RXF0SIDLbits.SID0 = 0x01;
		else
		    RXF0SIDLbits.SID0 = 0x00;

		if (filter &  0x00080000)
			RXF0SIDLbits.SID1 = 0x01;
		else
			RXF0SIDLbits.SID1 = 0x00;
		
		if (filter &  0x00100000)
			RXF0SIDLbits.SID2 = 0x01 ;
		else
			RXF0SIDLbits.SID2 = 0x00 ;

		tamp = (filter >> 21);
		tamp = tamp & 0x000000FF;				
		RXF0SIDH = (uint8_t) tamp;
	}

	// EXTENDED FILTER 2 BUFFER 1
	if ((filter_number == CAN_FILTER_B1_F2) && (type == CAN_CONFIG_XTD_MSG)) {
		
		//standard filter
		RXF1SIDLbits.EXIDEN = 0x01;	

		//EID0 - EID7 setting
		tamp = filter & 0x000000FF;		
		RXF1EIDL = (uint8_t) tamp;

		//EID8 - EID15 setting
		tamp = filter & 0x0000FF00;		
		tamp = tamp >> 8;	
		RXF1EIDH = (uint8_t) tamp;

		if (filter & 0x00010000) 
			RXF1SIDLbits.EID16 = 0x01; 
		else 
			RXF1SIDLbits.EID16 = 0x00; 

		if (filter & 0x00020000)
			RXF1SIDLbits.EID17 = 0x01;
		else
			RXF1SIDLbits.EID17 = 0x00;

		if (filter &  0x00040000)
			RXF1SIDLbits.SID0 = 0x01;
		else
		    RXF1SIDLbits.SID0 = 0x00;

		if (filter &  0x00080000)
			RXF1SIDLbits.SID1 = 0x01;
		else
			RXF1SIDLbits.SID1 = 0x00;
		
		if (filter &  0x00100000)
			RXF1SIDLbits.SID2 = 0x01 ;
		else
			RXF1SIDLbits.SID2 = 0x00 ;

		tamp = (filter >> 21);
		tamp = tamp & 0x000000FF;				
		RXF1SIDH = (uint8_t) tamp;
	}


	// EXTENDED FILTER 1 BUFFER 2
	if ((filter_number == CAN_FILTER_B2_F1) && (type == CAN_CONFIG_XTD_MSG)) {	
	
		//standard filter
		RXF2SIDLbits.EXIDEN = 0x01;	

		//EID0 - EID7 setting
		tamp = filter & 0x000000FF;		
		RXF2EIDL = (uint8_t) tamp;

		//EID8 - EID15 setting
		tamp = filter & 0x0000FF00;		
		tamp = tamp >> 8;	
		RXF2EIDH = (uint8_t) tamp;

		if (filter & 0x00010000) 
			RXF2SIDLbits.EID16 = 0x01; 
		else 
			RXF2SIDLbits.EID16 = 0x00; 

		if (filter & 0x00020000)
			RXF2SIDLbits.EID17 = 0x01;
		else
			RXF2SIDLbits.EID17 = 0x00;

		if (filter &  0x00040000)
			RXF2SIDLbits.SID0 = 0x01;
		else
		    RXF2SIDLbits.SID0 = 0x00;

		if (filter &  0x00080000)
			RXF2SIDLbits.SID1 = 0x01;
		else
			RXF2SIDLbits.SID1 = 0x00;
		
		if (filter &  0x00100000)
			RXF2SIDLbits.SID2 = 0x01 ;
		else
			RXF2SIDLbits.SID2 = 0x00 ;

		tamp = (filter >> 21);
		tamp = tamp & 0x000000FF;				
		RXF2SIDH = (uint8_t) tamp;
	}

	// EXTENDED FILTER 2 BUFFER 2
	if ((filter_number == CAN_FILTER_B2_F2) && (type == CAN_CONFIG_XTD_MSG)) {	
	
		//standard filter
		RXF3SIDLbits.EXIDEN = 0x01;	

		//EID0 - EID7 setting
		tamp = filter & 0x000000FF;		
		RXF3EIDL = (uint8_t) tamp;

		//EID8 - EID15 setting
		tamp = filter & 0x0000FF00;		
		tamp = tamp >> 8;	
		RXF3EIDH = (uint8_t) tamp;

		if (filter & 0x00010000) 
			RXF3SIDLbits.EID16 = 0x01; 
		else 
			RXF3SIDLbits.EID16 = 0x00; 

		if (filter & 0x00020000)
			RXF3SIDLbits.EID17 = 0x01;
		else
			RXF3SIDLbits.EID17 = 0x00;

		if (filter &  0x00040000)
			RXF3SIDLbits.SID0 = 0x01;
		else
		    RXF3SIDLbits.SID0 = 0x00;

		if (filter &  0x00080000)
			RXF3SIDLbits.SID1 = 0x01;
		else
			RXF3SIDLbits.SID1 = 0x00;
		
		if (filter &  0x00100000)
			RXF3SIDLbits.SID2 = 0x01 ;
		else
			RXF3SIDLbits.SID2 = 0x00 ;

		tamp = (filter >> 21);
		tamp = tamp & 0x000000FF;				
		RXF3SIDH = (uint8_t) tamp;
	}

	// EXTENDED FILTER 3 BUFFER 2
	if ((filter_number == CAN_FILTER_B2_F3) && (type == CAN_CONFIG_XTD_MSG)) {
		
		//standard filter
		RXF4SIDLbits.EXIDEN = 0x01;	

		//EID0 - EID7 setting
		tamp = filter & 0x000000FF;		
		RXF4EIDL = (uint8_t) tamp;

		//EID8 - EID15 setting
		tamp = filter & 0x0000FF00;		
		tamp = tamp >> 8;	
		RXF4EIDH = (uint8_t) tamp;

		if (filter & 0x00010000) 
			RXF4SIDLbits.EID16 = 0x01; 
		else 
			RXF4SIDLbits.EID16 = 0x00; 

		if (filter & 0x00020000)
			RXF4SIDLbits.EID17 = 0x01;
		else
			RXF4SIDLbits.EID17 = 0x00;

		if (filter &  0x00040000)
			RXF4SIDLbits.SID0 = 0x01;
		else
		    RXF4SIDLbits.SID0 = 0x00;

		if (filter &  0x00080000)
			RXF4SIDLbits.SID1 = 0x01;
		else
			RXF4SIDLbits.SID1 = 0x00;
		
		if (filter &  0x00100000)
			RXF4SIDLbits.SID2 = 0x01 ;
		else
			RXF4SIDLbits.SID2 = 0x00 ;

		tamp = (filter >> 21);
		tamp = tamp & 0x000000FF;				
		RXF4SIDH = (uint8_t) tamp;
	}

	// EXTENDED FILTER 4 BUFFER 2
	if ((filter_number == CAN_FILTER_B2_F4) && (type == CAN_CONFIG_XTD_MSG)) {	
	
		//standard filter
		RXF5SIDLbits.EXIDEN = 0x01;	

		//EID0 - EID7 setting
		tamp = filter & 0x000000FF;		
		RXF5EIDL = (uint8_t) tamp;

		//EID8 - EID15 setting
		tamp = filter & 0x0000FF00;		
		tamp = tamp >> 8;	
		RXF5EIDH = (uint8_t) tamp;

		if (filter & 0x00010000) 
			RXF5SIDLbits.EID16 = 0x01; 
		else 
			RXF5SIDLbits.EID16 = 0x00; 

		if (filter & 0x00020000)
			RXF5SIDLbits.EID17 = 0x01;
		else
			RXF5SIDLbits.EID17 = 0x00;

		if (filter &  0x00040000)
			RXF5SIDLbits.SID0 = 0x01;
		else
		    RXF5SIDLbits.SID0 = 0x00;

		if (filter &  0x00080000)
			RXF5SIDLbits.SID1 = 0x01;
		else
			RXF5SIDLbits.SID1 = 0x00;
		
		if (filter &  0x00100000)
			RXF5SIDLbits.SID2 = 0x01 ;
		else
			RXF5SIDLbits.SID2 = 0x00 ;

		tamp = (filter >> 21);
		tamp = tamp & 0x000000FF;				
		RXF5SIDH = (uint8_t) tamp;
	}
	
}

//*********************************************
//Set the CAN engine mode
//*********************************************

void CAN_operation_mode (enum CAN_OP_MODE mode) {
	CANCON = mode;
	while((CANSTAT & 0b11100000) != mode );
}


//*********************************************
// Abort all the pending messages
//*********************************************

void CAN_abort_messages (void) {
	CANCONbits.ABAT = 1;
}


//*********************************************
// Return 1 if the Bus is OFF
//*********************************************
BYTE CAN_is_BUS_OFF (void) {
	return (COMSTATbits.TXBO);		
}

//*********************************************
// Return 1 if there is a TX passive status
//*********************************************
BYTE CAN_is_TX_passive (void) {
	return (COMSTATbits.TXBP);
}


//*********************************************
// Return 1 if there is a TX passive status
//*********************************************
BYTE CAN_is_RX_passive (void) {
	return (COMSTATbits.RXBP);
}

//*********************************************
// Return 1 if TX warning is ON
//*********************************************
BYTE CAN_is_TX_warning_ON (void) {
	return (COMSTATbits.TXWARN);
}

//*********************************************
// Return 1 if RX warning is ON
//*********************************************
BYTE CAN_is_RX_warning_ON (void) {
	return (COMSTATbits.RXWARN);
}

//*********************************************
// Return TX error Count
//*********************************************

BYTE CAN_get_TX_error_count (void) {
	return (TXERRCNT);
}

//*********************************************
// Return RX error Count
//*********************************************

BYTE CAN_get_RX_error_count (void) {
	return (RXERRCNT);
}

//*********************************************
// Return 1 if at least 1 TX buffer is empty
//*********************************************

BYTE CAN_is_TX_ready (void) {
	//if at least one flag is 0, it returns return 1
	return (!TXB0CONbits.TXREQ || !TXB1CONbits.TXREQ || !TXB2CONbits.TXREQ); 
}

//*********************************************
// Return 1 if one or more RX buffer are full
//*********************************************

BYTE CAN_is_RX_ready (void) {
	return (RXB0CONbits.RXFUL || RXB1CONbits.RXFUL);	
}
