/*******************************************************************************
 
Author : Mauro Laurenti
Version : 1.3
 
Created on Date : 02/10/2010
Last update     : 03/02/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#ifdef __XC8
    #include <xc.h>
#endif

#include "module_EEPROM.h"


//************************************************************
//         EEPROM_write_byte function implementation
//************************************************************

int8_t EEPROM_write_byte (uint16_t address, uint8_t data) {
	
	// Flag used to store the GIE value
	uint8_t flagGIE = 0;	
	
	// Flag used to store the GIEH value
	uint8_t flagGIEH = 0;	
	
	// Flag used to store the GIEL value
	uint8_t flagGIEL = 0; 

    // Set the memory address that will be read    
    //depending on the meory size the address is set differently 
    
    #if (EEPROM_MODULE_SIZE == 0xFF)
        EEADR = (uint8_t) address;	           
    #endif

    #if (EEPROM_MODULE_SIZE == 0x3FF)
        EEADR = (uint8_t) address;  
        EEADRH = (uint8_t)(address >> 8); 	           
    #endif
    
    
	// Set the data that will be written
	EEDATA = data;		
	
	// EEPROM memory is pointed
	EECON1bits.EEPGD = 0; 

    #ifdef _PIC18
        // EEPROM access enable
        EECON1bits.CFGS = 0;
    #endif
	
	// Enable write 
	EECON1bits.WREN = 0x01;	

	// Check and store the Interrupt Status
	if (INTCONbits.GIE == 1) {  
			INTCONbits.GIE = 0; 
			flagGIE = 1;
	}

    #ifdef _PIC18
        if (INTCONbits.GIEH == 1) {
                        INTCONbits.GIEH = 0;
                        flagGIEH = 1;
        }

        if (INTCONbits.GIEL == 1) {
                        INTCONbits.GIEL = 0;
                        flagGIEL = 1;
        }
    #endif

	// Start the writing enabling sequence		
	EECON2 = 0x55;	
	EECON2 = 0xAA;

	// Initiate writing process
	EECON1bits.WR = 0x01; 	

	// Wait the end of the writing process
	while (EECON1bits.WR);	
	
	
	// Restore the previous interrupt status
	if (flagGIE == 1) {
		INTCONbits.GIE = 1; 
	}	

    #ifdef _PIC18
        if (flagGIEH == 1) {
                INTCONbits.GIEH = 1;
        }

        if (flagGIEL == 1) {
                INTCONbits.GIEL = 1;

        }
    #endif
	
	// Disable the writing process
	EECON1bits.WREN = 0x00;	
	
	
	// Check if the data has been properly written,
	// a simple read back is done
    if (EEPROM_read_byte (address) == data) {
		
		return (1);
		
	} else {
		
		return (0);
	}

}


//************************************************************
//           EEPROM_read_byte Function  Implementation
//************************************************************
uint8_t EEPROM_read_byte (uint16_t address) {
	
	uint8_t data = 0;

	// Set the memory address that will be read    
    //depending on the meory size the address is set differently 
    
    #ifdef EEPROM_MODULE_TYPE_1
        EEADR = (uint8_t) address;	           
    #endif

    #ifdef EEPROM_MODULE_TYPE_2
        EEADR = (uint8_t) address;  
        EEADRH = (uint8_t)(address >> 8); 	           
    #endif
        
	
	// EEPROM memory is pointed	
	EECON1bits.EEPGD = 0; 
    
    #ifdef _PIC18
        // EEPROM access enable
        EECON1bits.CFGS = 0;
    #endif
	
	// Initiate reading
	EECON1bits.RD = 0x01;	
	
	// Data is read from the register
	data = EEDATA;			
	
	return (data);				
}

