/*******************************************************************************
 
Author : Mauro Laurenti
Version : 1.1
 
Created on Date : 06/11/2018
Last update     : 31/01/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "module_FLASH.h"


//************************************************************
//
//************************************************************
void FLASH_read_memory (uint32_t memory_address, uint16_t block_size, uint8_t *buffer){
    
    uint16_t memory_offset = 0;
    
    //The address is assigned only one time
    //It gets incremented via Table Read instruction
    
    TBLPTRU = (uint8_t) ((memory_address >> 16) & 0x000000FF);
    TBLPTRH = (uint8_t) ((memory_address >> 8) & 0x000000FF);
    TBLPTRL = (uint8_t) (memory_address & 0x000000FF);
    
     for (memory_offset=0; memory_offset < block_size; memory_offset++){
         
        __asm("TBLRD*+");
        
        buffer[memory_offset] = TABLAT;        
    }
}

//************************************************************
//
//************************************************************
uint32_t FLASH_read_device_ID (void){
    uint8_t device_info[2];
    uint32_t device_ID = 0;
    
    FLASH_read_memory (DEVICE_ID_ADDRESS, DEVICE_ID_BYTES, device_info);
    
    device_ID |= device_info[0] >> 5;
    device_ID |= device_info[1] << 3;
    
    return (device_ID);
}


//************************************************************
//
//************************************************************
uint32_t FLASH_read_device_revision (void){
    uint8_t device_info[1];
    uint32_t device_revision = 0;
    
    FLASH_read_memory (DEVICE_REVISION_ADDRESS, DEVICE_REVISION_BYTES, device_info);
    
    device_revision |= device_info[0] & 0x1F;
    
    return (device_revision);
}


//************************************************************
//
//************************************************************
int8_t FLASH_erase_memory (uint32_t memory_address){
    
    //The bootloader does not use the interrupts.
    //This function assumes that the interrupts are not enabled
    
    //disable_interrupts( );
    
    //Set the start address for the block to be deleted
    TBLPTRU = (uint8_t) ((memory_address >> 16) & 0x000000FF);
    TBLPTRH = (uint8_t) ((memory_address >> 8) & 0x000000FF);
    TBLPTRL = (uint8_t) (memory_address & 0x000000FF);
    
    //Select flash memory and select delete option
    EECON1bits.EEPGD = 1;
    EECON1bits.CFGS = 0;
    EECON1bits.WREN = 1;
    EECON1bits.FREE = 1;
    
    //Flash unlock sequence
    EECON2 = 0x55;
    EECON2 = 0xAA;
    
    // Start the flash delete process
    EECON1bits.WR = 1;
    
    //Check for any errors    
    if (EECON1bits.WRERR == 1){
        return (-1);
    } else {
         return (0);
    }    
}


//************************************************************
//
//************************************************************
int8_t FLASH_write_memory (uint32_t memory_address, uint16_t block_size, uint8_t *buffer){
    //The bootloader does not use the interrupts.
    //This function assumes that the interrupts are not enabled
    
    //disable_interrupts( );
    
    uint16_t address_offset = 0;
    
    //Set the start address for the block to be written
    TBLPTRU = (uint8_t) ((memory_address >> 16) & 0x000000FF);
    TBLPTRH = (uint8_t) ((memory_address >> 8) & 0x000000FF);
    TBLPTRL = (uint8_t) (memory_address & 0x000000FF);
    
    //Load the RAM data within the holding register for a block writing
    for (address_offset = 0; address_offset < block_size ; address_offset++){
        
        TABLAT = buffer[address_offset];
        
        //Write the TBLAT data within the holding register
        __asm("TBLWT*+");        
    }
    
    //Set the start address for the block to be written
    //if you write 64 bytes you can get to the next fash block
    TBLPTRU = (uint8_t) ((memory_address >> 16) & 0x000000FF);
    TBLPTRH = (uint8_t) ((memory_address >> 8) & 0x000000FF);
    TBLPTRL = (uint8_t) (memory_address & 0x000000FF);
        
    //Select flash memory and select delete option
    EECON1bits.EEPGD = 1;
    EECON1bits.CFGS = 0;
    EECON1bits.WREN = 1;
    
    //Flash unlock sequence
    EECON2 = 0x55;
    EECON2 = 0xAA;
    
    // Start the flash delete process
    EECON1bits.WR = 1;
    
    //Check for any errors    
    if (EECON1bits.WRERR == 1){
        return (-1);
    } else {
         return (0);
    } 
}
