/*******************************************************************************
 
Autore : Mauro Laurenti
Versione : 1.4
 
Created on Date : 02/10/2010
Last update     : 31/01/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 
*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "module_I2C.h"

//************************************************************
//         I2C1_open function implementation
//************************************************************

void I2C1_open (unsigned char device_type){
  
    unsigned char dummy_read;
    
    //IO pin initialization
	#ifndef MCC_USED
		I2C1_SDA_LINE_TRIS = PIN_AS_INPUT;             
		I2C1_SCL_LINE_TRIS = PIN_AS_INPUT; 
	#endif
       
    //Register initialization (Starting known value)
    SSPSTAT &= 0x3F;                
    SSPCON1 = 0x00;                 
    SSPCON2 = 0x00;                
    
    //Device definition Master or Slave   
    SSPCON1 |= device_type; 
    
    //Slew Rate Control set to ON by default
    SSPSTAT |= I2C_SLEW_ON;                
  
    //Enable the I2C Module
    SSPCON1 |= 0x20;    
    
    //clear input buffer
    dummy_read = I2C1_read_byte ();
    
}

#ifdef I2C2_AVAILABLE 

void I2C2_open (unsigned char device_type){
    
    unsigned char dummy_read;
    
    //IO pin initialization
	#ifndef MCC_USED
		I2C2_SDA_LINE_TRIS = PIN_AS_INPUT;             
		I2C2_SCL_LINE_TRIS = PIN_AS_INPUT; 
	#endif
    
    //Register initialization (Starting known value)
    SSP2STAT &= 0x3F;                
    SSP2CON1 = 0x00;                 
    SSP2CON2 = 0x00;                
    
    //Device definition Master or Slave   
    SSP2CON1 |= device_type; 
    
    //Slew Rate Control set to ON by default
    SSP2STAT |= I2C_SLEW_ON;                
  
    //Enable the I2C Module
    SSP2CON1 |= 0x20;    
    
    //clear input buffer
    dummy_read = I2C2_read_byte (); 
    
}
#endif

//************************************************************
//         I2C1_close function implementation
//************************************************************
void I2C1_close (void){
    SSPCON1 &= 0xDF;  
}

#ifdef I2C2_AVAILABLE 
    void I2C2_close (void){
        SSP2CON1 &= 0xDF;  
    }
#endif

//************************************************************
//         I2C1_baud_rate function implementation
//************************************************************
void I2C1_baud_rate (unsigned int bus_baud_rate_KHZ){
    
    unsigned int div_a = (unsigned int) (I2C_CLOCK /1000);
    unsigned int div_b = (4 *bus_baud_rate_KHZ);    
    SSPADD = (unsigned char)(div_a/div_b)-1;
}

#ifdef I2C2_AVAILABLE 
void I2C2_baud_rate (unsigned int bus_baud_rate_KHZ){
    unsigned int div_a = (unsigned int) (I2C_CLOCK /1000);
    unsigned int div_b = (4 *bus_baud_rate_KHZ); 
    SSP2ADD = (unsigned char) (div_a/div_b)-1;
}
#endif

//************************************************************
//      I2C1_set_slave_address function implementation
//************************************************************
void I2C1_set_slave_address (unsigned int device_address){
    
    //Only one byte address is supported
    SSPADD = (unsigned char) device_address;
}

#ifdef I2C2_AVAILABLE 
void I2C2_set_slave_address (unsigned int device_address){
    SSP2ADD = (unsigned char) device_address;
}
#endif




//************************************************************
//         I2C1_start_bit function implementation
//************************************************************
void I2C1_start_bit (void){
    
    SSPCON2bits.SEN=1;
    //Waits util start bit is in idle
    while(SSPCON2bits.SEN);
}

 
#ifdef I2C2_AVAILABLE 
void I2C2_start_bit (void){
    SSP2CON2bits.SEN=1;
    //Waits util start bit is in idle
    while(SSP2CON2bits.SEN);
}
#endif   
    

//************************************************************
//         I2C1_stop_bit function implementation
//************************************************************  
    
void I2C1_stop_bit (void){
    SSPCON2bits.PEN=1;
    while(SSPCON2bits.PEN);
}

 
#ifdef I2C2_AVAILABLE 
void I2C2_stop_bit (void){

    SSP2CON2bits.PEN=1;
    while(SSP2CON2bits.PEN);
}
#endif 
   
//************************************************************
//         I2C1_write_byte function implementation
//************************************************************      
signed char I2C1_write_byte (unsigned char byte_to_send){
      
    //Writes the data to the output buffer
    SSPBUF = byte_to_send;           
    
    //Check for collision errors
    if ( SSPCON1bits.WCOL )      
      return ( -1 );              
    else {
    
    //slave mode only 
    if( ((SSPCON1&0x0F)!=0x08) && ((SSPCON1&0x0F)!=0x0B) ) {

      // release clock line   
      SSPCON1bits.CKP = 1;       
      
      // wait until the nineth clock pulse is received
      while ( !PIR1bits.SSPIF );  
      
      // if R/W=0 and BF=0, NOT ACK was received
      if ( ( !SSPSTATbits.R_W ) && ( !SSPSTATbits.BF )){
        return ( -2 );             
      }	
	  else 
          return(0);		
	
    }
	else 
        
    //master mode only	    
    if( ((SSPCON1&0x0F)==0x08) || ((SSPCON1&0x0F)==0x0B) ) {
        
        // wait until write cycle is complete    
		while( SSPSTATbits.BF );   
        
		I2C1_wait_bus_IDLE()
        ;                  
       
        // test for ACK condition received
	    if (SSPCON2bits.ACKSTAT ) 
	    	 return ( -2 );				
		else 
            return ( 0 );             
	}
  }
  
  //Unknown Error  
  return ( -3 );  
}

#ifdef I2C2_AVAILABLE 
signed char I2C2_write_byte (unsigned char byte_to_send){
    
    //Writes the data to the output buffer
    SSP2BUF = byte_to_send;           
    
    //Check for collision errors
    if ( SSP2CON1bits.WCOL )      
      return ( -1 );              
    else {
    
    //slave mode only 
    if( ((SSP2CON1&0x0F)!=0x08) && ((SSP2CON1&0x0F)!=0x0B) ) {
      // release clock line 
      SSP2CON1bits.CKP = 1;        
      
      // wait until ninth clock pulse received
      while ( !PIR3bits.SSP2IF );  
      
      // if R/W=0 and BF=0, NOT ACK was received
      if ( ( !SSP2STATbits.R_W ) && ( !SSP2STATbits.BF )){
        return ( -2 );             
      }	
	  else 
          return(0);		
	
    }
	else 
        
    //master mode only	    
    if( ((SSP2CON1&0x0F)==0x08) || ((SSP2CON1&0x0F)==0x0B) ) {
        
        // wait until write cycle is complete   
		while( SSP2STATbits.BF );    
        
		I2C2_wait_bus_IDLE()
        ; 
        
        // test for ACK condition received
	    if ( SSP2CON2bits.ACKSTAT ) 
	    	 return ( -2 );				
		else 
            return ( 0 );              
	}
  }
    
  //Unknown Error  
  return ( -3 );  
}
#endif        
    

//************************************************************
//     I2C1_reset_write_collision_flag function implementation
//************************************************************     
void I2C1_reset_write_collision_flag (void){
    SSPCON1bits.WCOL=0;
}
 
#ifdef I2C2_AVAILABLE 
void I2C2_reset_write_collision_flag (void){
    SSP2CON1bits.WCOL=0;       
}
#endif   


//************************************************************
//   I2C1_write_byte_to_external_device function implementation
//************************************************************  
signed char I2C1_write_byte_to_external_device (unsigned char control_byte, unsigned char register_address, unsigned char data){
  
    I2C1_wait_bus_IDLE();                     
    I2C1_start_bit();                     
  
    // test for bus collision
    if ( PIR2bits.BCLIF ){
        return ( -1 );                
    }
    else {
        // write byte - R/W bit should be 0    
        if ( I2C1_write_byte(control_byte)){
          I2C1_stop_bit(); 
          return (-3);              
        }

        // test for ACK condition received
        if ( !SSPCON2bits.ACKSTAT )   
        { 
          if (I2C1_write_byte(register_address)) {
            I2C1_stop_bit(); 
            return ( -3 );            
          }
        // test for ACK condition received
        if ( !SSPCON2bits.ACKSTAT ){ 
            if (I2C1_write_byte(data)) {
              I2C1_stop_bit(); 
              return ( -3 );          
            }
        }else {
            I2C1_stop_bit(); 
            return ( -2 );            
          }
        } else {
          I2C1_stop_bit(); 
          return ( -2 );              
        }
    }

    I2C1_stop_bit();                      

    // test for bus collision
    if ( PIR2bits.BCLIF ){
        return ( -1 );                
    }
    
    return ( 0 );                   
}
 
#ifdef I2C2_AVAILABLE 
signed char  I2C2_write_byte_to_external_device (unsigned char control_byte, unsigned char register_address, unsigned char data){
    
    I2C2_wait_bus_IDLE();                     
    I2C2_start_bit();                     
  
    // test for bus collision
    if (PIR3bits.BCL2IF){
        return (-1);                
    }
    else {
        // write byte - R/W bit should be 0    
        if ( I2C2_write_byte(control_byte)){
          I2C2_stop_bit(); 
          return (-3);              
        }

        // test for ACK condition received
        if (!SSP2CON2bits.ACKSTAT){ 
            
            if (I2C2_write_byte(register_address)) {
                I2C2_stop_bit(); 
                return (-3);            
            }
          
            // test for ACK condition received
            if ( !SSP2CON2bits.ACKSTAT ){ 
                if (I2C2_write_byte(data)) {
                  I2C2_stop_bit(); 
                  return (-3);          
                }
        }else {
            I2C2_stop_bit(); 
            return (-2);            
          }
        } else {
          I2C2_stop_bit(); 
          return (-2);              
        }
    }

    I2C2_stop_bit();                      

    // test for bus collision
    if (PIR3bits.BCL2IF){
        return (-1);                
    }
    
    return (0);       
}
#endif  

//************************************************************
//   I2C1_read_byte_from_external_device function implementation
//************************************************************  

signed char I2C1_read_byte_from_external_device (unsigned char control_byte, unsigned char register_address, unsigned char *data){
   
    I2C1_wait_bus_IDLE();                     
    I2C1_start_bit();                     

    // test for bus collision
    if (PIR2bits.BCLIF){
        return (-1);                
    } else {
        if (I2C1_write_byte(control_byte)) {
          I2C1_stop_bit(); 
          return (-3);             
        }
        
        // Check for ACK condition
        if (!SSPCON2bits.ACKSTAT){
           // WRITE word address for EEPROM 
          if (I2C1_write_byte(register_address)){
            I2C1_stop_bit(); 
            return (-3);            
          }
        
          // Check for ACK condition
          if (!SSPCON2bits.ACKSTAT) {
              
            I2C1_restart_communication();            
            
            // test for bus collision
            if ( PIR2bits.BCLIF ){
              return (-1);         
            }

            // write 1 byte - R/W bit should be 1
            if (I2C1_write_byte(control_byte+1)){
              I2C1_stop_bit(); 
              return (-3);         
            }

            // Check for ACK condition
            if (!SSPCON2bits.ACKSTAT ){
              // enable master for 1 byte reception  
              SSPCON2bits.RCEN = 1;   
              // check that receive sequence is over
              while ( SSPCON2bits.RCEN ); 

              // send ACK condition
              I2C1_negative_ACK();    
              
              // send STOP condition
              I2C1_stop_bit();             

              // test for bus collision
              if (PIR2bits.BCLIF) {
               return (-1);         
              }
            } else {
              I2C1_stop_bit(); 
              return (-2);          
            }
          } else {
            I2C1_stop_bit(); 
            return (-2);           
          }
        }
        else
        {
          I2C1_stop_bit(); 
          return (-2);              
        }
    }
    
    // if no error occured the data is valid!
    *data = SSPBUF;
    
    return (0);     
}
 
#ifdef I2C2_AVAILABLE 
signed char  I2C2_read_byte_from_external_device (unsigned char control_byte, unsigned char register_address, unsigned char *data){
    I2C2_wait_bus_IDLE();                     
    I2C2_start_bit();                     

    // test for bus collision
    if (PIR3bits.BCL2IF){
        return (-1);                
    } else {
        if (I2C2_write_byte(control_byte)) {
          I2C2_stop_bit(); 
          return (-3);             
        }
        
        // Check for ACK condition
        if (!SSP2CON2bits.ACKSTAT){
           // WRITE word address for EEPROM 
          if (I2C2_write_byte(register_address)){
            I2C2_stop_bit(); 
            return (-3);            
          }
        
          // Check for ACK condition
          if (!SSP2CON2bits.ACKSTAT) {
              
            I2C2_restart_communication();            
            
            // test for bus collision
            if (PIR3bits.BCL2IF){
              return (-1);         
            }

            // write 1 byte - R/W bit should be 1
            if (I2C2_write_byte(control_byte+1)){
              I2C1_stop_bit(); 
              return (-3);         
            }

            // Check for ACK condition
            if (!SSP2CON2bits.ACKSTAT){
              // enable master for 1 byte reception  
              SSP2CON2bits.RCEN = 1;   
              // check that receive sequence is over
              while (SSP2CON2bits.RCEN); 

              // send ACK condition
              I2C2_negative_ACK();    
              
              // send STOP condition
              I2C2_stop_bit();             

              // test for bus collision
              if (PIR3bits.BCL2IF) {
               return (-1);         
              }
            } else {
              I2C2_stop_bit(); 
              return (-2);          
            }
          } else {
            I2C2_stop_bit(); 
            return (-2);           
          }
        }
        else {
          I2C2_stop_bit(); 
          return (-2);              
        }
    }
    
    // if no error occured the data is valid!
    *data = SSP2BUF;
    
    return (0);  
}
#endif
