/*******************************************************************************
 
Autore : Mauro Laurenti
Versione : 1.2
 
Created on Date : 15/01/2016
Last update     : 23/01/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 
*******************************************************************************/

#ifdef COMPILER_XC
	#include <xc.h>
#endif

#include "module_IO.h"


//************************************************************
//      IO_set_all_port_as_inputs function implementation
//************************************************************
void IO_set_all_ports_as_inputs (void){
	
	#if (NUMBER_OF_IO_PORTS == 5)

        #ifdef _PIC18
            LATA = 0x00;
            LATB = 0x00;
            LATC = 0x00;
            LATD = 0x00;
            LATE = 0x00;    
        #endif

        #ifndef _PIC18
            PORTA = 0x00;
            PORTB = 0x00;
            PORTC = 0x00;
            PORTD = 0x00;
            PORTE = 0x00;     
        #endif               

        TRISA = 0xFF;        
		TRISB = 0xFF;
		TRISC = 0xFF;        
		TRISD = 0xFF;        
		TRISE = 0xFF;     
        
        //Disable ADC buffer        
        #ifdef _PIC18
            
            #ifdef ADC_WITH_ANSEL_REG
                ANSELA = 0x00;
                ANSELB = 0x00;
                ANSELC = 0x00;
                ANSELD = 0x00;
                ANSELE = 0x00;
            #endif  
            
            #ifdef ADC_WITH_ANSEL_2_REG
                ANSEL = 0x00;
                ANSELH = 0x00;
            #endif     
                
            #ifdef ADC_WITH_NO_ANSEL_REG
                //ADCON1 = 0x0F;                
            #endif  
                
        #endif

        #ifndef _PIC18
           
            #ifdef ADC_WITH_ANSEL_REG
                ANSEL = 0x00;
                ANSELH = 0x00;
            #endif

            #ifdef ADC_WITH_NO_ANSEL_REG
                ADCON1 = 0x07;
            #endif    
        #endif   
        
	#endif

                
    #if (NUMBER_OF_IO_PORTS == 3)

        #ifdef _PIC18
            LATA = 0x00;
            LATB = 0x00;
            LATC = 0x00; 
        #endif

        #ifndef _PIC18
            PORTA = 0x00;
            PORTB = 0x00;
            PORTC = 0x00;  
        #endif               

        TRISA = 0xFF;        
		TRISB = 0xFF;
		TRISC = 0xFF;        
        
        //Disable ADC buffer        
        #ifdef _PIC18
            
            #ifdef ADC_WITH_ANSEL_REG
                ANSELA = 0x00;
                ANSELB = 0x00;
                ANSELC = 0x00;
            #endif  
            
            #ifdef ADC_WITH_ANSEL_2_REG
                ANSEL = 0x00;
                ANSELH = 0x00;
            #endif    
                
            #ifdef ADC_WITH_NO_ANSEL_REG
                //ADCON1 = 0x0F;                
            #endif   
                
        #endif

        #ifndef _PIC18
           
            #ifdef ADC_WITH_ANSEL_REG
                ANSEL = 0x00;
                ANSELH = 0x00;
            #endif

            #ifndef ADC_WITH_ANSEL_REG
                ADCON1 = 0x07;
            #endif    
        #endif   
        
	#endif
	 	
}


//************************************************************
//      IO_set_port_direction function implementation
//************************************************************
void IO_set_port_direction (uint8_t mcu_port, port_size port_direction){
   
    #if (NUMBER_OF_IO_PORTS == 5)

        if (mcu_port == IO_PORTA){
            TRISA = port_direction;
        }

        if (mcu_port == IO_PORTB){
            TRISB = port_direction;
        }

        if (mcu_port == IO_PORTC){
            TRISC = port_direction;
        }

        if (mcu_port == IO_PORTD){
            TRISD = port_direction;
        }

        if (mcu_port == IO_PORTE){
            TRISE = port_direction;
        }
                
    #endif  
        
    
    #if (NUMBER_OF_IO_PORTS == 3)

        if (mcu_port == IO_PORTA){
            TRISA = port_direction;
        }

        if (mcu_port == IO_PORTB){
            TRISB = port_direction;
        }

        if (mcu_port == IO_PORTC){
            TRISC = port_direction;
        }
                
    #endif          
        
}


//************************************************************
//      IO_set_port_direction function implementation
//************************************************************
void IO_write_port (uint8_t mcu_port, port_size port_data){
    
    #if (NUMBER_OF_IO_PORTS == 5)

        #ifdef _PIC18
            if (mcu_port == IO_PORTA){
                LATA = port_data;
            }

            if (mcu_port == IO_PORTB){
                LATB = port_data;
            }

            if (mcu_port == IO_PORTC){
                LATC = port_data;
            }

            if (mcu_port == IO_PORTD){
                LATD = port_data;
            }

            if (mcu_port == IO_PORTE){
                LATE = port_data;
            }                
        #endif

        #ifndef _PIC18
            if (mcu_port == IO_PORTA){
                PORTA = port_data;
            }

            if (mcu_port == IO_PORTB){
                PORTB = port_data;
            }

            if (mcu_port == IO_PORTC){
                PORTC = port_data;
            }

            if (mcu_port == IO_PORTD){
                PORTD = port_data;
            }

            if (mcu_port == IO_PORTE){
                PORTE = port_data;
            } 
        #endif     
            
	#endif

    #if (NUMBER_OF_IO_PORTS == 3)

        #ifdef _PIC18
            if (mcu_port == IO_PORTA){
                LATA = port_data;
            }

            if (mcu_port == IO_PORTB){
                LATB = port_data;
            }

            if (mcu_port == IO_PORTC){
                LATC = port_data;
            }
              
        #endif

        #ifndef _PIC18
            if (mcu_port == IO_PORTA){
                PORTA = port_data;
            }

            if (mcu_port == IO_PORTB){
                PORTB = port_data;
            }

            if (mcu_port == IO_PORTC){
                PORTC = port_data;
            }
            
        #endif     
            
	#endif   
            
}


//************************************************************
//      IO_read_port function implementation
//************************************************************
port_size IO_read_port (uint8_t mcu_port){
    
    #if (NUMBER_OF_IO_PORTS == 5)

        if (mcu_port == IO_PORTA){
            return (PORTA);
        }

        if (mcu_port == IO_PORTB){
            return (PORTB);
        }

        if (mcu_port == IO_PORTC){
            return (PORTC);
        }

        if (mcu_port == IO_PORTD){
            return (PORTD);
        }

        if (mcu_port == IO_PORTE){
            return (PORTE);
        }    
    #endif 

        
    #if (NUMBER_OF_IO_PORTS == 3)

        if (mcu_port == IO_PORTA){
            return (PORTA);
        }

        if (mcu_port == IO_PORTB){
            return (PORTB);
        }

        if (mcu_port == IO_PORTC){
            return (PORTC);
        }
        
    #endif     
    
   return (0);     
}


//************************************************************
//      IO_read_port_bit function implementation
//************************************************************
port_size IO_read_port_bit (uint8_t mcu_port, port_size bit_mask ){

    port_size port_data = 0;

    #if (NUMBER_OF_IO_PORTS == 5)

        if (mcu_port == IO_PORTA){
            port_data = PORTA;
        }

        if (mcu_port == IO_PORTB){
            port_data = PORTB;
        }

        if (mcu_port == IO_PORTC){
            port_data = PORTC;
        }

        if (mcu_port == IO_PORTD){
            port_data = PORTD;
        }

        if (mcu_port == IO_PORTE){
            port_data = PORTE;
        }    

        if ( (port_data & bit_mask) == 0){
            return (0);
        } else {
            return (1);
        }

    #endif 

    
    #if (NUMBER_OF_IO_PORTS == 3)

        if (mcu_port == IO_PORTA){
            port_data = PORTA;
        }

        if (mcu_port == IO_PORTB){
            port_data = PORTB;
        }

        if (mcu_port == IO_PORTC){
            port_data = PORTC;
        }

        if ( (port_data & bit_mask) == 0){
            return (0);
        } else {
            return (1);
        }

    #endif 
  
}

//************************************************************
//      IO_enable_pull_up_resitors function implementation
//************************************************************
void IO_enable_pull_up_resistors (uint8_t mcu_port, port_size resistors_to_enable) {
    
     
    #ifdef PULL_UP_ENABLE_REGISTER_A
        if (mcu_port == IO_PORTA){
            #ifdef PULL_UP_SINGLE_BIT_ENABLE
                PULL_UP_ENABLE_REGISTER_A = resistors_to_enable;
            #endif  
        } 
    #endif 
    
    
    if (mcu_port == IO_PORTB){
        //Enable the resistor if at least one is required
        #ifdef PULL_UP_ENABLE_BIT
        if (resistors_to_enable != 0)
            PULL_UP_ENABLE_BIT = 0x00;
        else
            PULL_UP_ENABLE_BIT = 0x01;
        #endif 

        #ifdef PULL_UP_SINGLE_BIT_ENABLE
            PULL_UP_ENABLE_REGISTER_B = resistors_to_enable;
        #endif  
    }    
    
    #ifdef PULL_UP_ENABLE_REGISTER_C
        if (mcu_port == IO_PORTC){
            #ifdef PULL_UP_SINGLE_BIT_ENABLE
                PULL_UP_ENABLE_REGISTER_C = resistors_to_enable;
            #endif  
        } 
    #endif     
    
        
    #ifdef PULL_UP_ENABLE_REGISTER_D   
        if (mcu_port == IO_PORTD){
            #ifdef PULL_UP_SINGLE_BIT_ENABLE
                PULL_UP_ENABLE_REGISTER_D = resistors_to_enable;
            #endif  
        }
    #endif 
   
}
