/*******************************************************************************

Author : Mauro Laurenti
Version : 1.2
Created on Date : 09/03/2016
Last update     : 31/01/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This library contains all the functions that are handy for controlling 			
the SPI module
*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "module_SPI.h"


//************************************************************
//               SPI_open  Implementation
//************************************************************
void SPI1_open (uint8_t device_type){
    uint8_t data;  
    
    //IO pin initialization    
	#ifndef MCC_USED
		if (device_type == SPI_MASTER_DEVICE){
			SPI1_SCK_LINE_TRIS = PIN_AS_OUTPUT;            
			SPI1_SDI_LINE_TRIS = PIN_AS_INPUT; 
			SPI1_SDO_LINE_TRIS = PIN_AS_OUTPUT;   
			SPI1_SS_LINE_TRIS = PIN_AS_OUTPUT;  
		} 
	   
		if (device_type == SPI_SLAVE_DEVICE){
			SPI1_SCK_LINE_TRIS = PIN_AS_INPUT;            
			SPI1_SDI_LINE_TRIS = PIN_AS_INPUT; 
			SPI1_SDO_LINE_TRIS = PIN_AS_OUTPUT;   
			SPI1_SS_LINE_TRIS = PIN_AS_INPUT;  
			
			//SS initialization by Slave mode
			SSPCON1 &= 0xF0;
			SSPCON1 |= SPI1_SS_PIN;        
		} 
	#endif
   
    //Clear the register by reading it out
    data = SSPBUF;    
    
    //Clear history
    PIR1bits.SSPIF = 0;        
    SSPCON1bits.WCOL = 0;	
    SSPCON1bits.SSPOV = 0;
    
    //Data sampling in the midle of the bit
    SSPSTATbits.SMP = 0x00;
    
    //Enable SPI module 
    SSPCON1bits.SSPEN = 0x01; 
}

#ifdef SPI2_AVAILABLE
    void SPI2_open (uint8_t device_type){
        uint8_t data;  

        //IO pin initialization    
		#ifndef MCC_USED
			if (device_type == SPI_MASTER_DEVICE){
				SPI2_SCK_LINE_TRIS = PIN_AS_OUTPUT;            
				SPI2_SDI_LINE_TRIS = PIN_AS_INPUT; 
				SPI2_SDO_LINE_TRIS = PIN_AS_OUTPUT;   
				SPI2_SS_LINE_TRIS = PIN_AS_OUTPUT;  
			} 

			if (device_type == SPI_SLAVE_DEVICE){
				SPI2_SCK_LINE_TRIS = PIN_AS_INPUT;            
				SPI2_SDI_LINE_TRIS = PIN_AS_INPUT; 
				SPI2_SDO_LINE_TRIS = PIN_AS_OUTPUT;   
				SPI2_SS_LINE_TRIS = PIN_AS_INPUT;  

				//SS initialization by Slave mode
				SSP2CON1 &= 0xF0;
				SSP2CON1 |= SPI2_SS_PIN;

		   } 
	   #endif

        //Clear the register by reading it out
        data = SSP2BUF;    

        //Clear history
        PIR3bits.SSP2IF = 0;        
        SSP2CON1bits.WCOL = 0;	
        SSP2CON1bits.SSPOV = 0;

        //Data sampling in the midle of the bit
        SSP2STATbits.SMP = 0x00;

        //Enable SPI module 
        SSP2CON1bits.SSPEN = 0x01; 
    }
#endif


//************************************************************
//               SPI1_close  Implementation
//************************************************************
void SPI1_close (void){
   
   //Enable SPI module 
   SSPCON1bits.SSPEN = 0x00;
}

#ifdef SPI2_AVAILABLE
    void SPI2_close (void){

       //Enable SPI module 
       SSP2CON1bits.SSPEN = 0x00;
    }
#endif

//************************************************************
//               SPI1_baudrate  Implementation
//************************************************************
void SPI1_baudrate (uint8_t baudrate){
    
    SSPCON1 &= 0xFC;     //0b11111100
    SSPCON1 = baudrate;
}

#ifdef SPI2_AVAILABLE
    void SPI2_baudrate (uint8_t baudrate){

        SSP2CON1 &= 0xFC;   //0b11111100
        SSP2CON1 = baudrate;
    }
#endif

//************************************************************
//               SPI1_mode  Implementation
//************************************************************
void SPI1_mode (uint8_t communication_mode){
    
    SSPSTATbits.CKE = 0x00;
    
    
    switch (communication_mode){
        case SPI_MODE_0: // Data is sent on the rising edge
                        SSPSTATbits.CKE = 0x01;  
                        SSPCON1bits.CKP = 0x00;
                        break;
                        
        case SPI_MODE_1: 
                        SSPSTATbits.CKE = 0x00;  
                        SSPCON1bits.CKP = 0x00;      
                        break;
                        
        case SPI_MODE_2: 
                        SSPSTATbits.CKE = 0x01;  
                        SSPCON1bits.CKP = 0x01;        
                        break;
                        
        case SPI_MODE_3:
                        SSPSTATbits.CKE = 0x00;  
                        SSPCON1bits.CKP = 0x01; 
                        break;                
    }            
}

#ifdef SPI2_AVAILABLE
    void SPI2_mode (uint8_t communication_mode){

        SSP2STATbits.CKE = 0x00;

        switch (communication_mode){
            case SPI_MODE_0: // Data is sent on the rising edge
                            SSP2STATbits.CKE = 0x01;  
                            SSP2CON1bits.CKP = 0x00;
                            break;

            case SPI_MODE_1: 
                            SSP2STATbits.CKE = 0x00;  
                            SSP2CON1bits.CKP = 0x00;      
                            break;

            case SPI_MODE_2: 
                            SSP2STATbits.CKE = 0x01;  
                            SSP2CON1bits.CKP = 0x01;        
                            break;

            case SPI_MODE_3:
                            SSP2STATbits.CKE = 0x00;  
                            SSP2CON1bits.CKP = 0x01; 
                            break;                
        }            
    }
#endif

//************************************************************
//               SPI1_write_byte  Implementation
//************************************************************
signed char SPI1_write_byte (uint8_t byte_to_send){
  
    uint8_t data;  
  
    //Clear the register by reading it out
    data = SSPBUF;           

    //Clear history
    PIR1bits.SSPIF = 0;        
    SSPCON1bits.WCOL = 0;			

    //Send the data out
    SSPBUF = byte_to_send;  
    
    //Check for bus collision
    if (SSPCON1bits.WCOL == 0x01){
        return (-1);   
    } 
    
    //Wait the end of transmission
    while(!SPI1_is_TX_over()); 

    //No error occured
    return (0);                
}

#ifdef SPI2_AVAILABLE
    int8_t SPI2_write_byte (uint8_t byte_to_send){

        uint8_t data;  

        //Clear the register by reading it out
        data = SSP2BUF;           

        //Clear history
        PIR3bits.SSP2IF = 0;        
        SSP2CON1bits.WCOL = 0;			

        //Send the data out
        SSP2BUF = byte_to_send;  

        //Check for bus collision
        if (SSP2CON1bits.WCOL == 0x01){
            return (1);   
        } 

        //Wait end of transmission
        while(!SPI2_is_TX_over()); 

        //No error occured
        return (0);                
    }
#endif


//************************************************************
//               SPI1_read_byte  Implementation
//************************************************************
uint8_t SPI1_read_byte (void){
  
    uint8_t data;
    data = SSPBUF;       
  
    //Clear history  
    PIR1bits.SSPIF = 0;      

    //Write 0x00 to initiate a write
    SSPBUF = 0x00;           

    //Wait end of transmission
    while(!SPI1_is_TX_over());  
    
    //Return the new byte out of the 0x00 sent out
    return (SSPBUF);      
}

#ifdef SPI2_AVAILABLE
    uint8_t SPI2_read_byte (void){

        uint8_t data;
        data = SSP2BUF;       

        //Clear history  
        PIR3bits.SSP2IF = 0;      

        //Write 0x00 to initiate a write
        SSP2BUF = 0x00;           

        //Wait end of transmission
        while(!SPI2_is_TX_over());  

        //Return the new byte out of the 0x00 sent out
        return (SSP2BUF);      
    }
#endif


//************************************************************
//               SPI1_write_read_byte  Implementation
//************************************************************
uint8_t SPI1_write_read_byte (uint8_t write_byte){
  
    uint8_t data;
    data = SSPBUF;       
  
    //Clear history  
    PIR1bits.SSPIF = 0;      

    //Write 0x00 to initiate a write
    SSPBUF = write_byte;           

    //Wait end of transmission
    while(!SPI1_is_TX_over());  
    
    //Return the new byte out of the 0x00 sent out
    return (SSPBUF);      
}

#ifdef SPI2_AVAILABLE
    uint8_t SPI2_write_read_byte (uint8_t write_byte){

        uint8_t data;
        data = SSP2BUF;       

        //Clear history  
        PIR3bits.SSP2IF = 0;      

        //Write 0x00 to initiate a write
        SSP2BUF = write_byte;           

        //Wait end of transmission
        while(!SPI2_is_TX_over());  

        //Return the new byte out of the 0x00 sent out
        return (SSP2BUF);      
    }
#endif


