/*******************************************************************************
 
Author : Mauro Laurenti
Version : 1.1
 
Created on Date : 25/11/2015
Last update     : 26/01/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "module_UART.h"


//************************************************************
//         UART1_open function implementation
//************************************************************
void UART1_open (uint16_t baud_rate){
    
    //IO Pin Initialization 
	#ifndef MCC_USED	
		UART1_TX_LINE_TRIS = PIN_AS_INPUT;            
		UART1_RX_LINE_TRIS = PIN_AS_INPUT; 
	#endif
    
    //Module Initialization
    TXSTA = 0x00;
    RCSTA =0x00;
            
    //Module funtional settings    
    //Slave
    TXSTAbits.CSRC = 0;  
    //Asyncrone
    TXSTAbits.SYNC = 0;
    //High Boud Rate
    TXSTAbits.BRGH = 1;     
    // 8 bit mode
    TXSTAbits.TX9 = 0;	
    RCSTAbits.RX9 = 0;

    //Continuos Reception
    RCSTAbits.CREN = 1;
    RCSTAbits.SREN = 0;      
    
    //Baud rate
            
    switch(baud_rate) {
        case UART_BAUDRATE_9600:
                                SPBRG = UART_BAUDRATE_9600;
                                break;
        case UART_BAUDRATE_19200:
                                SPBRG = UART_BAUDRATE_19200;
                                break;                                
        case UART_BAUDRATE_38400:
                                SPBRG = UART_BAUDRATE_38400;
                                break;
        case UART_BAUDRATE_57600:
                                SPBRG = UART_BAUDRATE_57600;
                                break;
        case UART_BAUDRATE_115200:
                                SPBRG = UART_BAUDRATE_115200;
                                break;
        default:

                                SPBRG = UART_BAUDRATE_9600;
    }
    
    // Enable TX
    TXSTAbits.TXEN = 1;  
    //Enable RX
    RCSTAbits.SPEN = 1;   
       
}

#ifdef UART2_AVAILABLE 

    void UART2_open (uint16_t baud_rate){

        //IO Pin Initialization
		#ifndef MCC_USED        
			UART2_TX_LINE_TRIS = PIN_AS_INPUT;            
			UART2_RX_LINE_TRIS = PIN_AS_INPUT;  
		#endif
        
        //Module Initialization
        TXSTA2 = 0x00;
        RCSTA2 =0x00;

        //Module funtional settings    
        //Slave
        TXSTA2bits.CSRC = 0;  
        //Asyncrone
        TXSTA2bits.SYNC = 0;
        //High Boud Rate
        TXSTA2bits.BRGH = 1;     
        // 8 bit mode
        TXSTA2bits.TX9 = 0;	
        RCSTA2bits.RX9 = 0;

        //Continuos Reception
        RCSTA2bits.CREN = 1;
        RCSTA2bits.SREN = 0;      

        //Baud rate

        switch(baud_rate) {
            case UART_BAUDRATE_9600:
                                    SPBRG2 = UART_BAUDRATE_9600;
                                    break;
            case UART_BAUDRATE_19200:
                                    SPBRG2 = UART_BAUDRATE_19200;
                                    break;                                    
            case UART_BAUDRATE_38400:
                                    SPBRG2 = UART_BAUDRATE_38400;
                                    break;
            case UART_BAUDRATE_57600:
                                    SPBRG2 = UART_BAUDRATE_57600;
                                    break;
            case UART_BAUDRATE_115200:
                                    SPBRG2 = UART_BAUDRATE_115200;
                                    break;
            default:

                                    SPBRG2 = UART_BAUDRATE_9600;
        }

        // Enable TX
        TXSTA2bits.TXEN = 1;  
        //Enable RX
        RCSTA2bits.SPEN = 1;   
    }
#endif

//************************************************************
//         UART1_write_byte function implementation
//************************************************************
void UART1_write_byte (uint8_t data) { 
  TXREG = data;
  while (UART1_TX_busy( ));
}

#ifdef UART2_AVAILABLE 
    void UART2_write_byte (uint8_t data) { 
      TXREG2 = data;   
      while (UART2_TX_busy( ));
    }
#endif
    
//************************************************************
//         UART1_write_message function implementation
//************************************************************
 void UART1_write_message(const char *data){
   
    while(*data) {	    
        UART1_write_byte(*data);
        while(UART1_TX_busy());
        data++;              
    }  
}

#ifdef UART2_AVAILABLE 
    void UART2_write_message(const char *data){

        while(*data) {	    
            UART2_write_byte(*data);
            while(UART2_TX_busy());
            data++;              
        }  
    }
#endif

//************************************************************
//         UART1_write_string function implementation
//************************************************************
void UART1_write_string(uint8_t *data){
   
    while(*data) {	    
        UART1_write_byte(*data);
        while(UART1_TX_busy());
        data++;              
    }  
}

#ifdef UART2_AVAILABLE 
    void UART2_write_string(uint8_t *data){

        while(*data) {	    
            UART2_write_byte(*data);
            while(UART2_TX_busy());
            data++;              
        }  
    }
#endif
    
//************************************************************
//         UART1_read_byte function implementation
//************************************************************
uint8_t UART1_read_byte(void){
    
    uint8_t data;   

    data = RCREG;                      

    return (data);                     
}

#ifdef UART2_AVAILABLE 
    uint8_t UART2_read_byte(void){

        uint8_t data;   

        data = RCREG2;                      

        return (data);                     
    }
#endif
